﻿using HarmonyLib;
using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

namespace RIMMSqol
{
    [HarmonyPatch(typeof(ExpectationsUtility))]
    [HarmonyPatch("CurrentExpectationFor")]
    [HarmonyPatch(new[] { typeof(Pawn) })]
    [HarmonyPatchNamespace("PawnValueExpectations")]
    static class ExpectationsUtility_CurrentExpectationFor
    {
        static bool usePawnValueExpectations = false;
        static float[] wealthThresholds;

        static void Prepare()
        {
            QOLMod.addApplySettingsListener(mod =>
            {
                QOLModSettings settings = QOLMod.getSettings();
                wealthThresholds = new float[]{ settings.pawnValueExpectationsMV0, settings.pawnValueExpectationsMV1, settings.pawnValueExpectationsMV2,
                    settings.pawnValueExpectationsMV3, settings.pawnValueExpectationsMV4, settings.pawnValueExpectationsMV5};
                usePawnValueExpectations = false;
                for ( int i = 0; i < wealthThresholds.Length; i++ )
                {
                    if (wealthThresholds[i] < 0) wealthThresholds[i] = float.MaxValue;
                    else usePawnValueExpectations = true;
                }
            });
        }

        [HarmonyPriority(Priority.Last)]
        static void Postfix(Pawn p, ref ExpectationDef __result)
        {
            if (!usePawnValueExpectations) return;

            float value;
            try
            {
                value = p.GetStatValue(StatDefOf.MarketValue);
            } catch (Exception)
            {
                return;
            }

            int order = -1; //0 - 5

            for (int i = wealthThresholds.Length - 1; i >= 0; i--)
            {
                if ( value >= wealthThresholds[i] )
                {
                    order = i;
                    break;
                }
            }

            if (order < 0) return;

            foreach (ExpectationDef exp in DefDatabase<ExpectationDef>.AllDefsListForReading)
            {
                if ( exp.order == order && exp.WealthTriggered )
                {
                    if ( __result == null || __result.order < exp.order )
                    {
                        __result = exp;
                    }
                    return;
                }
            }
        }
    }
}
