﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 12.07.2019
 * Time: 02:43
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Reflection;
using HarmonyLib;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol
{
	/// <summary>
	/// Description of FloatMenuDecorations.
	/// </summary>
	[HarmonyPatch(typeof(Designator_Dropdown))]
	[HarmonyPatch("ProcessInput")]
	[HarmonyPatchNamespace("FloatMenuDecorations")]
	public static class FloatMenuDecorations_Designator_Dropdown_ProcessInput
	{
		static Func<Designator_Build, BuildableDef> getEntDef = UtilReflection.CreateGetter<Designator_Build, BuildableDef>(typeof(Designator_Build).GetField("entDef", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public));
		static Func<FloatMenu, List<FloatMenuOption>> getOptions = UtilReflection.CreateGetter<FloatMenu, List<FloatMenuOption>>(typeof(FloatMenu).GetField("options", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public));
		
		static void Postfix(Designator_Dropdown __instance, List<Designator> ___elements, Designator ___activeDesignator) {
			FloatMenu fm = Find.WindowStack.FloatMenu;
			if ( fm != null ) {
				List<FloatMenuOption> options = getOptions(fm);
				
				if ( !options.NullOrEmpty() && options.Count > 1 ) {
					bool isDropdown;
					
					using ( List<Designator>.Enumerator enDes = ___elements.GetEnumerator() ) {
						isDropdown = true;
						foreach ( FloatMenuOption fmo in options ) {
							if ( !enDes.MoveNext() ) {
								//No more designators found
								isDropdown = false;
								break;
							}
							while ( !enDes.Current.Visible ) {
								if ( !enDes.MoveNext() ) {
									//No more designators found while skipping invisible entries
									isDropdown = false;
									break;
								}
							}
							if ( !isDropdown ) break;
							if ( enDes.Current.LabelCap != fmo.Label ) {
								//Unexpected entry
								isDropdown = false;
								break;
							}
						}
						if ( enDes.MoveNext() ) {
							bool reachedEnd = false;
							while ( !enDes.Current.Visible ) {
								if ( !enDes.MoveNext() ) {
									reachedEnd = true;
									break;
								}
							}
							if ( !reachedEnd ) {
								//too many designators
								isDropdown = false;
							}
						}
					}
					
					if ( isDropdown ) {
						List<FloatMenuOption> newOptions = new List<FloatMenuOption>();
						using ( List<Designator>.Enumerator enumerator = ___elements.GetEnumerator() ) {
							foreach ( FloatMenuOption fmo in options ) {
								enumerator.MoveNext();
								while ( !enumerator.Current.Visible ) {
									enumerator.MoveNext();
								}
								Designator des = enumerator.Current;
								Material m = SolidColorMaterials.SimpleSolidColorMaterial(des.IconDrawColor);
								newOptions.Add(new FloatMenuOption(fmo.Label,fmo.action,fmo.Priority,fmo.mouseoverGuiAction,fmo.revalidateClickTarget,30f,r=>{
								                                   	r.x = fm.windowRect.width;
								                                   	GUI.color = des.IconDrawColor;
								                                   	Widgets.DrawTextureFitted(r, des.icon, 0.85f, des.iconProportions, des.iconTexCoords, des.iconAngle, 
								                                   	                          TexUI.GrayscaleGUI);
								                                   	GUI.color = Color.white;
								                                   	return false;
								                                   },fmo.revalidateWorldClickTarget));
							}
						}
						FloatMenu newFm = new FloatMenu(newOptions);
						newFm.vanishIfMouseDistant = true;
						newFm.onCloseCallback = fm.onCloseCallback;
						fm.onCloseCallback = null;
						Find.WindowStack.Add(newFm);
					}
				}
			}
		}
	}
	
	[HarmonyPatch(typeof(Designator_Build))]
	[HarmonyPatch("ProcessInput")]
	[HarmonyPatchNamespace("FloatMenuDecorations")]
	public static class FloatMenuDecorations_Designator_Build_ProcessInput
	{
		static Func<FloatMenu, List<FloatMenuOption>> getOptions = UtilReflection.CreateGetter<FloatMenu, List<FloatMenuOption>>(typeof(FloatMenu).GetField("options", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public));
		
		static void Postfix(Designator_Build __instance, BuildableDef ___entDef) {
			FloatMenu fm = Find.WindowStack.FloatMenu;
			if ( fm != null ) {
				List<FloatMenuOption> options = getOptions(fm);
				
				if ( !options.NullOrEmpty() && options.Count > 1 ) {
					bool isMaterialSelection;
					List<ThingDef> materials = null;
					
					isMaterialSelection = false;
					Designator_Build desiBuild = __instance;
					if ( desiBuild != null ) {
						BuildableDef thingDef = ___entDef;
						if ( thingDef != null && thingDef.MadeFromStuff ) {
							isMaterialSelection = true;
							using ( List<FloatMenuOption>.Enumerator enOpt = options.GetEnumerator() ) {
								materials = new List<ThingDef>();
								foreach (ThingDef current in Find.CurrentMap.resourceCounter.AllCountedAmounts.Keys) {
									if (current.IsStuff && current.stuffProps.CanMake(thingDef) && (DebugSettings.godMode || Find.CurrentMap.listerThings.ThingsOfDef(current).Count > 0)) {
										string label = GenLabel.ThingLabel(thingDef, current, 1).CapitalizeFirst();
										if ( !enOpt.MoveNext() || enOpt.Current.Label != label ) {
											isMaterialSelection = false;
											break;
										} else {
											materials.Add(current);
										}
									}
								}
								if ( enOpt.MoveNext() ) {
									isMaterialSelection = false;
								}
							}
						}
					}
					
					if ( isMaterialSelection ) {
						List<FloatMenuOption> newOptions = new List<FloatMenuOption>();
						using ( List<ThingDef>.Enumerator enumerator = materials.GetEnumerator() ) {
							foreach ( FloatMenuOption fmo in options ) {
								enumerator.MoveNext();
								ThingDef mat = enumerator.Current;
								Color cOrig = mat.stuffProps.color;
								Color cTrans = new Color(cOrig.r,cOrig.g,cOrig.b,0.5f);
								Material m = SolidColorMaterials.SimpleSolidColorMaterial(mat.stuffProps.color, true);
								
								newOptions.Add(new FloatMenuOption(fmo.Label,fmo.action,fmo.Priority,fmo.mouseoverGuiAction,fmo.revalidateClickTarget,30f,r=>{
								                                   	r.x = fm.windowRect.width;
								                                   	GenUI.DrawTextureWithMaterial(r, Command.BGTex, TexUI.GrayscaleGUI, default(Rect));
								                                   	GUI.color = mat.stuffProps.color;
								                                   	Widgets.DrawTextureFitted(r, mat.uiIcon, 0.85f * GenUI.IconDrawScale(mat), 
								                                   	                          mat.graphicData.drawSize.RotatedBy(mat.defaultPlacingRot),
								                                   	                          default(Rect), mat.uiIconAngle, TexUI.GrayscaleGUI);
								                                   	GUI.color = Color.white;
								                                   	return false;
								                                   },fmo.revalidateWorldClickTarget));
							}
						}
						FloatMenu newFm = new FloatMenu(newOptions);
						newFm.vanishIfMouseDistant = true;
						newFm.onCloseCallback = fm.onCloseCallback;
						fm.onCloseCallback = null;
						Find.WindowStack.Add(newFm);
					}
				}
			}
		}
	}
}
