﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HarmonyLib;
using System.Threading.Tasks;
using Verse;
using System.Xml;

namespace RIMMSqol
{
    [HarmonyPatch(typeof(LoadedModManager))]
    [HarmonyPatch("ParseAndProcessXML")]
    public static class LoadedModManager_ParseAndProcessXML
    {
        [HarmonyPriority(Priority.First)]
        static bool Prefix(XmlDocument xmlDoc, Dictionary<XmlNode, LoadableXmlAsset> assetlookup)
        {
            QOLMod.getModHandle().InitializeSettings();
            QOLModSettings settings = QOLMod.getSettings();
            if (settings == null) return true;

            if (settings.defCopies != null)
            {
                foreach (QOLModSettingsDefCopy defCopy in settings.defCopies)
                {
                    if (defCopy == null || !defCopy.IsValid) continue;
                    string defType = defCopy.defType, defNameOriginal = defCopy.defNameOriginal, defNameCopy = defCopy.defNameCopy, labelCopy = defCopy.labelCopy;
                    XmlNode originalDef = xmlDoc.SelectSingleNode("/Defs/" + defType + "[./defName[normalize-space() = '" + defNameOriginal + "']]");
                    //abstract nodes can be inherited but not IsNull with case insensitive "true"(pure sanity check should never occur)
                    if (originalDef != null && (originalDef.Attributes["IsNull"] == null || !"true".EqualsIgnoreCase(originalDef.Attributes["IsNull"].Value)))
                    {
                        string parentName;
                        XmlAttribute nameAttr = originalDef.Attributes["Name"];
                        if (nameAttr == null)
                        {
                            parentName = defType + "_" + defNameOriginal;
                            nameAttr = xmlDoc.CreateAttribute("Name");
                            nameAttr.Value = parentName;
                            originalDef.Attributes.Append(nameAttr);
                        }
                        else
                        {
                            parentName = nameAttr.Value;
                        }
                        XmlElement ele = xmlDoc.CreateElement(defType);
                        ele.SetAttribute("ParentName", parentName);
                        AppendTextNode(xmlDoc, ele, "defName", defNameCopy);
                        AppendTextNode(xmlDoc, ele, "label", labelCopy);
                        //if present we want to copy some of the originals attributes: "Class", "MayRequire"
                        nameAttr = originalDef.Attributes["Class"];
                        if (nameAttr != null) ele.SetAttribute("Class", nameAttr.Value);
                        nameAttr = originalDef.Attributes["MayRequire"];
                        if (nameAttr != null) ele.SetAttribute("MayRequire", nameAttr.Value);

                        xmlDoc.DocumentElement.AppendChild(ele);
                    }
                    else
                    {
                        Log.Warning("Failed to find original def of type \"" + defType + "\" with defName \"" + defNameOriginal + "\". Unable to create copy from it for defName \"" + defNameCopy + "\" and label \"" + labelCopy + "\".");
                    }
                }
            }

            return true;
        }

        static void AppendTextNode(XmlDocument doc, XmlNode parent, string name, string text)
        {
            XmlElement ele = doc.CreateElement(string.Empty, name, string.Empty);
            ele.AppendChild(doc.CreateTextNode(text));
            parent.AppendChild(ele);
        }
    }
}
