﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 24.08.2019
 * Time: 18:53
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol
{
	/// <summary>
	/// Description of MainButtonWorker_Dropdown.
	/// </summary>
	public class MainButtonWorker_Dropdown : MainButtonWorker
	{
        protected string toggleDescriptionWhenFloatMenuIsShowing = null;
        protected Rect lastDisplayedRect;

		public override bool Disabled {
			get {
				MainButtonDef_Dropdown d = def as MainButtonDef_Dropdown;
				if ( d == null ) return true;
				
				foreach (MainButtonDef button in d.buttons ) {
                    if (button == null) continue;
					if ( !button.Worker.Disabled ) return false;
				}
				
				return true;
			}
		}

        public override void DoButton(Rect rect)
        {
            base.DoButton(rect);
            lastDisplayedRect = rect;
        }


        public override void Activate() {
			MainButtonDef_Dropdown d = def as MainButtonDef_Dropdown;
			if ( d == null ) return;

            bool alwaysDisplayLabelInMBDD = QOLMod.getSettings().alwaysDisplayLabelInMBDD;
            TaggedString semiEmpty = " ";

            FloatMenu floatMenu = new FloatMenu(d.buttons.
                       Where(button=>button != null && !button.Worker.Disabled).
                       Select(button=>
                       {
                           //displayed as they would in the main menu bar: 
                           //if they have an icon the label is replaced with the texture in the option. 
                           //The tooltip is a combination of capitalized label and description.
                           if ( button.Icon != null )
                           {
                               return new FloatMenuOption(alwaysDisplayLabelInMBDD ? button.LabelCap : semiEmpty,
                                              delegate { button.Worker.InterfaceTryActivate(); }, button.Icon, GUI.color,
                                              MenuOptionPriority.Default, r => { TooltipHandler.TipRegion(r, () => button.LabelCap + "\n\n" + button.description, 1); }, null, 0f, null, null);
                           }
                           return new FloatMenuOption(button.LabelCap,
                                              delegate { button.Worker.InterfaceTryActivate(); },
                                              MenuOptionPriority.Default, r => { TooltipHandler.TipRegion(r, () => button.LabelCap + "\n\n" + button.description, 1); }, null, 0f, null, null);
                       }).ToList());
			floatMenu.vanishIfMouseDistant = true;
            floatMenu.onCloseCallback = () => { d.description = toggleDescriptionWhenFloatMenuIsShowing; toggleDescriptionWhenFloatMenuIsShowing = null; };

            Find.WindowStack.Add(floatMenu);
            toggleDescriptionWhenFloatMenuIsShowing = d.description;
            d.description = null;
            floatMenu.windowRect.x = Mathf.Min(lastDisplayedRect.x, floatMenu.windowRect.x);
            floatMenu.windowRect.y = lastDisplayedRect.y - floatMenu.InitialSize.y;
            if (floatMenu.windowRect.y < 0)
            {
                floatMenu.windowRect.y = lastDisplayedRect.y + lastDisplayedRect.height;
            }
        }
	}
}
