﻿using HarmonyLib;
using RIMMSqol.extensions;
using RIMMSqol.genericSettings;
using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;

namespace RIMMSqol
{
    [HarmonyPatch(typeof(MainTabWindow_Research))]
    [HarmonyPatch("DrawRightRect")]
    [HarmonyPatchNamespace("ResearchTabDrawer")]
    static public class MainTabWindow_Research_DrawRightRect
    {
        static bool translated = false;
        static string trnsTipSaveResearchProjectPositions = "RIMMSQoL.Research.SavePositions";

        static void Postfix(MainTabWindow_Research __instance, Rect rightOutRect)
        {
            if (Prefs.DevMode)
            {
                if (!translated)
                {
                    trnsTipSaveResearchProjectPositions.TryTranslate(out trnsTipSaveResearchProjectPositions);
                    translated = true;
                }

                Rect r = new Rect(rightOutRect.xMax - 110f, rightOutRect.yMin, 30f, 20f);
                if (Widgets.ButtonImageFitted(r, TexButton.Save))
                {
                    bool foundChanges = false;
                    FieldInfo fiPositionModified = typeof(ResearchProjectDef).GetField("positionModified", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
                    foreach (ResearchProjectDef researchProjectDef in from def in __instance.VisibleResearchProjects
                                                                      where def.Debug_IsPositionModified()
                                                                      select def)
                    {
                        ISettingsInstance si = SettingsDeepLink.getSettingsInstance("researchProjects", researchProjectDef);
                        si.set<float>("ResearchViewX", researchProjectDef.ResearchViewX);
                        si.set<float>("ResearchViewY", researchProjectDef.ResearchViewY);
                        fiPositionModified.SetValue(researchProjectDef, false);
                        foundChanges = true;
                    }
                    if (foundChanges)
                    {
                        SettingsInit.updateResearchLayout = true;
                        QOLMod.getModHandle().WriteSettings();
                    }
                }
                else if (Mouse.IsOver(r))
                {
                    TooltipHandler.TipRegion(r, () => trnsTipSaveResearchProjectPositions, 1);
                }
            }
        }
    }
}
