﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Verse;
using HarmonyLib;
using RimWorld;
using System.Threading;
using UnityEngine;
using System.Reflection;
using RIMMSqol.performance;
using System.Reflection.Emit;
using RIMMSqol.genericSettings;

namespace RIMMSqol
{
    [HarmonyPatch(typeof(PawnRenderer))]
    [HarmonyPatch("DrawPawnBody")]
    [HarmonyPatchNamespace("PawnApparelRendering")]
    static class PanwRenderer_DrawPawnBody
    {
        static IEnumerable<CodeInstruction> Transpiler(IEnumerable<CodeInstruction> instr, ILGenerator gen)
        {
            bool foundFirst = false;
            MethodInfo mi = typeof(GenDraw).GetMethod("DrawMeshNowOrLater", BindingFlags.Static | BindingFlags.Public, null, new Type[] { typeof(Mesh), typeof(Vector3), typeof(Quaternion), typeof(Material), typeof(bool) }, null);
            foreach (CodeInstruction ci in instr)
            {
                if (!foundFirst && ci.opcode == OpCodes.Call && (MethodInfo)ci.operand == mi)
                {
                    foundFirst = true;
                    yield return new CodeInstruction(OpCodes.Ldarg_0);

                    yield return new CodeInstruction(OpCodes.Ldarg_3);

                    yield return new CodeInstruction(OpCodes.Call, typeof(PanwRenderer_DrawPawnBody).GetMethod("DrawClothing", BindingFlags.Static | BindingFlags.NonPublic));
                }
                else yield return ci;
            }
        }
        
        static private void DrawClothing(Mesh mesh, Vector3 loc, Quaternion quat, Material mat, bool drawNow, PawnRenderer pawnRenderer, Rot4 facing)
        {
            Pawn wearer = pawnRenderer.graphics.pawn;
            WornGraphicData data = pawnRenderer.graphics.apparelGraphics.Where(agr => agr.graphic.MatAt(facing, null) == mat).
                Select(agr => agr.sourceApparel.def.apparel?.wornGraphicData).FirstOrDefault();
            if (data != null)
            {
                Vector2 offsetFactor = new Vector2(mesh.bounds.size.x, mesh.bounds.size.z);
                Vector2 vector1 = data.BeltOffsetAt(facing, wearer.story?.bodyType);
                Vector2 vector2 = data.BeltScaleAt(wearer.story?.bodyType);

                Matrix4x4 matrix = Matrix4x4.Translate(loc) * Matrix4x4.Rotate(quat) *
                    Matrix4x4.Translate(new Vector3(vector1.x * offsetFactor.x, 0f, vector1.y * offsetFactor.y)) * Matrix4x4.Scale(new Vector3(vector2.x, 1f, vector2.y));
                GenDraw.DrawMeshNowOrLater(mesh, matrix, mat, drawNow);
            }
            else GenDraw.DrawMeshNowOrLater(mesh, loc, quat, mat, drawNow);
        }
    }

    [HarmonyPatch(typeof(PawnRenderer))]
    [HarmonyPatch("DrawBodyApparel")]
    [HarmonyPatchNamespace("PawnApparelRendering")]
    static class PanwRenderer_DrawBodyApparel
    {
        static IEnumerable<CodeInstruction> Transpiler(IEnumerable<CodeInstruction> instr, ILGenerator gen)
        {
            MethodInfo mi = typeof(GenDraw).GetMethod("DrawMeshNowOrLater", BindingFlags.Static | BindingFlags.Public, null, new Type[] { typeof(Mesh), typeof(Vector3), typeof(Quaternion), typeof(Material), typeof(bool) }, null);
            bool foundFirst = false;

            foreach (CodeInstruction ci in instr)
            {
                if (!foundFirst && ci.opcode == OpCodes.Call && (MethodInfo)ci.operand == mi)
                {
                    foundFirst = true;
                    yield return new CodeInstruction(OpCodes.Ldloc_3);

                    yield return new CodeInstruction(OpCodes.Ldarg_S, 5);

                    yield return new CodeInstruction(OpCodes.Call, typeof(PanwRenderer_DrawBodyApparel).GetMethod("DrawShell", BindingFlags.Static | BindingFlags.NonPublic));
                }
                else yield return ci;
            }
        }

        static private void DrawShell(Mesh mesh, Vector3 loc, Quaternion quat, Material mat, bool drawNow, ApparelGraphicRecord apparelGraphicRecord, Rot4 facing)
        {
            Pawn wearer = apparelGraphicRecord.sourceApparel.Wearer;
            WornGraphicData data = apparelGraphicRecord.sourceApparel.def.apparel?.wornGraphicData;
            if (data != null)
            {
                Vector2 vector1 = data.BeltOffsetAt(facing, wearer.story?.bodyType);
                Vector2 vector2 = data.BeltScaleAt(wearer.story?.bodyType);
                Vector2 offsetFactor = new Vector2(mesh.bounds.size.x, mesh.bounds.size.z);
                Matrix4x4 matrix = Matrix4x4.Translate(loc) * Matrix4x4.Rotate(quat) *
                    Matrix4x4.Translate(new Vector3(vector1.x * offsetFactor.x, 0f, vector1.y * offsetFactor.y)) * Matrix4x4.Scale(new Vector3(vector2.x, 1f, vector2.y));
                GenDraw.DrawMeshNowOrLater(mesh, matrix, mat, drawNow);
            }
            else GenDraw.DrawMeshNowOrLater(mesh, loc, quat, mat, drawNow);
        }
    }

    [HarmonyPatch(typeof(PawnRenderer))]
    [HarmonyPatch("DrawHeadHair")]
    [HarmonyPatchNamespace("PawnApparelRendering")]
    static class PanwRenderer_DrawHeadHair
    {
        static IEnumerable<CodeInstruction> Transpiler(IEnumerable<CodeInstruction> instr, ILGenerator gen)
        {
            MethodInfo miFound = null;

            foreach (CodeInstruction ci in instr)
            {
                if (ci.opcode == OpCodes.Call && miFound == null)
                {
                    MethodInfo mi = (MethodInfo)ci.operand;
                    ParameterInfo[] parameters = mi.GetParameters();
                    if (parameters != null && parameters.Length == 2 && parameters[0].ParameterType == typeof(ApparelGraphicRecord))
                    {
                        miFound = mi;
                    }
                }
                yield return ci;
            }

            if ( miFound != null )
            {
                Harmony harmony = new Harmony("RIMMSqol");
                
                harmony.Patch(miFound, null, null, new HarmonyMethod(typeof(PanwRenderer_DrawHeadHair).
                    GetMethod("TranspileAnonymousApparelRenderer", BindingFlags.Static | BindingFlags.NonPublic)));
            }
        }

        static private IEnumerable<CodeInstruction> TranspileAnonymousApparelRenderer(IEnumerable<CodeInstruction> instr, ILGenerator gen)
        {
            MethodInfo mi = typeof(GenDraw).GetMethod("DrawMeshNowOrLater", BindingFlags.Static | BindingFlags.Public, null, new Type[] { typeof(Mesh), typeof(Vector3), typeof(Quaternion), typeof(Material), typeof(bool) }, null);
            FieldInfo bodyFacing = null;

            foreach (CodeInstruction ci in instr)
            {
                if (ci.opcode == OpCodes.Ldfld && bodyFacing == null && ((FieldInfo)ci.operand).Name == "bodyFacing")
                {
                    bodyFacing = (FieldInfo)ci.operand;
                }
                if (ci.opcode == OpCodes.Call && (MethodInfo)ci.operand == mi)
                {
                    if (bodyFacing != null)
                    {
                        yield return new CodeInstruction(OpCodes.Ldarg_1);

                        yield return new CodeInstruction(OpCodes.Ldarg_2);
                        yield return new CodeInstruction(OpCodes.Ldfld, bodyFacing);

                        yield return new CodeInstruction(OpCodes.Call, typeof(PanwRenderer_DrawHeadHair).GetMethod("DrawApparelGraphics", BindingFlags.Static | BindingFlags.NonPublic));
                    }
                    else
                    {
                        Log.Warning("Failed to find bodyFacing before trying to patch head apparel rendering. Skipping patching for headgear offset/scaling.");
                        yield return ci;
                    }
                }
                else yield return ci;
            }
        }

        static private void DrawApparelGraphics(Mesh mesh, Vector3 loc, Quaternion quat, Material mat, bool drawNow, ApparelGraphicRecord apparelGraphicRecord, Rot4 facing)
        {
            Pawn wearer = apparelGraphicRecord.sourceApparel.Wearer;
            WornGraphicData data = apparelGraphicRecord.sourceApparel.def.apparel?.wornGraphicData;
            if (data != null)
            {
                Vector2 offsetFactor = new Vector2(mesh.bounds.size.x, mesh.bounds.size.z);
                Vector2 vector1 = data.BeltOffsetAt(facing, wearer.story?.bodyType);
                Vector2 vector2 = data.BeltScaleAt(wearer.story?.bodyType);
                Matrix4x4 matrix = Matrix4x4.Translate(loc) * Matrix4x4.Rotate(quat) *
                    Matrix4x4.Translate(new Vector3(vector1.x * offsetFactor.x, 0f, vector1.y * offsetFactor.y)) * Matrix4x4.Scale(new Vector3(vector2.x, 1f, vector2.y));
                GenDraw.DrawMeshNowOrLater(mesh, matrix, mat, drawNow);
            }
            else GenDraw.DrawMeshNowOrLater(mesh, loc, quat, mat, drawNow);
        }
    }

    [HarmonyPatch(typeof(PawnRenderer))]
    [HarmonyPatch("DrawEquipmentAiming")]
    [HarmonyPatchNamespace("PawnApparelRendering")]
    static class PanwRenderer_DrawEquipmentAiming
    {
        [HarmonyAfter(new string[]{ "CombatExtended.HarmonyCE" })]
        static IEnumerable<CodeInstruction> Transpiler(IEnumerable<CodeInstruction> instr, ILGenerator gen)
        {
            //Since CE does pretty much the same, this won't work for both. CE users have to use the proprietary CE settings.
            MethodInfo mi = typeof(Graphics).GetMethod("DrawMesh", BindingFlags.Static | BindingFlags.Public, null, new Type[] { typeof(Mesh), typeof(Vector3), typeof(Quaternion), typeof(Material), typeof(int) }, null);
            bool foundFirst = false;

            foreach (CodeInstruction ci in instr)
            {
                if (!foundFirst && ci.opcode == OpCodes.Call && (MethodInfo)ci.operand == mi)
                {
                    foundFirst = true;
                    yield return new CodeInstruction(OpCodes.Ldarg_1);

                    yield return new CodeInstruction(OpCodes.Call, typeof(PanwRenderer_DrawEquipmentAiming).GetMethod("DrawEquipment", BindingFlags.Static | BindingFlags.NonPublic));
                }
                else yield return ci;
            }
        }

        static private void DrawEquipment(Mesh mesh, Vector3 loc, Quaternion quat, Material mat, int layer, Thing thing)
        {
            Pawn wearer = (thing.ParentHolder as Pawn_EquipmentTracker)?.pawn;
            
            if (wearer != null)
            {
                QOLDefModExtension_RenderingParameters data = thing.def.GetModExtension<QOLDefModExtension_RenderingParameters>();
                if (data != null)
                {
                    //wearer rotation is not necessarily correct if the rotation was overridden
                    //mesh for equipment is always of size (1, 1) no reason to scale offset so we can use the shorthand
                    Graphics.DrawMesh(mesh, data.BuildMatrix(loc, quat, wearer.Rotation, wearer.story?.bodyType), mat, layer);
                    return;
                }
            }

            Graphics.DrawMesh(mesh, loc, quat, mat, layer);
        }
    }
}
