﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 29.05.2017
 * Time: 01:19
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Reflection;
using RimWorld;
using UnityEngine;
using Verse;
using HarmonyLib;

namespace RIMMSqol
{	
	[HarmonyPatch(typeof(SkillRecord))]
	[HarmonyPatch("Learn")]
	[HarmonyPatchNamespace("TrackLevelUp")]
	static public class TrackLevelUp
	{
		[HarmonyPriority(Priority.First)]
		static bool Prefix(SkillRecord __instance, out int __state) {
			__state = __instance.levelInt;
			return true;
		}
		
		[HarmonyPriority(Priority.Last)]
		static void Postfix(SkillRecord __instance, int __state) {
			//if we detect that the level changed we add a symbol for levelUp or levelDown to the pawn whose skill changed.
			if ( __state != __instance.levelInt ) {
				foreach (Pawn p in Find.CurrentMap.mapPawns.FreeColonistsSpawned) {
					if ( p.skills.GetSkill(__instance.def) == __instance ) {
						ThoughtDef thought = DefDatabase<ThoughtDef>.GetNamed("QOLLeveled"+__instance.def.defName,false);
						if ( thought == null ) thought = DefDatabase<ThoughtDef>.GetNamed("QOLLeveled");
						Thought_Memory newThought = ThoughtMaker.MakeThought(thought,__state < __instance.levelInt ? 1 : 0);
						p.needs.mood.thoughts.memories.TryGainMemory(newThought, null);
						break;
					}
				}
			}
		}
	}
	
	[HarmonyPatch(typeof(SkillRecord))]
	[HarmonyPatch("Interval")]
	[HarmonyPatchNamespace("StopSkillDecay")]
	static public class StopSkillDecay
	{
		private static bool stopSkillDecay, stopSkillDown;
		private static int levelOffset;
		
		public static bool Prepare() {
			QOLMod.addApplySettingsListener(mod=> {
	            	stopSkillDecay = QOLMod.stopSkillDecay();
                    stopSkillDown = QOLMod.stopSkillDown();
                    levelOffset = QOLMod.skillDecayLevelOffset();
	            });
            stopSkillDown = QOLMod.stopSkillDown();
            stopSkillDecay = QOLMod.stopSkillDecay();
			levelOffset = QOLMod.skillDecayLevelOffset();
			return true;
		}
		
		static bool Prefix(SkillRecord __instance, Pawn ___pawn) {
			if ( stopSkillDecay ) {
				return false;
			}
            float xp;
			float num = ___pawn.story.traits.HasTrait(TraitDefOf.GreatMemory) ? 0.5f : 1f;
			switch (__instance.levelInt + levelOffset)
			{
			    case 10: xp = -0.1f * num; break;
			    case 11: xp = -0.2f * num; break;
			    case 12: xp = -0.4f * num; break;
                case 13: xp = -0.6f * num; break;
                case 14: xp = -1f * num; break;
                case 15: xp = -1.8f * num; break;
                case 16: xp = -2.8f * num; break;
                case 17: xp = -4f * num; break;
                case 18: xp = -6f * num; break;
                case 19: xp = -8f * num; break;
                case 20: xp = -12f * num; break;
                default: xp = 0; break;
            }
            if ( !stopSkillDown || __instance.xpSinceLastLevel + xp > -1000f )
                __instance.Learn(xp, false);
            else __instance.Learn(-999 - __instance.xpSinceLastLevel, false);
            return false;
		}
	}
}
