﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 20.06.2020
 * Time: 01:36
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Reflection;
using Verse;

namespace RIMMSqol.genericSettings
{
	/// <summary>
	/// Description of VanillaFurnitureExpandedConnectors.
	/// </summary>
	static public class VanillaFurnitureExpandedConnectors
	{
		/*
		 * <li Class="GasNetwork.CompProperties_GasStorage">
				<compClass>GasNetwork.CompGasStorage</compClass>
				<capacity>100</capacity>
				int capacity = 100
			</li>
			
			<li Class="GasNetwork.CompProperties_GasTrader">
				<compClass>GasNetwork.CompGasTrader</compClass>
				<gasConsumption>-300</gasConsumption>
				float gasConsumption = 0
				float gasConsumptionWhenUsed = 0
			</li>
		 */
		static bool initialized = false;
		static Type tCompPropertiesGasStorage;
		static Type tCompPropertiesGasTrader;
		static MethodInfo miGetCompPropertiesCompPropertiesGasStorage;
		static MethodInfo miGetCompPropertiesCompPropertiesGasTrader;
		
		static private Func<CompProperties,int> getterGasStorageCapacity;
		static private Func<CompProperties,float> getterGasTraderGasConsumption;
		static private Func<CompProperties,float> getterGasTraderGasConsumptionWhenUsed;
		
		static private Action<CompProperties,int> setterGasStorageCapacity;
		static private Action<CompProperties,float> setterGasTraderGasConsumption;
		static private Action<CompProperties,float> setterGasTraderGasConsumptionWhenUsed;
		
		static void initIfNecessary() {
			if ( !initialized ) {
				tCompPropertiesGasStorage = DefSettingsField.GetTypeFromAnyAssemblyVersion("GasNetwork.CompProperties_GasStorage, VanillaPowerExpanded, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null");
				miGetCompPropertiesCompPropertiesGasStorage = tCompPropertiesGasStorage != null ? typeof(ThingDef).GetMethod("GetCompProperties").MakeGenericMethod(tCompPropertiesGasStorage) : null;
				
				if ( tCompPropertiesGasStorage != null ) {
					getterGasStorageCapacity = UtilReflection.CreateGetter<CompProperties,int>(tCompPropertiesGasStorage.GetField("capacity"));
					setterGasStorageCapacity = UtilReflection.CreateSetter<CompProperties,int>(tCompPropertiesGasStorage.GetField("capacity"));
				}
				
				tCompPropertiesGasTrader = DefSettingsField.GetTypeFromAnyAssemblyVersion("GasNetwork.CompProperties_GasTrader, VanillaPowerExpanded, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null");
				miGetCompPropertiesCompPropertiesGasTrader = tCompPropertiesGasTrader != null ? typeof(ThingDef).GetMethod("GetCompProperties").MakeGenericMethod(tCompPropertiesGasTrader) : null;
				
				if ( tCompPropertiesGasTrader != null ) {
					getterGasTraderGasConsumption = UtilReflection.CreateGetter<CompProperties,float>(tCompPropertiesGasTrader.GetField("gasConsumption"));
					setterGasTraderGasConsumption = UtilReflection.CreateSetter<CompProperties,float>(tCompPropertiesGasTrader.GetField("gasConsumption"));
					
					getterGasTraderGasConsumptionWhenUsed = UtilReflection.CreateGetter<CompProperties,float>(tCompPropertiesGasTrader.GetField("gasConsumptionWhenUsed"));
					setterGasTraderGasConsumptionWhenUsed = UtilReflection.CreateSetter<CompProperties,float>(tCompPropertiesGasTrader.GetField("gasConsumptionWhenUsed"));
				}
				
				initialized = true;
			}
		}
		
		static public CompProperties GetCompPropertyGasStorage(ThingDef td) {
			initIfNecessary();
			if ( td == null || miGetCompPropertiesCompPropertiesGasStorage == null ) return null;
			return (CompProperties)miGetCompPropertiesCompPropertiesGasStorage.Invoke(td,null);
		}
			
		static public bool HasCompPropertyGasStorage(ThingDef td) {
			return GetCompPropertyGasStorage(td) != null;
		}
		
		static public int GetGasStorageCapacity(ThingDef td) {
			return GetGasStorageCapacity(GetCompPropertyGasStorage(td));
		}
		
		static public int GetGasStorageCapacity(CompProperties cp) {
			initIfNecessary();
			return cp != null ? getterGasStorageCapacity(cp) : 100;
		}
		
		static public void SetGasStorageCapacity(CompProperties cp, int capacity) {
			initIfNecessary();
			if ( cp != null ) setterGasStorageCapacity(cp,capacity);
		}
		
		static public CompProperties GetCompPropertyGasTrader(ThingDef td) {
			initIfNecessary();
			if ( td == null || miGetCompPropertiesCompPropertiesGasTrader == null ) return null;
			return (CompProperties)miGetCompPropertiesCompPropertiesGasTrader.Invoke(td,null);
		}
			
		static public bool HasCompPropertyGasTrader(ThingDef td) {
			return GetCompPropertyGasTrader(td) != null;
		}
		
		static public float GetGasTraderGasConsumption(ThingDef td) {
			return GetGasTraderGasConsumption(GetCompPropertyGasTrader(td));
		}
		
		static public float GetGasTraderGasConsumption(CompProperties cp) {
			initIfNecessary();
			return cp != null ? getterGasTraderGasConsumption(cp) : 0;
		}
		
		static public void SetGasTraderGasConsumption(CompProperties cp, float gasConsumption) {
			initIfNecessary();
			if ( cp != null ) setterGasTraderGasConsumption(cp,gasConsumption);
		}
		
		static public float GetGasTraderGasConsumptionWhenUsed(ThingDef td) {
			return GetGasTraderGasConsumptionWhenUsed(GetCompPropertyGasTrader(td));
		}
		
		static public float GetGasTraderGasConsumptionWhenUsed(CompProperties cp) {
			initIfNecessary();
			return cp != null ? getterGasTraderGasConsumptionWhenUsed(cp) : 0;
		}
		
		static public void SetGasTraderGasConsumptionWhenUsed(CompProperties cp, float gasConsumptionWhenUsed) {
			initIfNecessary();
			if ( cp != null ) setterGasTraderGasConsumptionWhenUsed(cp,gasConsumptionWhenUsed);
		}
	}
}
