﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 10.02.2021
 * Time: 04:26
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HarmonyLib;
using RimWorld;
using RimWorld.Planet;
using UnityEngine;
using Verse;
using Verse.AI;
using Verse.Sound;

namespace RIMMSqol.remnantcolony
{
	[HarmonyPatch(typeof(Dialog_FormCaravan))]
	[HarmonyPatch("Notify_ChoseRoute")]
	[HarmonyPatchNamespace("City")]
	public static class Dialog_FormCaravan_Notify_ChoseRoute {
		static bool Prefix(Dialog_FormCaravan __instance, int destinationTile) {
			Dialog_FormCaravanQOL qolVersion = __instance as Dialog_FormCaravanQOL;
			if ( qolVersion != null ) {
				qolVersion.Notify_ChoseRoute(destinationTile);
				return false;
			}
			return true;
		}
	}
	[HarmonyPatch(typeof(Dialog_FormCaravan))]
	[HarmonyPatch("CurrentTile", MethodType.Getter)]
	[HarmonyPatchNamespace("City")]
	public static class Dialog_FormCaravan_Get_CurrentTile {
		static bool Prefix(Dialog_FormCaravan __instance, ref int __result) {
			Dialog_FormCaravanQOL qolVersion = __instance as Dialog_FormCaravanQOL;
			if ( qolVersion != null ) {
				__result = qolVersion.CurrentTile;
				return false;
			}
			return true;
		}
	}
	/// <summary>
	/// Description of Dialog_FormCaravanQOL.
	/// </summary>
	public class Dialog_FormCaravanQOL : Dialog_FormCaravan
	{
		private enum Tab
		{
			Pawns,
			Items,
			FoodAndMedicine
		}
		
		private class Handler
		{
			public Handler() {}
			
			public static readonly Dialog_FormCaravanQOL.Handler handler = new Dialog_FormCaravanQOL.Handler();

			public static Predicate<Pawn> isCapableOfSocial;

			public static Func<string, string> cbLbParagraph;

			public static Predicate<Pawn> hasColonist;

			public static Predicate<TransferableOneWay> isPawn;

			public static Predicate<Thing> isNotSpawned;

			public static Func<Thing, bool> isSpawned;

			public static Predicate<Pawn> isDownedColonist;

			public static Func<TransferableOneWay, bool> isMedicine;

			public static Func<TransferableOneWay, bool> isEdibleNotDrugOrCorpse;

			public static Func<TransferableOneWay, float> getBedRestEffectiveness;

			internal bool canSocialize(Pawn pawn)
			{
				return CaravanUtility.IsOwner(pawn, Faction.OfPlayer) && !pawn.skills.GetSkill(SkillDefOf.Social).TotallyDisabled;
			}

			public string lbParagraph(string str)
			{
				return str + "\n\n";
			}

			public bool isColonist(Pawn x)
			{
				return CaravanUtility.IsOwner(x, Faction.OfPlayer);
			}

			public bool pawn(TransferableOneWay x)
			{
				return x.AnyThing is Pawn;
			}

			public bool notSpawned(Thing x)
			{
				return !x.Spawned;
			}

			public bool spawned(Thing x)
			{
				return x.Spawned;
			}

			public bool downedColonist(Pawn x)
			{
				return CaravanUtility.IsOwner(x, Faction.OfPlayer) && !x.Downed;
			}

			public bool medicine(TransferableOneWay x)
			{
				return x.ThingDef.category != ThingCategory.Pawn && !x.ThingDef.thingCategories.NullOrEmpty<ThingCategoryDef>() && x.ThingDef.thingCategories.Contains(ThingCategoryDefOf.Medicine);
			}

			public bool edibleNotDrugOrCorpse(TransferableOneWay x)
			{
				return x.ThingDef.IsIngestible && !x.ThingDef.IsDrug && !x.ThingDef.IsCorpse;
			}

			public float bedRestEffectiveness(TransferableOneWay x)
			{
				return x.AnyThing.GetStatValue(StatDefOf.BedRestEffectiveness, true);
			}
		}

		private bool reform;

		private Action onClosed;

		private bool canChooseRoute;

		private bool mapAboutToBeRemoved;

		//public bool choosingRoute;

		private bool thisWindowInstanceEverOpened;

		//public List<TransferableOneWay> transferables;

		private TransferableOneWayWidget pawnsTransfer;

		private TransferableOneWayWidget itemsTransfer;

		private TransferableOneWayWidget foodAndMedicineTransfer;

		private Dialog_FormCaravanQOL.Tab tab;

		private float lastMassFlashTime = -9999f;

		private int startingTile = -1;

		private int destinationTile = -1;

		private bool massUsageDirty = true;

		private float cachedMassUsage;

		private bool massCapacityDirty = true;

		private float cachedMassCapacity;

		private string cachedMassCapacityExplanation;

		private bool tilesPerDayDirty = true;

		private float cachedTilesPerDay;

		private string cachedTilesPerDayExplanation;

		private bool daysWorthOfFoodDirty = true;

		private Pair<float, float> cachedDaysWorthOfFood;

		private bool foragedFoodPerDayDirty = true;

		private Pair<ThingDef, float> cachedForagedFoodPerDay;

		private string cachedForagedFoodPerDayExplanation;

		private bool visibilityDirty = true;

		private float cachedVisibility;

		private string cachedVisibilityExplanation;

		private bool ticksToArriveDirty = true;

		private int cachedTicksToArrive;

		private bool autoSelectFoodAndMedicine;

		private const float TitleRectHeight = 35f;

		private const float BottomAreaHeight = 55f;

		private const float AutoSelectCheckBoxHeight = 35f;

		private readonly Vector2 BottomButtonSize = new Vector2(160f, 40f);

		private const float MaxDaysWorthOfFoodToShowWarningDialog = 5f;

		private const float AutoSelectFoodThresholdDays = 4f;

		private const int AutoMedicinePerColonist = 2;

		private static List<TabRecord> tabsList = new List<TabRecord>();

		private static List<Thing> tmpPackingSpots = new List<Thing>();

		private static List<Pair<int, int>> tmpTicksToArrive = new List<Pair<int, int>>();

		private static List<TransferableOneWay> tmpBeds = new List<TransferableOneWay>();
		
		public WorldObject wo;
		public List<Thing> items;
		public List<Pawn> pawns;

		new public int CurrentTile
		{
			get
			{
				return this.wo.Tile;
			}
		}

		public override Vector2 InitialSize
		{
			get
			{
				return new Vector2(1024f, (float)UI.screenHeight);
			}
		}

		protected override float Margin
		{
			get
			{
				return 0f;
			}
		}

		private bool AutoStripSpawnedCorpses
		{
			get
			{
				return this.reform;
			}
		}

		private bool ListPlayerPawnsInventorySeparately
		{
			get
			{
				return this.reform;
			}
		}

		private BiomeDef Biome
		{
			get
			{
				return this.wo.Biome;
			}
		}

		private bool MustChooseRoute
		{
			get
			{
				return this.canChooseRoute;
			}
		}

		private bool ShowCancelButton
		{
			get
			{
				return true;
			}
		}

		private IgnorePawnsInventoryMode IgnoreInventoryMode
		{
			get
			{
				if (!this.ListPlayerPawnsInventorySeparately)
				{
					return IgnorePawnsInventoryMode.IgnoreIfAssignedToUnload;
				}
				return IgnorePawnsInventoryMode.IgnoreIfAssignedToUnloadOrPlayerPawn;
			}
		}

		new public float MassUsage
		{
			get
			{
				if (this.massUsageDirty)
				{
					this.massUsageDirty = false;
					this.cachedMassUsage = CollectionsMassCalculator.MassUsageTransferables(this.transferables, this.IgnoreInventoryMode, false, this.AutoStripSpawnedCorpses);
				}
				return this.cachedMassUsage;
			}
		}

		new public float MassCapacity
		{
			get
			{
				if (this.massCapacityDirty)
				{
					this.massCapacityDirty = false;
					StringBuilder stringBuilder = new StringBuilder();
					this.cachedMassCapacity = CollectionsMassCalculator.CapacityTransferables(this.transferables, stringBuilder);
					this.cachedMassCapacityExplanation = stringBuilder.ToString();
				}
				return this.cachedMassCapacity;
			}
		}

		private float TilesPerDay
		{
			get
			{
				if (this.tilesPerDayDirty)
				{
					this.tilesPerDayDirty = false;
					StringBuilder stringBuilder = new StringBuilder();
					this.cachedTilesPerDay = TilesPerDayCalculator.ApproxTilesPerDay(this.transferables, this.MassUsage, this.MassCapacity, this.CurrentTile, this.startingTile, stringBuilder);
					this.cachedTilesPerDayExplanation = stringBuilder.ToString();
				}
				return this.cachedTilesPerDay;
			}
		}

		private Pair<float, float> DaysWorthOfFood
		{
			get
			{
				if (this.daysWorthOfFoodDirty)
				{
					this.daysWorthOfFoodDirty = false;
					float first;
					float second;
					if (this.destinationTile != -1)
					{
						using (WorldPath worldPath = Find.WorldPathFinder.FindPath(this.CurrentTile, this.destinationTile, null, null))
						{
							int ticksPerMove = CaravanTicksPerMoveUtility.GetTicksPerMove(new CaravanTicksPerMoveUtility.CaravanInfo(this), null);
							first = DaysWorthOfFoodCalculator.ApproxDaysWorthOfFood(this.transferables, this.CurrentTile, this.IgnoreInventoryMode, Faction.OfPlayer, worldPath, 0f, ticksPerMove);
							second = DaysUntilRotCalculator.ApproxDaysUntilRot(this.transferables, this.CurrentTile, this.IgnoreInventoryMode, worldPath, 0f, ticksPerMove);
							goto IL_DB;
						}
					}
					first = DaysWorthOfFoodCalculator.ApproxDaysWorthOfFood(this.transferables, this.CurrentTile, this.IgnoreInventoryMode, Faction.OfPlayer, null, 0f, 3300);
					second = DaysUntilRotCalculator.ApproxDaysUntilRot(this.transferables, this.CurrentTile, this.IgnoreInventoryMode, null, 0f, 3300);
					IL_DB:
					this.cachedDaysWorthOfFood = new Pair<float, float>(first, second);
				}
				return this.cachedDaysWorthOfFood;
			}
		}

		private Pair<ThingDef, float> ForagedFoodPerDay
		{
			get
			{
				if (this.foragedFoodPerDayDirty)
				{
					this.foragedFoodPerDayDirty = false;
					StringBuilder stringBuilder = new StringBuilder();
					this.cachedForagedFoodPerDay = ForagedFoodPerDayCalculator.ForagedFoodPerDay(this.transferables, this.Biome, Faction.OfPlayer, stringBuilder);
					this.cachedForagedFoodPerDayExplanation = stringBuilder.ToString();
				}
				return this.cachedForagedFoodPerDay;
			}
		}

		private float Visibility
		{
			get
			{
				if (this.visibilityDirty)
				{
					this.visibilityDirty = false;
					StringBuilder stringBuilder = new StringBuilder();
					this.cachedVisibility = CaravanVisibilityCalculator.Visibility(this.transferables, stringBuilder);
					this.cachedVisibilityExplanation = stringBuilder.ToString();
				}
				return this.cachedVisibility;
			}
		}

		private int TicksToArrive
		{
			get
			{
				if (this.destinationTile == -1)
				{
					return 0;
				}
				if (this.ticksToArriveDirty)
				{
					this.ticksToArriveDirty = false;
					using (WorldPath worldPath = Find.WorldPathFinder.FindPath(this.CurrentTile, this.destinationTile, null, null))
					{
						this.cachedTicksToArrive = CaravanArrivalTimeEstimator.EstimatedTicksToArrive(this.CurrentTile, this.destinationTile, worldPath, 0f, CaravanTicksPerMoveUtility.GetTicksPerMove(new CaravanTicksPerMoveUtility.CaravanInfo(this), null), Find.TickManager.TicksAbs);
					}
				}
				return this.cachedTicksToArrive;
			}
		}

		private bool MostFoodWillRotSoon
		{
			get
			{
				float num = 0f;
				float num2 = 0f;
				for (int i = 0; i < this.transferables.Count; i++)
				{
					TransferableOneWay transferableOneWay = this.transferables[i];
					if (transferableOneWay.HasAnyThing && transferableOneWay.CountToTransfer > 0 && transferableOneWay.ThingDef.IsNutritionGivingIngestible && !(transferableOneWay.AnyThing is Corpse))
					{
						float num3 = 600f;
						CompRottable compRottable = transferableOneWay.AnyThing.TryGetComp<CompRottable>();
						if (compRottable != null)
						{
							num3 = (float)DaysUntilRotCalculator.ApproxTicksUntilRot_AssumeTimePassesBy(compRottable, this.CurrentTile, null) / 60000f;
						}
						float num4 = transferableOneWay.ThingDef.GetStatValueAbstract(StatDefOf.Nutrition, null) * (float)transferableOneWay.CountToTransfer;
						if (num3 < 5f)
						{
							num += num4;
						}
						else
						{
							num2 += num4;
						}
					}
				}
				return (num != 0f || num2 != 0f) && num / (num + num2) >= 0.75f;
			}
		}

		public Dialog_FormCaravanQOL(WorldObject wo,List<Pawn> pawns, List<Thing> items, bool reform = false, Action onClosed = null, bool mapAboutToBeRemoved = false) : base(null, reform, onClosed, mapAboutToBeRemoved)
		{
			this.wo = wo;
			this.pawns = pawns;
			this.items = items;
			this.reform = reform;
			this.onClosed = onClosed;
			this.mapAboutToBeRemoved = mapAboutToBeRemoved;
			this.canChooseRoute = !reform;
			this.closeOnAccept = !reform;
			this.closeOnCancel = !reform;
			this.autoSelectFoodAndMedicine = !reform;
			this.forcePause = true;
			this.absorbInputAroundWindow = true;
		}

		public override void PostOpen()
		{
			if (this.soundAppear != null) {
				this.soundAppear.PlayOneShotOnCamera(null);
			}
			this.choosingRoute = false;
			if (!this.thisWindowInstanceEverOpened)
			{
				this.thisWindowInstanceEverOpened = true;
				this.CalculateAndRecacheTransferables();
				PlayerKnowledgeDatabase.KnowledgeDemonstrated(ConceptDefOf.FormCaravan, KnowledgeAmount.Total);
				
				Find.WorldRoutePlanner.Start(this);
			}
		}

		public override void PostClose()
		{
			if (this.onClosed != null && !this.choosingRoute)
			{
				this.onClosed();
			}
		}

		/*public void Notify_NoLongerChoosingRoute()
		{
			this.choosingRoute = false;
			if (!Find.WindowStack.IsOpen(this) && this.onClosed != null)
			{
				this.onClosed();
			}
		}*/

		public override void DoWindowContents(Rect inRect)
		{
			Rect arg_40_0 = new Rect(0f, 0f, inRect.width, 35f);
			Text.Font = GameFont.Medium;
			Text.Anchor = TextAnchor.MiddleCenter;
			Widgets.Label(arg_40_0, (this.reform ? "ReformCaravan" : "FormCaravan").Translate());
			Text.Font = GameFont.Small;
			Text.Anchor = TextAnchor.UpperLeft;
			CaravanUIUtility.DrawCaravanInfo(new CaravanUIUtility.CaravanInfo(this.MassUsage, this.MassCapacity, this.cachedMassCapacityExplanation, this.TilesPerDay, this.cachedTilesPerDayExplanation, this.DaysWorthOfFood, this.ForagedFoodPerDay, this.cachedForagedFoodPerDayExplanation, this.Visibility, this.cachedVisibilityExplanation, -1f, -1f, null), null, this.CurrentTile, (this.destinationTile == -1) ? null : new int?(this.TicksToArrive), this.lastMassFlashTime, new Rect(12f, 35f, inRect.width - 24f, 40f), true, (this.destinationTile == -1) ? null : ("\n" + "DaysWorthOfFoodTooltip_OnlyFirstWaypoint".Translate()), false);
			Dialog_FormCaravanQOL.tabsList.Clear();
			Dialog_FormCaravanQOL.tabsList.Add(new TabRecord("PawnsTab".Translate(), delegate
			{
				this.tab = Dialog_FormCaravanQOL.Tab.Pawns;
			}, this.tab == Dialog_FormCaravanQOL.Tab.Pawns));
			Dialog_FormCaravanQOL.tabsList.Add(new TabRecord("ItemsTab".Translate(), delegate
			{
				this.tab = Dialog_FormCaravanQOL.Tab.Items;
			}, this.tab == Dialog_FormCaravanQOL.Tab.Items));
			Dialog_FormCaravanQOL.tabsList.Add(new TabRecord("FoodAndMedicineTab".Translate(), delegate
			{
				this.tab = Dialog_FormCaravanQOL.Tab.FoodAndMedicine;
			}, this.tab == Dialog_FormCaravanQOL.Tab.FoodAndMedicine));
			inRect.yMin += 119f;
			Widgets.DrawMenuSection(inRect);
			TabDrawer.DrawTabs(inRect, Dialog_FormCaravanQOL.tabsList, 200f);
			Dialog_FormCaravanQOL.tabsList.Clear();
			inRect = inRect.ContractedBy(17f);
			inRect.height += 17f;
			GUI.BeginGroup(inRect);
			Rect rect = inRect.AtZero();
			this.DoBottomButtons(rect);
			Rect rect2 = rect;
			rect2.yMax -= 76f;
			bool flag = false;
			switch (this.tab)
			{
			case Dialog_FormCaravanQOL.Tab.Pawns:
				this.pawnsTransfer.OnGUI(rect2, out flag);
				break;
			case Dialog_FormCaravanQOL.Tab.Items:
				this.itemsTransfer.OnGUI(rect2, out flag);
				break;
			case Dialog_FormCaravanQOL.Tab.FoodAndMedicine:
				this.foodAndMedicineTransfer.extraHeaderSpace = 35f;
				this.foodAndMedicineTransfer.OnGUI(rect2, out flag);
				this.DrawAutoSelectCheckbox_Local(rect2, ref flag);
				break;
			}
			if (flag)
			{
				this.CountToTransferChanged();
			}
			GUI.EndGroup();
		}

		protected void DrawAutoSelectCheckbox_Local(Rect rect, ref bool anythingChanged)
		{
			rect.yMin += 37f;
			rect.height = 35f;
			bool arg_5B_0 = this.autoSelectFoodAndMedicine;
			Widgets.CheckboxLabeled(rect, "AutomaticallySelectFoodAndMedicine".Translate(), ref this.autoSelectFoodAndMedicine, false, null, null, true);
			this.foodAndMedicineTransfer.readOnly = this.autoSelectFoodAndMedicine;
			if (arg_5B_0 != this.autoSelectFoodAndMedicine)
			{
				anythingChanged = true;
			}
		}

		public override bool CausesMessageBackground()
		{
			return true;
		}

		new public void Notify_ChoseRoute(int destinationTile)
		{
			this.destinationTile = destinationTile;
			this.startingTile = this.wo.Tile;
			this.ticksToArriveDirty = true;
			this.daysWorthOfFoodDirty = true;
			this.soundAppear.PlayOneShotOnCamera(null);
			if (this.autoSelectFoodAndMedicine)
			{
				this.SelectApproximateBestFoodAndMedicine();
			}
		}

		private void AddToTransferables(Thing t, bool setToTransferMax = false)
		{
			TransferableOneWay transferableOneWay = TransferableUtility.TransferableMatching<TransferableOneWay>(t, this.transferables, TransferAsOneMode.PodsOrCaravanPacking);
			if (transferableOneWay == null)
			{
				transferableOneWay = new TransferableOneWay();
				this.transferables.Add(transferableOneWay);
			}
			if (transferableOneWay.things.Contains(t))
			{
				Log.Error("Tried to add the same thing twice to TransferableOneWay: " + t);
				return;
			}
			transferableOneWay.things.Add(t);
			if (setToTransferMax)
			{
				transferableOneWay.AdjustTo(transferableOneWay.CountToTransfer + t.stackCount);
			}
		}

		private void DoBottomButtons(Rect rect)
		{
			Rect rect2 = new Rect(rect.width - this.BottomButtonSize.x, rect.height - 55f - 17f, this.BottomButtonSize.x, this.BottomButtonSize.y);
			if (Widgets.ButtonText(rect2, "Send".Translate(), true, true, true))
			{
				if (this.reform)
				{
					if (this.TryReformCaravan())
					{
						SoundDefOf.Tick_High.PlayOneShotOnCamera(null);
						this.Close(false);
					}
				}
				else
				{
					List<string> list = new List<string>();
					Pair<float, float> daysWorthOfFood = this.DaysWorthOfFood;
					if (daysWorthOfFood.First < 5f)
					{
						list.Add((daysWorthOfFood.First < 0.1f) ? "DaysWorthOfFoodWarningDialog_NoFood".Translate() : "DaysWorthOfFoodWarningDialog".Translate(daysWorthOfFood.First.ToString("0.#")));
					}
					else if (this.MostFoodWillRotSoon)
					{
						list.Add("CaravanFoodWillRotSoonWarningDialog".Translate());
					}
					List<Pawn> arg_136_0 = TransferableUtility.GetPawnsFromTransferables(this.transferables);
					Predicate<Pawn> arg_136_1;
					if ((arg_136_1 = Dialog_FormCaravanQOL.Handler.isCapableOfSocial) == null)
					{
						arg_136_1 = (Dialog_FormCaravanQOL.Handler.isCapableOfSocial = new Predicate<Pawn>(Dialog_FormCaravanQOL.Handler.handler.canSocialize));
					}
					if (!arg_136_0.Any(arg_136_1))
					{
						list.Add("CaravanIncapableOfSocial".Translate());
					}
					if (list.Count > 0)
					{
						if (this.CheckForErrors(TransferableUtility.GetPawnsFromTransferables(this.transferables)))
						{
							IEnumerable<string> arg_194_0 = list;
							Func<string, string> arg_194_1;
							if ((arg_194_1 = Dialog_FormCaravanQOL.Handler.cbLbParagraph) == null)
							{
								arg_194_1 = (Dialog_FormCaravanQOL.Handler.cbLbParagraph = new Func<string, string>(Dialog_FormCaravanQOL.Handler.handler.lbParagraph));
							}
							string str = string.Concat(arg_194_0.Select(arg_194_1).ToArray<string>()) + "CaravanAreYouSure".Translate();
							Find.WindowStack.Add(Dialog_MessageBox.CreateConfirmation(str, delegate
							{
								if (this.TryFormAndSendCaravan())
								{
									this.Close(false);
								}
							}, false, null));
						}
					}
					else if (this.TryFormAndSendCaravan())
					{
						SoundDefOf.Tick_High.PlayOneShotOnCamera(null);
						this.Close(false);
					}
				}
			}
			if (this.ShowCancelButton && Widgets.ButtonText(new Rect(0f, rect2.y, this.BottomButtonSize.x, this.BottomButtonSize.y), "CancelButton".Translate(), true, true, true))
			{
				this.Close(true);
			}
			if (Widgets.ButtonText(new Rect(rect.width / 2f - this.BottomButtonSize.x - 8.5f, rect2.y, this.BottomButtonSize.x, this.BottomButtonSize.y), "ResetButton".Translate(), true, true, true))
			{
				SoundDefOf.Tick_Low.PlayOneShotOnCamera(null);
				this.CalculateAndRecacheTransferables();
			}
			if (this.canChooseRoute)
			{
				if (Widgets.ButtonText(new Rect(rect.width / 2f + 8.5f, rect2.y, this.BottomButtonSize.x, this.BottomButtonSize.y), "ChangeRouteButton".Translate(), true, true, true))
				{
					this.soundClose.PlayOneShotOnCamera(null);
					Find.WorldRoutePlanner.Start(this);
				}
				if (this.destinationTile != -1)
				{
					Rect rect3 = rect2;
					rect3.y += rect2.height + 4f;
					rect3.height = 200f;
					rect3.xMin -= 200f;
					Text.Anchor = TextAnchor.UpperRight;
					Widgets.Label(rect3, "CaravanEstimatedDaysToDestination".Translate(((float)this.TicksToArrive / 60000f).ToString("0.#")));
					Text.Anchor = TextAnchor.UpperLeft;
				}
			}
			if (Prefs.DevMode)
			{
				float width = 200f;
				float height = this.BottomButtonSize.y / 2f;
				if (Widgets.ButtonText(new Rect(0f, rect2.yMax + 4f, width, height), "Dev: Send instantly", true, true, true) && this.DebugTryFormCaravanInstantly())
				{
					SoundDefOf.Tick_High.PlayOneShotOnCamera(null);
					this.Close(false);
				}
				if (Widgets.ButtonText(new Rect(204f, rect2.yMax + 4f, width, height), "Dev: Select everything", true, true, true))
				{
					SoundDefOf.Tick_High.PlayOneShotOnCamera(null);
					this.SetToSendEverything();
				}
			}
		}

		private void CalculateAndRecacheTransferables()
		{
			this.transferables = new List<TransferableOneWay>();
			this.AddPawnsToTransferables();
			this.AddItemsToTransferables();
			CaravanUIUtility.CreateCaravanTransferableWidgets(this.transferables, out this.pawnsTransfer, out this.itemsTransfer, out this.foodAndMedicineTransfer, "FormCaravanColonyThingCountTip".Translate(), this.IgnoreInventoryMode, () => this.MassCapacity - this.MassUsage, this.AutoStripSpawnedCorpses, this.CurrentTile, this.mapAboutToBeRemoved);
			this.CountToTransferChanged();
		}

		private bool DebugTryFormCaravanInstantly()
		{
			List<Pawn> pawnsFromTransferables = TransferableUtility.GetPawnsFromTransferables(this.transferables);
			List<Pawn> arg_2C_0 = pawnsFromTransferables;
			Predicate<Pawn> arg_2C_1;
			if ((arg_2C_1 = Dialog_FormCaravanQOL.Handler.hasColonist) == null)
			{
				arg_2C_1 = (Dialog_FormCaravanQOL.Handler.hasColonist = new Predicate<Pawn>(Dialog_FormCaravanQOL.Handler.handler.isColonist));
			}
			if (!arg_2C_0.Any(arg_2C_1))
			{
				Messages.Message("CaravanMustHaveAtLeastOneColonist".Translate(), MessageTypeDefOf.RejectInput, false);
				return false;
			}
			this.AddItemsFromTransferablesToRandomInventories(pawnsFromTransferables);
			int num = this.startingTile;
			if (num < 0)
			{
				num = this.wo.Tile;
			}
			if (num < 0)
			{
				num = this.CurrentTile;
			}
			ExitCityAndCreateCaravan(pawnsFromTransferables, Faction.OfPlayer, this.CurrentTile, num, this.destinationTile);
			return true;
		}
		
		public Caravan ExitCityAndCreateCaravan(IEnumerable<Pawn> pawns, Faction faction, int exitFromTile, int directionTile, int destinationTile, bool sendMessage = true)
		{
			if (!GenWorldClosest.TryFindClosestPassableTile(exitFromTile, out exitFromTile)) {
				Log.Error("Could not find any passable tile for a new caravan.");
				return null;
			}
			if (Find.World.Impassable(directionTile)) {
				directionTile = exitFromTile;
			}
			List<Pawn> tmpPawns = new List<Pawn>(pawns);
			foreach ( Pawn pawn in tmpPawns ) {
				if (pawn.IsFreeColonist) {
					TaleRecorder.RecordTale(TaleDefOf.CaravanFormed, new object[]{pawn});
				}
			}
			Caravan caravan = CaravanMaker.MakeCaravan(tmpPawns, faction, exitFromTile, false);
			//Making this a callback like onClose would allow the broader use of the dialog.
			QOLCity city = this.wo as QOLCity;
			if ( city != null ) {
				foreach ( Pawn pawn in tmpPawns ) {
					city.RemoveCitizen(pawn);
				}
			}
			if (!caravan.pather.Moving && caravan.Tile != directionTile) {
				caravan.pather.StartPath(directionTile, null, true, true);
				caravan.pather.nextTileCostLeft /= 2f;
				caravan.tweener.ResetTweenedPosToRoot();
			}
			if (destinationTile != -1) {
				List<FloatMenuOption> choices = FloatMenuMakerWorld.ChoicesAtFor(destinationTile, caravan);
				if (choices.Any(choice => !choice.Disabled)) {
					choices.First(choice => !choice.Disabled).action();
				} else {
					caravan.pather.StartPath(destinationTile, null, true, true);
				}
			}
			if (sendMessage) {
				TaggedString taggedString = "MessageFormedCaravan".Translate(caravan.Name).CapitalizeFirst();
				if (caravan.pather.Moving && caravan.pather.ArrivalAction != null) {
					taggedString += " " + "MessageFormedCaravan_Orders".Translate() + ": " + caravan.pather.ArrivalAction.Label + ".";
				}
				Messages.Message(taggedString, caravan, MessageTypeDefOf.TaskCompletion, true);
			}
			return caravan;
		}

		private bool TryFormAndSendCaravan()
		{
			return this.DebugTryFormCaravanInstantly();
		}

		private bool TryReformCaravan()
		{
			List<Pawn> pawnsFromTransferables = TransferableUtility.GetPawnsFromTransferables(this.transferables);
			if (!this.CheckForErrors(pawnsFromTransferables))
			{
				return false;
			}
			this.AddItemsFromTransferablesToRandomInventories(pawnsFromTransferables);
			Caravan caravan = CaravanExitMapUtility.ExitMapAndCreateCaravan(pawnsFromTransferables, Faction.OfPlayer, this.CurrentTile, this.CurrentTile, this.destinationTile, false);
			TaggedString taggedString = "MessageReformedCaravan".Translate();
			if (caravan.pather.Moving && caravan.pather.ArrivalAction != null)
			{
				taggedString += " " + "MessageFormedCaravan_Orders".Translate() + ": " + caravan.pather.ArrivalAction.Label + ".";
			}
			Messages.Message(taggedString, caravan, MessageTypeDefOf.TaskCompletion, false);
			return true;
		}

		private void AddItemsFromTransferablesToRandomInventories(List<Pawn> pawns)
		{
			List<TransferableOneWay> arg_39_0 = this.transferables;
			Predicate<TransferableOneWay> arg_39_1;
			if ((arg_39_1 = Dialog_FormCaravanQOL.Handler.isPawn) == null)
			{
				arg_39_1 = (Dialog_FormCaravanQOL.Handler.isPawn = new Predicate<TransferableOneWay>(Dialog_FormCaravanQOL.Handler.handler.pawn));
			}
			arg_39_0.RemoveAll(arg_39_1);
			if (this.ListPlayerPawnsInventorySeparately)
			{
				for (int i = 0; i < pawns.Count; i++)
				{
					if (Dialog_FormCaravanQOL.CanListInventorySeparately(pawns[i]))
					{
						ThingOwner<Thing> innerContainer = pawns[i].inventory.innerContainer;
						for (int j = innerContainer.Count - 1; j >= 0; j--)
						{
							this.RemoveCarriedItemFromTransferablesOrDrop(innerContainer[j], pawns[i], this.transferables);
						}
					}
				}
				for (int k = 0; k < this.transferables.Count; k++)
				{
					List<Thing> arg_F2_0 = this.transferables[k].things;
					Predicate<Thing> arg_F2_1;
					if ((arg_F2_1 = Dialog_FormCaravanQOL.Handler.isNotSpawned) == null)
					{
						arg_F2_1 = (Dialog_FormCaravanQOL.Handler.isNotSpawned = new Predicate<Thing>(Dialog_FormCaravanQOL.Handler.handler.notSpawned));
					}
					if (arg_F2_0.Any(arg_F2_1))
					{
						List<Thing> arg_12A_0 = this.transferables[k].things;
						Func<Thing, bool> arg_12A_1;
						if ((arg_12A_1 = Dialog_FormCaravanQOL.Handler.isSpawned) == null)
						{
							arg_12A_1 = (Dialog_FormCaravanQOL.Handler.isSpawned = new Func<Thing, bool>(Dialog_FormCaravanQOL.Handler.handler.spawned));
						}
						arg_12A_0.SortBy(arg_12A_1);
					}
				}
			}
			Action<Thing, IThingHolder> addItem = null;
			for (int l = 0; l < this.transferables.Count; l++)
			{
				if (!(this.transferables[l].AnyThing is Corpse))
				{
					List<Thing> arg_1AA_0 = this.transferables[l].things;
					int arg_1AA_1 = this.transferables[l].CountToTransfer;
					Action<Thing, IThingHolder> arg_1AA_2;
					if ((arg_1AA_2 = addItem) == null)
					{
						arg_1AA_2 = (addItem = delegate(Thing splitPiece, IThingHolder originalHolder)
						{
							Thing item = splitPiece.TryMakeMinified();
							CaravanInventoryUtility.FindPawnToMoveInventoryTo(item, pawns, null, null).inventory.innerContainer.TryAdd(item, true);
						});
					}
					TransferableUtility.Transfer(arg_1AA_0, arg_1AA_1, arg_1AA_2);
				}
			}
			Action<Thing, int> addNoItems = null;
			for (int m = 0; m < this.transferables.Count; m++)
			{
				if (this.transferables[m].AnyThing is Corpse)
				{
					List<Thing> arg_229_0 = this.transferables[m].things;
					int arg_229_1 = this.transferables[m].CountToTransfer;
					Action<Thing, int> arg_229_2;
					if ((arg_229_2 = addNoItems) == null)
					{
						arg_229_2 = (addNoItems = delegate(Thing originalThing, int numToTake)
						{
							if (this.AutoStripSpawnedCorpses)
							{
								Corpse corpse = originalThing as Corpse;
								if (corpse != null && corpse.Spawned)
								{
									corpse.Strip();
								}
							}
							Thing item = originalThing.SplitOff(numToTake);
							CaravanInventoryUtility.FindPawnToMoveInventoryTo(item, pawns, null, null).inventory.innerContainer.TryAdd(item, true);
						});
					}
					TransferableUtility.TransferNoSplit(arg_229_0, arg_229_1, arg_229_2, true, true);
				}
			}
		}

		private bool CheckForErrors(List<Pawn> pawns)
		{
			if (this.MustChooseRoute && this.destinationTile < 0)
			{
				Messages.Message("MessageMustChooseRouteFirst".Translate(), MessageTypeDefOf.RejectInput, false);
				return false;
			}
			if (!this.reform && this.startingTile < 0)
			{
				Messages.Message("MessageNoValidExitTile".Translate(), MessageTypeDefOf.RejectInput, false);
				return false;
			}
			List<Pawn> arg_8C_0 = pawns;
			Predicate<Pawn> arg_8C_1;
			if ((arg_8C_1 = Dialog_FormCaravanQOL.Handler.isDownedColonist) == null)
			{
				arg_8C_1 = (Dialog_FormCaravanQOL.Handler.isDownedColonist = new Predicate<Pawn>(Dialog_FormCaravanQOL.Handler.handler.downedColonist));
			}
			if (!arg_8C_0.Any(arg_8C_1))
			{
				Messages.Message("CaravanMustHaveAtLeastOneColonist".Translate(), MessageTypeDefOf.RejectInput, false);
				return false;
			}
			if (!this.reform && this.MassUsage > this.MassCapacity)
			{
				this.FlashMass();
				Messages.Message("TooBigCaravanMassUsage".Translate(), MessageTypeDefOf.RejectInput, false);
				return false;
			}
			Pawn pawn = pawns.Find((Pawn x) => !x.IsColonist && !pawns.Any((Pawn y) => y.IsColonist && y.CanReach(x, PathEndMode.Touch, Danger.Deadly)));
			if (pawn != null)
			{
				Messages.Message("CaravanPawnIsUnreachable".Translate(pawn.LabelShort, pawn), pawn, MessageTypeDefOf.RejectInput, false);
				return false;
			}
			for (int i = 0; i < this.transferables.Count; i++)
			{
				if (this.transferables[i].ThingDef.category == ThingCategory.Item)
				{
					int countToTransfer = this.transferables[i].CountToTransfer;
					int num = 0;
					if (countToTransfer > 0)
					{
						for (int j = 0; j < this.transferables[i].things.Count; j++)
						{
							Thing t = this.transferables[i].things[j];
							if (!t.Spawned || pawns.Any((Pawn x) => x.IsColonist && x.CanReach(t, PathEndMode.Touch, Danger.Deadly)))
							{
								num += t.stackCount;
								if (num >= countToTransfer)
								{
									break;
								}
							}
						}
						if (num < countToTransfer)
						{
							if (countToTransfer == 1)
							{
								Messages.Message("CaravanItemIsUnreachableSingle".Translate(this.transferables[i].ThingDef.label), MessageTypeDefOf.RejectInput, false);
							}
							else
							{
								Messages.Message("CaravanItemIsUnreachableMulti".Translate(countToTransfer, this.transferables[i].ThingDef.label), MessageTypeDefOf.RejectInput, false);
							}
							return false;
						}
					}
				}
			}
			return true;
		}

		private void AddPawnsToTransferables()
		{
			foreach( Pawn pawn in this.pawns ) {
				this.AddToTransferables(pawn, (this.reform || this.mapAboutToBeRemoved) && !CaravanUtility.ShouldAutoCapture(pawn, Faction.OfPlayer));
			}
		}

		private void AddItemsToTransferables()
		{
			List<Thing> list = new List<Thing>(this.items);
			for (int i = 0; i < list.Count; i++)
			{
				this.AddToTransferables(list[i], false);
			}
			if (this.AutoStripSpawnedCorpses)
			{
				for (int j = 0; j < list.Count; j++)
				{
					if (list[j].Spawned)
					{
						this.TryAddCorpseInventoryAndGearToTransferables(list[j]);
					}
				}
			}
			if (this.ListPlayerPawnsInventorySeparately)
			{
				List<Pawn> list2 = new List<Pawn>(this.pawns);
				for (int k = 0; k < list2.Count; k++)
				{
					if (Dialog_FormCaravanQOL.CanListInventorySeparately(list2[k]))
					{
						ThingOwner<Thing> innerContainer = list2[k].inventory.innerContainer;
						for (int l = 0; l < innerContainer.Count; l++)
						{
							this.AddToTransferables(innerContainer[l], true);
							if (this.AutoStripSpawnedCorpses && innerContainer[l].Spawned)
							{
								this.TryAddCorpseInventoryAndGearToTransferables(innerContainer[l]);
							}
						}
					}
				}
			}
		}

		private void TryAddCorpseInventoryAndGearToTransferables(Thing potentiallyCorpse)
		{
			Corpse corpse = potentiallyCorpse as Corpse;
			if (corpse != null)
			{
				this.AddCorpseInventoryAndGearToTransferables(corpse);
			}
		}

		private void AddCorpseInventoryAndGearToTransferables(Corpse corpse)
		{
			Pawn_InventoryTracker inventory = corpse.InnerPawn.inventory;
			Pawn_ApparelTracker apparel = corpse.InnerPawn.apparel;
			Pawn_EquipmentTracker equipment = corpse.InnerPawn.equipment;
			for (int i = 0; i < inventory.innerContainer.Count; i++)
			{
				this.AddToTransferables(inventory.innerContainer[i], false);
			}
			if (apparel != null)
			{
				List<Apparel> wornApparel = apparel.WornApparel;
				for (int j = 0; j < wornApparel.Count; j++)
				{
					this.AddToTransferables(wornApparel[j], false);
				}
			}
			if (equipment != null)
			{
				List<ThingWithComps> allEquipmentListForReading = equipment.AllEquipmentListForReading;
				for (int k = 0; k < allEquipmentListForReading.Count; k++)
				{
					this.AddToTransferables(allEquipmentListForReading[k], false);
				}
			}
		}

		private void RemoveCarriedItemFromTransferablesOrDrop(Thing carried, Pawn carrier, List<TransferableOneWay> transferables)
		{
			TransferableOneWay transferableOneWay = TransferableUtility.TransferableMatchingDesperate(carried, transferables, TransferAsOneMode.PodsOrCaravanPacking);
			int num;
			if (transferableOneWay == null)
			{
				num = carried.stackCount;
			}
			else if (transferableOneWay.CountToTransfer >= carried.stackCount)
			{
				transferableOneWay.AdjustBy(-carried.stackCount);
				transferableOneWay.things.Remove(carried);
				num = 0;
			}
			else
			{
				num = carried.stackCount - transferableOneWay.CountToTransfer;
				transferableOneWay.AdjustTo(0);
			}
			if (num > 0)
			{
				Thing thing = carried.SplitOff(num);
				if (carrier.SpawnedOrAnyParentSpawned)
				{
					GenPlace.TryPlaceThing(thing, carrier.PositionHeld, carrier.MapHeld, ThingPlaceMode.Near, null, null, default(Rot4));
					return;
				}
				thing.Destroy(DestroyMode.Vanish);
			}
		}

		private void FlashMass()
		{
			this.lastMassFlashTime = Time.time;
		}

		/*public static bool CanListInventorySeparately(Pawn p)
		{
			return p.Faction == Faction.OfPlayer || p.HostFaction == Faction.OfPlayer;
		}*/

		private void SetToSendEverything()
		{
			for (int i = 0; i < this.transferables.Count; i++)
			{
				this.transferables[i].AdjustTo(this.transferables[i].GetMaximumToTransfer());
			}
			this.CountToTransferChanged();
		}

		private void CountToTransferChanged()
		{
			this.massUsageDirty = true;
			this.massCapacityDirty = true;
			this.tilesPerDayDirty = true;
			this.daysWorthOfFoodDirty = true;
			this.foragedFoodPerDayDirty = true;
			this.visibilityDirty = true;
			this.ticksToArriveDirty = true;
			if (this.autoSelectFoodAndMedicine)
			{
				this.SelectApproximateBestFoodAndMedicine();
			}
		}

		private void SelectApproximateBestFoodAndMedicine()
		{
			IEnumerable<TransferableOneWay> arg_25_0 = this.transferables;
			Func<TransferableOneWay, bool> arg_25_1;
			if ((arg_25_1 = Dialog_FormCaravanQOL.Handler.isMedicine) == null)
			{
				arg_25_1 = (Dialog_FormCaravanQOL.Handler.isMedicine = new Func<TransferableOneWay, bool>(Dialog_FormCaravanQOL.Handler.handler.medicine));
			}
			IEnumerable<TransferableOneWay> enumerable = arg_25_0.Where(arg_25_1);
			IEnumerable<TransferableOneWay> arg_50_0 = this.transferables;
			Func<TransferableOneWay, bool> arg_50_1;
			if ((arg_50_1 = Dialog_FormCaravanQOL.Handler.isEdibleNotDrugOrCorpse) == null)
			{
				arg_50_1 = (Dialog_FormCaravanQOL.Handler.isEdibleNotDrugOrCorpse = new Func<TransferableOneWay, bool>(Dialog_FormCaravanQOL.Handler.handler.edibleNotDrugOrCorpse));
			}
			IEnumerable<TransferableOneWay> enumerable2 = arg_50_0.Where(arg_50_1);
			Dialog_FormCaravanQOL.tmpBeds.Clear();
			for (int i = 0; i < this.transferables.Count; i++)
			{
				for (int j = 0; j < this.transferables[i].things.Count; j++)
				{
					Thing thing = this.transferables[i].things[j];
					for (int k = 0; k < thing.stackCount; k++)
					{
						Building_Bed building_Bed;
						if ((building_Bed = (thing.GetInnerIfMinified() as Building_Bed)) != null && building_Bed.def.building.bed_caravansCanUse)
						{
							for (int l = 0; l < building_Bed.SleepingSlotsCount; l++)
							{
								Dialog_FormCaravanQOL.tmpBeds.Add(this.transferables[i]);
							}
						}
					}
				}
			}
			List<TransferableOneWay> arg_14B_0 = Dialog_FormCaravanQOL.tmpBeds;
			Func<TransferableOneWay, float> arg_14B_1;
			if ((arg_14B_1 = Dialog_FormCaravanQOL.Handler.getBedRestEffectiveness) == null)
			{
				arg_14B_1 = (Dialog_FormCaravanQOL.Handler.getBedRestEffectiveness = new Func<TransferableOneWay, float>(Dialog_FormCaravanQOL.Handler.handler.bedRestEffectiveness));
			}
			arg_14B_0.SortByDescending(arg_14B_1);
			using (IEnumerator<TransferableOneWay> enumerator = enumerable.GetEnumerator())
			{
				while (enumerator.MoveNext())
				{
					enumerator.Current.AdjustTo(0);
				}
			}
			using (IEnumerator<TransferableOneWay> enumerator = enumerable2.GetEnumerator())
			{
				while (enumerator.MoveNext())
				{
					enumerator.Current.AdjustTo(0);
				}
			}
			using (List<TransferableOneWay>.Enumerator enumerator2 = Dialog_FormCaravanQOL.tmpBeds.GetEnumerator())
			{
				while (enumerator2.MoveNext())
				{
					enumerator2.Current.AdjustTo(0);
				}
			}
			List<Pawn> pawnsFromTransferables = TransferableUtility.GetPawnsFromTransferables(this.transferables);
			if (!pawnsFromTransferables.Any<Pawn>())
			{
				return;
			}
			foreach (Pawn current in pawnsFromTransferables)
			{
				TransferableOneWay transferableOneWay = this.BestBedFor(current);
				if (transferableOneWay != null)
				{
					Dialog_FormCaravanQOL.tmpBeds.Remove(transferableOneWay);
					if (transferableOneWay.CanAdjustBy(1).Accepted)
					{
						this.AddOneIfMassAllows(transferableOneWay);
					}
				}
				if (!current.AnimalOrWildMan() && !current.guest.IsPrisoner)
				{
					for (int m = 0; m < 2; m++)
					{
						Transferable transferable = this.BestMedicineItemFor(current, enumerable);
						if (transferable != null)
						{
							this.AddOneIfMassAllows(transferable);
						}
					}
				}
			}
			if (this.destinationTile == -1 || !DaysWorthOfFoodCalculator.AnyFoodEatingPawn(pawnsFromTransferables) || !enumerable2.Any<TransferableOneWay>())
			{
				return;
			}
			try
			{
				using (WorldPath worldPath = Find.WorldPathFinder.FindPath(this.CurrentTile, this.destinationTile, null, null))
				{
					int ticksPerMove = CaravanTicksPerMoveUtility.GetTicksPerMove(new CaravanTicksPerMoveUtility.CaravanInfo(this), null);
					CaravanArrivalTimeEstimator.EstimatedTicksToArriveToEvery(this.CurrentTile, this.destinationTile, worldPath, 0f, ticksPerMove, Find.TickManager.TicksAbs, Dialog_FormCaravanQOL.tmpTicksToArrive);
					float num = (float)Dialog_FormCaravanQOL.tmpTicksToArrive.Last<Pair<int, int>>().Second / 60000f + 4f;
					float num2;
					do
					{
						num2 = DaysWorthOfFoodCalculator.ApproxDaysWorthOfFood(this.transferables, this.CurrentTile, this.IgnoreInventoryMode, Faction.OfPlayer, worldPath, 0f, ticksPerMove);
						if (num2 >= num)
						{
							break;
						}
						bool flag = false;
						foreach (Pawn current2 in pawnsFromTransferables)
						{
							Transferable transferable2 = this.BestFoodItemFor(current2, enumerable2, Dialog_FormCaravanQOL.tmpTicksToArrive);
							if (transferable2 != null && this.AddOneIfMassAllows(transferable2))
							{
								flag = true;
							}
						}
						if (!flag)
						{
							break;
						}
					}
					while (num2 < num && this.MassUsage < this.MassCapacity);
				}
			}
			finally
			{
				Dialog_FormCaravanQOL.tmpTicksToArrive.Clear();
				this.daysWorthOfFoodDirty = true;
				this.massUsageDirty = true;
			}
		}

		private bool AddOneIfMassAllows(Transferable transferable)
		{
			if (transferable.CanAdjustBy(1).Accepted && this.MassUsage + transferable.ThingDef.BaseMass < this.MassCapacity)
			{
				transferable.AdjustBy(1);
				this.massUsageDirty = true;
				return true;
			}
			return false;
		}

		private TransferableOneWay BestBedFor(Pawn pawn)
		{
			for (int i = 0; i < Dialog_FormCaravanQOL.tmpBeds.Count; i++)
			{
				Thing innerIfMinified = Dialog_FormCaravanQOL.tmpBeds[i].AnyThing.GetInnerIfMinified();
				if (RestUtility.CanUseBedEver(pawn, innerIfMinified.def))
				{
					return Dialog_FormCaravanQOL.tmpBeds[i];
				}
			}
			return null;
		}

		private Transferable BestFoodItemFor(Pawn pawn, IEnumerable<TransferableOneWay> food, List<Pair<int, int>> ticksToArrive)
		{
			Transferable result = null;
			float num = 0f;
			if (!pawn.RaceProps.EatsFood)
			{
				return result;
			}
			foreach (TransferableOneWay current in food)
			{
				if (current.CanAdjustBy(1).Accepted)
				{
					float foodScore = this.GetFoodScore(pawn, current.AnyThing, ticksToArrive);
					if (foodScore > num)
					{
						result = current;
						num = foodScore;
					}
				}
			}
			return result;
		}

		private float GetFoodScore(Pawn pawn, Thing food, List<Pair<int, int>> ticksToArrive)
		{
			if (!CaravanPawnsNeedsUtility.CanEatForNutritionEver(food.def, pawn) || !food.def.ingestible.canAutoSelectAsFoodForCaravan)
			{
				return 0f;
			}
			float num = CaravanPawnsNeedsUtility.GetFoodScore(food.def, pawn, food.GetStatValue(StatDefOf.Nutrition, true));
			CompRottable compRottable = food.TryGetComp<CompRottable>();
			if (compRottable != null && compRottable.Active && DaysUntilRotCalculator.ApproxTicksUntilRot_AssumeTimePassesBy(compRottable, this.CurrentTile, ticksToArrive) < ticksToArrive.Last<Pair<int, int>>().Second)
			{
				num *= 0.1f;
			}
			return num;
		}

		private Transferable BestMedicineItemFor(Pawn pawn, IEnumerable<TransferableOneWay> medicine)
		{
			Transferable transferable = null;
			float num = 0f;
			foreach (TransferableOneWay current in medicine)
			{
				Thing anyThing = current.AnyThing;
				if (current.CanAdjustBy(1).Accepted && pawn.playerSettings.medCare.AllowsMedicine(anyThing.def))
				{
					float statValue = anyThing.GetStatValue(StatDefOf.MedicalPotency, true);
					if (transferable == null || statValue > num)
					{
						transferable = current;
						num = statValue;
					}
				}
			}
			return transferable;
		}
	}
}
