﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 08.10.2018
 * Time: 17:37
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using RIMMSqol.remnantcolony.actions;
using RIMMSqol.remnantcolony.listeners;
using RIMMSqol.remnantcolony.pawnvalues;
using RimWorld;
using RimWorld.Planet;
using UnityEngine;
using Verse;

namespace RIMMSqol.remnantcolony
{
	/// <summary>
	/// Description of QOLCity.
	/// </summary>
	public class QOLCity : WorldObject, ICommunicable, ITrader, IThingHolder {
		//TODO possibly add a mechanic for delayed jobs. Like exchanging citizens through comms with a colony(2 "toils" with a pause).
		//TODO interaction for trade and people exchange via caravan.
		//TODO a government or management resource that is filled through staffing a senate building. Management is consumed by external districts. Lack of management reduces productivity of external districts.
		public QOLCityDef defCity;
		//save state
		public string cityName = "unnamed";
		public List<QOLCityProject> projects = new List<QOLCityProject>();
		public QOLCityPopulation population = new QOLCityPopulation();
		public int lastSimulatedTick = 0;
		protected ThingOwner<Thing> stock;
		public List<QOLCityAction> actions = new List<QOLCityAction>();
		public int simulationInterval = 60000;
		
		//unsaved
		public QOLCityState stateLastSimulatedTick;
		public QOLCityState stateProjectedState;
		public QOLCityProject buildStateActive;
		protected Dictionary<string,List<Pair<int,QOLCityListener>>> listeners;
		protected Stack<QOLCityEvent> eventStack;
		protected List<IQOLCityListenerModification> listenerModifications = new List<IQOLCityListenerModification>();
		protected Dictionary<QOLCityResourceDef,HashSet<QOLCityProjectDef>> defsPerResource;
		
		#region fixedbehaviour
		public string GetCallLabel() { return cityName; }
		
		public string GetInfoText() { return defCity.description; }
		
		public Faction GetFaction() { return Faction; }
		
		public FloatMenuOption CommFloatMenuOption(Building_CommsConsole console, Pawn negotiator) { return new FloatMenuOption(Label, ()=>console.GiveUseCommsJob(negotiator,this)); }

		public TraderKindDef TraderKind { get { return defCity.traderKind; } }

		public IEnumerable<Thing> Goods { get { return this.stock.InnerListForReading; } }

		public int RandomPriceFactorSeed { get { return 1; } }

		public string TraderName { get { return GetCallLabel(); } }

		public bool CanTradeNow { get { return true; } }

		public float TradePriceImprovementOffsetForPlayer { get { return 0; } }
		#endregion
		
		override public void ExposeData() {
			base.ExposeData();
			Scribe_Defs.Look<QOLCityDef>(ref defCity, "defCity");
			Scribe_Deep.Look<QOLCityPopulation>(ref population, "population", new object[0]);
			Scribe_Values.Look<string>(ref cityName, "cityName", "unnamed");
			Scribe_Values.Look<int>(ref lastSimulatedTick, "lastSimulatedTick", 0);
			Scribe_Deep.Look<ThingOwner<Thing>>(ref this.stock, "stock", new object[]{this});
			Scribe_Collections.Look<QOLCityProject>(ref projects, "projects");
			Scribe_Collections.Look<QOLCityAction>(ref actions, "actions");
			Scribe_Values.Look<int>(ref simulationInterval, "simulationInterval", 60000);
			
			if (Scribe.mode == LoadSaveMode.PostLoadInit) {
				Init();
				Refresh();
			}
		}

		public TradeCurrency TradeCurrency {
			get {
				return TradeCurrency.Silver;
			}
		}
		
		virtual public void Init() {
			if ( stock == null ) stock = new ThingOwner<Thing>(this);
			if ( projects == null ) projects = new List<QOLCityProject>();
			if ( actions == null ) actions = new List<QOLCityAction>();
			if ( population == null ) population = new QOLCityPopulation();
			if ( listeners == null ) listeners = new Dictionary<string, List<Pair<int,QOLCityListener>>>();
			if ( eventStack == null ) eventStack = new Stack<QOLCityEvent>();
			
			//Removing projects that are no longer configured to be part of the city.
			List<QOLCityProject> tempProjects = new List<QOLCityProject>(projects);
			foreach ( QOLCityProject proj in tempProjects ) {
				if ( !defCity.projects.Contains(proj.def) ) {
					Log.Warning("Removing project "+proj.def.label+" from city "+cityName+" since it is no longer configured to be part of it.");
					RemoveProject(proj);
				}
			}
			
			//TODO adding methods for adding and removing actions.
			//Evaluating if the right amount of action instances exist, if not alter instance counter or remove/add new actions.
			//This is necessary if the configuration has changed.
			Dictionary<QOLCityActionDef,int> expectedActions = new Dictionary<QOLCityActionDef, int>();
			foreach ( QOLCityProject proj in projects ) {
				if ( !proj.IsBuilt ) continue;
				foreach ( QOLCityActionDef act in proj.def.actions ) {
					int v;
					if ( expectedActions.TryGetValue(act, out v) ) {
						expectedActions[act] = v + 1;
					} else {
						expectedActions[act] = 1;
					}
				}
			}
			List<QOLCityAction> tempActions = new List<QOLCityAction>(actions);
			foreach ( QOLCityAction act in tempActions ) {
				int v;
				if ( expectedActions.TryGetValue(act.def, out v) ) {
					//correcting instance counter if necessary
					if ( act.instances != v ) {
						Log.Warning("Correcting number of instances for action "+act.def.label+" on city "+cityName+".");
						act.instances = v;
					}
					expectedActions.Remove(act.def);
				} else {
					//removing actions that should no longer exist
					Log.Warning("Removing action "+act.def.label+" from city "+cityName+" since none of the projects provide it anymore.");
					RemoveActionFully(act);
					//actions.Remove(act);
				}
			}
			foreach ( QOLCityActionDef act in expectedActions.Keys ) {
				//adding missing actions
				Log.Warning("Adding action "+act.label+" to city "+cityName+" since existing projects started to provide it.");
				AddAction(act);
				//actions.Add(act.CreateAction());
			}
			
			foreach ( QOLCityNeedDef n in defCity.needs ) {
				RegisterListeners(n, false);
			}
			foreach ( QOLCityResourceDef r in defCity.resources ) {
				RegisterListeners(r, false);
			}
			foreach ( QOLSocietyIndicatorDef si in defCity.indicators ) {
				RegisterListeners(si, false);
			}
			RegisterListeners(defCity, false);
			foreach ( QOLCityProject proj in projects ) {
				RegisterListeners(proj, false);
			}
			foreach ( QOLCityAction action in actions ) {
				RegisterListeners(action, false);
			}
			foreach ( List<Pair<int,QOLCityListener>> listenersForEvent in listeners.Values ) {
				listenersForEvent.SortBy(p=>p.First);
			}
			
			autospawn();
		}
		
		public virtual void Remove() {
			List<QOLCityProject> temps = new List<QOLCityProject>(projects);
			foreach ( QOLCityProject proj in temps ) {
				RemoveProject(proj);
			}
			stock.ClearAndDestroyContents();
			Find.WorldObjects.Remove(this);
		}
		
		#region listener
		protected void RegisterListeners(IQOLCityListenerOwner owner, bool sort = true) {
			if ( owner == null || owner.Listeners.NullOrEmpty() ) return;
			
			foreach ( QOLCityListenerDef ld in owner.Listeners ) {
				QOLCityListener l = ld.CreateListener(owner);
				if ( l != null ) {
					foreach ( QOLCityListenerDef.EventProcessingProperties eventProperties in ld.eventIds ) {
						RegisterListener(eventProperties, l, false);
					}
				} else {
					Log.Error("Failed to create listener "+ld.defName+"! Skipping registration.");
				}
			}
			
			if ( sort ) {
				List<string> listenersToSort = new List<string>();
				foreach ( QOLCityListenerDef ld in owner.Listeners ) {
					foreach ( QOLCityListenerDef.EventProcessingProperties eventProperties in ld.eventIds ) {
						listenersToSort.AddDistinct(eventProperties.eventId);
					}
				}
				foreach ( string eventId in listenersToSort ) {
					List<Pair<int,QOLCityListener>> listenersForEvent;
					if ( listeners.TryGetValue(eventId, out listenersForEvent) ) {
						listenersForEvent.SortBy(p=>p.First);
					}
				}
			}
		}
		
		protected void RegisterListener(QOLCityListenerDef.EventProcessingProperties eventProperties, QOLCityListener listener, bool sort = true) {
			if ( eventProperties == null || eventProperties.eventId == null || listener == null ) return;
			List<Pair<int,QOLCityListener>> listenersForEvent;
			if ( !listeners.TryGetValue(eventProperties.eventId, out listenersForEvent) ) {
				listenersForEvent = new List<Pair<int,QOLCityListener>>();
				listeners.Add(eventProperties.eventId,listenersForEvent);
			}
			if ( !listenersForEvent.Any(p=>p.Second.def == listener.def && p.Second.owner == listener.owner) ) {
				//new copy to allow concurrent access
				listenersForEvent = new List<Pair<int, QOLCityListener>>(listenersForEvent);
				listenersForEvent.Add(new Pair<int, QOLCityListener>(eventProperties.order,listener));
				listeners[eventProperties.eventId] = listenersForEvent;
				if ( sort ) listenersForEvent.SortBy(p=>p.First);
			} else {
				Log.Warning("Tried to register duplicate listener with definition \""+listener.def.defName+"\" for event \""+eventProperties.eventId+"\".");
			}
		}
		
		protected void DeregisterListeners(IQOLCityListenerOwner owner) {
			if ( owner == null || owner.Listeners.NullOrEmpty() ) return;
			
			foreach ( QOLCityListenerDef ld in owner.Listeners ) {
				foreach ( QOLCityListenerDef.EventProcessingProperties eventProperties in ld.eventIds ) {
					DeregisterListener(eventProperties, ld, owner);
				}
			}
		}
		
		protected void DeregisterListener(QOLCityListenerDef.EventProcessingProperties eventProperties, QOLCityListenerDef def, IQOLCityListenerOwner owner) {
			if ( eventProperties == null || eventProperties.eventId == null ) return;
			List<Pair<int,QOLCityListener>> listenersForEvent;
			if ( listeners.TryGetValue(eventProperties.eventId, out listenersForEvent) ) {
				listenersForEvent = new List<Pair<int, QOLCityListener>>(listenersForEvent);
				listenersForEvent.RemoveAll(p=>(def == null || p.Second.def == def) && (owner == null || p.Second.owner == owner));
				listeners[eventProperties.eventId] = listenersForEvent;
			}
		}
		
		public void QueueDeregisteringOfListeners(IQOLCityListenerOwner owner) {
			if ( owner == null || owner.Listeners.NullOrEmpty() ) return;
			
			foreach ( QOLCityListenerDef ld in owner.Listeners ) {
				foreach ( QOLCityListenerDef.EventProcessingProperties eventProperties in ld.eventIds ) {
					QueueDeregisteringOfListener(eventProperties, ld, owner);
				}
			}
		}
		
		protected void ModifyListeners() {
			foreach ( IQOLCityListenerModification modification in listenerModifications ) {
				modification.apply(this);
			}
			listenerModifications.Clear();
		}
		
		public void QueueRegisteringOfListener(QOLCityListenerDef.EventProcessingProperties eventProperties, QOLCityListener listener) {
			listenerModifications.Add(new QOLCityListenerModificationRegister(eventProperties, listener));
		}
		
		public void QueueDeregisteringOfListener(QOLCityListenerDef.EventProcessingProperties eventProperties, QOLCityListenerDef def, IQOLCityListenerOwner owner) {
			listenerModifications.Add(new QOLCityListenerModificationDeregister(eventProperties, def, owner));
		}
		
		public void QueueDeregisteringOfListener(QOLCityListener listener, string eventId = null) {
			if ( listener == null ) return;
			foreach ( QOLCityListenerDef.EventProcessingProperties eventProperties in listener.def.eventIds ) {
				if ( eventId == null || eventId == eventProperties.eventId ) QueueDeregisteringOfListener(eventProperties, listener.def, listener.owner);
			}
		}
		
		protected QOLCityEvent currentEvent;
		
		public void RaiseEvent(QOLCityEvent e) {
			if ( currentEvent != null ) {
				eventStack.Push(e);
			} else {
				eventStack.Push(e);
				while ( eventStack.Count > 0 ) {
					if ( !listenerModifications.NullOrEmpty() ) {
						ModifyListeners();
					}
					currentEvent = eventStack.Pop();
					List<Pair<int,QOLCityListener>> listenersForEvent;
					if ( listeners.TryGetValue(currentEvent.EventId, out listenersForEvent) ) {
						//Log.Message("Processing event(with listeners): "+currentEvent.EventId);
						foreach ( Pair<int,QOLCityListener> p in listenersForEvent ) {
							//Log.Message("Processing listener: "+p.Second.def);
							if ( currentEvent.ShouldProcess ) p.Second.OnEvent(this,currentEvent);
							else break;
						}
					}
				}
				currentEvent = null;
			}
		}
		
		protected interface IQOLCityListenerModification {
			void apply(QOLCity city);
		}
		
		protected class QOLCityListenerModificationRegister : IQOLCityListenerModification {
			protected readonly QOLCityListenerDef.EventProcessingProperties eventProperties; 
			protected readonly QOLCityListener listener;
			public QOLCityListenerModificationRegister(QOLCityListenerDef.EventProcessingProperties eventProperties, QOLCityListener listener) {
				this.eventProperties = eventProperties;
				this.listener = listener;
			}
			public void apply(QOLCity city) {
				city.RegisterListener(eventProperties,listener);
			}
		}
		
		protected class QOLCityListenerModificationDeregister : IQOLCityListenerModification {
			protected readonly QOLCityListenerDef.EventProcessingProperties eventProperties; 
			protected readonly QOLCityListenerDef def; 
			protected readonly IQOLCityListenerOwner owner;
			public QOLCityListenerModificationDeregister(QOLCityListenerDef.EventProcessingProperties eventProperties, QOLCityListenerDef def, IQOLCityListenerOwner owner) {
				this.eventProperties = eventProperties;
				this.def = def;
				this.owner = owner;
			}
			public void apply(QOLCity city) {
				city.DeregisterListener(eventProperties,def,owner);
			}
		}
		#endregion
		
		virtual public bool AddCitizen(Pawn p) {
			if ( defCity.population.IsImmigrationAllowed(this,p) ) {
				//TODO determine social standing(slave,prisoner,citizen)
				//TODO unpack inventory into storage
				population.citizens.Add(p,defCity.population.socialStandings[0]);
				return true;
			}
			return false;
		}
		
		virtual public bool RemoveCitizen(Pawn p) {
			return population.citizens.Remove(p);
		}
		
		virtual public void KillCitizen(Pawn p, DamageInfo? dinfo = null, Hediff exactCulprit = null) {
			if ( RemoveCitizen(p) ) {
				p.Kill(dinfo, exactCulprit);
				p.health.NotifyPlayerOfKilled(dinfo, exactCulprit, null);
			}
		}
		
		virtual public float NeedBalance(QOLCityNeedDef need, QOLCityState state = null) {
			if ( state == null ) state = stateProjectedState;
			ProvidedAndRequired par;
			if ( state != null && state.needs.TryGetValue(need, out par) ) {
				return par.provided.Value - par.required.Value;
			}
			return float.NaN;
		}
		
		virtual public ProvidedAndRequired NeedPaR(QOLCityNeedDef need, QOLCityState state = null) {
			if ( state == null ) state = stateProjectedState;
			ProvidedAndRequired par;
			if ( state != null && state.needs.TryGetValue(need, out par) ) {
				return par;
			}
			return null;
		}
		
		virtual public float LabourBalance(QOLCityState state = null) {
			return NeedBalance(defCity.labourDef, state);
		}
		
		virtual public ProvidedAndRequired LabourPaR(QOLCityState state = null) {
			return NeedPaR(defCity.labourDef, state);
		}
		
		virtual protected void ProjectWasBuilt(QOLCityProject proj) {
			if ( stateProjectedState != null ) {
				if ( !proj.def.spawnsInactive )
					stateProjectedState.Add(proj.def.states.active);
				else {
					stateProjectedState.Add(proj.def.states.inactive);
					DeactivateProject(proj, true, false);
				}
			}
			
			if ( !proj.def.actions.NullOrEmpty() ) {
				foreach ( QOLCityActionDef ad in proj.def.actions ) {
					AddAction(ad);
				}
			}
			
			if ( proj.def.sendCompletedNotification ) {
				Find.LetterStack.ReceiveLetter("ProjectCompleted".Translate(proj.def.label), 
									                               "ProjectCompletedNotification".Translate(GetCallLabel(), proj.def.label),
									                               LetterDefOf.PositiveEvent, (proj.worldObject ?? this), null, null);
			}
		}
		
		virtual public float TryToAddLabourToProject(QOLCityProject proj, float labour) {
			if ( labour > 0f && CanBuildProject(proj) ) {
				RaiseEvent(new QOLCityEvent_Project(QOLCityEvents.PreProjectAddingLabour,proj));
				float labourCombined = proj.labourInvested + labour;
				
				if ( proj.def.labour <= labourCombined ) {
					proj.labourInvested = proj.def.labour;
					RaiseEvent(new QOLCityEvent_Project(QOLCityEvents.PostProjectAddingLabour,proj));
					if ( proj.IsBuilt ) {
						ProjectWasBuilt(proj);
						RaiseEvent(new QOLCityEvent_Project(QOLCityEvents.PostProjectBuild,proj));
					}
					return labourCombined - proj.def.labour;
				}
				proj.labourInvested = labourCombined;
				stateProjectedState.Add(proj.def.states.build);
				buildStateActive = proj;
				RaiseEvent(new QOLCityEvent_Project(QOLCityEvents.PostProjectAddingLabour,proj));
				return 0f;
			}
			return labour;
		}
		
		virtual public bool CanBuildProject(QOLCityProject proj) {
			if ( proj.IsBuilt || !proj.IsActive ) {
				return false;
			}
			return CanBuildProject(proj.def);
		}
		
		virtual public bool CanBuildProject(QOLCityProjectDef proj) {
			if ( proj.requirements.Any(r=>!r.IsValid(this)) ) {
		    	return false;
		    }
			//if labour invested is greater 0 it means work has begun but not finished
			//if the project has not been built it won't influence the city unless it is building
			//So the resources are not affected through this proj by the time we check this.
			//Once a proj gets actual labour allocated during a turn it starts affecting the city. 
			QOLCityStateContribution stateBuild = proj.states.build;
			bool enoughResources = true;
			if ( !stateBuild.resources.NullOrEmpty() ) {
				foreach ( QOLCityStateContributionResource cpr in stateBuild.resources ) {
					NumericAdjustment na;
					if ( stateProjectedState.resources.TryGetValue(cpr.def, out na) ) {
						if ( -cpr.Value > na.Value ) {
							enoughResources = false;
							break;
						}
					}
				}
			}
			return enoughResources;
		}
		
		virtual public bool CanAddProject(QOLCityProjectDef def) {
			if ( !defCity.projects.Contains(def) ) {
				return false;
			}
			if ( !def.AllowsNumberOfInstances(projects.Count(p=>p.def == def)+1) ) {
				return false;
			}
			if ( def.requirements.Any(r=>!r.IsValid(this)) ) {
		    	return false;
		    }
			return true;
		}
		
		virtual public QOLCityAction AddAction(QOLCityActionDef act) {
			QOLCityAction instance = actions.Find(a=>a.def == act);
			if ( instance != null ) {
				instance.instances++;
			} else {
				instance = act.CreateAction();
				actions.Add(instance);
				RegisterListeners(instance);
			}
			return instance;
		}
		
		virtual public QOLCityProject AddProjectAndRefresh(QOLCityProjectDef def, int tileID = -1) {
			try {
				return AddProject(def, tileID);
			} finally {
				Refresh();
			}
		}
		
		virtual public QOLCityProject AddProject(QOLCityProjectDef def, int tileID = -1) {
			if ( !CanAddProject(def) ) {
				return null;
			}
			if ( def.worldObject != null && tileID < 0 ) {
				return null;
			}
			QOLCityProject proj;
			if ( def.stateClass != null ) {
				proj = (QOLCityProject)Activator.CreateInstance(def.stateClass, new object[]{def});
			} else {
				proj = new QOLCityProject(def);
			}
			if ( def.worldObject != null ) {
				proj.worldObject = WorldObjectMaker.MakeWorldObject(def.worldObject);
				if ( tileID != -1 ) proj.worldObject.Tile = tileID;
				else proj.worldObject.Tile = this.Tile;
				proj.worldObject.SetFaction(this.Faction);
				Find.WorldObjects.Add(proj.worldObject);
			} else proj.worldObject = null;
			proj.timeAdded = Find.TickManager.TicksGame;
			proj.Init();
			RaiseEvent(new QOLCityEvent_Project(QOLCityEvents.PreProjectAdded, proj));
			projects.Add(proj);
			RegisterListeners(proj);
			RaiseEvent(new QOLCityEvent_Project(QOLCityEvents.PostProjectAdded, proj));
			if ( proj.IsBuilt ) {
				ProjectWasBuilt(proj);
			}
			
			return proj;
		}
		
		virtual public bool RemoveActionFully(QOLCityAction act) {
			if ( actions.Remove(act) ) {
				DeregisterListeners(act);
				return true;
			}
			return false;
		}
		
		virtual public bool RemoveAction(QOLCityAction act) {
			if ( --act.instances <= 0 ) {
				return RemoveActionFully(act);
			}
			return false;
		}
		
		virtual public bool RemoveProject(QOLCityProject proj) {
			if ( projects.Remove(proj) ) {
				DeregisterListeners(proj);
				proj.Remove();
				
				if ( !proj.def.actions.NullOrEmpty() ) {
					List<QOLCityAction> tempActions = new List<QOLCityAction>(actions);
					foreach ( QOLCityAction act in tempActions ) {
						if ( proj.def.actions.Contains(act.def) ) RemoveAction(act);
					}
				}
				return true;
			}
			return false;
		}
		
		virtual public bool ActivateProject(QOLCityProject proj, bool force = false, bool refresh = true) {
			if ( proj == null ) return false;
			if ( !proj.IsActive && ( proj.def.toggleable || force ) ) {
				proj.active = true;
				if ( refresh ) Refresh();
				return true;
			}
			return false;
		}
		
		virtual public int ActivateProjects(List<QOLCityProject> projs, bool force = false, bool refresh = true) {
			if ( projs.NullOrEmpty() ) return 0;
			int activated = 0;
			foreach ( QOLCityProject proj in projs ) {
				if ( ActivateProject(proj, force, false) ) activated++;
			}
			if ( activated > 0 && refresh ) Refresh();
			return activated;
		}
		
		virtual public int DeactivateProjects(List<QOLCityProject> projs, bool force = false, bool refresh = true) {
			if ( projs.NullOrEmpty() ) return 0;
			int deactivated = 0;
			foreach ( QOLCityProject proj in projs ) {
				if ( DeactivateProject(proj, force, false) ) deactivated++;
			}
			if ( deactivated > 0 && refresh ) Refresh();
			return deactivated;
		}
		
		virtual public bool DeactivateProject(QOLCityProject proj, bool force = false, bool refresh = true) {
			if ( proj == null ) return false;
			if ( proj.IsActive && ( proj.def.toggleable || force ) ) {
				proj.active = false;
				if ( refresh ) Refresh();
				return true;
			}
			return false;
		}
		
		virtual public void Refresh() {
			RaiseEvent(new QOLCityEvent_State(QOLCityEvents.PreCalculateStaticContributions, stateProjectedState));

			QOLCityState state = new QOLCityState(this);
			
			foreach ( QOLCityProject project in projects ) {
				project.CityContribution(state);
			}
			if ( buildStateActive != null ) buildStateActive.CityContribution(state,true);
			
			population.CityContribution(this,state);
			
			foreach ( Pair<QOLSocietyIndicatorStage, float> p in state.activeStages.Values ) {
				p.First.CityContribution(state);
			}
			
			stateProjectedState = state;
			
			RaiseEvent(new QOLCityEvent_State(QOLCityEvents.PostCalculateStaticContributions, state));
			
			RaiseEvent(new QOLCityEvent_State(QOLCityEvents.PreCalculateDynamicContributions, state));
			
			RaiseEvent(new QOLCityEvent_State(QOLCityEvents.PostCalculateDynamicContributions, state));
		}
		
		virtual public void TryOpenComms(Pawn negotiator) {
			//Ensuring that the world camera gets initialized and we get correct positions for the hexagon tiles. Requires the game to run for a frame or so, which is why it's here.
			Find.WorldCamera.gameObject.SetActive(true);
			
			Refresh();
		}
		
		virtual public void autospawn() {
			//TODO a configurable autopicker for the world tile if autospawn is used. Something like a nearby bandit camp that needs to be destroyed by the player.
			/*
			 * Creating a project from the defs can only happen until the number of instances are filled.
			 * Only projects with free instances need to be checked.
			 */
			Dictionary<QOLCityProjectDef,KeyValuePair<int,int>> projectInstances = new Dictionary<QOLCityProjectDef,KeyValuePair<int,int>>();
			foreach ( QOLCityProject proj in projects ) {
				if ( !projectInstances.ContainsKey(proj.def) ) {
					projectInstances.Add(proj.def,new KeyValuePair<int, int>(1,proj.timeAdded));
				} else {
					KeyValuePair<int, int> kvp = projectInstances[proj.def];
					projectInstances[proj.def] = new KeyValuePair<int, int>(kvp.Key+1,Math.Max(proj.timeAdded,kvp.Value));
				}
			}
			int turnsSinceCreation = Find.TickManager.TicksGame - creationGameTicks + 1;
			bool projectWasAdded = false;
			foreach ( QOLCityProjectDef proj in defCity.projects ) {
				KeyValuePair<int,int> kvp = projectInstances.TryGetValue(proj,new KeyValuePair<int, int>(0,turnsSinceCreation));
				int instances = kvp.Key;
				if ( !proj.AllowsNumberOfInstances(instances+1) ) continue;
				
				if ( proj.autospawnTimer > 0 ) {
					//Maximum instances can be used to spawn multiple instances at once when using a single autospawn.
					if ( proj.autospawnTimer <= turnsSinceCreation ) {
						for ( int i = instances; i < Math.Max(1,proj.maximumInstances); i++ ) projectWasAdded = projectWasAdded || AddProject(proj) != null;
					}
				} else if ( proj.autospawnTimer < 0 ) {
					//For intervals we use turns since the last project of this type has been created(or turns since the city was created if no project of type exists).
					//If an instance got delayed due to requirements not met its better this way.
					if ( -proj.autospawnTimer <= kvp.Value ) {
						projectWasAdded = projectWasAdded || AddProject(proj) != null;
					}
				}
			}
			
			if ( projectWasAdded ) {
				//in case these new projects allow projects to be added that were previously not valid we do a lazy recursion
				Refresh();
				autospawn();
			}
		}
		
		virtual public void timeoutProjects() {
			List<QOLCityProject> despawn = new List<QOLCityProject>();
			List<QOLCityProject> deactivate = new List<QOLCityProject>();
			foreach ( QOLCityProject proj in projects ) {
				if ( proj.IsActive && proj.turnsActive < int.MaxValue ) proj.turnsActive++;
				if ( proj.def.lifetime > 0 && proj.IsActive && proj.turnsActive >= proj.def.lifetime ) {
					if ( proj.def.autospawnTimer == 0 ) {
						despawn.Add(proj);
					} else {
						//Autospawning projects lifetime triggers inactivity not cancelling the project. Otherwise it would respawn.
						deactivate.Add(proj);
					}
				}
			}
			
			DeactivateProjects(deactivate, true);
			foreach ( QOLCityProject proj in despawn ) {
				RemoveProject(proj);
			}
		}
		
		protected void assembleProjectsToDeactivate(HashSet<QOLCityProjectDef> projectsToDeactivate) {
			if ( defsPerResource == null ) {
				defsPerResource = new Dictionary<QOLCityResourceDef, HashSet<QOLCityProjectDef>>();
				foreach ( QOLCityProjectDef d in defCity.projects ) {
					if ( d.states != null && d.states.active != null && d.states.active.resources != null ) {
						foreach ( QOLCityStateContributionResource c in d.states.active.resources ) {
							if ( c.Value < 0 ) {
								HashSet<QOLCityProjectDef> hs;
								if ( !defsPerResource.TryGetValue(c.def, out hs) ) {
									hs = new HashSet<QOLCityProjectDef>();
									defsPerResource[c.def] = hs;
								}
								hs.Add(d);
							}
						}
					}
				}
			}
			
			projectsToDeactivate.Clear();
			foreach ( QOLCityResourceDef rd in stateProjectedState.resources.Where(p=>p.Value.Value < 0).Select(p=>p.Key) ) {
				HashSet<QOLCityProjectDef> hs;
				if( defsPerResource.TryGetValue(rd, out hs) ) {
					projectsToDeactivate.AddRange(hs);
				}
			}
		}
		
		virtual public void autodeactive() {
			HashSet<QOLCityProjectDef> projectsToDeactivate = new HashSet<QOLCityProjectDef>();
			assembleProjectsToDeactivate(projectsToDeactivate);
			
			if ( projectsToDeactivate.Count <= 0 ) return;
			
			List<QOLCityProject> deactivatedProjects = null;
			List<QOLCityProject>.Enumerator enumerator = projects.GetEnumerator();
			while ( enumerator.MoveNext() ) {
				QOLCityProject proj = enumerator.Current;
				if ( !proj.IsActive ) continue;
				if ( projectsToDeactivate.Contains(proj.def) && DeactivateProject(proj, false, false) ) {
					stateProjectedState.Remove(proj.def.states.active);
					if ( deactivatedProjects == null ) deactivatedProjects = new List<QOLCityProject>();
					deactivatedProjects.Add(proj);
					
					assembleProjectsToDeactivate(projectsToDeactivate);
					
					enumerator.Dispose();
					if ( projectsToDeactivate.Count <= 0 ) return;
					enumerator = projects.GetEnumerator();
				}
			}
			
			if ( !deactivatedProjects.NullOrEmpty() ) {
				StringBuilder sb = new StringBuilder();
				foreach ( QOLCityProject proj in deactivatedProjects ) sb.AppendWithComma(proj.def.label);
				
				Find.LetterStack.ReceiveLetter("ProjectsDeactivatedDueToInsufficentResourcesTitle".Translate(), 
				                               "ProjectsDeactivatedDueToInsufficentResources".Translate(
				                               	new NamedArgument(GetCallLabel(),"city"),
			                               		new NamedArgument(deactivatedProjects.Select(p=>p.def.label).ToCommaList(true),"projects")),
				                               LetterDefOf.NegativeEvent, this, null, null);
			}
		}

        virtual public void spawnThings()
        {
            foreach (KeyValuePair<KeyValuePair<ThingDef, QualityCategory>, NumericAdjustment> kvp in stateProjectedState.things )
            {
                int stackCount = Mathf.FloorToInt(kvp.Value.Value+0.0001f);
                ThingDef td = kvp.Key.Key;
                if ( stackCount == 0 )
                {
                    continue;
                }
                else if ( stackCount > 0 )
                {
                    Thing t = ThingMaker.MakeThing(td, td.MadeFromStuff ? GenStuff.DefaultStuffFor(td) : null);
                    t.stackCount = stackCount;
                    MinifiedThing minifiedThing = t as MinifiedThing;
                    CompQuality compQuality = (minifiedThing == null) ? t.TryGetComp<CompQuality>() : minifiedThing.InnerThing.TryGetComp<CompQuality>();
                    if (compQuality != null) compQuality.SetQuality(kvp.Key.Value, ArtGenerationContext.Outsider);
                    GiveSoldThingToTrader(t, t.stackCount, null);
                }
                else
                {
                    int countToGive = stackCount;
                    Thing thingToSplit = null;
                    ThingOwner owner = GetDirectlyHeldThings();
                    owner.RemoveAll(t => {
                        if (td == t.def)
                        {
                            if (countToGive >= t.stackCount)
                            {
                                countToGive -= t.stackCount;
                                return true;
                            }
                            thingToSplit = t;
                        }
                        return false;
                    });
                    if (countToGive > 0)
                    {
                        if (thingToSplit != null)
                        {
                            owner.Take(thingToSplit, countToGive);
                        }
                    }
                }
            }
        }

        override public void Tick() {
			RaiseEvent(new QOLCityEvent(QOLCityEvents.Tick));
			base.Tick();
			if ( Find.TickManager.TicksGame - lastSimulatedTick > simulationInterval ) {
				RaiseEvent(new QOLCityEvent(QOLCityEvents.DailyTick));
				
				buildStateActive = null;
				
				autospawn();
				
				Refresh();
				
				autodeactive();

                spawnThings();
				
				RaiseEvent(new QOLCityEvent(QOLCityEvents.ConsumeContributions));
				
				timeoutProjects();
				
				stateLastSimulatedTick = stateProjectedState;
				lastSimulatedTick = Find.TickManager.TicksGame;
			}
		}
		
		virtual public IEnumerable<QOLCityProject> BuildQueue {
			get {
				for ( int i = 0; i < projects.Count; i++ ) {
					QOLCityProject proj = projects[i];
					if ( !proj.IsBuilt ) yield return proj;
				}
			}
		}
		
		virtual public void GetChildHolders(List<IThingHolder> outChildren) {
			ThingOwnerUtility.AppendThingHoldersFromThings(outChildren, this.GetDirectlyHeldThings());
		}

		virtual public ThingOwner GetDirectlyHeldThings() {
			return stock;
		}

		virtual public IEnumerable<Thing> ColonyThingsWillingToBuy(Pawn playerNegotiator) {
			//The result is a list of things that the pawn HAS, that are in a sellable position and that the trader accepts.
			if ( playerNegotiator != null ) {
				if ( playerNegotiator.Map != null ) {
					return TradeUtility.AllLaunchableThingsForTrade(playerNegotiator.Map).Where(t=>TraderKind.WillTrade(t.def));
				} else if ( playerNegotiator.GetCaravan() != null ) {
					return playerNegotiator.GetCaravan().trader.Goods.Where(t=>TraderKind.WillTrade(t.def));
				}
			}
			return new List<Thing>();
		}

		virtual public void GiveSoldThingToTrader(Thing toGive, int countToGive, Pawn playerNegotiator) {
			if ( toGive as Pawn != null ) {
				Log.Error("Slave trade with a city is not supported!");
				return;
			}
			Thing thing = toGive.SplitOff(countToGive);
			thing.PreTraded(TradeAction.PlayerSells, playerNegotiator, this);
			if (!stock.TryAdd(thing, false)) {
				thing.Destroy(DestroyMode.Vanish);
				Log.Error("Failed to add bought thing to trader stock in city!");
			}
		}

		virtual public void GiveSoldThingToPlayer(Thing toGive, int countToGive, Pawn playerNegotiator) {
			if ( toGive as Pawn != null ) {
				Log.Error("Slave trade with a city is not supported!");
				return;
			}
			Thing thing = toGive.SplitOff(countToGive);
			thing.PreTraded(TradeAction.PlayerBuys, playerNegotiator, this);
			Caravan caravan = playerNegotiator.GetCaravan();
			if ( playerNegotiator.Map != null ) {
				TradeUtility.SpawnDropPod(DropCellFinder.TradeDropSpot(playerNegotiator.Map), playerNegotiator.Map, thing);
			} else if ( caravan != null ) {
				Pawn pawn = CaravanInventoryUtility.FindPawnToMoveInventoryTo(thing, caravan.PawnsListForReading, null, null);
				if (pawn == null) {
					Log.Error("Could not find any pawn to give sold thing to.");
					thing.Destroy(DestroyMode.Vanish);
				} else if (!pawn.inventory.innerContainer.TryAdd(thing, true)) {
					Log.Error("Could not add sold thing to inventory.");
					thing.Destroy(DestroyMode.Vanish);
				}
			} else {
				Log.Error("Trade with a city is expected to take place via a com console or caravans!");
			}
		}
	}
	
	public class QOLCityState {
		public SortedDictionary<QOLCityNeedDef,ProvidedAndRequired> needs;
		public SortedDictionary<QOLCityResourceDef,NumericAdjustment> resources;
		public Dictionary<KeyValuePair<ThingDef, QualityCategory>, NumericAdjustment> things;
		public QOLCity city;
		public Dictionary<Pawn,QOLCityStatePawnProperties> pawnProperties;
		public Dictionary<QOLSocietyIndicatorDef,Pair<QOLSocietyIndicatorStage,float>> activeStages;
		public Dictionary<QOLSocietyIndicatorDef,Dictionary<QOLSocietyIndicatorStage,float>> stageValues;
		
		public QOLCityState(QOLCity city) {
			this.city = city;
			
			this.needs = new SortedDictionary<QOLCityNeedDef, ProvidedAndRequired>();
			foreach ( QOLCityNeedDef need in city.defCity.needs ) {
				this.needs.Add(need, new ProvidedAndRequired());
			}
			
			this.resources = new SortedDictionary<QOLCityResourceDef, NumericAdjustment>();
			foreach ( QOLCityResourceDef res in city.defCity.resources ) {
				this.resources.Add(res, new NumericAdjustment());
			}
			
			this.things = new Dictionary<KeyValuePair<ThingDef,QualityCategory>, NumericAdjustment>();
			
			this.pawnProperties = new Dictionary<Pawn,QOLCityStatePawnProperties>();
			this.stageValues = new Dictionary<QOLSocietyIndicatorDef, Dictionary<QOLSocietyIndicatorStage, float>>();
			this.activeStages = new Dictionary<QOLSocietyIndicatorDef, Pair<QOLSocietyIndicatorStage, float>>();
			foreach ( QOLSocietyIndicatorDef indicator in city.defCity.indicators ) {
				Dictionary<QOLSocietyIndicatorStage,float> sts = new Dictionary<QOLSocietyIndicatorStage, float>();
				foreach ( QOLSocietyIndicatorStage stage in indicator.stages ) {
					sts.Add(stage,0f);
				}
				this.stageValues.Add(indicator, sts);
				this.activeStages.Add(indicator, new Pair<QOLSocietyIndicatorStage, float>(indicator.stages[0],0f));
			}
			
			foreach ( Pawn citizen in city.population.citizens.Keys ) {
				QOLCityStatePawnProperties props = new QOLCityStatePawnProperties(citizen);
				foreach ( QOLSocietyIndicatorDef indicator in city.defCity.indicators ) {
					Dictionary<QOLSocietyIndicatorStage,QOLCityStateSocietyIndicatorStageProperties> dic = new Dictionary<QOLSocietyIndicatorStage, QOLCityStateSocietyIndicatorStageProperties>();
					foreach ( QOLSocietyIndicatorStage stage in indicator.stages ) {
						dic.Add(stage,new QOLCityStateSocietyIndicatorStageProperties());
					}
					props.sip.Add(indicator,dic);
				}
				this.pawnProperties.Add(citizen,props);
			}
		}
		
		public void Add(QOLCityStateContribution contribution) {
			if ( contribution == null ) return;
			if ( contribution.needs != null ) {
				foreach ( QOLCityStateContributionNeed pn in contribution.needs ) {
					ProvidedAndRequired par;
					if ( needs.TryGetValue(pn.def, out par) ) {
						par.Add(pn);
					}
				}
			}
			if ( contribution.indicators != null ) {
				List<QOLSocietyIndicatorDef> uniqueDefs = new List<QOLSocietyIndicatorDef>();
				foreach ( QOLCityStateContributionIndicator ci in contribution.indicators ) {
					if ( !uniqueDefs.Contains(ci.def) ) {
						uniqueDefs.Add(ci.def);
					}
					
					Func<Pawn,bool> predicate;
					if ( ci.pawnFilters.NullOrEmpty() ) {
						predicate = p=>true;
					} else {
						predicate = p=>!ci.pawnFilters.Any(pf=>!pf.IsAllowed(city,p));
					}
					
					foreach ( Pawn p in city.population.citizens.Keys.Where(predicate) ) {
						QOLCityStatePawnProperties props;
						if ( pawnProperties.TryGetValue(p, out props) ) {
							Dictionary<QOLSocietyIndicatorStage,QOLCityStateSocietyIndicatorStageProperties> stages;
							if ( props.sip.TryGetValue(ci.def, out stages) ) {
								QOLCityStateSocietyIndicatorStageProperties isprops;
								if ( stages.TryGetValue(ci.Stage,out isprops) ) {
									isprops.affinity = (1-isprops.affinity) * ci.affinity + isprops.affinity;
									isprops.suppression = (1-isprops.suppression) * ci.suppression + isprops.suppression;
								}
							}
						}
					}
				}
				//Rebuilding the city wide values for modified indicators.
				//For the avg each person on each stage lowers affinity by suppression and then caps it between 0% and 100%. 
				//The results are summed up and divided by the number of people.
				//A persons social standing multiplies all affinities of that person and its population count. One person can therfore count as four.
				float totalPopulationPower = 0;
				foreach ( QOLCityStateContributionIndicator ci in contribution.indicators ) {
					Dictionary<QOLSocietyIndicatorStage, float> stages;
					if ( stageValues.TryGetValue(ci.def, out stages) ) {
						float newValue = 0;
						foreach ( QOLCityStatePawnProperties props in pawnProperties.Values ) {
							Dictionary<QOLSocietyIndicatorStage,QOLCityStateSocietyIndicatorStageProperties> stagesProp;
							if ( props.sip.TryGetValue(ci.def, out stagesProp) ) {
								QOLCityStateSocietyIndicatorStageProperties isprops;
								if ( stagesProp.TryGetValue(ci.Stage,out isprops) ) {
									float affinity = Mathf.Clamp(isprops.affinity - isprops.suppression, 0f, 1f);
									float power = city.population.citizens[props.p].power;
									newValue += affinity * power;
									totalPopulationPower += power;
								}
							}
						}
						if ( totalPopulationPower > 0f ) stages[ci.Stage] = Mathf.Clamp(newValue / totalPopulationPower,0f,1f);
						else stages[ci.Stage] = 0f;
					}
				}
				//The active stage is the one with the highest avg across all citizens.
				foreach ( QOLSocietyIndicatorDef def in uniqueDefs ) {
					Dictionary<QOLSocietyIndicatorStage, float> stages = stageValues[def];
					float maxValue = float.MinValue;
					QOLSocietyIndicatorStage maxStage = null;
					foreach ( KeyValuePair<QOLSocietyIndicatorStage, float> kvp in stages ) {
						if ( kvp.Value > maxValue ) {
							maxValue = kvp.Value;
							maxStage = kvp.Key;
						}
					}
					activeStages[def] = new Pair<QOLSocietyIndicatorStage,float>(maxStage,maxValue);
				}
			}
			if ( contribution.resources != null ) {
				foreach ( QOLCityStateContributionResource pr in contribution.resources ) {
					NumericAdjustment na;
					if ( resources.TryGetValue(pr.def, out na) ) {
						na.Add(pr);
					}
				}
			}
			if ( contribution.things != null ) {
				foreach ( QOLCityStateContributionThing pr in contribution.things ) {
					NumericAdjustment na;
                    KeyValuePair<ThingDef, QualityCategory> kvp = new KeyValuePair<ThingDef, QualityCategory>(pr.def, pr.quality);
                    if ( things.TryGetValue(kvp, out na) ) {
						na.Add(pr);
					} else {
						na = new NumericAdjustment();
						na.Add(pr);
						things.Add(kvp, na);
					}
				}
			}
		}
		
		public void Remove(QOLCityStateContribution contribution) {
			if ( contribution == null ) return;
			if ( contribution.needs != null ) {
				foreach ( QOLCityStateContributionNeed pn in contribution.needs ) {
					ProvidedAndRequired par;
					if ( needs.TryGetValue(pn.def, out par) ) {
						par.Remove(pn);
					}
				}
			}
			if ( contribution.indicators != null ) {
				List<QOLSocietyIndicatorDef> uniqueDefs = new List<QOLSocietyIndicatorDef>();
				foreach ( QOLCityStateContributionIndicator ci in contribution.indicators ) {
					if ( !uniqueDefs.Contains(ci.def) ) {
						uniqueDefs.Add(ci.def);
					}
					
					Func<Pawn,bool> predicate;
					if ( ci.pawnFilters.NullOrEmpty() ) {
						predicate = p=>true;
					} else {
						predicate = p=>!ci.pawnFilters.Any(pf=>!pf.IsAllowed(city,p));
					}
					
					foreach ( Pawn p in city.population.citizens.Keys.Where(predicate) ) {
						QOLCityStatePawnProperties props;
						if ( pawnProperties.TryGetValue(p, out props) ) {
							Dictionary<QOLSocietyIndicatorStage,QOLCityStateSocietyIndicatorStageProperties> stages;
							if ( props.sip.TryGetValue(ci.def, out stages) ) {
								QOLCityStateSocietyIndicatorStageProperties isprops;
								if ( stages.TryGetValue(ci.Stage,out isprops) ) {
									//A = (C - B) / (1 - B)
									if (Math.Abs(ci.affinity) > float.Epsilon) {
										isprops.affinity = (isprops.affinity - ci.affinity) / (1 - ci.affinity);
									}
									if (Math.Abs(ci.suppression) > float.Epsilon) {
										isprops.suppression = (isprops.suppression - ci.suppression) / (1 - ci.suppression);
									}
								}
							}
						}
					}
				}
				//Rebuilding the city wide values for modified indicators.
				//For the avg each person on each stage lowers affinity by suppression and then caps it between 0% and 100%. 
				//The results are summed up and divided by the number of people.
				//A persons social standing multiplies all affinities of that person and its population count. One person can therfore count as four.
				float totalPopulationPower = 0;
				foreach ( QOLCityStateContributionIndicator ci in contribution.indicators ) {
					Dictionary<QOLSocietyIndicatorStage, float> stages;
					if ( stageValues.TryGetValue(ci.def, out stages) ) {
						float newValue = 0;
						foreach ( QOLCityStatePawnProperties props in pawnProperties.Values ) {
							Dictionary<QOLSocietyIndicatorStage,QOLCityStateSocietyIndicatorStageProperties> stagesProp;
							if ( props.sip.TryGetValue(ci.def, out stagesProp) ) {
								QOLCityStateSocietyIndicatorStageProperties isprops;
								if ( stagesProp.TryGetValue(ci.Stage,out isprops) ) {
									float affinity = Mathf.Clamp(isprops.affinity - isprops.suppression, 0f, 1f);
									float power = city.population.citizens[props.p].power;
									newValue += affinity * power;
									totalPopulationPower += power;
								}
							}
						}
						if ( totalPopulationPower > 0f ) stages[ci.Stage] = Mathf.Clamp(newValue / totalPopulationPower,0f,1f);
						else stages[ci.Stage] = 0f;
					}
				}
				//The active stage is the one with the highest avg across all citizens.
				foreach ( QOLSocietyIndicatorDef def in uniqueDefs ) {
					Dictionary<QOLSocietyIndicatorStage, float> stages = stageValues[def];
					float maxValue = float.MinValue;
					QOLSocietyIndicatorStage maxStage = null;
					foreach ( KeyValuePair<QOLSocietyIndicatorStage, float> kvp in stages ) {
						if ( kvp.Value > maxValue ) {
							maxValue = kvp.Value;
							maxStage = kvp.Key;
						}
					}
					activeStages[def] = new Pair<QOLSocietyIndicatorStage,float>(maxStage,maxValue);
				}
			}
			if ( contribution.resources != null ) {
				foreach ( QOLCityStateContributionResource pr in contribution.resources ) {
					NumericAdjustment na;
					if ( resources.TryGetValue(pr.def, out na) ) {
						na.Remove(pr);
					}
				}
			}
			if ( contribution.things != null ) {
				foreach ( QOLCityStateContributionThing pr in contribution.things ) {
					NumericAdjustment na;
                    KeyValuePair<ThingDef, QualityCategory> kvp = new KeyValuePair<ThingDef, QualityCategory>(pr.def,pr.quality);
					if ( things.TryGetValue(kvp, out na) ) {
						na.Remove(pr);
					} else {
						na = new NumericAdjustment();
						na.Remove(pr);
						things.Add(kvp, na);
					}
				}
			}
		}
		
		public int Accommodate(QOLCityStateContribution contribution, int limit) {
			if ( limit > 0 ) {
				foreach ( QOLCityStateContributionNeed nc in contribution.needs ) {
					ProvidedAndRequired par;
					if ( needs.TryGetValue(nc.def, out par) ) {
						int i; int added = 0;
						for ( i = 0; i < limit; i++ ) {
							par.Add(nc); added++;
							if ( par.provided.Value - par.required.Value < 0.0f ) break;
						}
						limit = i;
						for ( ; added > 0; added-- ) {
							par.Remove(nc);
						}
						if ( limit == 0 ) break;
					}
				}
			}
			if ( limit > 0 ) {
				foreach ( QOLCityStateContributionResource rc in contribution.resources ) {
					NumericAdjustment na;
					if ( resources.TryGetValue(rc.def, out na) ) {
						int i; int added = 0;
						for ( i = 0; i < limit; i++ ) {
							na.Add(rc); added++;
							if ( na.Value < 0.0f ) break;
						}
						limit = i;
						for ( ; added > 0; added-- ) {
							na.Remove(rc);
						}
						if ( limit == 0 ) break;
					}
				}
			}
			if ( limit > 0 ) {
				foreach ( QOLCityStateContributionThing tc in contribution.things ) {
					NumericAdjustment na;
                    KeyValuePair<ThingDef, QualityCategory> kvp = new KeyValuePair<ThingDef, QualityCategory>(tc.def, tc.quality);
                    if ( things.TryGetValue(kvp, out na) ) {
						int i; int added = 0;
						for ( i = 0; i < limit; i++ ) {
							na.Add(tc); added++;
							if ( na.Value < 0.0f ) break;
						}
						limit = i;
						for ( ; added > 0; added-- ) {
							na.Remove(tc);
						}
						if ( limit == 0 ) break;
					}
				}
			}
			return Math.Max(0,limit);
		}
	}
	
	public class QOLCityStatePawnProperties {
		public Pawn p;
		public Dictionary<QOLSocietyIndicatorDef,Dictionary<QOLSocietyIndicatorStage,QOLCityStateSocietyIndicatorStageProperties>> sip;
		public Dictionary<string,QOLCityValueMutator> props;
		public QOLCityStatePawnProperties(Pawn p) {
			this.p = p;
			sip = new Dictionary<QOLSocietyIndicatorDef, Dictionary<QOLSocietyIndicatorStage, QOLCityStateSocietyIndicatorStageProperties>>();
			props = null;
		}
		
		public float getProp(QOLCityState state, string key, float baseValue = 1f) {
			if ( props == null ) {
				props = state.city.defCity.population.calculateProperties(state, p);
			}
			
			QOLCityValueMutator mutator;
			if ( props.TryGetValue(key, out mutator) ) {
				return mutator.mutate(baseValue);
			}
			
			return baseValue;
		}
	}
	
	public class QOLCityStateSocietyIndicatorStageProperties {
		public float affinity, suppression;
		public float oppression {get {return Mathf.Max(0f,Mathf.Min(affinity,suppression));}}
	}
}
