﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 09.02.2019
 * Time: 02:24
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using RIMMSqol.remnantcolony.listeners;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol.remnantcolony.actions
{
	public class QOLCityAction_Convert : QOLCityAction {
		protected int cyclesAllocatedInt;
		protected QOLCityActionConfiguration_Convert conf;
		public int CyclesAllocated { get {return cyclesAllocatedInt;} }
		
		public QOLCityAction_Convert() : base(null) {}
		
		public QOLCityAction_Convert(QOLCityActionDef def) : base(def) {
			// disable once DoNotCallOverridableMethodsInConstructor
			Init();
			cyclesAllocatedInt = 0;
		}

		public override void ExposeData() {
			base.ExposeData();
			Scribe_Values.Look<int>(ref cyclesAllocatedInt, "cyclesAllocated", 0);
			if (Scribe.mode == LoadSaveMode.ResolvingCrossRefs) {
				Init();
			}
		}
		
		protected virtual void Init() {
			if ( def != null ) {
				conf = def.action as QOLCityActionConfiguration_Convert;
				if ( conf == null ) {
					Log.Error("No matching conf found in "+typeof(QOLCityAction_Convert).Name+"!");
				}
			} else {
				Log.Error("No definition found in "+typeof(QOLCityAction_Convert).Name+"!");
			}
		}
		
		public override void OnClick(QOLCity city, Pawn negotiator) {
			Find.WindowStack.Add(new Dialog_Spinner(cyclesAllocatedInt, 
			                                        newValue => newValue >= 0 && newValue <= conf.MaxCyclePerInstance * instances, 
			                                        newValue => { 
			                                        	cyclesAllocatedInt = newValue;
														city.Refresh();			                                        	
			                                        }, 1, def.label, def.description));
		}
	}
	
	public class QOLCityActionConfiguration_Convert : QOLCityActionConfiguration, IQOLCityListener_ConversionConfiguration
	{
		protected QOLCityStateContribution inputPerCycle;
		protected QOLCityStateContribution outputPerCycle;
		protected float labourPerCycle;
		protected int maxCyclePerInstance;
		protected bool onlyNecessaryProduction;
		// disable ConvertToAutoProperty
		public QOLCityStateContribution InputPerCycle {get{return inputPerCycle;}set{inputPerCycle = value;}}
		public QOLCityStateContribution OutputPerCycle {get{return outputPerCycle;}set{outputPerCycle = value;}}
		public float LabourPerCycle {get{return labourPerCycle;}set{labourPerCycle = value;}}
		public int MaxCyclePerInstance {get{return maxCyclePerInstance;}set{maxCyclePerInstance = value;}}
		public bool OnlyNecessaryProduction {get{return onlyNecessaryProduction;}set{onlyNecessaryProduction = value;}}
		public int ActiveInstances(QOLCity city, IQOLCityListenerOwner owner, QOLCityState state) {
			QOLCityActionDef action = ((QOLCityAction_Convert)owner).def;
			return city.projects.Count(p=>p.IsActive && p.def.actions.Contains(action));
		}
		public int CyclesAllocated(QOLCity city, IQOLCityListenerOwner owner, QOLCityState state) {
			return ((QOLCityAction_Convert)owner).CyclesAllocated;
		}
		public void CyclesPerformed(QOLCity city, IQOLCityListenerOwner owner, QOLCityState state, int cyclesPerformed) {}
        public override QOLCityAction CreateAction(QOLCityActionDef def) {
			return new QOLCityAction_Convert(def);
		}
	}
}
