﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 27.02.2019
 * Time: 00:00
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using RIMMSqol.extensions;
using RIMMSqol.remnantcolony.requirements;
using Verse;

namespace RIMMSqol.remnantcolony.listeners
{
	/// <summary>
	/// Description of QOLCityListener_EventProducer.
	/// </summary>
	public class QOLCityListener_EventProducer : QOLCityListener
	{
		public QOLCityListener_EventProducer(QOLCityListenerDef def,IQOLCityListenerOwner owner) : base(def,owner) {}
		
		public override void OnEvent(QOLCity city, QOLCityEvent e) {
			IEventProducerOwner epo = owner as IEventProducerOwner;
			if ( epo != null ) {
				Dictionary<string,DictionaryExposable<string,int>> epoc = epo.eventProducerCooldowns;
				if ( epoc == null ) {
					epoc = new Dictionary<string, DictionaryExposable<string, int>>();
					epo.eventProducerCooldowns = epoc;
				}
				DictionaryExposable<string,int> eventCooldowns;
				if ( !epoc.TryGetValue(def.defName, out eventCooldowns) ) {
					eventCooldowns = new DictionaryExposable<string, int>();
					epoc.Add(def.defName, eventCooldowns);
				}
				
				//each cooldown entry is lowered by 1 and if its 0 it is removed
				List<string> keys = new List<string>(eventCooldowns.Keys);
				foreach ( string key in keys ) {
					eventCooldowns[key] = eventCooldowns[key] - 1;
				}
				eventCooldowns.RemoveAll(kvp=>kvp.Value <= 0);
				
				if ( eventCooldowns.ContainsKey("all") ) {
					return;
				}
				
				QOLCityListenerDef_EventProducer conf = def as QOLCityListenerDef_EventProducer;
				if ( conf != null ) {
					List<QOLCityListenerDef_EventProducerEventItem> list = new List<QOLCityListenerDef_EventProducerEventItem>();
					foreach ( QOLCityListenerDef_EventProducerEventItem epei in conf.events ) {
						if ( !eventCooldowns.ContainsKey(epei.eventId) && ( epei.requirements.NullOrEmpty() || epei.requirements.TrueForAll(req=>req.IsValid(city)) ) ) {
							list.Add(epei);
						}
					}
					
					if ( conf.stopAllIfOneProcessed ) {
						Action<QOLCityEvent_EventChain> onStop = eve => {
							eventCooldowns["all"] = UnityEngine.Random.Range(conf.cooldownOnProcessedMin,conf.cooldownOnProcessedMaxExclusive);
							eventCooldowns[eve.EventId] = UnityEngine.Random.Range(conf.cooldownOnProcessedIndividualMin,conf.cooldownOnProcessedIndividualMaxExclusive);};
						QOLCityEvent_EventChain lastEvent = null;
						foreach ( QOLCityListenerDef_EventProducerEventItem epei in list.ScrambleByWeight(epei=>epei.weight) ) {
							QOLCityEvent_EventChain currentEvent = new QOLCityEvent_EventChain(epei.eventId, lastEvent, onStop);
							//Log.Message("Raising chain event: "+epei.eventId);
							city.RaiseEvent(currentEvent);
							lastEvent = currentEvent;
						}
					} else {
						eventCooldowns["all"] = UnityEngine.Random.Range(conf.cooldownOnProcessedMin,conf.cooldownOnProcessedMaxExclusive);
						foreach ( QOLCityListenerDef_EventProducerEventItem epei in list.ScrambleByWeight(epei=>epei.weight) ) {
							//Log.Message("Raising event: "+epei.eventId);
							eventCooldowns[epei.eventId] = UnityEngine.Random.Range(conf.cooldownOnProcessedIndividualMin,conf.cooldownOnProcessedIndividualMaxExclusive);
							city.RaiseEvent(new QOLCityEvent(epei.eventId));
						}
					}
				}
			}
		}
	}
	
	public class QOLCityEvent_EventChain : QOLCityEvent {
		protected Action<QOLCityEvent_EventChain> onStop;
		
		public override bool ShouldProcess { get { return (ParentEvent == null || ParentEvent.ShouldProcess) && base.ShouldProcess; } }
		
		public QOLCityEvent_EventChain(string eventId, Action<QOLCityEvent_EventChain> onStop) : this(eventId, null, onStop) {}
		
		public QOLCityEvent_EventChain(string eventId, QOLCityEvent parentEvent, Action<QOLCityEvent_EventChain> onStop) : base(eventId, parentEvent) {
			this.onStop = onStop;
		}
		
		public override void StopProcessing() {
			base.StopProcessing();
			if ( onStop != null ) onStop(this);
		}
	}
	
	public interface IEventProducerOwner {
		Dictionary<string,DictionaryExposable<string,int>> eventProducerCooldowns{get;set;}
	}
	
	public class DictionaryExposable<K, V> : Dictionary<K, V>, IExposable {
		protected string label;
		
		public DictionaryExposable() : this("dic") {}
		
		public DictionaryExposable(string label) : base() {
			this.label = label;
		}
		
		public void ExposeData()
		{
			if (Scribe.mode == LoadSaveMode.Saving) {
				//when saving we copy the data into a new dictionary and save that
				Dictionary<K,V> dic = new Dictionary<K, V>(this);
				Scribe_Collections.Look<K, V>(ref dic, label, LookMode.Value, LookMode.Value);
			} else if (Scribe.mode == LoadSaveMode.LoadingVars) {
				//when loading we replace the current data with the new one
				Dictionary<K,V> dic = null;
				Scribe_Collections.Look<K, V>(ref dic, label, LookMode.Value, LookMode.Value);
				Clear();
				if ( dic != null ) {
					foreach ( KeyValuePair<K,V> kvp in dic ) {
						Add(kvp.Key,kvp.Value);
					}
				}
			}
		}
	}
	
	public class QOLCityListenerDef_EventProducer : QOLCityListenerDef {
		public int cooldownOnProcessedMin = 0, cooldownOnProcessedIndividualMin = 0, cooldownOnProcessedMax = 0, cooldownOnProcessedIndividualMax = 0;
		public bool stopAllIfOneProcessed = true;
		public List<QOLCityListenerDef_EventProducerEventItem> events;
		[Unsaved]
		public int cooldownOnProcessedMaxExclusive = 0, cooldownOnProcessedIndividualMaxExclusive = 0;
		
		public override void PostLoad() {
			base.PostLoad();
			if ( cooldownOnProcessedMin < cooldownOnProcessedMax ) {
				int i = cooldownOnProcessedMax;
				cooldownOnProcessedMax = cooldownOnProcessedMin;
				cooldownOnProcessedMin = i;
			}
			if ( cooldownOnProcessedIndividualMin < cooldownOnProcessedIndividualMax ) {
				int i = cooldownOnProcessedIndividualMax;
				cooldownOnProcessedIndividualMax = cooldownOnProcessedIndividualMin;
				cooldownOnProcessedIndividualMin = i;
			}
			cooldownOnProcessedMaxExclusive = cooldownOnProcessedMax + 1;
			cooldownOnProcessedIndividualMaxExclusive = cooldownOnProcessedIndividualMax + 1;
		}
	}
	
	public class QOLCityListenerDef_EventProducerEventItem {
		public float weight;
		public string eventId;
		public List<QOLCityRequirement> requirements;
	}
}
