﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 22.01.2019
 * Time: 19:53
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Text;
using RimWorld;
using RimWorld.Planet;
using Verse;

namespace RIMMSqol.remnantcolony.listeners
{
	/// <summary>
	/// Description of QOLCityListener_TriggerVictory.
	/// </summary>
	public class QOLCityListener_TriggerVictory : QOLCityListener
	{
		public QOLCityListener_TriggerVictory(QOLCityListenerDef def,IQOLCityListenerOwner owner) : base(def,owner) {}
		
		public override void OnEvent(QOLCity city, QOLCityEvent e) {
			//Listener removes itself from a project after it fired once and uses the active flag in the projects save state to remember that decision.
			//TODO If used on other objects it fires once every time the game is loaded. Maybe something like removing the owner - that would be the cleanest approach
			
			QOLCityProject project = owner as QOLCityProject;
			if ( project != null && !project.IsBuilt || !project.IsActive ) {
				if ( !project.IsActive ) {
					city.QueueDeregisteringOfListener(this);
				}
				return;
			}

			StringBuilder sb = new StringBuilder();
			foreach (Pawn current in PawnsFinder.AllMaps_FreeColonistsSpawned)
			{
				sb.AppendLine(current.LabelCap);
			}
			List<Caravan> caravans = Find.WorldObjects.Caravans;
			for (int i = 0; i < caravans.Count; i++)
			{
				Caravan caravan = caravans[i];
				if (caravan.IsPlayerControlled)
				{
					List<Pawn> pawnsListForReading = caravan.PawnsListForReading;
					for (int j = 0; j < pawnsListForReading.Count; j++)
					{
						Pawn pawn = pawnsListForReading[j];
						if (pawn.IsColonist && pawn.HostFaction == null)
						{
							sb.AppendLine(pawn.LabelCap);
						}
					}
				}
			}
			foreach ( KeyValuePair<Pawn,QOLCitySocialStandingDef> p in city.population.citizens ) {
				sb.AppendLine(p.Key.LabelCap);
			}
			QOLCityListenerDef_TriggerVictory conf = def as QOLCityListenerDef_TriggerVictory;
			string victoryMessage = conf != null ? (!conf.victoryMessage.NullOrEmpty() ? conf.victoryMessage : "QOLCityGameOverGeneric") : "QOLCityGameOverGeneric";
			
			GameVictoryUtility.ShowCredits(victoryMessage.Translate(sb.ToString()));
			city.QueueDeregisteringOfListener(this);
			
			if ( project != null ) project.active = false;
		}
	}
	
	public class QOLCityListenerDef_TriggerVictory : QOLCityListenerDef {
		[Description("The translation key to use when creating the victory message."), NoTranslate]
		public string victoryMessage = "QOLCityGameOverGeneric";
	}
}
