﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 05.01.2019
 * Time: 10:38
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Text;
using RIMMSqol.remnantcolony.requirements;
using RIMMSqol.remnantcolony.triggers;
using Verse;

namespace RIMMSqol.remnantcolony.projects
{
	/// <summary>
	/// Description of QOLCityProject_Crisis.
	/// </summary>
	public class QOLCityProject_Crisis : QOLCityProject {
		public float progress;
		
		public QOLCityProject_Crisis() : base() {}
		
		public QOLCityProject_Crisis(QOLCityProjectDef def) : base(def) {}
		
		public override void ExposeData() {
			base.ExposeData();
			Scribe_Values.Look<float>(ref this.progress, "progress");
		}
	}
	
	public class QOLCityProjectDef_Crisis_RequirementProgress {
		[DefaultValue(null), Description("If all requirements are fulfilled the daily progress is added to the crisis progress.")]
		public List<QOLCityRequirement> requirements;
		[DefaultValue(0), Description("By how much the progress of the crisis is increased every day if the requirement is fulfilled.")]
		public float dailyProgress;
	}
	
	public class QOLCityProjectDef_Crisis_RequirementTrigger {
		[DefaultValue(null), Description("If all requirements are fulfilled the trigger is invoked during the daily tick.")]
		public List<QOLCityRequirement> requirements;
		[DefaultValue(null), Description("What to execute when the requirements are met.")]
		public QOLCityTrigger trigger;
	}
	
	public class QOLCityProjectDef_Crisis : QOLCityProjectDef {
		[DefaultValue(0),Description("By how much the crisis's progress decays every day.")]
		public float dailyDecrease;
		[DefaultValue(null),Description("Defines when and how much the progress of the crisis changes on a daily basis.")]
		public List<QOLCityProjectDef_Crisis_RequirementProgress> progression;
		[DefaultValue(null),Description("Defines when and what should be done.")]
		public List<QOLCityProjectDef_Crisis_RequirementTrigger> triggers;
		[DefaultValue(10),Description("The upper cap for progression.")]
		public float maxProgression;
		[DefaultValue(0),Description("The lower cap for progression.")]
		public float minProgression;
		[DefaultValue(10),Description("The upper cap for displaying progression in a progress bar.")]
		public float maxDisplayProgression;
		[DefaultValue(0),Description("The lower cap for displaying progression in a progress bar.")]
		public float minDisplayProgression;
		
		public override string Tooltip () {
			StringBuilder sb = new StringBuilder(base.Tooltip());
			
			if (Math.Abs(dailyDecrease) > float.Epsilon) sb.Append("Daily reduction in progress ").Append(dailyDecrease).AppendLine(".");
			
			if ( !progression.NullOrEmpty() ) {
				foreach ( QOLCityProjectDef_Crisis_RequirementProgress pg in progression ) {
					sb.Append("Daily progress ").Append(pg.dailyProgress);
					if ( !pg.requirements.NullOrEmpty() ) {
						sb.AppendLine(" with conditions.");
						foreach ( QOLCityRequirement req in pg.requirements ) {
							sb.Append("\t").AppendLine(req.Description());
						}
					} else {
						sb.AppendLine();
					}
				}
			}
			
			if ( !triggers.NullOrEmpty() ) {
				foreach ( QOLCityProjectDef_Crisis_RequirementTrigger rt in triggers ) {
					sb.Append(rt.trigger.Description());
					if ( !rt.requirements.NullOrEmpty() ) {
						sb.AppendLine(" with conditions.");
						foreach ( QOLCityRequirement req in rt.requirements ) {
							sb.Append("\t").AppendLine(req.Description());
						}
					} else {
						sb.AppendLine();
					}
				}
			}
			
			return sb.ToString();
		}
		
		public override IEnumerable<string> ConfigErrors()
		{
			foreach (string e in base.ConfigErrors()) {
				yield return e;
			}
			if ( minDisplayProgression < minProgression || maxDisplayProgression > maxProgression ) {
				yield return "display progression exceeds progression limits";
			}
		}
	}
}
