﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 23.10.2018
 * Time: 06:01
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using UnityEngine;
using Verse;

namespace RIMMSqol.renderers
{
	/// <summary>
	/// Description of HorizontalSplitRenderer.
	/// </summary>
	public class HorizontalSplitRenderer : ParentRenderer
	{
		protected Vector2 scrollPosition;
		protected BaseRenderer topChild, bottomChild;
		protected float width, heightTop, heightBottom;
		public HorizontalSplitRenderer(BaseRenderer topChild, BaseRenderer bottomChild, float width = -1f, float heightTop = -1f, float heightBottom = -1f) {
			this.topChild = topChild;
			this.bottomChild = bottomChild;
			this.AddChild(this.topChild).AddChild(this.bottomChild);
			this.width = width;
			this.heightTop = heightTop;
			this.heightBottom = heightBottom;
		}
		public float getHeightTop() {
			return ( heightTop < 0 ? topChild.getHeight() : heightTop );
		}
		public float getHeightBottom() {
			return ( heightBottom < 0 ? bottomChild.getHeight() : heightBottom );
		}
		public override float getHeight() {
			return getHeightTop() + getHeightBottom();
		}
		public override float getWidth() {
			if ( width < 0 )
				return Mathf.Max(topChild.getWidth(), bottomChild.getWidth());
			return width;
		}
		public override void DoComponentContents(Rect inRect, Flow flow) {
			if ( inRect.height >= getHeight() && inRect.width >= getWidth() ) {
				DoChildren(inRect,flow);
			} else {
				//adding a scrollpane if necessary
				Rect viewRect = new Rect(0,0,getWidth(),getHeight());
				Widgets.BeginScrollView(inRect, ref scrollPosition, viewRect);
				DoChildren(viewRect,flow);
				Widgets.EndScrollView();
			}
		}
		protected void DoChildren(Rect inRect, Flow flow) {
			float w = getWidth(), ht = getHeightTop();
			topChild.DoComponentContents(new Rect(inRect.x,inRect.y,w,ht), flow);
			bottomChild.DoComponentContents(new Rect(inRect.x,inRect.y+ht,w,getHeightBottom()), flow);
		}
	}
}
