﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 26.11.2019
 * Time: 00:04
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using UnityEngine;
using Verse;

namespace RIMMSqol.renderers
{
	/// <summary>
	/// Description of TextRenderer.
	/// </summary>
	public class TextRenderer : BaseRenderer {
        protected Func<Flow, string> valueExpr;
        protected Action<Flow, string> onChange;
        protected string text;
		protected GameFont font;
		protected float width, height;
        public bool EmptyIsNull { get; set; } = false;
        protected Vector2 scrollPos = Vector2.zero;
		public override float getHeight() {
			if ( height < 0 ) {
				GameFont gf = Text.Font;
				Text.Font = font;
				height = Text.CalcHeight(text, getWidth() - QOLMod.VerticalScrollbarWidth());
				Text.Font = gf;
			}
			return height;
		}
		public override float getWidth() {
			if ( width >= 0 ) return width;
			return 200f;
		}
        public TextRenderer(Func<Flow, string> valueExpr, Action<Flow, string> onChange, float width = 40f, GameFont font = GameFont.Small, float height = -1f)
        {
            this.valueExpr = valueExpr;
            this.onChange = onChange;
            this.width = width;
            this.height = height;
            this.font = font;
        }
        /*public TextRenderer(string text, float width = 40f, GameFont font = GameFont.Small, float height = -1f) {
			this.text = text;
			this.width = width;
			this.height = height;
			this.font = font;
		}*/
		public override void DoComponentContents(Rect inRect, Flow flow) {
            Text.Font = font;
            string value;
            float width = getWidth();
            if (width >= 0 && inRect.width > width) inRect.width = width;
            float height = getHeight();
            if (height >= 0 && inRect.height > height) inRect.height = height;
            if (valueExpr != null) value = valueExpr(flow);
            else value = text;
			displayToolTipIfNecessary(inRect, flow);
            string originalValue = value;

            value = TextAreaScrollable(inRect, value, ref scrollPos, onChange == null);
            if (EmptyIsNull && value != null && value.Length == 0) value = null;
            if (onChange != null && ((originalValue == null && value != null) || (originalValue != null && !originalValue.Equals(value))))
            {
                onChange(flow, value);
            }
        }
		public string TextAreaScrollable(Rect rect, string text, ref Vector2 scrollbarPosition, bool readOnly = false) {
			Rect rect2 = new Rect(0f, 0f, rect.width - QOLMod.VerticalScrollbarWidth(), Mathf.Max(Text.CalcHeight(text, rect.width - QOLMod.VerticalScrollbarWidth()) + Text.LineHeight, rect.height));
			Widgets.BeginScrollView(rect, ref scrollbarPosition, rect2, true);
			string result = Widgets.TextArea(rect2, text, readOnly);
			Widgets.EndScrollView();
			return result;
		}
	}
}
