﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 08.11.2019
 * Time: 20:14
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections;
using System.Collections.Generic;
using RIMMSqol.genericSettings;
using RIMMSqol.renderers;
using UnityEngine;
using Verse;
using RIMMSqol.extensions;

namespace RIMMSqol.storyteller
{
	/// <summary>
	/// Description of Dialog_EditObject.
	/// </summary>
	public class Dialog_EditObject<T> : Window where T : StorytellerTreeNode
	{
		protected SettingsProperties props;
		protected SettingsInstance si;
		protected T o;
		protected Action<SettingsInstance,T> onClose;
		protected Flow flow;
		
		protected float initialWidth, initialHeight;
		
		override public Vector2 InitialSize {
			get {
				return new Vector2(initialWidth, initialHeight);
			}
		}
		
		public Dialog_EditObject(T o, string propertiesId, Action<SettingsInstance,T> onClose, float initialWidth = 500f, float initialHeight = 500f)
		{
			this.closeOnCancel = true;
			this.doCloseX = true;
			this.onlyOneOfTypeAllowed = false;
			this.absorbInputAroundWindow = true;
			this.forcePause = true;
			this.initialWidth = initialWidth;
			this.initialHeight = initialHeight;
			
			props = SettingsStorage.getSettingsProperties(propertiesId);
			if ( props != null ) {
				foreach ( ISettingsFieldProperties field in props.fields ) {
					TaggedString ts;
					string translation;
					string tagName = props.id+"."+field.id;
					if ( tagName.TryTranslate(out ts) && !(translation=ts).NullOrEmpty() ) {
						field.label = translation;
					}
					tagName = tagName+".tooltip";
					if ( tagName.TryTranslate(out ts) && !(translation=ts).NullOrEmpty() ) {
						field.labelTooltip = translation;
					}
				}
			}
			
			si = new SettingsInstance(propertiesId,o);
			this.o = o;
			this.onClose = onClose;
		}
		
		public override void PreOpen() {
			base.PreOpen();
			
			Dictionary<string, PageRenderer> pages = new Dictionary<string, PageRenderer>();
			pages.Add("main",editPage(props,null,"main",pages));
			flow = new Flow(pages,"main");
			flow.flowScope["edit"] = si;
		}
		
		public override void PostClose()
		{
			base.PostClose();
			
			if ( onClose != null ) onClose(si,o);
			props = null;
			si = null;
			o = null;
			if ( flow != null ) flow.destroy();
			flow = null;
		}
		
		public override void DoWindowContents(Rect inRect)
		{
			if ( flow != null ) flow.DoFlowContents(inRect);
		}
		
		protected PageRenderer editPage(SettingsProperties props, string backPage, string currentPage, Dictionary<string, PageRenderer> pages, Action<Flow,PageRenderer> onNavigationHandler = null) {
			RowLayoutRenderer rows = new RowLayoutRenderer();
			
			string objectDescription;
			if ( (props.id+".description").TryTranslate(out objectDescription) ) {
				rows.AddChild(new TextRenderer(f => objectDescription, null, 404f, GameFont.Small, QOLMod.LineHeight(GameFont.Small) * 3) {  });
			}
			
			List<ISettingsFieldProperties> fis = props.fields;
			foreach ( ISettingsFieldProperties p in fis ) {
				int startChildCount = rows.childs.Count;
				
				if ( !p.isList && p.isSelectable ) {
					ISettingsFieldPropertiesSelectSingle pSelect = (ISettingsFieldPropertiesSelectSingle)p;
					rows.AddChild(new ColumnLayoutRenderer()
					              .AddChild(new LabelRenderer(pSelect.label, 200f).attachTooltip(p))
				            .AddChild(new ButtonTextRenderer(f => pSelect.labelProducer(((ISettingsInstance)f.flowScope["edit"]).get<object>(p.id)),
				                                            f => Find.WindowStack.Add(new Dialog_Select<object>(pSelect.onSelectItem(f), pSelect.selectableItems(f), pSelect.labelProducer)),
                                                       		GameFont.Small,200f))
					             );
				} else if ( !p.isList && p.isSettings ) {
					SettingsFieldPropertiesSettings pSetting = (SettingsFieldPropertiesSettings)p;
					SettingsProperties props2 = SettingsStorage.getSettingsProperties(pSetting.idEnclosedSettings);
					string deepEditPageId = currentPage+"edit"+pSetting.id+props2.id;
					rows.AddChild(new ColumnLayoutRenderer()
					            .AddChild(new LabelRenderer(p.label, 200f).attachTooltip(p))
					            .AddChild(new ButtonTextRenderer(f => {
					                                             	SettingsInstance si = ((ISettingsInstance)f.flowScope["edit"]).get<SettingsInstance>(p.id);
					                                             	if ( si != null ) return si.getLabel();
					                                             	else return "NONE";
					                                             },
					                                               f=>{
					                                             	SettingsInstance si = ((ISettingsInstance)f.flowScope["edit"]).get<SettingsInstance>(p.id);
					                                             	if ( si == null ) {
                                                                        if (pSetting.createNewEntry != null) si = (SettingsInstance)pSetting.createNewEntry();
                                                                        else si = (SettingsInstance)p.defaultValue;
					                                             		if ( si != null ) 
					                                             			((ISettingsInstance)f.flowScope["edit"]).set<SettingsInstance>(p.id, si);
					                                             	}
					                                             	if ( si != null ) {
					                                               		f.addPostRenderCallback(fl=>fl.flowScope["edit"] = si);
					                                               		f.navigate(deepEditPageId);
					                                             	} else {
					                                             		Log.Warning("Can not edit settings that are null and have no non null default value.");
					                                             	}
					                                               }, GameFont.Small, 200f - (pSetting.allowRemoval ? 24f : 0f)).attachTooltip(p))
                                                                   .AddChild(pSetting.allowRemoval ? new ButtonTextRenderer(f => {
                                                                       ISettingsInstance setting = (ISettingsInstance)f.flowScope["edit"];
                                                                       if (((ISettingsInstance)f.flowScope["edit"]).get<SettingsInstance>(p.id) == null)
                                                                       {
                                                                           if (pSetting.createNewEntry != null) return "+";
                                                                           else return "";
                                                                       }
                                                                       else return "-";
                                                                   }, f =>
                                                                   {
                                                                       ISettingsInstance setting = (ISettingsInstance)f.flowScope["edit"];
                                                                       SettingsInstance si = ((ISettingsInstance)f.flowScope["edit"]).get<SettingsInstance>(p.id);
                                                                       if (si != null)
                                                                       {
                                                                           ((ISettingsInstance)f.flowScope["edit"]).set<SettingsInstance>(p.id, null);
                                                                       }
                                                                       else if (pSetting.createNewEntry != null)
                                                                       {
                                                                           ((ISettingsInstance)f.flowScope["edit"]).set<SettingsInstance>(p.id, (SettingsInstance)pSetting.createNewEntry());
                                                                       }
                                                                   }, GameFont.Small, 20f) : null)
                        );
					
					pages.Add(deepEditPageId,editPage(props2,currentPage,deepEditPageId,pages,(f,nextPage)=>{
					                                  	if ( nextPage.onBack() != deepEditPageId ) {
						                                  	f.flowScope["edit"] = ((ISettingsInstance)f.flowScope["edit"]).getParentObject();
					                                  	}
					                                  }));
				} else if ( p.type == typeof(bool) ) {
					rows.AddChild(new ColumnLayoutRenderer()
			            	.AddChild(new LabelRenderer(p.label, 200f).attachTooltip(p))
			            	.AddChild(new CheckboxRenderer(f => ((ISettingsInstance)f.flowScope["edit"]).get<bool>(p.id), (f,v) => ((ISettingsInstance)f.flowScope["edit"]).set(p.id, v)))
			            );
				} else if ( p.type == typeof(int) ) {
					rows.AddChild(new ColumnLayoutRenderer()
				            .AddChild(new LabelRenderer(p.label, 200f).attachTooltip(p))
				            .AddChild(new EditNumericRenderer<int>(f=>((ISettingsInstance)f.flowScope["edit"]).get<int>(p.id),(f,v) => ((ISettingsInstance)f.flowScope["edit"]).set(p.id, v),
																200f,((ISettingsFieldPropertiesPrimitiveInt)p).minValue,((ISettingsFieldPropertiesPrimitiveInt)p).maxValue))
				           );
				} else if ( p.type == typeof(float) ) {
					rows.AddChild(new ColumnLayoutRenderer()
				            .AddChild(new LabelRenderer(p.label, 200f).attachTooltip(p))
				            .AddChild(new EditNumericRenderer<float>(f=>((ISettingsInstance)f.flowScope["edit"]).get<float>(p.id),(f,v) => ((ISettingsInstance)f.flowScope["edit"]).set(p.id, v),
			                                                    200f,((ISettingsFieldPropertiesPrimitiveFloat)p).minValue,((ISettingsFieldPropertiesPrimitiveFloat)p).maxValue))
				           );
				} else if ( p.type == typeof(string) ) {
                    rows.AddChild(new ColumnLayoutRenderer()
                            .AddChild(new LabelRenderer(p.label, 200f).attachTooltip(p))
                            .AddChild(new EditTextRenderer(f => ((ISettingsInstance)f.flowScope["edit"]).get<string>(p.id), (f, v) => ((ISettingsInstance)f.flowScope["edit"]).set(p.id, v),
                                                                   200f, ((ISettingsFieldPropertiesPrimitiveString)p).maxNumOfChars, ((ISettingsFieldPropertiesPrimitiveString)p).inputValidator)
                            { EmptyIsNull = ((ISettingsFieldPropertiesPrimitiveString)p).EmptyIsNull })
				           );
				} else if ( p.type == typeof(Color) ) {
					rows.AddChild(new ColumnLayoutRenderer()
				            .AddChild(new LabelRenderer(p.label, 200f).attachTooltip(p))
				            .AddChild(new EditColorRenderer(f=>((ISettingsInstance)f.flowScope["edit"]).get<Color>(p.id), 
				                                         (f,c) =>((ISettingsInstance)f.flowScope["edit"]).set<Color>(p.id,c),
				                                         200f))
				           );
                }
                else if (p.isList && ((ISettingsFieldPropertiesList)p).isListSettings)
                {
                    if (p is SettingsFieldPropertiesListSettings)
                    {
                        SettingsFieldPropertiesListSettings pList = (SettingsFieldPropertiesListSettings)p;
                        SettingsProperties props2 = SettingsStorage.getSettingsProperties(pList.idEnclosedSettings);
                        string deepEditPageId = currentPage + "edit" + pList.id + props2.id;
                        rows.AddChild(new ColumnLayoutRenderer()
                                .AddChild(new LabelRenderer(p.label, 400f, GameFont.Small, TextAnchor.MiddleCenter).attachTooltip(p))
                                .AddChild(new EmptyRenderer(0f))
                                .AddChild(new ColumnLayoutRenderer().setPaddingBetweenColumns(0f)
                                    .AddChild(new HideCollapseRenderer(f => ((ISettingsInstance)f.flowScope["edit"]).isFieldConfigured(p.id), new ButtonRenderer(Materials.radioButtonOff, f => ((ISettingsInstance)f.flowScope["edit"]).markFieldAsConfigured(p.id), 24f, 24f)))
                                    .AddChild(new HideCollapseRenderer(f => !((ISettingsInstance)f.flowScope["edit"]).isFieldConfigured(p.id) || !((ISettingsInstance)f.flowScope["edit"]).hasFieldConfiguredStatus(), new ButtonRenderer(Materials.radioButtonOn, f => ((ISettingsInstance)f.flowScope["edit"]).markFieldAsNotConfigured(p.id), 24f, 24f)))
                                )
                               );
                        rows.AddChild(new ListRenderer(f =>
                        {
                            List<SettingsInstance> lst = ((ISettingsInstance)f.flowScope["edit"]).get<List<SettingsInstance>>(pList.id);
                            //avoiding nullpointer if the default value for a list is null or if a save state didnt contain the list
                            if (lst == null)
                            {
                                lst = new List<SettingsInstance>();
                                ((ISettingsInstance)f.flowScope["edit"]).set<List<SettingsInstance>>(pList.id, lst);
                            }
                            return lst;
                        }, 400f + QOLMod.VerticalScrollbarWidth(), QOLMod.LineHeight(GameFont.Small) * 3)
                        { resizeVertically = QOLMod.getSettings().growLists }

                                      .AddChild(new ButtonTextRenderer(f => ((ISettingsInstance)((IterationItem)f.pageScope["curItem"]).curItem).getLabel(),
                                                                       f =>
                                                                       {
                                                                           f.navigate(deepEditPageId);
                                                                           object curItem = ((IterationItem)f.pageScope["curItem"]).curItem;
                                                                           f.addPostRenderCallback(fl => fl.flowScope["edit"] = curItem);
                                                                       }, GameFont.Small, pList.isRemovalAllowed ? 380f : 400f))
                                      .AddChild(pList.isRemovalAllowed ? new ButtonTextRenderer("-", f =>
                                      {
                                          ISettingsInstance setting = (ISettingsInstance)f.flowScope["edit"];
                                          SettingsInstance lstItem = (SettingsInstance)((IterationItem)f.pageScope["curItem"]).curItem;
                                          setting.markFieldAsConfigured(p.id, true, false);
                                          f.addPostRenderCallback(fl =>
                                          {
                                              if (setting.get<List<SettingsInstance>>(p.id).Remove(lstItem)) setting.markAsChanged();
                                          });
                                      }, GameFont.Small, 20f) : null));
                        rows.AddChild(pList.isAddingAllowed ? new ColumnLayoutRenderer()
                                      .AddChild(new ButtonTextRenderer("+",
                                                                       f =>
                                                                       {
                                                                           SettingsInstance newItem = (SettingsInstance)pList.createNewEntry();
                                                                           if (newItem != null)
                                                                           {
                                                                               ((ISettingsInstance)f.flowScope["edit"]).get<List<SettingsInstance>>(p.id).Add(newItem);
                                                                               newItem.attach(((ISettingsInstance)f.flowScope["edit"]), pList);
                                                                               ((ISettingsInstance)f.flowScope["edit"]).markFieldAsConfigured(p.id, true, false);
                                                                               ((ISettingsInstance)f.flowScope["edit"]).markAsChanged();
                                                                           }
                                                                       },
                                                                       GameFont.Small, 400f)) : null);
                        pages.Add(deepEditPageId, editPage(props2, currentPage, deepEditPageId, pages, (f, nextPage) =>
                        {
                            if (nextPage.onBack() != deepEditPageId)
                            {
                                f.flowScope["edit"] = ((ISettingsInstance)f.flowScope["edit"]).getParentObject();
                            }
                        }));
                    }
                    else if (p is SettingsFieldPropertiesListSettingsReferences)
                    {
                        //TODO won't actually work since live object from save games do not use overrides(they have no original state only the last saved one)
                        SettingsFieldPropertiesListSettingsReferences pList = (SettingsFieldPropertiesListSettingsReferences)p;
                        SettingsProperties props2 = SettingsStorage.getSettingsProperties(pList.idEnclosedSettings);
                        string deepEditPageId = currentPage + "edit" + pList.id + props2.id;
                        rows.AddChild(new ColumnLayoutRenderer()
                                .AddChild(new LabelRenderer(p.label, 400f, GameFont.Small, TextAnchor.MiddleCenter).attachTooltip(p))
                                .AddChild(new EmptyRenderer(0f))
                                .AddChild(new ColumnLayoutRenderer().setPaddingBetweenColumns(0f)
                                    .AddChild(new HideCollapseRenderer(f => ((ISettingsInstance)f.flowScope["edit"]).isFieldConfigured(p.id), new ButtonRenderer(Materials.radioButtonOff, f => ((ISettingsInstance)f.flowScope["edit"]).markFieldAsConfigured(p.id), 24f, 24f)))
                                    .AddChild(new HideCollapseRenderer(f => !((ISettingsInstance)f.flowScope["edit"]).isFieldConfigured(p.id) || !((ISettingsInstance)f.flowScope["edit"]).hasFieldConfiguredStatus(), new ButtonRenderer(Materials.radioButtonOn, f => ((ISettingsInstance)f.flowScope["edit"]).markFieldAsNotConfigured(p.id), 24f, 24f)))
                                )
                               );
                        rows.AddChild(new ListRenderer(f =>
                        {
                            List<SettingsInstance> lst = ((ISettingsInstance)f.flowScope["edit"]).get<List<SettingsInstance>>(pList.id);
                            //avoiding nullpointer if the default value for a list is null or if a save state didnt contain the list
                            if (lst == null)
                            {
                                lst = new List<SettingsInstance>();
                                ((ISettingsInstance)f.flowScope["edit"]).set<List<SettingsInstance>>(pList.id, lst);
                            }
                            return lst;
                        }, 400f + QOLMod.VerticalScrollbarWidth(), QOLMod.LineHeight(GameFont.Small) * 3)
                        { resizeVertically = QOLMod.getSettings().growLists }

                                      .AddChild(new ButtonTextRenderer(f => ((ISettingsInstance)((IterationItem)f.pageScope["curItem"]).curItem).getLabel(),
                                                                       f =>
                                                                       {
                                                                           f.navigate(deepEditPageId);
                                                                           object curItem = ((IterationItem)f.pageScope["curItem"]).curItem;
                                                                           f.addPostRenderCallback(fl => fl.flowScope["edit"] = curItem);
                                                                       }, GameFont.Small, pList.isRemovalAllowed ? 380f : 400f))
                                      .AddChild(pList.isRemovalAllowed ? new ButtonTextRenderer("-", f =>
                                      {
                                          ISettingsInstance setting = (ISettingsInstance)f.flowScope["edit"];
                                          SettingsInstance lstItem = (SettingsInstance)((IterationItem)f.pageScope["curItem"]).curItem;
                                          setting.markFieldAsConfigured(p.id, true, false);
                                          f.addPostRenderCallback(fl =>
                                          {
                                              if (setting.get<List<SettingsInstance>>(p.id).Remove(lstItem)) setting.markAsChanged();
                                          });
                                      }, GameFont.Small, 20f) : null));
                        rows.AddChild(pList.isAddingAllowed ? new ColumnLayoutRenderer()
                                      .AddChild(new ButtonTextRenderer("+",
                                                                       f =>
                                                                       {
                                                                           List<SettingsInstance> choices = SettingsStorage.getSettingsInstancesForSelection((ISettingsInstance)f.flowScope["edit"], pList);
                                                                           Find.WindowStack.Add(new Dialog_Select<SettingsInstance>(
                                                                               si => {
                                                                                   ((ISettingsInstance)f.flowScope["edit"]).get<List<SettingsInstance>>(p.id).Add(si);
                                                                                   si.attach(((ISettingsInstance)f.flowScope["edit"]), pList);
                                                                                   ((ISettingsInstance)f.flowScope["edit"]).markFieldAsConfigured(p.id, true, false);
                                                                                   ((ISettingsInstance)f.flowScope["edit"]).markAsChanged();
                                                                                   return true;
                                                                               }, choices, si => si.getLabel(), null, null));
                                                                       },
                                                                       GameFont.Small, 400f)) : null);
                        pages.Add(deepEditPageId, editPage(props2, currentPage, deepEditPageId, pages, (f, nextPage) =>
                        {
                            if (nextPage.onBack() != deepEditPageId)
                            {
                                f.flowScope["edit"] = ((ISettingsInstance)f.flowScope["edit"]).getParentObject();
                            }
                        }));
                    }
                } else if ( p.isList && ((ISettingsFieldPropertiesList)p).isListPrimitive ) {
					SettingsFieldPropertiesListPrimitive pList = (SettingsFieldPropertiesListPrimitive)p;
					Type listItemType = pList.type.GetGenericArguments()[0];
					rows.AddChild(new ColumnLayoutRenderer()
				            .AddChild(new LabelRenderer(p.label, 400f, GameFont.Small, TextAnchor.MiddleCenter).attachTooltip(p))
				            .AddChild(new EmptyRenderer(0f))
				           );
					ListRenderer lstRenderer = new ListRenderer(f => {
					                                                  	IList lst = ((ISettingsInstance)f.flowScope["edit"]).get<IList>(pList.id);
					                                                  	//avoiding nullpointer if the default value for a list is null or if a save state didnt contain the list
					                                                  	if ( lst == null ) {
                                                                            lst = (IList)Activator.CreateInstance(typeof(List<>).MakeGenericType(listItemType),null);
					                                                  		((ISettingsInstance)f.flowScope["edit"]).setUnchanged<IList>(pList.id,lst);
					                                                  	}
					                                                  	return lst;
		                                                              },400f+QOLMod.VerticalScrollbarWidth(),QOLMod.LineHeight(GameFont.Small)*3);
					
					float editRendererWidth;
					if ( pList.listLabelProducer != null ) {
						lstRenderer.AddChild(new LabelRenderer(f => pList.listLabelProducer((ISettingsInstance)f.flowScope["edit"],f), 200f));
						editRendererWidth = pList.isRemovalAllowed ? 180f : 200f;
					} else editRendererWidth = pList.isRemovalAllowed ? 380f : 400f;
					//Currently no min max etc for the list item type
					if ( listItemType == typeof(float) ) {
						lstRenderer.AddChild(new EditNumericRenderer<float>(f=>(float)((IterationItem)f.pageScope["curItem"]).curItem,
						                                                    (f,v) => { ((ISettingsInstance)f.flowScope["edit"]).get<IList>(pList.id)[((IterationItem)f.pageScope["curItem"]).index] = v; ((ISettingsInstance)f.flowScope["edit"]).markAsChanged(); },
						                                                    editRendererWidth/*,((ISettingsFieldPropertiesPrimitiveFloat)p).minValue,((ISettingsFieldPropertiesPrimitiveFloat)p).maxValue*/));
					} else if ( listItemType == typeof(int) ) {
						lstRenderer.AddChild(new EditNumericRenderer<int>(f=>(int)((IterationItem)f.pageScope["curItem"]).curItem,
						                                                  (f,v) => { ((ISettingsInstance)f.flowScope["edit"]).get<IList>(pList.id)[((IterationItem)f.pageScope["curItem"]).index] = v; ((ISettingsInstance)f.flowScope["edit"]).markAsChanged(); },
						                                                  editRendererWidth/*,((ISettingsFieldPropertiesPrimitiveInt)p).minValue,((ISettingsFieldPropertiesPrimitiveInt)p).maxValue*/));
					} else if ( listItemType == typeof(bool) ) {
						lstRenderer.AddChild(new CheckboxRenderer(f=>(bool)((IterationItem)f.pageScope["curItem"]).curItem,
						                                          (f,v) => { ((ISettingsInstance)f.flowScope["edit"]).get<IList>(pList.id)[((IterationItem)f.pageScope["curItem"]).index] = v; ((ISettingsInstance)f.flowScope["edit"]).markAsChanged(); }));
					} else if ( listItemType == typeof(string) ) {
						lstRenderer.AddChild(new EditTextRenderer(f=>(string)((IterationItem)f.pageScope["curItem"]).curItem,
						                                          (f,v) => { ((ISettingsInstance)f.flowScope["edit"]).get<IList>(pList.id)[((IterationItem)f.pageScope["curItem"]).index] = v; ((ISettingsInstance)f.flowScope["edit"]).markAsChanged(); },
						                                          editRendererWidth/*,((ISettingsFieldPropertiesPrimitiveString)p).maxNumOfChars,((ISettingsFieldPropertiesPrimitiveString)p).inputValidator*/));
					}
					
					lstRenderer.AddChild(pList.isRemovalAllowed ? new ButtonTextRenderer("-", f => {
                     	ISettingsInstance setting = (ISettingsInstance)f.flowScope["edit"];
                     	int indx = setting.get<IList>(p.id).IndexOf(((IterationItem)f.pageScope["curItem"]).curItem);
                     	if ( indx >= 0 ) {
	                     	f.addPostRenderCallback(fl => {
								IList lst = setting.get<IList>(p.id);
								if (indx < lst.Count) {
									lst.RemoveAt(indx);
									setting.markAsChanged();
								}});
                     	}},
                     	GameFont.Small, 20f):null);
					
					rows.AddChild(lstRenderer);
					
					rows.AddChild(pList.isAddingAllowed ? new ColumnLayoutRenderer()
			              .AddChild(new ButtonTextRenderer("+", 
	                           f=> {
	                           	object newItem = pList.createNewEntry();
	                           	if ( newItem != null ) {
	                           		ISettingsInstance currentSettings = ((ISettingsInstance)f.flowScope["edit"]);
	                           		IList lst = currentSettings.get<IList>(p.id);
	                           		f.addPostRenderCallback(fl=>{
       		                        		lst.Add(newItem);
			                               	if ( pList.isSettings ) ((ISettingsInstance)newItem).attach(currentSettings, pList);
			                               	currentSettings.markAsChanged();
           		                        });
	                           	}
	                           },
	                           GameFont.Small, 400f)) : null);
				} else if ( p.isList && ((ISettingsFieldPropertiesList)p).isSelectable ) {
					SettingsFieldPropertiesListSelectable pList = (SettingsFieldPropertiesListSelectable)p;
					Type listItemType = pList.type.GetGenericArguments()[0];
					rows.AddChild(new ColumnLayoutRenderer()
				            .AddChild(new LabelRenderer(p.label, 400f, GameFont.Small, TextAnchor.MiddleCenter).attachTooltip(p))
				            .AddChild(new EmptyRenderer(0f))
				           );
					ListRenderer lstRenderer = new ListRenderer(f => {
                      	IList lst = ((ISettingsInstance)f.flowScope["edit"]).get<IList>(pList.id);
                      	//avoiding nullpointer if the default value for a list is null or if a save state didnt contain the list
                      	if ( lst == null ) {
                      		lst = (IList)Activator.CreateInstance(typeof(List<>).MakeGenericType(listItemType),null);
                      		((ISettingsInstance)f.flowScope["edit"]).setUnchanged<IList>(pList.id,lst);
                      	}
                      	return lst;
                      },400f+QOLMod.VerticalScrollbarWidth(),QOLMod.LineHeight(GameFont.Small)*3);
					
					lstRenderer.AddChild(new ButtonTextRenderer(f => pList.labelProducer(((IterationItem)f.pageScope["curItem"]).curItem),
					                                            f => Find.WindowStack.Add(new Dialog_Select<object>(pList.onSelectItem(f), 
					                                            	pList.selectableItems(f), pList.labelProducer, null, pList.tooltipProducer)),
                                                       		GameFont.Small,pList.isRemovalAllowed ? 380f : 400f));
					if ( pList.tooltipProducer != null ) {
						lstRenderer.childs[0].attachTooltip(f => pList.tooltipProducer(((IterationItem)f.pageScope["curItem"]).curItem));
					}
					lstRenderer.AddChild(pList.isRemovalAllowed ? new ButtonTextRenderer("-", f => {
                     	ISettingsInstance setting = (ISettingsInstance)f.flowScope["edit"];
                     	object lstItem = ((IterationItem)f.pageScope["curItem"]).curItem;
                     	f.addPostRenderCallback(fl => {
							IList lst = setting.get<IList>(p.id);
							if (lst.Contains(lstItem)) {
								lst.Remove(lstItem);
								setting.markAsChanged();
							}});},
                     	GameFont.Small, 20f):null);
					
					rows.AddChild(lstRenderer);
					
					rows.AddChild(pList.isAddingAllowed ? new ColumnLayoutRenderer()
			              .AddChild(new ButtonTextRenderer("+", 
	                           f=> {
	                           	object newItem = pList.createNewEntry();
	                           	if ( newItem != null ) {
	                           		ISettingsInstance currentSettings = ((ISettingsInstance)f.flowScope["edit"]);
	                           		IList lst = currentSettings.get<IList>(p.id);
	                           		f.addPostRenderCallback(fl=>{
       		                        		lst.Add(newItem);
			                               	if ( pList.isSettings ) ((ISettingsInstance)newItem).attach(currentSettings, pList);
			                               	currentSettings.markAsChanged();
           		                        });
	                           	}
	                           },
	                           GameFont.Small, 400f)) : null);
				}
				
				Func<Flow,bool> isHidden = p.isHidden;
				if ( isHidden != null ) {
					int stopChildCount = rows.childs.Count;
					for ( int i = startChildCount; i < stopChildCount; i++ ) {
						HideCollapseRenderer hr = new HideCollapseRenderer(isHidden, rows.childs[i]);
						rows.childs[i] = hr;
					}
				}
			}
			
			Func<string> back;
			if ( backPage == null ) {
				back = null;
			} else back = ()=>backPage;
            
            //TODO for now keeping the start stop editing callbacks out since they create overrides which are not intended to be used with live objects from a savegame
            return new PageRenderer(f=>"", null, back, 
                (f, p) => { /*if (f.GetPage(backPage) == p) ((ISettingsInstance)f.flowScope["edit"]).OnStopEditing();*/ onNavigationHandler?.Invoke(f, p); },
                (f, p) => { /*if (f.GetPage(backPage) == p) ((ISettingsInstance)f.flowScope["edit"]).OnStartEditing();*/ }).AddChild(new VerticalScrollViewRenderer().AddChild(rows));
		}
	}
}
