﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 31.10.2019
 * Time: 13:06
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol.storyteller
{
	/// <summary>
	/// Description of StorytellerTreeComp_TargetFilter.
	/// </summary>
	public class StorytellerTreeComp_TargetFilter : StorytellerTreeComp
	{
		protected List<Func<IIncidentTarget,bool>> predicates = new List<Func<IIncidentTarget, bool>>();
		protected Dictionary<IIncidentTarget,float> loopCounter = new Dictionary<IIncidentTarget, float>();
		
		public IEnumerable<IIncidentTarget> FilteredTargetsNoLoop {
			get {
				return Tree.lastTargets.Where(target => predicates.TrueForAll(p=>p(target)));
			}
		}
		
		protected IEnumerable<IIncidentTarget> Loop(IEnumerable<IIncidentTarget> targets) {
			foreach ( IIncidentTarget target in targets ) {
				float f;
				if ( loopCounter.TryGetValue(target, out f) ) {
					for ( float i = 1f; i < f + 0.005f; i++ ) {
						yield return target;
					}
				} else yield return target;
			}
		}
		
		public IEnumerable<IIncidentTarget> FilteredTargets {
			get {
				return Loop(Tree.lastTargets.Where(target => predicates.TrueForAll(p=>p(target))));
			}
		}
		
		public void ApplyFilter(Func<IIncidentTarget,bool> predicate) {
			predicates.Add(predicate);
		}
		
		public void RemoveFilter(Func<IIncidentTarget,bool> predicate) {
			predicates.Remove(predicate);
		}
		
		public float getLoopCounter(IIncidentTarget target) {
			float loop;
			if ( loopCounter.TryGetValue(target, out loop) ) {
				return loop;
			}
			return 1f;
		}
		
		public void AddLoopMultiplier(IIncidentTarget target, float loop) {
			float oldLoop;
			if ( loopCounter.TryGetValue(target, out oldLoop) ) {
				loopCounter[target] = oldLoop * loop;
			} else {
				loopCounter[target] = loop;
			}
		}
		
		public void setLoopCounter(IIncidentTarget target, float loop) {
			loopCounter[target] = loop;
		}
		
		public override void EndInterval() {
			loopCounter.Clear();
			predicates.Clear();
		}
	}
}
