﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 25.11.2019
 * Time: 16:48
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using RIMMSqol.genericSettings;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol.storyteller
{
	/// <summary>
	/// Description of StorytellerTreeNode_IncidentProperties.
	/// </summary>
	public class StorytellerTreeNode_IncidentProperties : StorytellerTreeNode
	{
		public int delay = 0, retryDurationTicks = 0;
		public float pointsMultiplier = 1f, pointsMin = 0, pointsMax = float.MaxValue;
		public bool forced = false;
		public RaidStrategyDef raidStrategy;
		
		public StorytellerTreeNode_IncidentProperties() : this(null) { }
		
		public StorytellerTreeNode_IncidentProperties(StorytellerTreeNode parent) : base(parent) { }
		
		public override IEnumerable<StorytellerTreeIncident> TryStartIncident(StorytellerTree tree) {
			foreach ( StorytellerTreeIncident fi in base.TryStartIncident(tree) ) {
				fi.delay = delay;
				fi.retryDurationTicks = retryDurationTicks;
				fi.incident.parms.points *= pointsMultiplier;
				fi.incident.parms.points = Mathf.Clamp(fi.incident.parms.points, pointsMin, pointsMax);
				fi.incident.parms.forced = forced;
				if ( raidStrategy != null ) fi.incident.parms.raidStrategy = raidStrategy;
				yield return fi;
			}
		}
		
		public override void ExposeData() {
			base.ExposeData();
			Scribe_Values.Look<int>(ref delay, "delay", 0);
			Scribe_Values.Look<int>(ref retryDurationTicks, "retryDurationTicks", 0);
			Scribe_Values.Look<float>(ref pointsMultiplier, "pointsMultiplier", 1f);
			Scribe_Values.Look<float>(ref pointsMin, "pointsMin", 0f);
			Scribe_Values.Look<float>(ref pointsMax, "pointsMax", float.MaxValue);
			Scribe_Values.Look<bool>(ref forced, "forced", false);
			Scribe_Defs.Look<RaidStrategyDef>(ref raidStrategy, "raidStrategy");
		}
		
		protected override void InitSettings(SettingsProperties p) {
			base.InitSettings(p);
			p.fields.Add(new SettingsFieldPropertiesPrimitiveInt<StorytellerTreeNode_IncidentProperties>("Delay","Delay",10,0,t=>t.delay,0));
			p.fields.Add(new SettingsFieldPropertiesPrimitiveInt<StorytellerTreeNode_IncidentProperties>("RetryDurationTicks","Retries",20,0,t=>t.retryDurationTicks,0));
			p.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<StorytellerTreeNode_IncidentProperties>("PointsMultiplier","Points Multiplier",30,1f,t=>t.pointsMultiplier,0f));
			p.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<StorytellerTreeNode_IncidentProperties>("PointsMin","Points Min",31,0f,t=>t.pointsMin,0f));
			p.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<StorytellerTreeNode_IncidentProperties>("PointsMax","Points Max",32,float.MaxValue,t=>t.pointsMax,0f));
			p.fields.Add(new SettingsFieldPropertiesPrimitiveBool<StorytellerTreeNode_IncidentProperties>("Forced","Forced",40,false,t=>t.forced));
			p.fields.Add(new SettingsFieldPropertiesSelectable("RaidStrategy","Raid Strategy",50,typeof(RaidStrategyDef),null,
			                                                   t=>((StorytellerTreeNode_IncidentProperties)t).raidStrategy,
			                                                   f=>from object d in DefDatabase<RaidStrategyDef>.AllDefsListForReading select d,
			                                                   f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<RaidStrategyDef>("RaidStrategy",(RaidStrategyDef)def); return true;},
			                                                   SettingsFieldPropertiesSelectable.DefLabelProducer,true));
		}
		
		protected override void AdoptSettings(SettingsInstance si, StorytellerTreeNode obj) {
			base.AdoptSettings(si, obj);
			StorytellerTreeNode_IncidentProperties sttn = (StorytellerTreeNode_IncidentProperties)obj;
			sttn.delay = si.get<int>("Delay");
			sttn.retryDurationTicks = si.get<int>("RetryDurationTicks");
			sttn.pointsMultiplier = si.get<float>("PointsMultiplier");
			sttn.pointsMin = si.get<float>("PointsMin");
			sttn.pointsMax = si.get<float>("PointsMax");
			sttn.forced = si.get<bool>("Forced");
			sttn.raidStrategy = si.get<RaidStrategyDef>("RaidStrategy");
		}
		
		public override void DoSettingsDialog(StorytellerTree tree) {
			Find.WindowStack.Add(new Dialog_EditObject<StorytellerTreeNode_IncidentProperties>(this,GetSettings().id,AdoptSettings));
		}
	}
	
	public class StorytellerTreeNodeConfiguration_IncidentProperties : StorytellerTreeNodeConfiguration {
		public int delay = 0, retryDurationTicks = 0;
		public float pointsMultiplier = 1f, pointsMin = 0f, pointsMax = float.MaxValue;
		public RaidStrategyDef raidStrategy;
		public bool forced = false;
		
		public override StorytellerTreeNode CreateNode(StorytellerTreeNode parent) {
			StorytellerTreeNode_IncidentProperties n = new StorytellerTreeNode_IncidentProperties(parent);
			n.label = label;
			n.delay = delay;
			n.retryDurationTicks = retryDurationTicks;
			n.pointsMultiplier = pointsMultiplier;
			n.pointsMin = pointsMin;
			n.pointsMax = pointsMax;
			n.raidStrategy = raidStrategy;
			n.forced = forced;
			return n;
		}
	}
}
