﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 28.10.2019
 * Time: 06:26
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using RIMMSqol.genericSettings;
using RimWorld;
using Verse;

namespace RIMMSqol.storyteller
{
	/// <summary>
	/// Description of StorytellerTreeNode_Chance.
	/// </summary>
	public class StorytellerTreeNode_Pick : StorytellerTreeNode
	{
		public List<float> chances;
		
		public StorytellerTreeNode_Pick() : this(null) { }
		
		public StorytellerTreeNode_Pick(StorytellerTreeNode parent) : base(parent) { }
		
		override protected void AddChild(StorytellerTreeNode child) {
			base.AddChild(child);
			if ( chances == null ) chances = new List<float>();
			while ( chances.Count < children.Count ) chances.Add(1f);
		}
		
		public override IEnumerable<StorytellerTreeIncident> TryStartIncident(StorytellerTree tree) {
			if ( !children.NullOrEmpty() ) {
				StorytellerTreeNode pick;
				if ( children.TryRandomElementByWeight(n=>chances[children.IndexOf(n)], out pick) ) {
					//Log.Message("Picked "+pick.label);
					foreach ( StorytellerTreeIncident fi in pick.TryStartIncident(tree) ) {
						yield return fi;
					}
				}
			}
		}
		
		public override void ExposeData() {
			base.ExposeData();
			Scribe_Collections.Look<float>(ref chances, "chances");
		}
		
		protected override void InitSettings(SettingsProperties p) {
			base.InitSettings(p);
			p.fields.Add(new SettingsFieldPropertiesListPrimitive("Chances","Chances",10,typeof(List<float>),null,
				                                                   t=>((StorytellerTreeNode_Pick)t).chances,null,null,false));
		}
		
		protected override void AdoptSettings(SettingsInstance si, StorytellerTreeNode obj) {
			base.AdoptSettings(si, obj);
			StorytellerTreeNode_Pick sttn = (StorytellerTreeNode_Pick)obj;
			sttn.chances = si.get<List<float>>("Chances");
		}
		
		public override void DoSettingsDialog(StorytellerTree tree) {
			Find.WindowStack.Add(new Dialog_EditObject<StorytellerTreeNode_Pick>(this,GetSettings().id,AdoptSettings));
		}
	}
	
	public class StorytellerTreeNodeConfiguration_Pick : StorytellerTreeNodeConfiguration {
		public List<float> chances;
		
		public override StorytellerTreeNode CreateNode(StorytellerTreeNode parent) {
			StorytellerTreeNode_Pick n = new StorytellerTreeNode_Pick(parent);
			n.label = label;
			n.chances = chances;
			return n;
		}
	}
}
