﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 28.10.2019
 * Time: 06:25
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using RIMMSqol.genericSettings;
using RimWorld;
using Verse;

namespace RIMMSqol.storyteller
{
	/// <summary>
	/// Description of StorytellerTreeNode_WaitFor.
	/// </summary>
	public class StorytellerTreeNode_WaitFor : StorytellerTreeNode
	{
		public int minInterval = 0, maxInterval = int.MaxValue;
		
		public StorytellerTreeNode_WaitFor() : this(null) { }
		
		public StorytellerTreeNode_WaitFor(StorytellerTreeNode parent) : base(parent) { }
		
		public override IEnumerable<StorytellerTreeIncident> TryStartIncident(StorytellerTree tree) {
			if ( tree.lastInterval < minInterval ) yield break;
			if ( tree.lastInterval > maxInterval ) yield break;
			
			foreach ( StorytellerTreeIncident fi in base.TryStartIncident(tree) ) yield return fi;
		}
		
		public override void ExposeData() {
			base.ExposeData();
			Scribe_Values.Look<int>(ref minInterval, "minInterval");
			Scribe_Values.Look<int>(ref maxInterval, "maxInterval");
		}
		
		protected override void InitSettings(SettingsProperties p) {
			base.InitSettings(p);
			p.fields.Add(new SettingsFieldPropertiesPrimitiveInt<StorytellerTreeNode_WaitFor>("MinInterval","min interval",10,0,t=>t.minInterval));
			p.fields.Add(new SettingsFieldPropertiesPrimitiveInt<StorytellerTreeNode_WaitFor>("MaxInterval","max interval",20,int.MaxValue,t=>t.maxInterval));
		}
		
		protected override void AdoptSettings(SettingsInstance si, StorytellerTreeNode obj) {
			base.AdoptSettings(si, obj);
			StorytellerTreeNode_WaitFor sttn = (StorytellerTreeNode_WaitFor)obj;
			sttn.minInterval = si.get<int>("MinInterval");
			sttn.maxInterval = si.get<int>("MaxInterval");
		}
		
		public override void DoSettingsDialog(StorytellerTree tree) {
			Find.WindowStack.Add(new Dialog_EditObject<StorytellerTreeNode_WaitFor>(this,GetSettings().id,AdoptSettings));
		}
	}
	
	public class StorytellerTreeNodeConfiguration_WaitFor : StorytellerTreeNodeConfiguration {
		public int minInterval = 0, maxInterval = int.MaxValue;
		
		public override StorytellerTreeNode CreateNode(StorytellerTreeNode parent) {
			StorytellerTreeNode_WaitFor n = new StorytellerTreeNode_WaitFor(parent);
			n.label = label;
			n.minInterval = minInterval;
			n.maxInterval = maxInterval;
			return n;
		}
	}
}
