﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 06.11.2019
 * Time: 20:17
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using RIMMSqol.genericSettings;
using RimWorld;
using UnityEngine;
using Verse;

namespace RIMMSqol.storyteller
{
	/// <summary>
	/// Description of StorytellerWindow.
	/// </summary>
	public class StorytellerWindow :  RimWorld.MainTabWindow
	{
		protected StorytellerTreeDef currentlySelectedTemplate;
		protected StorytellerTreeNode copiedNode;
		protected bool translated = false, disableVanillaStoryteller;
		protected string chooseTemplateLabel = "Choose";
		protected string loadTemplateLabel = "LoadTemplate";
		protected string loadTemplateWarning = "LoadTemplateWarning";
		protected string disableVanillaStorytellerLabel = "DisableVanillaStoryteller";
        protected string disabledIncidentsLabel = "DisabledIncidentsLabel";
        protected string disabledQuestScriptsLabel = "DisabledQuestScriptsLabel";
        protected List<FloatMenuOption> templateOptions;
		protected Dictionary<Type,string> translatedNodeLabels = new Dictionary<Type, string>();
		protected float spx = 8f, spy = 4f;
		protected float contentWidth, contentHeight;
		protected Vector2 scrollPosition;
		
		public override void PreOpen() {
			base.PreOpen();
			if ( !translated ) {
				TaggedString ts;
				chooseTemplateLabel.TryTranslate(out ts); chooseTemplateLabel = ts;
				loadTemplateLabel.TryTranslate(out ts); loadTemplateLabel = ts;
				loadTemplateWarning.TryTranslate(out ts); loadTemplateWarning = ts;
				disableVanillaStorytellerLabel.TryTranslate(out ts); disableVanillaStorytellerLabel = ts;
                disabledIncidentsLabel.TryTranslate(out ts); disabledIncidentsLabel = ts;
                disabledQuestScriptsLabel.TryTranslate(out ts); disabledQuestScriptsLabel = ts;

                TaggedString s;
				foreach ( Type t in typeof(StorytellerTreeNode).AllSubclassesNonAbstract() ) {
					t.Name.TryTranslate(out s);
					translatedNodeLabels.Add(t,s);
				}
				
				translated = true;
			}
			
			StorytellerGameComponent stgc = Current.Game.GetComponent<StorytellerGameComponent>();
			disableVanillaStoryteller = stgc.DisableVanillaStoryteller;
			
			currentlySelectedTemplate = null;
			copiedNode = null;
			
			contentWidth = 0;
			contentHeight = 0;
			scrollPosition = Vector2.zero;
		}
		
		public override void PostClose() {
			base.PostClose();
			if ( templateOptions != null ) {
				templateOptions.Clear();
				templateOptions = null;
			}
			currentlySelectedTemplate = null;
			copiedNode = null;
			StorytellerGameComponent stgc = Current.Game.GetComponent<StorytellerGameComponent>();
			stgc.DisableVanillaStoryteller = disableVanillaStoryteller;
		}
		
		public override void DoWindowContents(Rect inRect) {
            StorytellerGameComponent stgc = Current.Game.GetComponent<StorytellerGameComponent>();
            Rect contentRect = new Rect(inRect);
			contentRect.height = Mathf.Max(contentHeight,contentRect.height-QOLMod.HorizontalScrollbarHeight());
			contentRect.width = Mathf.Max(contentWidth,contentRect.width-QOLMod.VerticalScrollbarWidth());
			Widgets.BeginScrollView(inRect, ref scrollPosition, contentRect);
			
			Rect r = new Rect();
			r.Set(contentRect.x, contentRect.y, 200f, Text.LineHeight);
			if ( Widgets.ButtonText(r,currentlySelectedTemplate != null ? currentlySelectedTemplate.LabelCap.ToString() : chooseTemplateLabel) ) {
				if ( templateOptions == null ) {
					List<FloatMenuOption> op = new List<FloatMenuOption>();
					foreach ( StorytellerTreeDef d in DefDatabase<StorytellerTreeDef>.AllDefsListForReading ) {
						op.Add(new FloatMenuOption(d.label, ()=>currentlySelectedTemplate=d));
					}
					templateOptions = op;
				}
				FloatMenu fm = new FloatMenu(templateOptions);
				fm.vanishIfMouseDistant = true;
				//fm.onCloseCallback = delegate {};
				Find.WindowStack.Add(fm);
			}
			if ( currentlySelectedTemplate != null && Mouse.IsOver(r) ) {
				TooltipHandler.TipRegion(r, () => currentlySelectedTemplate.description, 1);
			}
			
			r.x += r.width + spx;
			if ( Widgets.ButtonText(r,loadTemplateLabel) && currentlySelectedTemplate != null ) {
				Find.WindowStack.Add(Dialog_MessageBox.CreateConfirmation(loadTemplateWarning, delegate {
                  	StorytellerGameComponent stgcc = Current.Game.GetComponent<StorytellerGameComponent>();
                  	stgcc.LoadStorytellerTree(currentlySelectedTemplate);
				}, false, null));
			}
			
			r.Set(contentRect.x, r.y + r.height + spy, r.x + r.width - contentRect.x, Text.LineHeight);
			Widgets.CheckboxLabeled(r, disableVanillaStorytellerLabel, ref disableVanillaStoryteller);

            r.Set(contentRect.x, r.y + r.height + spy, r.x + r.width - contentRect.x, Text.LineHeight);
            if (Widgets.ButtonText(r, disabledIncidentsLabel))
            {
                Find.WindowStack.Add(new Dialog_SelectDef<IncidentDef>(ic => { stgc.AddDisabledIncident(ic); return true; }));
                Find.WindowStack.Add(new Dialog_Select<IncidentDef>(ic =>
                {
                    stgc.ToggleDisabledIncident(ic);
                    return false;
                },
                DefDatabase<IncidentDef>.AllDefsListForReading,
                ic => { return stgc.DisabledIncidents.Contains(ic) ? '*' + SettingsFieldPropertiesSelectable.DefLabelProducer(ic) : SettingsFieldPropertiesSelectable.DefLabelProducer(ic); })
                { LabelColors = ic => stgc.DisabledIncidents.Contains(ic) ? Color.red : Color.clear });
            }

            r.Set(contentRect.x, r.y + r.height + spy, r.x + r.width - contentRect.x, Text.LineHeight);
            if (Widgets.ButtonText(r, disabledQuestScriptsLabel))
            {
                Find.WindowStack.Add(new Dialog_Select<QuestScriptDef>(qs =>
                {
                    stgc.ToggleDisabledQuestScript(qs);
                    return false;
                },
                DefDatabase<QuestScriptDef>.AllDefsListForReading.Where(q => q.IsRootRandomSelected || stgc.DisabledQuestScripts.Contains(q)),
                qs => { return stgc.DisabledQuestScripts.Contains(qs) ? '*' + SettingsFieldPropertiesSelectable.DefLabelProducer(qs) : SettingsFieldPropertiesSelectable.DefLabelProducer(qs); })
                { LabelColors = qs => stgc.DisabledQuestScripts.Contains(qs) ? Color.red : Color.clear });
            }

            r.Set(contentRect.x, r.y + r.height + spy, contentRect.width, contentRect.height - (r.y + r.height + spy));
			float requiredWidth, requiredHeight;
			stgc.Tree.DoSettingsContent(r, out requiredWidth, out requiredHeight);
			
			r.height = requiredHeight;
			
			StorytellerTreeNode n = stgc.Tree.Root;
			int indent = 0;
			
			while ( n != null ) {
				//render n
				r.Set(contentRect.x + indent * 40f, r.y + r.height + spy, 200f, Text.LineHeight);
								
				if ( Widgets.ButtonText(r, n.label.NullOrEmpty() ? translatedNodeLabels[n.GetType()] : n.label) ) {
					//open dialog for editing node created by the node itself
					n.DoSettingsDialog(stgc.Tree);
				}
				r.x += r.width + spx;
				r.width = 20f;
				if ( !n.IsLeaf && Widgets.ButtonText(r,"+") ) {
					//add a child to the end of the child list after picking the type of node from a float menu
					StorytellerTreeNode parentForSpawning = n;
					List<FloatMenuOption> opts = new List<FloatMenuOption>();
					foreach ( Type t in typeof(StorytellerTreeNode).AllSubclassesNonAbstract() ) {
						if ( typeof(StorytellerTreeNode_Root).IsAssignableFrom(t) ) continue;
						opts.Add(new FloatMenuOption(translatedNodeLabels[t], () => Activator.CreateInstance(t,parentForSpawning)));
					}
					FloatMenu fm = new FloatMenu(opts);
					fm.vanishIfMouseDistant = true;
					//fm.onCloseCallback = delegate {};
					Find.WindowStack.Add(fm);
				}
				r.x += r.width + spx;
				if ( n.Parent != null && Widgets.ButtonText(r,"<") ) {
					//move this node forward in the parents child list
					n.Parent.MoveChildForward(n);
				}
				r.x += r.width + spx;
				if ( n.Parent != null && Widgets.ButtonText(r,">") ) {
					//move this node backward in the parents child list
					n.Parent.MoveChildBackwards(n);
				}
				r.x += r.width + spx;
				if ( Widgets.ButtonText(r,"C") ) {
					//Copy this subtree
					copiedNode = n;
				}
				r.x += r.width + spx;
				if ( copiedNode != null && !copiedNode.IsAncestorOf(n) && Widgets.ButtonText(r,"P") ) {
					//Paste the last copied subtree to the end of the child list
					copiedNode.DeepCopySubtree(n);
				}
				r.x += r.width + spx;
				if ( n.Parent != null && Widgets.ButtonText(r,"X") ) {
					//delete this subtree
					n.Parent.RemoveChild(n);
				}
				contentWidth = Mathf.Max(contentWidth,r.xMax-contentRect.x);
				
				StorytellerTreeNode firstSibling = n.FirstSibling;
				if ( firstSibling != null ) {
					n = firstSibling;
					indent++;
				} else {
					StorytellerTreeNode last = n;
					StorytellerTreeNode parent = null;
					StorytellerTreeNode sibling = null;
					indent++;
					while ( sibling == null && (parent = last.Parent) != null ) {
						sibling = parent.NextSibling(last);
						last = parent;
						indent--;
					}
					n = sibling;
				}
			}
			
			Widgets.EndScrollView();
			
			contentHeight = r.yMax;
		}
		
		public override void ExtraOnGUI() {
			
		}
	}
}
