﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 15.11.2018
 * Time: 01:50
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Verse;

namespace RIMMSqol.translations
{
	/// <summary>
	/// Description of TranslationCache.
	/// </summary>
	public static class TranslationCache
	{
		static readonly Dictionary<TranslationRequest,string> translations;
		static public string getFormattedTranslation(TranslationRequest req) {
			string translation;
			if ( !translations.TryGetValue(req, out translation) ) {
				TaggedString translation2;
				if ( req.translationKey.TryTranslate(out translation2) ) {
					translation = translation2.Formatted(req.arguments);
				}
				translations.Add(req,translation);
			}
			return translation;
		}
		static public bool removeFormattedTranslation(TranslationRequest req) {
			return req != null && translations.Remove(req);
		}
		static TranslationCache() {
			translations = new Dictionary<TranslationRequest,string>();
			LanguageChangeListener.translationActions.Add(translations.Clear);
		}
	}
	
	public class EqualityComparer_NamedArgument : IEqualityComparer<NamedArgument> {
		static readonly public EqualityComparer_NamedArgument comparer = new EqualityComparer_NamedArgument();
		
		public bool Equals(NamedArgument x, NamedArgument y) {
			return x.label.Equals(y.label) && x.arg.Equals(y.arg);
		}

		public int GetHashCode(NamedArgument na) {
			int hashCode = 17;
			unchecked {
				hashCode = hashCode * 23 + na.label.GetHashCode();
				if ( na.arg != null ) hashCode = hashCode * 23 + na.arg.GetHashCode();
			}
			return hashCode;
		}
	}
	
	public class TranslationRequest {
		public readonly string translationKey;
		public readonly NamedArgument[] arguments;
		public TranslationRequest(string translationKey) : this(translationKey, null, null) {}
		public TranslationRequest(string translationKey, string[] parameterNames, object[] parameters) {
			this.translationKey = translationKey;
			if ( parameterNames != null ) {
				arguments = new NamedArgument[parameterNames.Length];
				for ( int i = 0; i < parameterNames.Length; i++ ) {
					this.arguments[i] = new NamedArgument(parameters[i],parameterNames[i]);
				}
			} else {
				arguments = new NamedArgument[0];
			}
		}
		
		public override bool Equals(object obj) {
			TranslationRequest other = obj as TranslationRequest;
			if (other == null)
				return false;
			return this.translationKey == other.translationKey && this.arguments.SequenceEqual(other.arguments,EqualityComparer_NamedArgument.comparer);
		}

		public override int GetHashCode() {
			int hashCode = 17;
			unchecked {
				if (translationKey != null)
					hashCode = hashCode * 23 + translationKey.GetHashCode();
				if (arguments != null) {
					EqualityComparer_NamedArgument comparer = EqualityComparer_NamedArgument.comparer;
					foreach ( NamedArgument na in arguments ) {
						hashCode = comparer.GetHashCode(na);
					}
				}
			}
			return hashCode;
		}

		public static bool operator ==(TranslationRequest lhs, TranslationRequest rhs) {
			if (ReferenceEquals(lhs, rhs))
				return true;
			if (ReferenceEquals(lhs, null) || ReferenceEquals(rhs, null))
				return false;
			return lhs.Equals(rhs);
		}

		public static bool operator !=(TranslationRequest lhs, TranslationRequest rhs) {
			return !(lhs == rhs);
		}
	}
}
