﻿/*
 * Created by SharpDevelop.
 * User: Malte Schulze
 * Date: 31.10.2017
 * Time: 00:06
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Reflection;
using System.Reflection.Emit;
using System.Runtime.CompilerServices;
using System.Text;
using System.Text.RegularExpressions;
using HarmonyLib;
using RIMMSqol.genericSettings;
using RIMMSqol.pathfinding;
using RIMMSqol.renderers;
using RIMMSqol.translations;
using RimWorld;
using UnityEngine;
using Verse;
using Verse.AI;
using RIMMSqol.extensions;
using Verse.Sound;
using RIMMSqol.performance;

namespace RIMMSqol
{
	/// <summary>
	/// Description of SettingsInit.
	/// </summary>
	public static class SettingsInit
	{
		private static bool initialized = false;
		private static int applyCounter = 1;
		private static bool updatePawns = false;
		private static bool updateThingFilters, minifiableHasChanged = false;
        public static bool updateWatermills = false, updateResearchLayout = false;
        private static List<string> restartRequiredMessages = new List<string>();
        public static bool IsInitialized { get => initialized; }

        public static void init() {
			if ( !initialized ) {
				generalSettings();
				traitCosts();
				recordFactors();
				//powerBuildings();
				mainButtons();
				architectButtons();
				recipes();
				floorsAndTerrain();
				thoughts();
				apparel();
				weapons();
                projectiles();
                damage();
                plants();
				materials();
				foods();
				resourceRocks();
				buildings();
				items();
				hediffs();
				pawns();
				traits();
				abilities();
				researchProjects();
				researchTabs();
				backstories();
				biomes();
                filth();
                factions();
                worldObjects();
                if (ModsConfig.IsActive(ModContentPack.RoyaltyModPackageId) && !Prefs.SimulateNotOwningRoyalty) {
                    weaponTraits();
                }

                //materials are split under: ressources raw,textiles, isStuff

                //common flushing of caches that gets always applied regardless of changes in settings
                QOLMod.addApplySettingsListener(mod=> {
                    if ( !restartRequiredMessages.NullOrEmpty() )
                    {
                        if (Current.Game != null && Find.WindowStack != null)
                        {
                            StringBuilder sb = new StringBuilder().AppendLine("Following changes require a reload:");
                            foreach (string s in restartRequiredMessages)
                            {
                                sb.AppendLine(s);
                            }
                            Find.WindowStack.Add(new Dialog_MessageBox(sb.ToString(), null, null, null, null, "Reload required", true, null, null));
                        }
                        restartRequiredMessages.Clear();
                    }


                    CostListCalculator.Reset();
                    try
                    {
                        //recalculate wealth for terrain and floors
                        WealthWatcher.ResetStaticData();
                    } catch (Exception ex)
                    {
                        Log.Error("Failed to recalculate terrain wealth!\n"+ex.Message+"\n"+ex.StackTrace);
                    }
                    try
                    {
                        //recalculates weapon stuff and commonality for pawn redressing - weapon means primary equipment slot with at least one weapon tag
                        PawnWeaponGenerator.Reset();
                    }
                    catch (Exception ex)
                    {
                        Log.Error("Failed to recalculate weapon cache for weapon generation!\n" + ex.Message + "\n" + ex.StackTrace);
                    }
                    QOLMod.reloadPFAlgorithms();
                	GenLabel.ClearCache();
                	if ( Current.Game != null ) {
                        FieldInfo fiArtificialBuildingsPerCell = typeof(ListerArtificialBuildingsForMeditation).GetField("artificialBuildingsPerCell", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public);

                        foreach ( Map m in Find.Maps ) {
							//recalculates e.g. beauty ratings of rooms
							//this also resets the temperature. In cases where volcanic winter or other extremes influence the heat this can result in an immediate heating/cooling failure
							//m.regionAndRoomUpdater.RebuildAllRegionsAndRooms();
							//In order to avoid loosing the game state for temperature we trigger recalculation of room stats.
							foreach (Room room in m.regionGrid.allRooms) {
								room.Notify_TerrainChanged();
							}
                            //removing cached information about tile-speed for all map cells
                            //m.pathGrid.ResetPathGrid(); doing this will reset temperature in conjunction with recalculating path costs
                            m.pathing.RecalculateAllPerceivedPathCosts();
                            //m.pathGrid.RecalculateAllPerceivedPathCosts();

                            ((Dictionary<CellWithRadius, List<Thing>>)fiArtificialBuildingsPerCell?.GetValue(m.listerArtificialBuildingsForMeditation))?.Clear();
                        }
						//applies a new ordering to the main menu while a game is running
						System.Reflection.FieldInfo fi = Find.MainButtonsRoot.GetType().GetField("allButtonsInOrder", System.Reflection.BindingFlags.NonPublic | System.Reflection.BindingFlags.Instance);
						fi.SetValue(Find.MainButtonsRoot, (from x in DefDatabase<MainButtonDef>.AllDefs orderby x.order select x).ToList<MainButtonDef>());
                	}
                	MainTabWindow_Research_PostOpen.ClearCachedData();
                	if ( int.MaxValue != applyCounter ) applyCounter++;
                });
				
				QOLMod.addApplySettingsListener(mod=>{
                	if ( SettingsInit.updatePawns ) {
                        SettingsInit.updatePawns = false;
                        //keeping local copy since the returned list gets cached and modified whenever it is requested for some retarded reason.
                        List<Pawn> lstPreserved = new List<Pawn>(PawnsFinder.AllMapsWorldAndTemporary_Alive);
                        foreach ( Pawn pawn in lstPreserved ) {
                			RPGWindow.UpdatePawn(pawn);
                		}
                	}
                });
                QOLMod.addApplySettingsListener(mod =>
                {
                    if (SettingsInit.minifiableHasChanged)
                    {
                        SettingsInit.minifiableHasChanged = false;
                        //caches all generateable things, which can change if minifiable status changes
                        ThingSetMakerUtility.Reset();
                    }
                });

                QOLMod.addApplySettingsListener(mod=>{
                	if ( SettingsInit.updateThingFilters ) {
                        SettingsInit.updateThingFilters = false;
                        foreach (ThingCategoryDef tc in DefDatabase<ThingCategoryDef>.AllDefsListForReading)
                        {
                            tc.ResolveReferences();
                        }

                        FieldInfo fAllowedDefs = typeof(ThingFilter).GetField("allowedDefs", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
                        FieldInfo fPremultipliedSmallIngredients = typeof(RecipeDef).GetField("premultipliedSmallIngredients", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public);
                        if ( fAllowedDefs != null ) {
	                		foreach ( RecipeDef recipe in DefDatabase<RecipeDef>.AllDefsListForReading ) {
                                fPremultipliedSmallIngredients.SetValue(recipe, null);
                                if ( !recipe.products.NullOrEmpty() ) {
		            				ThingDef td = recipe.products[0].thingDef;
                                    if ( td == null )
                                    {
                                        Log.Warning("Found recipe \""+recipe.label+"\"[" + recipe.defName+"] with null entry as first product. Fix this to avoid errors!");
                                    }
                                    if ( td == null || td.MadeFromStuff ) {
		            					if ( recipe.fixedIngredientFilter != null && ThingFilterHasDefinitions(recipe.fixedIngredientFilter) ) {
		            						((HashSet<ThingDef>)fAllowedDefs.GetValue(recipe.fixedIngredientFilter)).Clear();
		            						recipe.fixedIngredientFilter.ResolveReferences();
		            					}
		            					if ( recipe.defaultIngredientFilter != null && ThingFilterHasDefinitions(recipe.defaultIngredientFilter)) {
		            						((HashSet<ThingDef>)fAllowedDefs.GetValue(recipe.defaultIngredientFilter)).Clear();
		            						recipe.defaultIngredientFilter.ResolveReferences();
		            					}
		            				}
		            			}
                                if (!recipe.ingredients.NullOrEmpty())
                                {
                                    foreach (IngredientCount ing in recipe.ingredients)
                                    {
                                        if (ing != null && ing.filter != null)
                                        {
                                            //Some modders build their filters by allowing things but not registering them in the corresponding collections.
                                            if (ThingFilterHasDefinitions(ing.filter))
                                            {
                                                ((HashSet<ThingDef>)fAllowedDefs.GetValue(ing.filter)).Clear();
                                                ing.filter.ResolveReferences();
                                            }
                                        }
                                    }
                                }
                            }
                		}
                	}
                });

                QOLMod.addApplySettingsListener(mod => {
                    if (SettingsInit.updateWatermills)
                    {
                        if (Current.Game != null)
                        {
                            MethodInfo mi = typeof(CompPowerPlantWater).GetMethod("ClearCache", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public);
                            foreach (Map m in Find.Maps)
                            {
                                using (IEnumerator<Building> enumerator = m.listerBuildings.AllBuildingsColonistOfDef(ThingDefOf.WatermillGenerator).GetEnumerator())
                                {
                                    while (enumerator.MoveNext())
                                    {
                                        mi.Invoke(enumerator.Current.GetComp<CompPowerPlantWater>(), null);
                                    }
                                }
                            }
                        }
                        SettingsInit.updateWatermills = false;
                    }
                });

                QOLMod.addApplySettingsListener(mod => {
                    if (SettingsInit.updateResearchLayout)
                    {
                        ResearchProjectDef.GenerateNonOverlappingCoordinates();
                        SettingsInit.updateResearchLayout = false;
                    }
                });

                SettingsStorage.initializeSettingsInstances();
				
				foreach ( SettingsProperties props in SettingsStorage.getAllSettingsProperties() ) {
					props.fields.Sort((a,b)=>a.order.CompareTo(b.order));
				}
				
				//create language file output in the console
				/*
				StringBuilder sb = new StringBuilder();
				foreach ( SettingsProperties props in SettingsStorage.getAllSettingsProperties() ) {
					foreach ( ISettingsFieldProperties field in props.fields ) {
						string tagName = props.id+"."+field.id;
						sb.Append("<"+tagName+">"+field.label+"</"+tagName+">\n");
						tagName = tagName+".tooltip";
						sb.Append("<"+tagName+">"+field.labelTooltip+"</"+tagName+">\n\n");
					}
				}
				Log.Message(sb.ToString());
				*/
				
				LanguageChangeListener.translationActions.Add(()=>{
                  	foreach ( SettingsProperties props in SettingsStorage.getAllSettingsProperties() ) {
						foreach ( ISettingsFieldProperties field in props.fields ) {
							string translation;
							string tagName = props.id+"."+field.id;
							if ( tagName.TryTranslate(out translation) && !translation.NullOrEmpty() ) {
								field.label = translation;
							}
							tagName = tagName+".tooltip";
							if ( tagName.TryTranslate(out translation) && !translation.NullOrEmpty() ) {
								field.labelTooltip = translation;
                            }
						}
					}});
                LanguageChangeListener.translationActions.Last()();

                initialized = true;
            }
		}

        static private bool ThingFilterHasDefinitions(ThingFilter tf)
        {
            //a filter must define at least one allowing rule to qualify for a reload.
            if (
                !ThingFilterAdapter.ThingFilterThingDefsGet(tf).NullOrEmpty() ||
                !ThingFilterAdapter.ThingFilterCategoriesGet(tf).NullOrEmpty() ||
                !ThingFilterAdapter.ThingFilterTradeTagsToAllowGet(tf).NullOrEmpty() ||
                //!ThingFilterAdapter.ThingFilterTradeTagsToDisallowGet(tf).NullOrEmpty() ||
                !ThingFilterAdapter.ThingFilterThingSetMakerTagsToAllowGet(tf).NullOrEmpty() ||
                //!ThingFilterAdapter.ThingFilterThingSetMakerTagsToDisallowGet(tf).NullOrEmpty() ||
                //!ThingFilterAdapter.ThingFilterDisallowedCategoriesGet(tf).NullOrEmpty() ||
                //!ThingFilterAdapter.ThingFilterSpecialFiltersToDisallowGet(tf).NullOrEmpty() ||
                !ThingFilterAdapter.ThingFilterSpecialFiltersToAllowGet(tf).NullOrEmpty() ||
                !ThingFilterAdapter.ThingFilterStuffCategoriesToAllowGet(tf).NullOrEmpty() ||
                !ThingFilterAdapter.ThingFilterAllowAllWhoCanMakeGet(tf).NullOrEmpty() ||
                //ThingFilterAdapter.ThingFilterDisallowWorsePreferabilityGet(tf) != FoodPreferability.Undefined ||
                //ThingFilterAdapter.ThingFilterDisallowInedibleByHumanGet(tf) ||
                //ThingFilterAdapter.ThingFilterDisallowNotEverStorableGet(tf) ||
                ThingFilterAdapter.ThingFilterAllowWithCompGet(tf) != null //||
                //ThingFilterAdapter.ThingFilterDisallowWithCompGet(tf) != null ||
                //ThingFilterAdapter.ThingFilterDisallowCheaperThanGet(tf) != float.MinValue ||
                //!ThingFilterAdapter.ThingFilterDisallowedThingDefsGet(tf).NullOrEmpty()
                )
            {
                return true;
            }
            //Log.Message("Thingfilter has no definitions and is skipped: "+tf);
            return false;
        }

        private static void Slavery()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("Slavery");
            if (props.fields.NullOrEmpty())
            {
                CurvePoint();
                props.fields.Add(new SettingsFieldPropertiesListSettings("MovingCapacityFactorCurve", "Moving Capacity Curve", 10, typeof(List<SettingsInstance>),
                    new SimpleCurve { { new CurvePoint(0f, 0.01f), true }, { new CurvePoint(0.5f, 0.5f), true }, { new CurvePoint(1f, 1f), true } }.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                        t => ((SimpleCurve)typeof(SlaveRebellionUtility).GetField("MovingCapacityFactorCurve", BindingFlags.Static | BindingFlags.NonPublic | BindingFlags.Public).GetValue(null)).Points.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                        "curvePoint", () => new SettingsInstance("curvePoint", new CurvePoint(0f, 0f), null, null), true));
                props.fields.Add(new SettingsFieldPropertiesListSettings("SuppresionRebellionFactorCurve", "Rebellion Suppression Curve", 20, typeof(List<SettingsInstance>),
                    new SimpleCurve{{new CurvePoint(0f, 5f),true},{new CurvePoint(0.333f, 1.5f),true},{new CurvePoint(0.5f, 1f),true},{new CurvePoint(1f, 0.25f),true}}.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                        t => ((SimpleCurve)typeof(SlaveRebellionUtility).GetField("SuppresionRebellionFactorCurve", BindingFlags.Static | BindingFlags.NonPublic | BindingFlags.Public).GetValue(null)).Points.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                        "curvePoint", () => new SettingsInstance("curvePoint", new CurvePoint(0f, 0f), null, null), true));
                props.fields.Add(new SettingsFieldPropertiesListSettings("SlaveCountFactorCurve", "Slave Count Curve", 30, typeof(List<SettingsInstance>),
                    new SimpleCurve { { new CurvePoint(1f, 1f), true }, { new CurvePoint(2f, 0.75f), true }, { new CurvePoint(5f, 0.5f), true }, { new CurvePoint(10f, 0.3f), true }, { new CurvePoint(20f, 0.2f), true } }.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                        t => ((SimpleCurve)typeof(SlaveRebellionUtility).GetField("SlaveCountFactorCurve", BindingFlags.Static | BindingFlags.NonPublic | BindingFlags.Public).GetValue(null)).Points.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                        "curvePoint", () => new SettingsInstance("curvePoint", new CurvePoint(0f, 0f), null, null), true));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Slavery Properties");
            }
        }

        private static void Slavery_Merge(QOLModSettings modSettings, SettingsInstance slaverySettings)
        {
            if (slaverySettings == null) return;
            List<SettingsInstance> suppresionRebellionFactorCurve = slaverySettings.get<List<SettingsInstance>>("SuppresionRebellionFactorCurve");
            if (suppresionRebellionFactorCurve != null)
            {
                FieldInfo fi = typeof(SlaveRebellionUtility).GetField("SuppresionRebellionFactorCurve", BindingFlags.Static|BindingFlags.NonPublic|BindingFlags.Public);
                SimpleCurve sc = (SimpleCurve)fi.GetValue(null);
                sc.Points.Clear();
                sc.Points.AddRange(suppresionRebellionFactorCurve.Select(si => new CurvePoint(si.get<float>("X"), si.get<float>("Y"))));
                sc.SortPoints();
            }
            List<SettingsInstance> movingCapacityFactorCurve = slaverySettings.get<List<SettingsInstance>>("MovingCapacityFactorCurve");
            if (movingCapacityFactorCurve != null)
            {
                FieldInfo fi = typeof(SlaveRebellionUtility).GetField("MovingCapacityFactorCurve", BindingFlags.Static | BindingFlags.NonPublic | BindingFlags.Public);
                SimpleCurve sc = (SimpleCurve)fi.GetValue(null);
                sc.Points.Clear();
                sc.Points.AddRange(movingCapacityFactorCurve.Select(si => new CurvePoint(si.get<float>("X"), si.get<float>("Y"))));
                sc.SortPoints();
            }
            List<SettingsInstance> slaveCountFactorCurve = slaverySettings.get<List<SettingsInstance>>("SlaveCountFactorCurve");
            if (slaveCountFactorCurve != null)
            {
                FieldInfo fi = typeof(SlaveRebellionUtility).GetField("SlaveCountFactorCurve", BindingFlags.Static | BindingFlags.NonPublic | BindingFlags.Public);
                SimpleCurve sc = (SimpleCurve)fi.GetValue(null);
                sc.Points.Clear();
                sc.Points.AddRange(slaveCountFactorCurve.Select(si => new CurvePoint(si.get<float>("X"), si.get<float>("Y"))));
                sc.SortPoints();
            }
        }

        private static string ExtractDefTypeName(Type t)
        {
            if (t == null) return null;
            return GenTypes.GetTypeNameWithoutIgnoredNamespaces(t);
        }

        private static Type ExtractDefTypeFromName(string name)
        {
            if (name == null) return null;
            return GenTypes.GetTypeInAnyAssembly(name, null);
        }

        private static Def GetFirstDefOfType(Type t)
        {
            if (t == null) return null;
            System.Collections.IEnumerable enumerable = ((System.Collections.IEnumerable)typeof(DefDatabase<>).MakeGenericType(t)
                .GetProperty("AllDefsListForReading").GetGetMethod().Invoke(null, null));
            if ( enumerable != null )
            {
                System.Collections.IEnumerator rator = enumerable.GetEnumerator();
                if (rator.MoveNext()) return (Def)rator.Current;
            }
            return null;
        }

        private static IEnumerable<Def> GetAllDefsOfType(Type t)
        {
            if (t == null) yield break;
            System.Collections.IEnumerable enumerable = ((System.Collections.IEnumerable)typeof(DefDatabase<>).MakeGenericType(t)
                .GetProperty("AllDefsListForReading").GetGetMethod().Invoke(null, null));
            if (enumerable != null)
            {
                foreach (object o in enumerable) yield return (Def)o;
            }
        }

        private static void QOLModSettingsDefCopy()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("QOLModSettingsDefCopy");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesSelectable("DefType", "Type", 10, typeof(string), ExtractDefTypeName(GenDefDatabase.AllDefTypesWithDatabases().FirstOrDefault()),
                    t => ((QOLModSettingsDefCopy)t).defType,
                    f => from Type t in GenDefDatabase.AllDefTypesWithDatabases() select ExtractDefTypeName(t),
                    f => s => {
                        ((ISettingsInstance)f.flowScope["edit"]).set<string>("DefType", (string)s);
                        ((ISettingsInstance)f.flowScope["edit"]).set<string>("DefNameOriginal", GetFirstDefOfType(ExtractDefTypeFromName((string)s))?.defName);
                        return true;
                    },
                    o=>(string)o, false));
                props.fields.Add(new SettingsFieldPropertiesSelectable("DefNameOriginal", "Name", 20, typeof(string), null,
                    t => ((QOLModSettingsDefCopy)t).defNameOriginal,
                    f => {
                        string s = ((ISettingsInstance)f.flowScope["edit"]).get<string>("DefType");
                        if ( s != null ) return GetAllDefsOfType(ExtractDefTypeFromName(s)).Where(d=>!d.generated).Select(d=> d.defName);
                        return null;
                    },
                    f => s => {
                        ((ISettingsInstance)f.flowScope["edit"]).set<string>("DefNameOriginal", (string)s);
                        return true;
                    },
                    o=>(string)o, false));

                props.fields.Add(new SettingsFieldPropertiesPrimitiveString<QOLModSettingsDefCopy>("DefNameCopy", "New Name", 30, null, t => t.defNameCopy));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveString<QOLModSettingsDefCopy>("LabelCopy", "New Label", 40, null, t => t.labelCopy));

                props.dynamicLabel = settings => settings.get<string>("LabelCopy");

                props.visualization = new SettingsVisualizationEdit(f => "Edit \"" + ((ISettingsInstance)f.flowScope["edit"]).getLabel() + "\"");
            }
        }

        private static void generalSettings() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("settings");
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<QOLModSettings>("SkillDecayLevelOffset","Skill decay level offset",9,0,t=>t.skillDecayLevelOffset));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("StopSkillDecay","Stop skill decay",10,default(bool),t=>t.stopSkillDecay));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("StopSkillDown", "Stop skill down", 11, default(bool), t => t.stopSkillDown));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("StopTamenessDecay","Stop tameness decay",12,default(bool),t=>t.stopTamenessDecay));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("FallRatePerPsyfocusBand0", "Psy Decay 0 <- 25%", 13, 0.035f, t => Pawn_PsychicEntropyTracker.FallRatePerPsyfocusBand[0], -400, 400));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("FallRatePerPsyfocusBand1", "Psy Decay 25 <- 50%", 14, 0.055f, t => Pawn_PsychicEntropyTracker.FallRatePerPsyfocusBand[1], -400, 400));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("FallRatePerPsyfocusBand2", "Psy Decay 50 <-> 100%", 15, 0.075f, t => Pawn_PsychicEntropyTracker.FallRatePerPsyfocusBand[2], -400, 400));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("PreventAnimalFamilies","No animal families",17,default(bool),t=>t.preventAnimalFamilies));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("AllowNamedAnimalFamilies", "but allow named animals", 18, default(bool), t => t.allowNamedAnimalFamilies));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("UseFixedNumericTextfields","Fixed Numberinput",19,default(bool),t=>t.useFixedNumericTextfields));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("CuMBaseCost","CuM base cost",20,20f,t=>t.cumBaseCost));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("CumCostIdeo", "CuM cost Ideology", 21, 10f, t => t.cumCostIdeo));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("CumCostPassionMinor", "CuM cost P-Minor", 24, 2f, t => t.cumCostPassionMinor));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("CumCostPassionMajor", "CuM cost P-Major", 25, 3f, t => t.cumCostPassionMajor));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("UseSyrchalistTraitvalue", "TVF Support", 27, default(bool), t => t.useSyrchalistTraitvalue));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("CuMTax","CuM tax",28,0f,t=>t.cumTax));

            CurvePoint();
            props.fields.Add(new SettingsFieldPropertiesListSettings("CuMInflationCurve", "CuM Inflation", 29, typeof(List<SettingsInstance>), new List<SettingsInstance>(),
                    t => ((QOLModSettings)t).cumInflationCurve == null ? null : 
                        ((QOLModSettings)t).cumInflationCurve.Points.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                    "curvePoint", () => new SettingsInstance("curvePoint", new CurvePoint(0f, 0f), null, null), true));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("CuMConversionPawn","CuM point conversion from pawns",30,0f,t=>t.cumPointPawnToPoolConversionFactor, 0f, 1f));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("CuMConversionRemnants","CuM point conversion from remnants",40,0.25f,t=>t.cumPointRemnantsToPoolConversionFactor, 0f));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("RemnantsOrderPriceFactor","Remnants-Order price factor",50,0.5f,t=>t.remnantOrderPriceFactor));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("StarvingThreshold","Starving Threshold (Food Rules)",55,0.4f,t=>t.starvingThreshold));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("DisplayFoundCityGizmo","Display Found City",57,true,t=>t.displayFoundCityGizmo));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("TooltipMaxWidth", "Tooltip Max Width", 58, 260f, t => t.tooltipMaxWidth, 260f));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("GrowLists", "Grow Lists", 59, false, t => t.growLists));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("AlwaysDisplayLabelInMBDD", "MB DD always labeled", 60, false, t => t.alwaysDisplayLabelInMBDD));
            props.fields.Add(new SettingsFieldPropertiesSelectable("DateStyle", "Date Style", 68, typeof(DateStyleDef), DefDatabase<DateStyleDef>.AllDefsListForReading.FirstOrDefault(),
                                                                   t => ((QOLModSettings)t).dateStyle,
                                                             f => from object d in DefDatabase<DateStyleDef>.AllDefsListForReading select d,
                                                             f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<DateStyleDef>("DateStyle", (DateStyleDef)def); return true; },
                                                             SettingsFieldPropertiesSelectable.DefLabelProducer, false));

            /*
			 * We want to configure 4 groups for pathfinding:
			 * - colonists
			 * - tame animals
			 * - wild animals
			 * - other
			 * That way colonists can gain vip pathfinding and the rest gets aproximations to save time.
			 * We select an algorithm(null means vanilla) and we define a text property that can be used to configure the pf algorithm instance(ignored for vanilla).
			 * The algorithms declare a class to instantiate for the algorithm. That class must use a constructor that accepts a string argument and it must implement
			 * the findpath method. Reflexion is used to grab the method. That way other mods implementing pathfinding algorithms do not need to link my assembly for
			 * their compilation.
			 */
            props.fields.Add(new SettingsFieldPropertiesSelectable("ColonistPF","Colonist PF",69,typeof(QOLPFDef),default(QOLPFDef),
			                                                       t=> {
			                                                       	string defName = ((QOLModSettings)t).pfColonist;
			                                                       	if ( defName == null ) return null;
			                                                       	return DefDatabase<QOLPFDef>.GetNamed(defName,false);
			                                                       },
				                                             f=>from object d in DefDatabase<QOLPFDef>.AllDefsListForReading select d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<QOLPFDef>("ColonistPF",(QOLPFDef)def); return true; },
				                                             SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveString<QOLModSettings>("ColonistPFConf", "Configuration", 70, default(string), t => t.pfColonistConf) { EmptyIsNull = true });
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("AnimalTamePF","Tame Animal PF",80,typeof(QOLPFDef),default(QOLPFDef),
			                                                       t=> {
			                                                       	string defName = ((QOLModSettings)t).pfAnimalTame;
			                                                       	if ( defName == null ) return null;
			                                                       	return DefDatabase<QOLPFDef>.GetNamed(defName,false);
			                                                       },
				                                             f=>from object d in DefDatabase<QOLPFDef>.AllDefsListForReading select d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<QOLPFDef>("AnimalTamePF",(QOLPFDef)def); return true; },
				                                             SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveString<QOLModSettings>("AnimalTamePFConf","Configuration",90,default(string),t=>t.pfAnimalTameConf) { EmptyIsNull = true });
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("AnimalWildPF","Wild Animal PF",100,typeof(QOLPFDef),default(QOLPFDef),
			                                                       t=> {
			                                                       	string defName = ((QOLModSettings)t).pfAnimalWild;
			                                                       	if ( defName == null ) return null;
			                                                       	return DefDatabase<QOLPFDef>.GetNamed(defName,false);
			                                                       },
				                                             f=>from object d in DefDatabase<QOLPFDef>.AllDefsListForReading select d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<QOLPFDef>("AnimalWildPF",(QOLPFDef)def); return true; },
				                                             SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveString<QOLModSettings>("AnimalWildPFConf","Configuration",110,default(string),t=>t.pfAnimalWildConf) { EmptyIsNull = true });
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("OtherPF","Other PF",120,typeof(QOLPFDef),default(QOLPFDef),
			                                                       t=> {
			                                                       	string defName = ((QOLModSettings)t).pfOther;
			                                                       	if ( defName == null ) return null;
			                                                       	return DefDatabase<QOLPFDef>.GetNamed(defName,false);
			                                                       },
				                                             f=>from object d in DefDatabase<QOLPFDef>.AllDefsListForReading select d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<QOLPFDef>("OtherPF",(QOLPFDef)def); return true; },
				                                             SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveString<QOLModSettings>("OtherPFConf","Configuration",130,default(string),t=>t.pfOtherConf) { EmptyIsNull = true });
			
			props.fields.Add(new SettingsFieldPropertiesListSelectable("PFRestrictionExcemption","Free Moving Jobs",140,typeof(List<string>),new List<string>(),
			                                                           t=> ((QOLModSettings)t).pfRestrictionExcemptions != null ? new List<string>(((QOLModSettings)t).pfRestrictionExcemptions) : new List<string>(),
			                                                           f=>DefDatabase<JobDef>.AllDefsListForReading.Select(d=>d.defName).Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<string>>("PFRestrictionExcemption")[listIndex] = (string)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                           entry=>{
			                                                           	JobDef jd = DefDatabase<JobDef>.GetNamed((string)entry,false);
			                                                           	if ( jd != null ) return SettingsFieldPropertiesSelectable.DefLabelProducer(jd);
			                                                           	return "Outdated";
			                                                           },()=>DefDatabase<JobDef>.AllDefsListForReading.First().defName,true));
			
			props.fields.Add(new SettingsFieldPropertiesListSelectable("HiddenAlerts","Hidden Alerts",145,typeof(List<string>),new List<string>(),
			                                                           t=> ((QOLModSettings)t).hiddenAlerts != null ? new List<string>(((QOLModSettings)t).hiddenAlerts) : new List<string>(),
			                                                           f=>typeof(Alert).AllSubclassesNonAbstract().Select(t=>t.Name).Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<string>>("HiddenAlerts")[listIndex] = (string)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
																		entry => (string)entry,
																		()=>typeof(Alert).AllSubclassesNonAbstract().First().Name,true));

            props.fields.Add(new SettingsFieldPropertiesListPrimitive("HiddenLetters", "Hidden Letters", 147, typeof(List<string>), new List<string>(),
                                      t => ((QOLModSettings)t).hiddenLetters != null ? ((QOLModSettings)t).hiddenLetters.ToList() : new List<string>(),
                                          null, () => (object)"Label Expression", true));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("ForbiddenPatches","Forbidden patches",150,typeof(List<string>),new List<string>(),
			                                                           t=> ((QOLModSettings)t).forbiddenPatchNamespaces,
			                                                           f=>HarmonyPatchNamespace.namespaces.Keys.Cast<object>().ToList(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<string>>("ForbiddenPatches")[listIndex] = (string)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                           entry=>{
			                                                           	HarmonyPatchNamespaceProperties nsprops;
			                                                           	if ( entry != null && HarmonyPatchNamespace.namespaces.TryGetValue((string)entry, out nsprops) ) 
			                                                           		return nsprops.label;
			                                                           	return "Select";
			                                                           },()=>"Select",true,
			                                                          entry=>{
			                                                           	HarmonyPatchNamespaceProperties nsprops;
			                                                           	if ( entry != null && HarmonyPatchNamespace.namespaces.TryGetValue((string)entry, out nsprops) ) 
			                                                           		return nsprops.description;
			                                                           	return null;
			                                                           }));
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("AvoidDesignatorRefresh","Avoid designator refresh",160,default(bool),t=>t.avoidDesignatorRefresh));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("ReapplyOnGameLoad","Reapply on game load",170,default(bool),t=>t.reapplyOnGameLoad));
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<QOLModSettings>("StopRoofCollapse","Stop roof collapse",180,default(bool),t=>t.stopRoofCollapse));

            Slavery();
            props.fields.Add(new SettingsFieldPropertiesSettings("Slavery", "Slavery", 190, typeof(SettingsInstance), new SettingsInstance("Slavery", null, null, null),
                                                                 t => new SettingsInstance("Slavery", null, null, null), "Slavery"));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("PawnValueExpectationsMV0", "Pawn Value Exp Extremely Low", 200, -1, t => t.pawnValueExpectationsMV0, -1));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("PawnValueExpectationsMV1", "Pawn Value Exp Very Low", 201, -1, t => t.pawnValueExpectationsMV1, -1));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("PawnValueExpectationsMV2", "Pawn Value Exp Low", 202, -1, t => t.pawnValueExpectationsMV2, -1));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("PawnValueExpectationsMV3", "Pawn Value Exp Moderate", 203, -1, t => t.pawnValueExpectationsMV3, -1));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("PawnValueExpectationsMV4", "Pawn Value Exp High", 204, -1, t => t.pawnValueExpectationsMV4, -1));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLModSettings>("PawnValueExpectationsMV5", "Pawn Value Exp SkyHigh", 205, -1, t => t.pawnValueExpectationsMV5, -1));

            props.fields.Add(new SettingsFieldPropertiesListSettings("ProgressPerParticipantCurve", "Anima Linking Curve", 250, typeof(List<SettingsInstance>),
                    new SimpleCurve { { new CurvePoint(1f, 1f), true }, { new CurvePoint(2f, 1.2f), true }, { new CurvePoint(4f, 1.5f), true }, { new CurvePoint(6f, 2f), true }, { new CurvePoint(8f, 3f), true } }.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                        t => ((SimpleCurve)typeof(RitualStage_AnimaTreeLinking).GetField("ProgressPerParticipantCurve", BindingFlags.Static | BindingFlags.NonPublic | BindingFlags.Public).GetValue(null)).Points.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                        "curvePoint", () => new SettingsInstance("curvePoint", new CurvePoint(0f, 0f), null, null), true));

            QOLModSettingsDefCopy();
            props.fields.Add(new SettingsFieldPropertiesListSettings("DefCopies", "Def Copies", 300, typeof(List<SettingsInstance>), null,
                                     t => ((QOLModSettings)t).defCopies == null ? new List<SettingsInstance>() : ((QOLModSettings)t).defCopies.Select(tc => new SettingsInstance("QOLModSettingsDefCopy", tc, null, null)).ToList(),
                                     "QOLModSettingsDefCopy", () => new SettingsInstance("QOLModSettingsDefCopy", new QOLModSettingsDefCopy(), null, null), true));

            props.mergers.Add(setting => {
                QOLModSettings settings = QOLMod.getSettings();
                settings.stopSkillDecay = setting.get<bool>("StopSkillDecay");
                settings.stopSkillDown = setting.get<bool>("StopSkillDown");
                settings.stopTamenessDecay = setting.get<bool>("StopTamenessDecay");
                settings.preventAnimalFamilies = setting.get<bool>("PreventAnimalFamilies");
                settings.allowNamedAnimalFamilies = setting.get<bool>("AllowNamedAnimalFamilies");
                settings.useSyrchalistTraitvalue = setting.get<bool>("UseSyrchalistTraitvalue");
                settings.useFixedNumericTextfields = setting.get<bool>("UseFixedNumericTextfields");
                settings.cumBaseCost = setting.get<float>("CuMBaseCost");
                settings.cumCostIdeo = setting.get<float>("CumCostIdeo");
                settings.cumCostPassionMinor = setting.get<float>("CumCostPassionMinor");
                settings.cumCostPassionMajor = setting.get<float>("CumCostPassionMajor");
                settings.cumTax = setting.get<float>("CuMTax");
                settings.cumPointPawnToPoolConversionFactor = setting.get<float>("CuMConversionPawn");
                settings.cumPointRemnantsToPoolConversionFactor = setting.get<float>("CuMConversionRemnants");
                settings.remnantOrderPriceFactor = setting.get<float>("RemnantsOrderPriceFactor");
                settings.avoidDesignatorRefresh = setting.get<bool>("AvoidDesignatorRefresh");
                settings.reapplyOnGameLoad = setting.get<bool>("ReapplyOnGameLoad");
                settings.starvingThreshold = setting.get<float>("StarvingThreshold");
                settings.displayFoundCityGizmo = setting.get<bool>("DisplayFoundCityGizmo");
                settings.alwaysDisplayLabelInMBDD = setting.get<bool>("AlwaysDisplayLabelInMBDD");
                settings.skillDecayLevelOffset = setting.get<int>("SkillDecayLevelOffset");
                settings.stopRoofCollapse = setting.get<bool>("StopRoofCollapse");
                settings.tooltipMaxWidth = setting.get<float>("TooltipMaxWidth");
                settings.growLists = setting.get<bool>("GrowLists");
                settings.dateStyle = setting.get<DateStyleDef>("DateStyle");

                settings.pfColonist = setting.get<QOLPFDef>("ColonistPF") != null ? setting.get<QOLPFDef>("ColonistPF").defName : null;
                settings.pfAnimalTame = setting.get<QOLPFDef>("AnimalTamePF") != null ? setting.get<QOLPFDef>("AnimalTamePF").defName : null;
                settings.pfAnimalWild = setting.get<QOLPFDef>("AnimalWildPF") != null ? setting.get<QOLPFDef>("AnimalWildPF").defName : null;
                settings.pfOther = setting.get<QOLPFDef>("OtherPF") != null ? setting.get<QOLPFDef>("OtherPF").defName : null;
                settings.pfColonistConf = setting.get<string>("ColonistPFConf");
                settings.pfAnimalTameConf = setting.get<string>("AnimalTamePFConf");
                settings.pfAnimalWildConf = setting.get<string>("AnimalWildPFConf");
                settings.pfOtherConf = setting.get<string>("OtherPFConf");

                settings.pawnValueExpectationsMV0 = setting.get<float>("PawnValueExpectationsMV0");
                settings.pawnValueExpectationsMV1 = setting.get<float>("PawnValueExpectationsMV1");
                settings.pawnValueExpectationsMV2 = setting.get<float>("PawnValueExpectationsMV2");
                settings.pawnValueExpectationsMV3 = setting.get<float>("PawnValueExpectationsMV3");
                settings.pawnValueExpectationsMV4 = setting.get<float>("PawnValueExpectationsMV4");
                settings.pawnValueExpectationsMV5 = setting.get<float>("PawnValueExpectationsMV5");


                Dictionary<int,ReflectionMethodInvoker> pfAlgorithms = new Dictionary<int,ReflectionMethodInvoker>();
                AddPFAlgorithm(settings.pfColonist,settings.pfColonistConf,1,pfAlgorithms);
                AddPFAlgorithm(settings.pfAnimalTame,settings.pfAnimalTameConf,2,pfAlgorithms);
                AddPFAlgorithm(settings.pfAnimalWild,settings.pfAnimalWildConf,3,pfAlgorithms);
                AddPFAlgorithm(settings.pfOther,settings.pfOtherConf,4,pfAlgorithms);
                QOLMod.setPFAlgorithms(pfAlgorithms);
                	
                List<string> pfRestictionExcemptions = setting.get<List<string>>("PFRestrictionExcemption");
                if ( pfRestictionExcemptions != null ) {
                	if ( settings.pfRestrictionExcemptions == null ) {
                		settings.pfRestrictionExcemptions = new List<string>(pfRestictionExcemptions);
                	} else {
                		settings.pfRestrictionExcemptions.Clear();
                		settings.pfRestrictionExcemptions.AddRange(pfRestictionExcemptions);
                	}
                }
                	
                List<string> hiddenAlerts = setting.get<List<string>>("HiddenAlerts");
                if ( hiddenAlerts != null ) {
                	if ( settings.hiddenAlerts == null ) {
                		settings.hiddenAlerts = new HashSet<string>(hiddenAlerts);
                	} else {
                		settings.hiddenAlerts.Clear();
                		settings.hiddenAlerts.AddRange(hiddenAlerts);
                	}
                } else {
                	if ( settings.hiddenAlerts == null ) {
                		settings.hiddenAlerts = new HashSet<string>();
                	} else {
                		settings.hiddenAlerts.Clear();
                	}
                }

                List<string> hiddenLetters = setting.get<List<string>>("HiddenLetters");
                if (hiddenLetters != null)
                {
                    if (hiddenLetters.Count == 0)
                    {
                        settings.hiddenLetters = null;
                    }
                    else
                    {
                        if (settings.hiddenLetters == null) settings.hiddenLetters = new List<string>();
                        else settings.hiddenLetters.Clear();
                        settings.hiddenLetters.AddRange(hiddenLetters);
                    }
                }

                List<SettingsInstance> cumInflationCurve = setting.get<List<SettingsInstance>>("CuMInflationCurve");
                if (!cumInflationCurve.NullOrEmpty())
                {
                    if (settings.cumInflationCurve == null) settings.cumInflationCurve = new SimpleCurve();
                    SimpleCurve sc = settings.cumInflationCurve;
                    sc.Points.Clear();
                    sc.Points.AddRange(cumInflationCurve.Select(si => new CurvePoint(si.get<float>("X"), si.get<float>("Y"))));
                    sc.SortPoints();
                }
                else settings.cumInflationCurve = null;

                bool changesOccurred = false;


                if (settings.forbiddenPatchNamespaces == null) settings.forbiddenPatchNamespaces = new List<string>();
                else settings.forbiddenPatchNamespaces.Clear();
                List<string> forbiddenPatches = setting.get<List<string>>("ForbiddenPatches");
                if (forbiddenPatches != null )
                {
                    settings.forbiddenPatchNamespaces.AddRange(forbiddenPatches);
                }
                /*settings.forbiddenPatchNamespaces = setting.get<List<string>>("ForbiddenPatches");
                if ( settings.forbiddenPatchNamespaces == null ) settings.forbiddenPatchNamespaces = new List<string>();*/
                foreach ( KeyValuePair<string,HarmonyPatchNamespaceProperties> p in HarmonyPatchNamespace.namespaces ) {
                	if ( !p.Value.active && !settings.forbiddenPatchNamespaces.Contains(p.Key) ) {
                		p.Value.active = true;
                		changesOccurred = true;
                	}
                }
                foreach ( string id in settings.forbiddenPatchNamespaces ) {
                	HarmonyPatchNamespaceProperties nsprops;
                	if ( HarmonyPatchNamespace.namespaces.TryGetValue(id,out nsprops) ) {
                		if ( nsprops.active ) {
                			nsprops.active = false;
                			changesOccurred = true;
                		}
                	}
                }
                if ( changesOccurred ) {
                	if ( Find.WindowStack != null )
                		Find.WindowStack.Add(new Dialog_MessageBox("Changing the forbidden patches requires a restart to take effect.", null, null, null, null, "Restart required", true, null, null));
                }

                Pawn_PsychicEntropyTracker.FallRatePerPsyfocusBand[0] = setting.get<float>("FallRatePerPsyfocusBand0");
                Pawn_PsychicEntropyTracker.FallRatePerPsyfocusBand[1] = setting.get<float>("FallRatePerPsyfocusBand1");
                Pawn_PsychicEntropyTracker.FallRatePerPsyfocusBand[2] = setting.get<float>("FallRatePerPsyfocusBand2");
                //resetting tooltip to show the new values.
                Pawn_PsychicEntropyTracker.psyfocusLevelInfoCached = null;

                Slavery_Merge(settings, setting.get<SettingsInstance>("Slavery"));

                List<SettingsInstance> progressPerParticipantCurve = setting.get<List<SettingsInstance>>("ProgressPerParticipantCurve");
                if (progressPerParticipantCurve != null)
                {
                    FieldInfo fi = typeof(RitualStage_AnimaTreeLinking).GetField("ProgressPerParticipantCurve", BindingFlags.Static | BindingFlags.NonPublic | BindingFlags.Public);
                    SimpleCurve sc = (SimpleCurve)fi.GetValue(null);
                    sc.Points.Clear();
                    sc.Points.AddRange(progressPerParticipantCurve.Select(si => new CurvePoint(si.get<float>("X"), si.get<float>("Y"))));
                    sc.SortPoints();
                }

                List<SettingsInstance> defCopies = setting.get<List<SettingsInstance>>("DefCopies");
                if ( defCopies != null )
                {
                    if (settings.defCopies == null) settings.defCopies = new List<QOLModSettingsDefCopy>();
                    else settings.defCopies.Clear();
                    foreach (SettingsInstance si in defCopies)
                    {
                        QOLModSettingsDefCopy dc = new QOLModSettingsDefCopy();
                        dc.defType = si.get<string>("DefType");
                        dc.defNameOriginal = si.get<string>("DefNameOriginal");
                        dc.defNameCopy = si.get<string>("DefNameCopy");
                        dc.labelCopy = si.get<string>("LabelCopy");
                        if (dc.IsValid)
                        {
                            settings.defCopies.Add(dc);
                        }
                        else
                        {
                            Log.Warning("Ignoring invalid def copy entry. "+dc);
                        }
                    }
                }

            });
			
			props.visualization = new SettingsVisualizationMenuEdit<QOLModSettings>("General Settings", QOLMod.getSettings, "Change General Settings for QOL Mod");
            props.ResetToDefaults = true;
		}
		
		private static void AddPFAlgorithm(string defName, string conf, int algId, Dictionary<int,ReflectionMethodInvoker> pfAlgorithms) {
			if ( defName != null ) {
        		ReflectionMethodInvoker rmi = GetPFAlgorithm(defName,conf);
        		if ( rmi != null ) pfAlgorithms.Add(algId,rmi);
        	}
		}
		
		private static ReflectionMethodInvoker GetPFAlgorithm(string defName, string conf) {
			QOLPFDef def = DefDatabase<QOLPFDef>.GetNamed(defName,false);
    		return new ReflectionMethodInvoker(def.GetConstructor().Invoke(new object[]{conf}),def.GetFindPathMethod());
		}
		
		private static QOLTraitCostDef getDefinedOrDefaultQOLTraitCostDef(TraitDef trait, TraitDegreeData degree) {
			if ( trait == null || degree == null ) return null;
			QOLTraitCostDef def = DefDatabase<QOLTraitCostDef>.GetNamed("QOLTraitCost"+trait.defName+degree.degree,false);
			if ( def == null ) {
				//Log.Message("Trait "+trait.defName+"/"+degree+" is missing CuM cost definitions.");
				def = new QOLTraitCostDef(){defName="QOLTraitCost"+trait.defName+degree.degree,traitDef=trait,traitDegree=degree.degree,cost=2};
				def.ResolveReferences();
				DefDatabase<QOLTraitCostDef>.Add(def);
			}
			return def;
		}
		
		private static QOLTraitCostDef getDefinedOrDefaultQOLTraitCostDef(string key) {
			Regex regexpr = new Regex(@"QOLTraitCost(.*)(?<!-)([-]?[0123456789]+)");
			Match m = regexpr.Match(key);
			int degreeInt = int.Parse(m.Groups[2].Value);
			TraitDef trait = DefDatabase<TraitDef>.GetNamed(m.Groups[1].Value,false);
			TraitDegreeData degree = trait.DataAtDegree(degreeInt);
			return getDefinedOrDefaultQOLTraitCostDef(trait,degree);
		}
		
		private static List<QOLTraitCostDef> getDefinedOrDefaultQOLTraitCostDef() {
			foreach ( TraitDef trait in DefDatabase<TraitDef>.AllDefsListForReading ) {
				foreach ( TraitDegreeData degree in trait.degreeDatas ) {
					getDefinedOrDefaultQOLTraitCostDef(trait,degree);
				}
			}
			return DefDatabase<QOLTraitCostDef>.AllDefsListForReading;
		}
		
		private static void traitCosts() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("traitCosts");
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLTraitCostDef>("Cost","Cost",10,default(float),t=>t.cost));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLTraitCostDef>("RemovalCost","Removal Cost",20,default(float),t=>t.costRemoval));
			
			props.mergers.Add(setting => {
	        	QOLTraitCostDef traitCostDef = getDefinedOrDefaultQOLTraitCostDef(setting.getKey());
	        	traitCostDef.cost = setting.get<float>("Cost");
	        	traitCostDef.costRemoval = setting.get<float>("RemovalCost");
	        	});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<QOLTraitCostDef>("CuM Trait Costs", "Select Trait", getDefinedOrDefaultQOLTraitCostDef, 
				def => def.defName, 
				getTraitCostTooltip,//DefLabelProducerGeneric<QOLTraitCostDef>.LabelProducerNotNull,
				getDefinedOrDefaultQOLTraitCostDef,
				f => "Set Cost for Trait");//"Set Cost for Trait \"" + ((ISettingsInstance)f.flowScope["edit"]).getLabel() + "\"");
		}
		
		private static Dictionary<string,string> tcdDescriptions = new Dictionary<string, string>();
		
		private static string getTraitCostTooltip(QOLTraitCostDef tcd) {
			string desc;
			if ( !tcdDescriptions.TryGetValue(tcd.defName, out desc) ) {
				try {
					desc = (tcd.label.NullOrEmpty() ? tcd.defName : tcd.label) + "\n" + new Trait(tcd.traitDef,tcd.traitDegree,false).TipString(QOLMod.getDummyPawn());
				} catch (Exception ex) {
					Log.Error("Exception creating trait description!\n"+ex.Message+"\n"+ex.StackTrace);
					desc = "error "+ex.Message;
				}
				tcdDescriptions.Add(tcd.defName,desc);
			}
			return desc;
		}
		
		private static QOLRecordFactorDef getDefinedOrDefaultQOLRecordFactorDef(RecordDef record) {
			if ( record == null ) return null;
			QOLRecordFactorDef def = DefDatabase<QOLRecordFactorDef>.GetNamed("QOLRecordFactor"+record.defName,false);
			if ( def == null ) {
				def = new QOLRecordFactorDef(){defName="QOLRecordFactor"+record.defName,recordDef=record,factor=0};
				def.ResolveReferences();
				DefDatabase<QOLRecordFactorDef>.Add(def);
			}
			return def;
		}
		
		private static QOLRecordFactorDef getDefinedOrDefaultQOLRecordFactorDef(string key) {
			return getDefinedOrDefaultQOLRecordFactorDef(DefDatabase<RecordDef>.GetNamed(key.Substring("QOLRecordFactor".Length),false));
		}
		
		private static List<QOLRecordFactorDef> getDefinedOrDefaultQOLRecordFactorDef() {
			foreach ( RecordDef record in DefDatabase<RecordDef>.AllDefsListForReading ) {
				getDefinedOrDefaultQOLRecordFactorDef(record);
			}
			return DefDatabase<QOLRecordFactorDef>.AllDefsListForReading;
		}
		
		private static void recordFactors() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("recordFactors");
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLRecordFactorDef>("Factor","Factor",10,default(float),t=>t.factor));
			
			props.mergers.Add(setting => {
              	QOLRecordFactorDef recordFactorDef = getDefinedOrDefaultQOLRecordFactorDef(setting.getKey());
        		recordFactorDef.factor = setting.get<float>("Factor");
              	});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<QOLRecordFactorDef>("CuM Record Points", "Select Record", getDefinedOrDefaultQOLRecordFactorDef, 
                  def => def.defName, DefLabelProducerGeneric<QOLRecordFactorDef>.LabelProducerNotNull, getDefinedOrDefaultQOLRecordFactorDef,
				f=>"Set Point-Gain-Factor for record \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}

        private static void factions()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("factions");

            props.fields.Add(new SettingsFieldPropertiesPrimitiveString<FactionDef>("Label", "Label", 10, "", t => t.label) { EmptyIsNull = true });
            props.fields.Add(new SettingsFieldPropertiesSelectableIcon("FactionIconPath", "Icon", 20, null, o => ((FactionDef)o).factionIconPath,
                new string[] { "World/WorldObjects/Expanding", "World/WorldObjects/Expanding/Sites" }, null, true));

            props.mergers.Add(setting => {
                FactionDef faction = DefDatabase<FactionDef>.GetNamed(setting.getKey());
                faction.label = setting.get<string>("Label");
                if (SettingsFieldPropertiesSelectableIcon.UpdateIconpath(ref faction.factionIconPath, setting.get<KeyValuePair<string, Texture2D>?>("FactionIconPath")))
                {
                    //refreshing the cached icon
                    typeof(FactionDef).GetField("factionIcon", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public).SetValue(faction, null);
                }

                faction.ClearCachedData();
            });

            props.visualization = new SettingsVisualizationMenuSelectEdit<FactionDef>("Factions", "Select Faction", ()=> DefDatabase<FactionDef>.AllDefsListForReading,
                  def => def.defName, DefLabelProducerGeneric<FactionDef>.LabelProducerNotNull, s=>DefDatabase<FactionDef>.GetNamed(s),
                f => "Edit " + ((ISettingsInstance)f.flowScope["edit"]).getLabel());
        }

        private static void weaponTraits()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("weaponTraits");

            props.fields.Add(new SettingsFieldPropertiesPrimitiveString<WeaponTraitDef>("Label", "Label", 10, "", t => t.label) { EmptyIsNull = true });
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<WeaponTraitDef>("Commonality", "Commonality", 20, default(float), t => t.commonality));
            props.fields.Add(new SettingsFieldPropertiesListPrimitive("ExclusionTags", "Exclusion Tags", 70, typeof(List<string>), new List<string>(),
                                      t => ((WeaponTraitDef)t).exclusionTags != null ? ((WeaponTraitDef)t).exclusionTags.ToList() : new List<string>(),
                                          null, () => (object)"", true));

            props.mergers.Add(setting => {
                WeaponTraitDef wt = DefDatabase<WeaponTraitDef>.GetNamed(setting.getKey());
                wt.label = setting.get<string>("Label");
                wt.commonality = setting.get<float>("Commonality");
                List<string> exclusionTags = setting.get<List<string>>("ExclusionTags");
                if (exclusionTags != null)
                {
                    if (exclusionTags.Count == 0)
                    {
                        wt.exclusionTags = null;
                    }
                    else
                    {
                        if (wt.exclusionTags == null) wt.exclusionTags = new List<string>();
                        else wt.exclusionTags.Clear();
                        wt.exclusionTags.AddRange(exclusionTags);
                    }
                }

                wt.ClearCachedData();
            });

            props.visualization = new SettingsVisualizationMenuSelectEdit<WeaponTraitDef>("Weapon Traits", "Select Weapon Traits", () => DefDatabase<WeaponTraitDef>.AllDefsListForReading,
                  def => def.defName, DefLabelProducerGeneric<WeaponTraitDef>.LabelProducerNotNull, s => DefDatabase<WeaponTraitDef>.GetNamed(s),
                f => "Edit " + ((ISettingsInstance)f.flowScope["edit"]).getLabel());
        }

        private static void worldObjects()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("worldObject");

            props.fields.Add(new SettingsFieldPropertiesPrimitiveString<WorldObjectDef>("Label", "Label", 10, null, t => t.label) { EmptyIsNull = true });
            props.fields.Add(new SettingsFieldPropertiesSelectableIcon("Texture", "Icon", 20, null, o => ((WorldObjectDef)o).texture,
                new string[] { "World/WorldObjects/Expanding", "World/WorldObjects/Expanding/Sites" }, null, true));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<WorldObjectDef>("ExpandingIcon", "Use Expanding Icon", 30, default(bool), t => t.expandingIcon));
            props.fields.Add(new SettingsFieldPropertiesSelectableIcon("ExpandingIconTexture", "Expanding Icon", 40, null, o => ((WorldObjectDef)o).expandingIconTexture,
                new string[] { "World/WorldObjects/Expanding", "World/WorldObjects/Expanding/Sites" }, null, true));

            props.mergers.Add(setting => {
                WorldObjectDef wo = DefDatabase<WorldObjectDef>.GetNamed(setting.getKey());
                wo.label = setting.get<string>("Label");
                wo.expandingIcon = setting.get<bool>("ExpandingIcon");
                if (SettingsFieldPropertiesSelectableIcon.UpdateIconpath(ref wo.texture, setting.get<KeyValuePair<string, Texture2D>?>("Texture")))
                {
                    //refreshing the cached icon
                    typeof(WorldObjectDef).GetField("material", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public).SetValue(wo, null);
                    if (!wo.useDynamicDrawer) Find.World?.renderer?.Notify_StaticWorldObjectPosChanged();
                }
                if (SettingsFieldPropertiesSelectableIcon.UpdateIconpath(ref wo.expandingIconTexture, setting.get<KeyValuePair<string, Texture2D>?>("ExpandingIconTexture")))
                {
                    //refreshing the cached icon
                    typeof(WorldObjectDef).GetField("expandingIconTextureInt", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public).SetValue(wo, null);
                    if (!wo.useDynamicDrawer) Find.World?.renderer?.Notify_StaticWorldObjectPosChanged();
                }
                wo.ClearCachedData();
            });

            props.visualization = new SettingsVisualizationMenuSelectEdit<WorldObjectDef>("World Objects", "Select World Object", () => DefDatabase<WorldObjectDef>.AllDefsListForReading,
                  def => def.defName, DefLabelProducerGeneric<WorldObjectDef>.LabelProducerNotNull, s => DefDatabase<WorldObjectDef>.GetNamed(s),
                f => "Edit " + ((ISettingsInstance)f.flowScope["edit"]).getLabel());
        }

        private static void powerBuildings() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("powerBuildings");
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("BasePower","Base Power",10,default(float),t=>t.GetCompProperties<CompProperties_Power>().basePowerConsumption));
			
			props.mergers.Add(setting => {
              	ThingDef thing = DefDatabase<ThingDef>.GetNamed(setting.getKey());
				CompProperties_Power cpp = thing.GetCompProperties<CompProperties_Power>();
				float oldBasePowerConsumption = cpp.basePowerConsumption;
				cpp.basePowerConsumption = setting.get<float>("BasePower");
				if ( Current.Game != null ) {
					HarmonyPatchNamespaceProperties hpnsp;
					HarmonyPatchNamespace.namespaces.TryGetValue("PowerTraderZero", out hpnsp);
					foreach ( Map m in Find.Maps ) {
						m.listerThings.ThingsOfDef(thing).ForEach(t=>{CompPowerTrader pt = t.TryGetComp<CompPowerTrader>(); if ( pt != null ) {
                          		if ( hpnsp != null && hpnsp.active ) {
                              		if (Math.Abs(oldBasePowerConsumption) < float.Epsilon && Math.Abs(pt.Props.basePowerConsumption) > float.Epsilon ) pt.ResetPowerVars();
                          			if (Math.Abs(pt.Props.basePowerConsumption) < float.Epsilon && Math.Abs(oldBasePowerConsumption) > float.Epsilon ) pt.PowerOn = true;
                          		}
                          		pt.SetUpPowerVars();
                          	}});
					}
				}});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ThingDef>("Power Buildings","Select Power Building",()=>DefDatabase<ThingDef>.AllDefsListForReading.Where(d=>d.GetCompProperties<CompProperties_Power>() != null).ToList(),
               def => def.defName, DefLabelProducerGeneric<ThingDef>.LabelProducerNotNull,key => DefDatabase<ThingDef>.GetNamed(key),
               f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}

        private static void mainButtons() {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("mainButtons");
            props.props.Add(new SettingsPropertyProperties("IsDD", o => o is MainButtonDef_Dropdown));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveString<MainButtonDef>("Label", "Label", 5, "", t => t.label));

            props.fields.Add(new SettingsFieldPropertiesSelectableIcon("IconPath", "Icon", 6, null, o=> ((MainButtonDef)o).iconPath, 
                new string[] { "ui/Buttons/MainButtons", "UI/Buttons/MainButtons/", "GD/UI/Icons/MainButton" }, null, true));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveText<MainButtonDef>("Description","Description",7,"",t=>t.description));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<MainButtonDef>("Visible","Visible",10,default(bool),t=>t.buttonVisible));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<MainButtonDef>("Minimized", "Half-Size", 15, default(bool), t => t.minimized));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<MainButtonDef>("Order", "Order", 20, default(int), t => t.order) {
                labelTooltipProducer = (flow) => {
                    ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                    SettingsProperties p = SettingsStorage.getSettingsProperties(setting.getId());
                    ISettingsVisualizationSelection vps = ((ISettingsVisualizationSelection)props.visualization);
                    List<ISettingsInstance> sis = SettingsStorage.generateListForSelection(p);
                    Dictionary<string, ISettingsInstance> map = null;
                    StringBuilder sb = new StringBuilder();
                    sis.Sort((a, b) => a.get<int>("Order").CompareTo(b.get<int>("Order")));
                    foreach (ISettingsInstance si in sis)
                    {
                        if (!si.get<bool>("Visible")) continue;

                        if ( si == setting )
                        {
                            sb.Append("<color=#FFD800>");
                        }
                        sb.Append(si.getLabel()).Append(" : ").Append(si.get<int>("Order"));
                        List<MainButtonDef> btns = si.get<List<MainButtonDef>>("Buttons");
                        if (!btns.NullOrEmpty())
                        {
                            if ( map == null ) map = sis.ToDictionary(s => s.getKey());
                            sb.Append(" [");
                            for ( int j = 0; j < btns.Count; j++ )
                            {
                                ISettingsInstance btnReferenced = null;
                                if (j != 0) sb.Append(", ");
                                if (btns[j] != null) map.TryGetValue(vps.getKeyFromBaseObject(btns[j]), out btnReferenced);
                                if (btnReferenced != null)
                                    sb.Append(btnReferenced.getLabel());
                                else
                                    sb.Append("None".Translate());
                            }
                            sb.Append("]");
                        }
                        if (si == setting)
                        {
                            sb.Append("</color>");
                        }
                        sb.AppendLine();
                    }
                    bool atLeastOneInvisibleFound = false;
                    foreach (ISettingsInstance si in sis)
                    {
                        if (si.get<bool>("Visible")) continue;
                        if ( !atLeastOneInvisibleFound )
                        {
                            sb.Append("RIMMSQoL.Invisible".Translate()).Append(": [");
                            atLeastOneInvisibleFound = true;
                        }
                        else
                        {
                            sb.Append(", ");
                        }
                        sb.Append(si.getLabel());
                    }
                    if (atLeastOneInvisibleFound)
                    {
                        sb.Append("]").AppendLine();
                    }
                    return sb.ToString();
                }
            });
         	props.fields.Add(new SettingsFieldPropertiesListSelectable("Buttons","Buttons",40,typeof(List<MainButtonDef>),new List<MainButtonDef>(),
			                                                           t=> { MainButtonDef_Dropdown dd = t as MainButtonDef_Dropdown; return dd != null ? dd.buttons : new List<MainButtonDef>(); },
			                                                           f=>DefDatabase<MainButtonDef>.AllDefsListForReading.Where(btn=>!(btn is MainButtonDef_Dropdown)).Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<MainButtonDef>>("Buttons")[listIndex] = (MainButtonDef)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<MainButtonDef>.AllDefsListForReading.First(),
			                                                          true).setHidden(f=>!((ISettingsInstance)f.flowScope["edit"]).prop<bool>("IsDD")));
			
			props.mergers.Add(setting => {
              	MainButtonDef btn = DefDatabase<MainButtonDef>.GetNamed(setting.getKey());

              	setting.resolveProps(props,btn);
				btn.buttonVisible = setting.get<bool>("Visible");
                btn.minimized = setting.get<bool>("Minimized");
                btn.order = setting.get<int>("Order");
                if ( SettingsFieldPropertiesSelectableIcon.UpdateIconpath(ref btn.iconPath, setting.get<KeyValuePair<string, Texture2D>?>("IconPath")) )
                {
                    //refreshing the cached icon
                    typeof(MainButtonDef).GetField("icon", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public).SetValue(btn, null);
                }

				string configuredLabel = setting.get<string>("Label");
				if ( !configuredLabel.NullOrEmpty() && configuredLabel.Trim().Length != 0 ) {
					btn.label = configuredLabel;
                    DefCachedLabelCapSet(btn, null);
                }
				string configuredDescription = setting.get<string>("Description");
				if ( !configuredDescription.NullOrEmpty() && configuredDescription.Trim().Length != 0 ) {
					btn.description = configuredDescription;
				}
				MainButtonDef_Dropdown btnDD = btn as MainButtonDef_Dropdown;
				if ( btnDD != null ) {
					List<MainButtonDef> buttons = setting.get<List<MainButtonDef>>("Buttons");
					if ( buttons != null ) {
      					if ( btnDD.buttons == null ) btnDD.buttons = new List<MainButtonDef>();
	      				btnDD.buttons.Clear();
	      				btnDD.buttons.AddRange(buttons);
      				}
				}
				
				btn.ClearCachedData();
				});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<MainButtonDef>("Main Buttons","Select Main Button",()=>DefDatabase<MainButtonDef>.AllDefsListForReading,
               def => def.defName, DefLabelProducerGeneric<MainButtonDef>.LabelProducerNotNull,key => DefDatabase<MainButtonDef>.GetNamed(key),
               f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}
		
		private static void architectButtons() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("architectButtons");
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DesignationCategoryDef>("Visible","Visible",10,default(bool),t=> {
                 	QOLDefModExtension_DesignationCategoryDef modExtension = t.GetModExtension<QOLDefModExtension_DesignationCategoryDef>();
                 	if ( modExtension != null ) return modExtension.visible;
                 	return true;
                 }));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveString<DesignationCategoryDef>("Label", "Label", 15, "", t => t.label));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<DesignationCategoryDef>("Order","Order",20,default(int),t=>t.order)
            {
                labelTooltipProducer = (flow) => {
                    ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                    List<ISettingsInstance> sis = SettingsStorage.generateListForSelection(SettingsStorage.getSettingsProperties(setting.getId()));
                    StringBuilder sb = new StringBuilder();
                    foreach (ISettingsInstance si in sis.OrderBy(si => si.get<int>("Order")))
                    {
                        if (si == setting)
                        {
                            sb.Append("<color=#FFD800>");
                        }
                        sb.Append(si.get<bool>("Visible") ? "+ " : "- ").Append(si.getLabel()).Append(" : ").Append(si.get<int>("Order")).AppendLine();
                        if (si == setting)
                        {
                            sb.Append("</color>");
                        }
                    }
                    return sb.ToString();
                }
            });
			props.fields.Add(new SettingsFieldPropertiesListSelectable("SpecialDesignatorClasses","Designators",30,typeof(List<Type>),new List<Type>(),
			                                                          dc=>((DesignationCategoryDef)dc).specialDesignatorClasses.ToList(),
			                                                          f=>typeof(Designator).AllSubclassesNonAbstract().Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<Type>>("SpecialDesignatorClasses")[listIndex] = (Type)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          o=>o!=null?((Type)o).Name:"Undefined",()=>typeof(Designator).AllSubclassesNonAbstract().First(),
			                                                          true));
			
			props.mergers.Add(setting => {
              	DesignationCategoryDef btn = null;
              	btn = DefDatabase<DesignationCategoryDef>.GetNamed(setting.getKey());
            					
      			QOLDefModExtension_DesignationCategoryDef modExtension = btn.GetModExtension<QOLDefModExtension_DesignationCategoryDef>();
      			if ( modExtension == null ) {
      				if ( btn.modExtensions == null ) btn.modExtensions = new List<DefModExtension>();
      				modExtension = new QOLDefModExtension_DesignationCategoryDef();
      				btn.modExtensions.Add(modExtension);
      			}
  				modExtension.visible = setting.get<bool>("Visible");
      			btn.order = setting.get<int>("Order");
                btn.label = setting.get<string>("Label");
                List<Type> specialDesignatorClasses = setting.get<List<Type>>("SpecialDesignatorClasses");
      			if ( specialDesignatorClasses != null ) {
      				if ( btn.specialDesignatorClasses == null ) btn.specialDesignatorClasses = new List<Type>();
      				btn.specialDesignatorClasses.Clear();
      				btn.specialDesignatorClasses.AddRange(specialDesignatorClasses);
      				if ( !QOLMod.getSettings().avoidDesignatorRefresh || applyCounter == 1 ) {
	      				MethodInfo mi = typeof(DesignationCategoryDef).GetMethod("ResolveDesignators", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
	      				mi.Invoke(btn, null);
      				}
      			}

                btn.ClearCachedData();
                });
			
			QOLMod.addApplySettingsListener(mod=> {
                	if ( MainButtonDefOf.Architect.TabWindow != null ) 
                		MainButtonDefOf.Architect.tabWindowClass.GetMethod("CacheDesPanels", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic).Invoke(MainButtonDefOf.Architect.TabWindow, null);
                	
                	MainButtonDef mbMintArchitect = DefDatabase<MainButtonDef>.GetNamed("MintMenus",false);
                	if ( mbMintArchitect != null ) {
                		Type mainTabWindow_MintArchitect = DefSettingsField.GetTypeFromAnyAssemblyVersion("DubsMintMenus.MainTabWindow_MintArchitect, DubsMintMenus, Version=1.2.7123.42234, Culture=neutral, PublicKeyToken=null");
	                	if ( mainTabWindow_MintArchitect != null ) {
	                		mainTabWindow_MintArchitect.GetField("desPanelsCached",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public).SetValue(mbMintArchitect.TabWindow,null);
	                	}
                	}
                });
			
			props.validators.Add(setting => {
                 	DesignationCategoryDef dc = DefDatabase<DesignationCategoryDef>.GetNamed(setting.getKey());
                 	if ( dc != null && setting.get<List<Type>>("SpecialDesignatorClasses") == null ) {
                 		List<Type> types = setting.get<List<Type>>("SpecialDesignatorClasses");
                 		setting.set<List<Type>>("SpecialDesignatorClasses",dc.specialDesignatorClasses.ToList<Type>());
                 	}
                 	return dc != null;
                 });
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<DesignationCategoryDef>("Architect Buttons","Select Architect Button",()=>DefDatabase<DesignationCategoryDef>.AllDefsListForReading,
               def => def.defName, DefLabelProducerGeneric<DesignationCategoryDef>.LabelProducerNotNull,key => DefDatabase<DesignationCategoryDef>.GetNamed(key, false),
               f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}
		
		private static void ThingDefCountClass() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("thingDefCountClass");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("Thing","Thing",10,typeof(ThingDef),default(ThingDef),t=>((ThingDefCountClass)t).thingDef,
				                                             f=>from object d in DefDatabase<ThingDef>.AllDefsListForReading where ((ThingDef)d).category == ThingCategory.Item select d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("Thing",(ThingDef)def); return true; },
				                                             SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDefCountClass>("Count","Count",20,default(int),t=>t.count));

				props.dynamicLabel = settings => settings.get<int>("Count")+"x"+(settings.get<ThingDef>("Thing")!=null ? settings.get<ThingDef>("Thing").label : "ERROR");
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void SkillRequirement() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("skillRequirement");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("Skill","Skill",10,typeof(SkillDef),default(SkillDef),t=>((SkillRequirement)t).skill,
				                                             f=>from object d in DefDatabase<SkillDef>.AllDefsListForReading select d,
				                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<SkillDef>("Skill",(SkillDef)def); return true; },
				                                           SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<SkillRequirement>("MinLevel","Min Level",20,default(int),t=>t.minLevel));

				props.dynamicLabel = settings => settings.get<SkillDef>("Skill").label+" lvl "+settings.get<int>("MinLevel");
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void IngredientCount() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("ingredientCount");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<IngredientCount>("Count", "count", 10, 1f, t=>t.GetBaseCount()));
				ThingFilter();
				props.fields.Add(new SettingsFieldPropertiesSettings("Filter","filter",20,typeof(SettingsInstance),null,t=>new SettingsInstance("thingFilter",((IngredientCount)t).filter,null,null),"thingFilter"));
				
				props.dynamicLabel = settings => settings.get<float>("Count") + "x " + settings.get<ISettingsInstance>("Filter").getLabel();
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		public static class ThingFilterAdapter {
			public static Func<ThingFilter,float> ThingFilterDisallowCheaperThanGet = 
				UtilReflection.CreateGetter<ThingFilter,float>(typeof(ThingFilter).GetField("disallowCheaperThan", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,float> ThingFilterDisallowCheaperThanSet = 
				UtilReflection.CreateSetter<ThingFilter,float>(typeof(ThingFilter).GetField("disallowCheaperThan", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,QualityRange> ThingFilterAllowedQualitiesGet = 
				UtilReflection.CreateGetter<ThingFilter,QualityRange>(typeof(ThingFilter).GetField("allowedQualities", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,QualityRange> ThingFilterAllowedQualitiesSet = 
				UtilReflection.CreateSetter<ThingFilter,QualityRange>(typeof(ThingFilter).GetField("allowedQualities", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,FoodPreferability> ThingFilterDisallowWorsePreferabilityGet = 
				UtilReflection.CreateGetter<ThingFilter,FoodPreferability>(typeof(ThingFilter).GetField("disallowWorsePreferability", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,FoodPreferability> ThingFilterDisallowWorsePreferabilitySet = 
				UtilReflection.CreateSetter<ThingFilter,FoodPreferability>(typeof(ThingFilter).GetField("disallowWorsePreferability", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,bool> ThingFilterDisallowInedibleByHumanGet = 
				UtilReflection.CreateGetter<ThingFilter,bool>(typeof(ThingFilter).GetField("disallowInedibleByHuman", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,bool> ThingFilterDisallowInedibleByHumanSet = 
				UtilReflection.CreateSetter<ThingFilter,bool>(typeof(ThingFilter).GetField("disallowInedibleByHuman", UtilReflection.BindingInstanceFlags));
            public static Func<ThingFilter, bool> ThingFilterDisallowNotEverStorableGet =
                UtilReflection.CreateGetter<ThingFilter, bool>(typeof(ThingFilter).GetField("disallowNotEverStorable", UtilReflection.BindingInstanceFlags));
            public static Action<ThingFilter, bool> ThingFilterDisallowNotEverStorableSet =
                UtilReflection.CreateSetter<ThingFilter, bool>(typeof(ThingFilter).GetField("disallowNotEverStorable", UtilReflection.BindingInstanceFlags));
            public static Func<ThingFilter,Type> ThingFilterAllowWithCompGet = 
				UtilReflection.CreateGetter<ThingFilter,Type>(typeof(ThingFilter).GetField("allowWithComp", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,Type> ThingFilterAllowWithCompSet = 
				UtilReflection.CreateSetter<ThingFilter,Type>(typeof(ThingFilter).GetField("allowWithComp", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,Type> ThingFilterDisallowWithCompGet = 
				UtilReflection.CreateGetter<ThingFilter,Type>(typeof(ThingFilter).GetField("disallowWithComp", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,Type> ThingFilterDisallowWithCompSet = 
				UtilReflection.CreateSetter<ThingFilter,Type>(typeof(ThingFilter).GetField("disallowWithComp", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,FloatRange> ThingFilterAllowedHitPointsPercentsGet = 
				UtilReflection.CreateGetter<ThingFilter,FloatRange>(typeof(ThingFilter).GetField("allowedHitPointsPercents", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,FloatRange> ThingFilterAllowedHitPointsPercentsSet = 
				UtilReflection.CreateSetter<ThingFilter,FloatRange>(typeof(ThingFilter).GetField("allowedHitPointsPercents", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,List<StuffCategoryDef>> ThingFilterStuffCategoriesToAllowGet = 
				UtilReflection.CreateGetter<ThingFilter,List<StuffCategoryDef>>(typeof(ThingFilter).GetField("stuffCategoriesToAllow", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,List<StuffCategoryDef>> ThingFilterStuffCategoriesToAllowSet = 
				UtilReflection.CreateSetter<ThingFilter,List<StuffCategoryDef>>(typeof(ThingFilter).GetField("stuffCategoriesToAllow", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,List<ThingDef>> ThingFilterAllowAllWhoCanMakeGet = 
				UtilReflection.CreateGetter<ThingFilter,List<ThingDef>>(typeof(ThingFilter).GetField("allowAllWhoCanMake", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,List<ThingDef>> ThingFilterAllowAllWhoCanMakeSet = 
				UtilReflection.CreateSetter<ThingFilter,List<ThingDef>>(typeof(ThingFilter).GetField("allowAllWhoCanMake", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,List<ThingDef>> ThingFilterDisallowedThingDefsGet = 
				UtilReflection.CreateGetter<ThingFilter,List<ThingDef>>(typeof(ThingFilter).GetField("disallowedThingDefs", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,List<ThingDef>> ThingFilterDisallowedThingDefsSet = 
				UtilReflection.CreateSetter<ThingFilter,List<ThingDef>>(typeof(ThingFilter).GetField("disallowedThingDefs", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,List<ThingDef>> ThingFilterThingDefsGet = 
				UtilReflection.CreateGetter<ThingFilter,List<ThingDef>>(typeof(ThingFilter).GetField("thingDefs", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,List<ThingDef>> ThingFilterThingDefsSet = 
				UtilReflection.CreateSetter<ThingFilter,List<ThingDef>>(typeof(ThingFilter).GetField("thingDefs", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,List<string>> ThingFilterCategoriesGet = 
				UtilReflection.CreateGetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("categories", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,List<string>> ThingFilterCategoriesSet = 
				UtilReflection.CreateSetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("categories", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,List<string>> ThingFilterDisallowedCategoriesGet = 
				UtilReflection.CreateGetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("disallowedCategories", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,List<string>> ThingFilterDisallowedCategoriesSet = 
				UtilReflection.CreateSetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("disallowedCategories", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,List<string>> ThingFilterSpecialFiltersToAllowGet = 
				UtilReflection.CreateGetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("specialFiltersToAllow", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,List<string>> ThingFilterSpecialFiltersToAllowSet = 
				UtilReflection.CreateSetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("specialFiltersToAllow", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,List<string>> ThingFilterSpecialFiltersToDisallowGet = 
				UtilReflection.CreateGetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("specialFiltersToDisallow", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,List<string>> ThingFilterSpecialFiltersToDisallowSet = 
				UtilReflection.CreateSetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("specialFiltersToDisallow", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,List<string>> ThingFilterTradeTagsToAllowGet = 
				UtilReflection.CreateGetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("tradeTagsToAllow", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,List<string>> ThingFilterTradeTagsToAllowSet = 
				UtilReflection.CreateSetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("tradeTagsToAllow", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,List<string>> ThingFilterTradeTagsToDisallowGet = 
				UtilReflection.CreateGetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("tradeTagsToDisallow", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,List<string>> ThingFilterTradeTagsToDisallowSet = 
				UtilReflection.CreateSetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("tradeTagsToDisallow", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,List<string>> ThingFilterThingSetMakerTagsToAllowGet = 
				UtilReflection.CreateGetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("thingSetMakerTagsToAllow", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,List<string>> ThingFilterThingSetMakerTagsToAllowSet = 
				UtilReflection.CreateSetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("thingSetMakerTagsToAllow", UtilReflection.BindingInstanceFlags));
			public static Func<ThingFilter,List<string>> ThingFilterThingSetMakerTagsToDisallowGet = 
				UtilReflection.CreateGetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("thingSetMakerTagsToDisallow", UtilReflection.BindingInstanceFlags));
			public static Action<ThingFilter,List<string>> ThingFilterThingSetMakerTagsToDisallowSet = 
				UtilReflection.CreateSetter<ThingFilter,List<string>>(typeof(ThingFilter).GetField("thingSetMakerTagsToDisallow", UtilReflection.BindingInstanceFlags));
			
			public static ThingFilter AsThingFilter(ISettingsInstance settings) {
				if ( settings == null ) return null;
				if ( settings.getId() != "thingFilter" ) {
					Log.Error("Invalid conversion to ThingFilter. Settings must be of id thingFilter but were "+settings.getId()+"!");
					return null;
				}
				ThingFilter tf = new ThingFilter();
				tf.allowedHitPointsConfigurable = settings.get<bool>("AllowedHitPointsConfigurable");
				tf.customSummary = settings.get<string>("CustomSummary");
				//if smallest possible input then convert to smallest possible float since it is checked against as equal in vanilla code
				float dct = settings.get<float>("DisallowCheaperThan");
				if (Math.Abs(dct - float.Parse(float.MinValue.ToString())) < float.Epsilon) {
					dct = float.MinValue;
				}
				ThingFilterDisallowCheaperThanSet(tf,settings.get<float>("DisallowCheaperThan"));
				ThingFilterAllowedQualitiesSet(tf,new QualityRange((QualityCategory)Enum.Parse(typeof(QualityCategory),settings.get<string>("AllowedQualitiesMin")),
				                                                   (QualityCategory)Enum.Parse(typeof(QualityCategory),settings.get<string>("AllowedQualitiesMax"))));
				ThingFilterDisallowWorsePreferabilitySet(tf,(FoodPreferability)Enum.Parse(typeof(FoodPreferability),settings.get<string>("DisallowWorsePreferability")));
				ThingFilterDisallowInedibleByHumanSet(tf,settings.get<bool>("DisallowInedibleByHuman"));
                ThingFilterDisallowNotEverStorableSet(tf, settings.get<bool>("DisallowNotEverStorable"));
                ThingFilterAllowWithCompSet(tf,settings.get<Type>("AllowWithComp"));
				ThingFilterDisallowWithCompSet(tf,settings.get<Type>("DisallowWithComp"));
				ThingFilterAllowedHitPointsPercentsSet(tf,new FloatRange(settings.get<float>("AllowedHitPointsPercentsMin"),settings.get<float>("AllowedHitPointsPercentsMax")));
				ThingFilterStuffCategoriesToAllowSet(tf,settings.get<List<StuffCategoryDef>>("StuffCategoriesToAllow"));
				ThingFilterAllowAllWhoCanMakeSet(tf,settings.get<List<ThingDef>>("AllowAllWhoCanMake"));
				ThingFilterDisallowedThingDefsSet(tf,settings.get<List<ThingDef>>("DisallowedThingDefs"));
				ThingFilterThingDefsSet(tf,settings.get<List<ThingDef>>("ThingDefs"));
				ThingFilterCategoriesSet(tf,settings.get<List<ThingCategoryDef>>("Categories") != null ? (from tcd in settings.get<List<ThingCategoryDef>>("Categories") select tcd.defName).ToList() : null);
				ThingFilterDisallowedCategoriesSet(tf,settings.get<List<ThingCategoryDef>>("DisallowedCategories") != null ? (from tcd in settings.get<List<ThingCategoryDef>>("DisallowedCategories") select tcd.defName).ToList() : null);
				ThingFilterSpecialFiltersToAllowSet(tf,settings.get<List<SpecialThingFilterDef>>("SpecialFiltersToAllow") != null ? (from tcd in settings.get<List<SpecialThingFilterDef>>("SpecialFiltersToAllow") select tcd.defName).ToList() : null);
				ThingFilterSpecialFiltersToDisallowSet(tf,settings.get<List<SpecialThingFilterDef>>("SpecialFiltersToDisallow") != null ? (from tcd in settings.get<List<SpecialThingFilterDef>>("SpecialFiltersToDisallow") select tcd.defName).ToList() : null);
				ThingFilterTradeTagsToAllowSet(tf,settings.get<List<string>>("TradeTagsToAllow"));
				ThingFilterTradeTagsToDisallowSet(tf,settings.get<List<string>>("TradeTagsToDisallow"));
				ThingFilterThingSetMakerTagsToAllowSet(tf,settings.get<List<string>>("ThingSetMakerTagsToAllow"));
				ThingFilterThingSetMakerTagsToDisallowSet(tf,settings.get<List<string>>("ThingSetMakerTagsToDisallow"));
				
				//Ideally it would be avoided to call ResolveReferences but all code has been packed into this loading event. 
				//Maybe just copying the content to prevent any unwanted mod interactions.
				tf.ResolveReferences();
				
				return tf;
			}
		}
		
		private static void ThingFilter() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("thingFilter");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveString<ThingFilter>("CustomSummary", "customSummary", 10, null, t => t.customSummary) { EmptyIsNull = true });
				props.fields.Add(new SettingsFieldPropertiesSelectable("AllowedQualitiesMin","allowedQualitiesMin",320,typeof(string),Enum.GetName(typeof(QualityCategory),QualityCategory.Awful),
		                                                       t=>Enum.GetName(typeof(QualityCategory),ThingFilterAdapter.ThingFilterAllowedQualitiesGet((ThingFilter)t).min),
                                                       		f=>from QualityCategory d in Enum.GetValues(typeof(QualityCategory)) orderby Enum.GetName(typeof(QualityCategory),d) select (object)Enum.GetName(typeof(QualityCategory),d),
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("AllowedQualitiesMin",(string)def); return true; },
				                                             o=>(string)o,false));
				props.fields.Add(new SettingsFieldPropertiesSelectable("AllowedQualitiesMax","allowedQualitiesMax",330,typeof(string),Enum.GetName(typeof(QualityCategory),QualityCategory.Legendary),
		                                                       t=>Enum.GetName(typeof(QualityCategory),ThingFilterAdapter.ThingFilterAllowedQualitiesGet((ThingFilter)t).max),
	                                                       f=>from QualityCategory d in Enum.GetValues(typeof(QualityCategory)) orderby Enum.GetName(typeof(QualityCategory),d) select (object)Enum.GetName(typeof(QualityCategory),d),
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("AllowedQualitiesMax",(string)def); return true; },
				                                             o=>(string)o,false));
				props.fields.Add(new SettingsFieldPropertiesSelectable("DisallowWorsePreferability","disallowWorsePreferability",160,typeof(string),Enum.GetName(typeof(FoodPreferability),FoodPreferability.Undefined),
				                                                       t=>Enum.GetName(typeof(FoodPreferability),ThingFilterAdapter.ThingFilterDisallowWorsePreferabilityGet((ThingFilter)t)),
	                                                       f=>from FoodPreferability d in Enum.GetValues(typeof(FoodPreferability)) orderby Enum.GetName(typeof(FoodPreferability),d) select (object)Enum.GetName(typeof(FoodPreferability),d),
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("DisallowWorsePreferability",(string)def); return true; },
				                                             o=>(string)o,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingFilter>("DisallowInedibleByHuman","disallowInedibleByHuman",170,default(bool),ThingFilterAdapter.ThingFilterDisallowInedibleByHumanGet));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingFilter>("DisallowNotEverStorable", "disallowNotEverStorable", 171, default(bool), ThingFilterAdapter.ThingFilterDisallowNotEverStorableGet));
                props.fields.Add(new SettingsFieldPropertiesSelectable("AllowWithComp","allowWithComp",180,typeof(Type),null,
		                                                       t=>ThingFilterAdapter.ThingFilterAllowWithCompGet((ThingFilter)t),
	                                                       f=>from Type d in typeof(ThingComp).AllSubclassesNonAbstract() orderby d.Name select (object)d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<Type>("AllowWithComp",(Type)def); return true; },
				                                             o=>o != null ? ((Type)o).Name : "NONE",false));
				props.fields.Add(new SettingsFieldPropertiesSelectable("DisallowWithComp","disallowWithComp",190,typeof(Type),null,
		                                                       t=>ThingFilterAdapter.ThingFilterDisallowWithCompGet((ThingFilter)t),
	                                                       f=>from Type d in typeof(ThingComp).AllSubclassesNonAbstract() orderby d.Name select (object)d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<Type>("DisallowWithComp",(Type)def); return true; },
				                                             o=>o != null ? ((Type)o).Name : "NONE",false));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingFilter>("DisallowCheaperThan", "disallowCheaperThan", 200, default(float), t => float.Parse(ThingFilterAdapter.ThingFilterDisallowCheaperThanGet(t).ToString())));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingFilter>("AllowedHitPointsPercentsMin", "allowedHitPointsPercentsMin", 300, 0f, t=>ThingFilterAdapter.ThingFilterAllowedHitPointsPercentsGet(t).min));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingFilter>("AllowedHitPointsPercentsMax", "allowedHitPointsPercentsMax", 310, 1f, t=>ThingFilterAdapter.ThingFilterAllowedHitPointsPercentsGet(t).max));
				props.fields.Add(new SettingsFieldPropertiesListSelectable("StuffCategoriesToAllow","stuffCategoriesToAllow",140,typeof(List<StuffCategoryDef>),new List<StuffCategoryDef>(),
				                                                           t=>ThingFilterAdapter.ThingFilterStuffCategoriesToAllowGet((ThingFilter)t) != null ? ThingFilterAdapter.ThingFilterStuffCategoriesToAllowGet((ThingFilter)t).ToList() : null,
			                                                          f=>DefDatabase<StuffCategoryDef>.AllDefsListForReading.Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<StuffCategoryDef>>("StuffCategoriesToAllow")[listIndex] = (StuffCategoryDef)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<StuffCategoryDef>.AllDefsListForReading.First(),
			                                                          true));
				props.fields.Add(new SettingsFieldPropertiesListSelectable("AllowAllWhoCanMake","allowAllWhoCanMake",150,typeof(List<ThingDef>),new List<ThingDef>(),
				                                                           t=>ThingFilterAdapter.ThingFilterAllowAllWhoCanMakeGet((ThingFilter)t) != null ? ThingFilterAdapter.ThingFilterAllowAllWhoCanMakeGet((ThingFilter)t).ToList() : null,
			                                                          f=>DefDatabase<ThingDef>.AllDefsListForReading.Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<ThingDef>>("AllowAllWhoCanMake")[listIndex] = (ThingDef)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<ThingDef>.AllDefsListForReading.First(),
			                                                          true));
				props.fields.Add(new SettingsFieldPropertiesListSelectable("DisallowedThingDefs","disallowedThingDefs",210,typeof(List<ThingDef>),new List<ThingDef>(),
				                                                           t=>ThingFilterAdapter.ThingFilterDisallowedThingDefsGet((ThingFilter)t) != null ? ThingFilterAdapter.ThingFilterDisallowedThingDefsGet((ThingFilter)t).ToList() : null,
			                                                          f=>DefDatabase<ThingDef>.AllDefsListForReading.Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<ThingDef>>("DisallowedThingDefs")[listIndex] = (ThingDef)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<ThingDef>.AllDefsListForReading.First(),
			                                                          true));
				props.fields.Add(new SettingsFieldPropertiesListSelectable("ThingDefs","thingDefs",50,typeof(List<ThingDef>),new List<ThingDef>(),
				                                                           t=>ThingFilterAdapter.ThingFilterThingDefsGet((ThingFilter)t) != null ? ThingFilterAdapter.ThingFilterThingDefsGet((ThingFilter)t).ToList() : null,
			                                                          f=>DefDatabase<ThingDef>.AllDefsListForReading.Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<ThingDef>>("ThingDefs")[listIndex] = (ThingDef)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<ThingDef>.AllDefsListForReading.First(),
			                                                          true));
				props.fields.Add(new SettingsFieldPropertiesListSelectable("Categories","categories",60,typeof(List<ThingCategoryDef>),new List<ThingCategoryDef>(),
				                                                           t=>ThingFilterAdapter.ThingFilterCategoriesGet((ThingFilter)t) != null ? (from string tfs in ThingFilterAdapter.ThingFilterCategoriesGet((ThingFilter)t) let tf = tfs != null ? DefDatabase<ThingCategoryDef>.GetNamed(tfs, false) : null where tf != null select tf).ToList() : null,
			                                                          f=>DefDatabase<ThingCategoryDef>.AllDefsListForReading.Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<ThingCategoryDef>>("Categories")[listIndex] = (ThingCategoryDef)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<ThingCategoryDef>.AllDefsListForReading.First(),
			                                                          true));
				props.fields.Add(new SettingsFieldPropertiesListSelectable("DisallowedCategories","disallowedCategories",110,typeof(List<ThingCategoryDef>),new List<ThingCategoryDef>(),
				                                                           t=>ThingFilterAdapter.ThingFilterDisallowedCategoriesGet((ThingFilter)t) != null ? (from string tfs in ThingFilterAdapter.ThingFilterDisallowedCategoriesGet((ThingFilter)t) let tf = tfs != null ? DefDatabase<ThingCategoryDef>.GetNamed(tfs, false) : null where tf != null select tf).ToList() : null,
			                                                          f=>DefDatabase<ThingCategoryDef>.AllDefsListForReading.Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<ThingCategoryDef>>("DisallowedCategories")[listIndex] = (ThingCategoryDef)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<ThingCategoryDef>.AllDefsListForReading.First(),
			                                                          true));
				props.fields.Add(new SettingsFieldPropertiesListSelectable("SpecialFiltersToAllow","specialFiltersToAllow",120,typeof(List<SpecialThingFilterDef>),new List<SpecialThingFilterDef>(),
				                                                           t=>ThingFilterAdapter.ThingFilterSpecialFiltersToAllowGet((ThingFilter)t) != null ? (from string tfs in ThingFilterAdapter.ThingFilterSpecialFiltersToAllowGet((ThingFilter)t) let tf = tfs != null ? DefDatabase<SpecialThingFilterDef>.GetNamed(tfs, false) : null where tf != null select tf).ToList() : null,
				                                                           f=>DefDatabase<SpecialThingFilterDef>.AllDefsListForReading.Where(stfd=>stfd.configurable).Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<SpecialThingFilterDef>>("SpecialFiltersToAllow")[listIndex] = (SpecialThingFilterDef)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<SpecialThingFilterDef>.AllDefsListForReading.First(),
			                                                          true));
				props.fields.Add(new SettingsFieldPropertiesListSelectable("SpecialFiltersToDisallow","specialFiltersToDisallow",130,typeof(List<SpecialThingFilterDef>),new List<SpecialThingFilterDef>(),
				                                                           t=>ThingFilterAdapter.ThingFilterSpecialFiltersToDisallowGet((ThingFilter)t) != null ? (from string tfs in ThingFilterAdapter.ThingFilterSpecialFiltersToDisallowGet((ThingFilter)t) let tf = tfs != null ? DefDatabase<SpecialThingFilterDef>.GetNamed(tfs, false) : null where tf != null select tf).ToList() : null,
			                                                          f=>DefDatabase<SpecialThingFilterDef>.AllDefsListForReading.Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<SpecialThingFilterDef>>("SpecialFiltersToDisallow")[listIndex] = (SpecialThingFilterDef)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<SpecialThingFilterDef>.AllDefsListForReading.First(),
			                                                          true));
				props.fields.Add(new SettingsFieldPropertiesListPrimitive("TradeTagsToAllow","tradeTagsToAllow",70,typeof(List<string>),null,
				                      t=> ThingFilterAdapter.ThingFilterTradeTagsToAllowGet((ThingFilter)t) != null ? ThingFilterAdapter.ThingFilterTradeTagsToAllowGet((ThingFilter)t).ToList() : null,
                              			null,()=>(object)"",true));
				props.fields.Add(new SettingsFieldPropertiesListPrimitive("TradeTagsToDisallow","tradeTagsToDisallow",80,typeof(List<string>),null,
				                      t=> ThingFilterAdapter.ThingFilterTradeTagsToDisallowGet((ThingFilter)t) != null ? ThingFilterAdapter.ThingFilterTradeTagsToDisallowGet((ThingFilter)t).ToList() : null,
                              			null,()=>(object)"",true));
				props.fields.Add(new SettingsFieldPropertiesListPrimitive("ThingSetMakerTagsToAllow","thingSetMakerTagsToAllow",90,typeof(List<string>),null,
				                      t=> ThingFilterAdapter.ThingFilterThingSetMakerTagsToAllowGet((ThingFilter)t) != null ? ThingFilterAdapter.ThingFilterThingSetMakerTagsToAllowGet((ThingFilter)t).ToList() : null,
                              			null,()=>(object)"",true));
				props.fields.Add(new SettingsFieldPropertiesListPrimitive("ThingSetMakerTagsToDisallow","thingSetMakerTagsToDisallow",100,typeof(List<string>),null,
				                      t=> ThingFilterAdapter.ThingFilterThingSetMakerTagsToDisallowGet((ThingFilter)t) != null ? ThingFilterAdapter.ThingFilterThingSetMakerTagsToDisallowGet((ThingFilter)t).ToList() : null,
                              			null,()=>(object)"",true));
				
				
				props.dynamicLabel = settings => {
					ThingFilter tf = ThingFilterAdapter.AsThingFilter(settings);
					if ( tf != null ) return tf.Summary;
					else return "NONE";
				};
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static Func<RecipeDef,Type> RecipeDefIngredientValueGetterClassGet = UtilReflection.CreateGetter<RecipeDef,Type>(typeof(RecipeDef).GetField("ingredientValueGetterClass", UtilReflection.BindingInstanceFlags));
		private static Action<RecipeDef,Type> RecipeDefIngredientValueGetterClassSet = UtilReflection.CreateSetter<RecipeDef,Type>(typeof(RecipeDef).GetField("ingredientValueGetterClass", UtilReflection.BindingInstanceFlags));
		private static Action<RecipeDef,List<ThingDef>> RecipeDefPremultipliedSmallIngredientsSet = UtilReflection.CreateSetter<RecipeDef,List<ThingDef>>(typeof(RecipeDef).GetField("premultipliedSmallIngredients", UtilReflection.BindingInstanceFlags));
		private static Action<RecipeDef,IngredientValueGetter> RecipeDefIngredientValueGetterIntSet = UtilReflection.CreateSetter<RecipeDef,IngredientValueGetter>(typeof(RecipeDef).GetField("ingredientValueGetterInt", UtilReflection.BindingInstanceFlags));
		private static Action<RecipeDef,RecipeWorkerCounter> RecipeDefWorkerCounterIntSet = UtilReflection.CreateSetter<RecipeDef,RecipeWorkerCounter>(typeof(RecipeDef).GetField("workerCounterInt", UtilReflection.BindingInstanceFlags));
		private static Action<RecipeDef,RecipeWorker> RecipeDefWorkerIntSet = UtilReflection.CreateSetter<RecipeDef,RecipeWorker>(typeof(RecipeDef).GetField("workerInt", UtilReflection.BindingInstanceFlags));
		private static Action<ThingDef,List<RecipeDef>> ThingDefAllRecipesCachedSet = UtilReflection.CreateSetter<ThingDef,List<RecipeDef>>(typeof(ThingDef).GetField("allRecipesCached", UtilReflection.BindingInstanceFlags));
        private static Action<RecipeDef, bool?> RecipeDefIsSurgeryCachedSet = UtilReflection.CreateSetter<RecipeDef, bool?>(typeof(RecipeDef).GetField("isSurgeryCached", UtilReflection.BindingInstanceFlags));
        private static Action<Def, TaggedString> DefCachedLabelCapSet = UtilReflection.CreateSetter<Def, TaggedString>(typeof(Def).GetField("cachedLabelCap", UtilReflection.BindingInstanceFlags));
        private static Action<TraitDegreeData, string> TraitDegreeDataCachedLabelCapSet = UtilReflection.CreateSetter<TraitDegreeData, string>(typeof(TraitDegreeData).GetField("cachedLabelCap", UtilReflection.BindingInstanceFlags));
        private static Action<TraitDegreeData, string> TraitDegreeDataCachedLabelMaleCapSet = UtilReflection.CreateSetter<TraitDegreeData, string>(typeof(TraitDegreeData).GetField("cachedLabelMaleCap", UtilReflection.BindingInstanceFlags));
        private static Action<TraitDegreeData, string> TraitDegreeDataCachedLabelFemaleCapSet = UtilReflection.CreateSetter<TraitDegreeData, string>(typeof(TraitDegreeData).GetField("cachedLabelFemaleCap", UtilReflection.BindingInstanceFlags));

        private static void recipes() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("recipes");
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<RecipeDef>("Version","version",0,2,t=>2).setHidden(f=>true));
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveString<Def>("Label","Label",5,null,t=>t.label));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RecipeDef>("WorkAmount","Work Amount",10,default(float),t=>t.workAmount));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RecipeDef>("LearnFactor","Learn Factor",20,default(float),t=>t.workSkillLearnFactor));
			props.fields.Add(new SettingsFieldPropertiesSelectable("WorkSkill","Work Skill",30,typeof(SkillDef),default(SkillDef),t=>((RecipeDef)t).workSkill,
			                                             f=>from object d in DefDatabase<SkillDef>.AllDefsListForReading select d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<SkillDef>("WorkSkill",(SkillDef)def); return true; },
			                                            SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("WorkSpeedStat","Work Speed",40,typeof(StatDef),default(StatDef),t=>((RecipeDef)t).workSpeedStat,
			                                             f=>from StatDef d in DefDatabase<StatDef>.AllDefsListForReading where d.category == StatCategoryDefOf.PawnWork select (object)d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<StatDef>("WorkSpeedStat",(StatDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			ThingDefCountClass();
			props.fields.Add(new SettingsFieldPropertiesListSettings("Products","Products",50,typeof(List<SettingsInstance>),null,
                                     t=> ((RecipeDef)t).products == null ? new List<SettingsInstance>() : ((RecipeDef)t).products.Select(tc=>new SettingsInstance("thingDefCountClass",tc,null,null)).ToList(),
                                     "thingDefCountClass", ()=>new SettingsInstance("thingDefCountClass",new ThingDefCountClass(DefDatabase<ThingDef>.AllDefsListForReading.Find(d=>d.category == ThingCategory.Item),1),null,null),true));
			SkillRequirement();
			props.fields.Add(new SettingsFieldPropertiesListSettings("SkillRequirements","Skill Req.",60,typeof(List<SettingsInstance>),null,
                                 t=> ((RecipeDef)t).skillRequirements == null ? new List<SettingsInstance>() : ((RecipeDef)t).skillRequirements.Select(tc=>new SettingsInstance("skillRequirement",tc,null,null)).ToList(),
                                 "skillRequirement",()=>new SettingsInstance("skillRequirement",new SkillRequirement() { skill = DefDatabase<SkillDef>.AllDefsListForReading[0], minLevel = 1 },null,null),true));
			/*props.fields.Add(new SettingsFieldPropertiesListPrimitive("Ingredients","Ingredients",70,typeof(List<float>),null,
                                      t=> ((RecipeDef)t).ingredients == null ? new List<float>() : ((RecipeDef)t).ingredients.Select(tc=>tc.GetBaseCount()).ToList(),
                              			(settings,f)=>{
					                 		RecipeDef def = DefDatabase<RecipeDef>.GetNamed(settings.getKey(),false);
					                 		int indx = ((IterationItem)f.pageScope["curItem"]).index;
					                 		if ( def != null && def.ingredients != null && def.ingredients.Count > indx ) return def.ingredients[indx].filter.Summary; return "undefined";
                                          },null,false));*/

			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RecipeDef>("AllowMixingIngredients","allowMixingIngredients",100,default(bool),t=>t.allowMixingIngredients));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RecipeDef>("AutoStripCorpses","autoStripCorpses",110,true,t=>t.autoStripCorpses));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RecipeDef>("InterruptIfIngredientIsRotting","Rotting Ingredient Forbidden",115,default(bool),t=>t.interruptIfIngredientIsRotting));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RecipeDef>("ProductHasIngredientStuff","productHasIngredientStuff",120,true,t=>t.productHasIngredientStuff));
			//multiplier for + and minus buttons in the bill configuration
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<RecipeDef>("TargetCountAdjustment","targetCountAdjustment",130,1,t=>t.targetCountAdjustment));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RecipeDef>("HideBodyPartNames","hideBodyPartNames",140,default(bool),t=>t.hideBodyPartNames));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RecipeDef>("IsViolation","isViolation",150,default(bool),t=>t.isViolation));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RecipeDef>("SurgerySuccessChanceFactor","surgerySuccessChanceFactor",160,1f,t=>t.surgerySuccessChanceFactor));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RecipeDef>("DeathOnFailedSurgeryChance","deathOnFailedSurgeryChance",170,default(float),t=>t.deathOnFailedSurgeryChance));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RecipeDef>("TargetsBodyPart","targetsBodyPart",180,true,t=>t.targetsBodyPart));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RecipeDef>("Anesthetize","anesthetize",190,true,t=>t.anesthetize));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RecipeDef>("DontShowIfAnyIngredientMissing","dontShowIfAnyIngredientMissing",200,default(bool),t=>t.dontShowIfAnyIngredientMissing));
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("EfficiencyStat","efficiencyStat",210,typeof(StatDef),default(StatDef),t=>((RecipeDef)t).efficiencyStat,
			                                             f=>from StatDef d in DefDatabase<StatDef>.AllDefsListForReading where d.category == StatCategoryDefOf.PawnWork select (object)d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<StatDef>("EfficiencyStat",(StatDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("WorkTableEfficiencyStat","workTableEfficiencyStat",220,typeof(StatDef),default(StatDef),t=>((RecipeDef)t).workTableEfficiencyStat,
			                                             f=>from StatDef d in DefDatabase<StatDef>.AllDefsListForReading where d.category == StatCategoryDefOf.Building select (object)d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<StatDef>("WorkTableEfficiencyStat",(StatDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("WorkTableSpeedStat","workTableSpeedStat",230,typeof(StatDef),default(StatDef),t=>((RecipeDef)t).workTableSpeedStat,
			                                             f=>from StatDef d in DefDatabase<StatDef>.AllDefsListForReading where d.category == StatCategoryDefOf.Building select (object)d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<StatDef>("WorkTableSpeedStat",(StatDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("RequiredGiverWorkType","requiredGiverWorkType",240,typeof(WorkTypeDef),default(WorkTypeDef),t=>((RecipeDef)t).requiredGiverWorkType,
			                                             f=>from WorkTypeDef d in DefDatabase<WorkTypeDef>.AllDefsListForReading select (object)d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<WorkTypeDef>("RequiredGiverWorkType",(WorkTypeDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("UnfinishedThingDef","unfinishedThingDef",250,typeof(ThingDef),default(ThingDef),t=>((RecipeDef)t).unfinishedThingDef,
			                                                       f=>from ThingDef d in DefDatabase<ThingDef>.AllDefsListForReading where typeof(UnfinishedThing).IsAssignableFrom(d.thingClass) select (object)d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("UnfinishedThingDef",(ThingDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("EffectWorking","effectWorking",260,typeof(EffecterDef),default(EffecterDef),t=>((RecipeDef)t).effectWorking,
			                                             f=>from EffecterDef d in DefDatabase<EffecterDef>.AllDefsListForReading select (object)d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<EffecterDef>("EffectWorking",(EffecterDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("SoundWorking","soundWorking",270,typeof(SoundDef),default(SoundDef),t=>((RecipeDef)t).soundWorking,
			                                             f=>from SoundDef d in DefDatabase<SoundDef>.AllDefsListForReading select (object)d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<SoundDef>("SoundWorking",(SoundDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("AddsHediff","addsHediff",280,typeof(HediffDef),default(HediffDef),t=>((RecipeDef)t).addsHediff,
			                                             f=>from HediffDef d in DefDatabase<HediffDef>.AllDefsListForReading select (object)d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<HediffDef>("AddsHediff",(HediffDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("RemovesHediff","removesHediff",290,typeof(HediffDef),default(HediffDef),t=>((RecipeDef)t).removesHediff,
			                                             f=>from HediffDef d in DefDatabase<HediffDef>.AllDefsListForReading select (object)d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<HediffDef>("RemovesHediff",(HediffDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("ResearchPrerequisite","researchPrerequisite",300,typeof(ResearchProjectDef),default(ResearchProjectDef),t=>((RecipeDef)t).researchPrerequisite,
			                                             f=>from ResearchProjectDef d in DefDatabase<ResearchProjectDef>.AllDefsListForReading select (object)d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ResearchProjectDef>("ResearchPrerequisite",(ResearchProjectDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("WorkerClass","workerClass",310,typeof(Type),typeof(RecipeWorker),t=>((RecipeDef)t).workerClass,
			                                                       f=>new []{typeof(RecipeWorker)}.Concat(typeof(RecipeWorker).AllSubclassesNonAbstract()).Cast<object>(),
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<Type>("WorkerClass",(Type)def); return true; },
			                                            o=>o == null ? "None" : ((Type)o).Name,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("WorkerCounterClass","workerCounterClass",320,typeof(Type),typeof(RecipeWorkerCounter),t=>((RecipeDef)t).workerCounterClass,
			                                                       f=>new []{typeof(RecipeWorkerCounter)}.Concat(typeof(RecipeWorkerCounter).AllSubclassesNonAbstract()).Cast<object>(),
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<Type>("WorkerCounterClass",(Type)def); return true; },
			                                            o=>o == null ? "None" : ((Type)o).Name,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("IngredientValueGetterClass","ingredientValueGetterClass",330,typeof(Type),typeof(IngredientValueGetter_Volume),t=>RecipeDefIngredientValueGetterClassGet((RecipeDef)t),
			                                           f=>typeof(IngredientValueGetter).AllSubclassesNonAbstract().Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<Type>("IngredientValueGetterClass",(Type)def); return true; },
			                                           o=>o == null ? "None" : ((Type)o).Name,true));
			
			props.fields.Add(new SettingsFieldPropertiesListSelectable("ForceHiddenSpecialFilters","forceHiddenSpecialFilters",340,typeof(List<SpecialThingFilterDef>),new List<SpecialThingFilterDef>(),
			                                          t=>((RecipeDef)t).forceHiddenSpecialFilters != null ? ((RecipeDef)t).forceHiddenSpecialFilters.ToList<SpecialThingFilterDef>() : new List<SpecialThingFilterDef>(),
                                                      f=>DefDatabase<SpecialThingFilterDef>.AllDefsListForReading.Cast<object>(),
                                                      flow=>{
                                                      	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                                                      	return listEntry=>{
                                                       	flow.addPostRenderCallback(fl=>{setting.get<List<SpecialThingFilterDef>>("ForceHiddenSpecialFilters")[listIndex] = (SpecialThingFilterDef)listEntry; setting.markAsChanged();});
                                                      	return true;
                                                   		};},
                                                      SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<SpecialThingFilterDef>.AllDefsListForReading.First(),
                                                      true));
			
			props.fields.Add(new SettingsFieldPropertiesListSelectable("SpecialProducts","specialProducts",350,typeof(List<string>),new List<string>(),
			        t=> ((RecipeDef)t).specialProducts != null ? (from SpecialProductType spt in ((RecipeDef)t).specialProducts select Enum.GetName(typeof(SpecialProductType),spt)).ToList() : new List<string>(),
			        f => Enum.GetNames(typeof(SpecialProductType)).Cast<object>(),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
							flow.addPostRenderCallback(fl=>{setting.get<List<string>>("SpecialProducts")[listIndex] = (string)listEntry; setting.markAsChanged();});
							return true;
						};},
					entry => (string)entry,
					()=>Enum.GetName(typeof(SpecialProductType),SpecialProductType.Butchery),
					true));
			
			props.fields.Add(new SettingsFieldPropertiesListSelectable("RecipeUsers","recipeUsers",360,typeof(List<ThingDef>),new List<ThingDef>(),
			                                                           t=>((RecipeDef)t).recipeUsers != null ? ((RecipeDef)t).recipeUsers.ToList<ThingDef>() : new List<ThingDef>(),
                                                      f=>DefDatabase<ThingDef>.AllDefsListForReading.Cast<object>(),
                                                      flow=>{
                                                      	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                                                      	return listEntry=>{
                                                       	flow.addPostRenderCallback(fl=>{setting.get<List<ThingDef>>("RecipeUsers")[listIndex] = (ThingDef)listEntry; setting.markAsChanged();});
                                                      	return true;
                                                   		};},
                                                      SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<ThingDef>.AllDefsListForReading.First(),
                                                      true));
			
			props.fields.Add(new SettingsFieldPropertiesListSelectable("AppliedOnFixedBodyParts","appliedOnFixedBodyParts",370,typeof(List<BodyPartDef>),new List<BodyPartDef>(),
			                                                           t=>((RecipeDef)t).appliedOnFixedBodyParts != null ? ((RecipeDef)t).appliedOnFixedBodyParts.ToList<BodyPartDef>() : null,
                                                      f=>DefDatabase<BodyPartDef>.AllDefsListForReading.Cast<object>(),
                                                      flow=>{
                                                      	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                                                      	return listEntry=>{
                                                       	flow.addPostRenderCallback(fl=>{setting.get<List<BodyPartDef>>("AppliedOnFixedBodyParts")[listIndex] = (BodyPartDef)listEntry; setting.markAsChanged();});
                                                      	return true;
                                                   		};},
                                                      SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<BodyPartDef>.AllDefsListForReading.First(),
                                                      true));
			
			
			props.fields.Add(new SettingsFieldPropertiesListPrimitive("FactionPrerequisiteTags","factionPrerequisiteTags",380,typeof(List<string>),new List<string>(),
                                              t=> ((RecipeDef)t).factionPrerequisiteTags != null ? ((RecipeDef)t).factionPrerequisiteTags.ToList() : new List<string>(),null,()=>"",true));
			
			IngredientCount();
			
			//The filter in tc must be customized in case it is a recipe maker recipe. Then the configuration is not set only the cache.
			props.fields.Add(new SettingsFieldPropertiesListSettings("Ingredients2","ingredients",390,typeof(List<SettingsInstance>),null,
                        t=> ((RecipeDef)t).ingredients.NullOrEmpty() ? new List<SettingsInstance>() : ((RecipeDef)t).ingredients.Where(tc => tc != null).Select(tc=>new SettingsInstance("ingredientCount",tc,null,null)).ToList(),
                        "ingredientCount",()=>new SettingsInstance("ingredientCount",new IngredientCount(),null,null),true));
			
			ThingFilter();
			
			props.fields.Add(new SettingsFieldPropertiesSettings("FixedIngredientFilter","fixedIngredientFilter",400,typeof(SettingsInstance),new SettingsInstance("thingFilter",new ThingFilter(),null,null),
			                                                     t=>new SettingsInstance("thingFilter",((RecipeDef)t).fixedIngredientFilter,null,null),"thingFilter"));
			props.fields.Add(new SettingsFieldPropertiesSettings("DefaultIngredientFilter","defaultIngredientFilter",410,typeof(SettingsInstance),null,
			                                                     t=>new SettingsInstance("thingFilter",((RecipeDef)t).defaultIngredientFilter,null,null),"thingFilter"));
			
			//props.fields.Add(new SettingsFieldPropertiesPrimitiveString<RecipeDef>("SuccessfullyRemovedHediffMessage","successfullyRemovedHediffMessage",310,null,t=>t.successfullyRemovedHediffMessage));
			
			
			/*
				--- skipped
				//we don't want to change what the recipe is in nature only change it's configuration
				public ConceptDef conceptLearned;
				
				--- Caches must be nulled after merging
				private List<ThingDef> premultipliedSmallIngredients;
				private IngredientValueGetter ingredientValueGetterInt;
				private RecipeWorkerCounter workerCounterInt;
				private RecipeWorker workerInt;
			 */
			
			props.mergers.Add(setting => {
              	RecipeDef def = DefDatabase<RecipeDef>.GetNamed(setting.getKey());
              	int version = setting.get<int>("Version");
              	
              	string configuredLabel = setting.get<string>("Label");
				if ( !configuredLabel.NullOrEmpty() && configuredLabel.Trim().Length != 0 ) {
					def.label = configuredLabel;
                    DefCachedLabelCapSet(def, null);
                }
         		def.workAmount = setting.get<float>("WorkAmount");
         		def.workSkill = setting.get<SkillDef>("WorkSkill");
         		def.workSkillLearnFactor = setting.get<float>("LearnFactor");
         		def.workSpeedStat = setting.get<StatDef>("WorkSpeedStat");
				
				if ( def.products != null ) def.products.Clear();
         		List<SettingsInstance> products = setting.get<List<SettingsInstance>>("Products");
				if ( !products.NullOrEmpty() ) {
					if ( def.products == null ) def.products = new List<ThingDefCountClass>(products.Count);
					foreach ( ISettingsInstance e in products ) {
						if ( e.get<ThingDef>("Thing") != null ) {
							def.products.Add(new ThingDefCountClass(e.get<ThingDef>("Thing"),e.get<int>("Count")));
						}
					}
				}
         		
				if ( def.skillRequirements != null ) def.skillRequirements.Clear();
				List<SettingsInstance> skillRequirements = setting.get<List<SettingsInstance>>("SkillRequirements");
				if ( !skillRequirements.NullOrEmpty() ) {
					if ( def.skillRequirements == null ) def.skillRequirements = new List<SkillRequirement>(skillRequirements.Count);
					foreach ( ISettingsInstance e in skillRequirements ) {
						if ( e.get<SkillDef>("Skill") != null ) {
							SkillRequirement skrq = new SkillRequirement();
							skrq.skill = e.get<SkillDef>("Skill");
							skrq.minLevel = e.get<int>("MinLevel");
							def.skillRequirements.Add(skrq);
						}
					}
				}
				
				/*List<float> ingredients = setting.get<List<float>>("Ingredients");
				if ( !ingredients.NullOrEmpty() && !def.ingredients.NullOrEmpty() ) {
					for ( int i = 0; i < def.ingredients.Count && i < ingredients.Count; i++ ) {
						def.ingredients[i].SetBaseCount(ingredients[i]);
					}
				}*/
				
				if ( version > 1 ) {
					def.allowMixingIngredients = setting.get<bool>("AllowMixingIngredients");
					def.autoStripCorpses = setting.get<bool>("AutoStripCorpses");
					def.interruptIfIngredientIsRotting = setting.get<bool>("InterruptIfIngredientIsRotting");
					def.productHasIngredientStuff = setting.get<bool>("ProductHasIngredientStuff");
					def.targetCountAdjustment = setting.get<int>("TargetCountAdjustment");
					def.hideBodyPartNames = setting.get<bool>("HideBodyPartNames");
					def.isViolation = setting.get<bool>("IsViolation");
					def.surgerySuccessChanceFactor = setting.get<float>("SurgerySuccessChanceFactor");
					def.deathOnFailedSurgeryChance = setting.get<float>("DeathOnFailedSurgeryChance");
					def.targetsBodyPart = setting.get<bool>("TargetsBodyPart");
					def.anesthetize = setting.get<bool>("Anesthetize");
					def.dontShowIfAnyIngredientMissing = setting.get<bool>("DontShowIfAnyIngredientMissing");
					def.efficiencyStat = setting.get<StatDef>("EfficiencyStat");
					def.workTableEfficiencyStat = setting.get<StatDef>("WorkTableEfficiencyStat");
					def.workTableSpeedStat = setting.get<StatDef>("WorkTableSpeedStat");
					def.requiredGiverWorkType = setting.get<WorkTypeDef>("RequiredGiverWorkType");
					def.unfinishedThingDef = setting.get<ThingDef>("UnfinishedThingDef");
					def.effectWorking = setting.get<EffecterDef>("EffectWorking");
					def.soundWorking = setting.get<SoundDef>("SoundWorking");
					def.addsHediff = setting.get<HediffDef>("AddsHediff");
					def.removesHediff = setting.get<HediffDef>("RemovesHediff");
					def.researchPrerequisite = setting.get<ResearchProjectDef>("ResearchPrerequisite");
					def.workerClass = setting.get<Type>("WorkerClass");
					def.workerCounterClass = setting.get<Type>("WorkerCounterClass");
					RecipeDefIngredientValueGetterClassSet(def, setting.get<Type>("IngredientValueGetterClass"));
					
					List<SpecialThingFilterDef> forceHiddenSpecialFilters = setting.get<List<SpecialThingFilterDef>>("ForceHiddenSpecialFilters");
	            	if ( forceHiddenSpecialFilters != null ) {
						if ( forceHiddenSpecialFilters.Count == 0 ) {
							def.forceHiddenSpecialFilters = null;
						} else {
		            		if ( def.forceHiddenSpecialFilters == null ) def.forceHiddenSpecialFilters = new List<SpecialThingFilterDef>();
		            		else def.forceHiddenSpecialFilters.Clear();
		            		foreach ( SpecialThingFilterDef stf in forceHiddenSpecialFilters ) {
		            			def.forceHiddenSpecialFilters.Add(stf);
		            		}
						}
	            	}
					List<string> specialProducts = setting.get<List<string>>("SpecialProducts");
	            	if ( specialProducts != null ) {
						if ( specialProducts.Count == 0 ) {
							def.specialProducts = null;
						} else {
		            		if ( def.specialProducts == null ) def.specialProducts = new List<SpecialProductType>();
		            		else def.specialProducts.Clear();
		            		foreach ( string s in specialProducts ) {
								def.specialProducts.Add((SpecialProductType)Enum.Parse(typeof(SpecialProductType), s));
		            		}
						}
	            	}
					List<ThingDef> oldRecipeUsers = def.recipeUsers != null ? new List<ThingDef>(def.recipeUsers) : new List<ThingDef>();
					List<ThingDef> recipeUsers = setting.get<List<ThingDef>>("RecipeUsers");
	            	if ( recipeUsers != null ) {
						if ( recipeUsers.Count == 0 ) {
							def.recipeUsers = null;
						} else {
		            		if ( def.recipeUsers == null ) def.recipeUsers = new List<ThingDef>();
		            		else def.recipeUsers.Clear();
		            		foreach ( ThingDef td in recipeUsers ) {
		            			def.recipeUsers.Add(td);
		            		}
						}
	            	}
					List<BodyPartDef> appliedOnFixedBodyParts = setting.get<List<BodyPartDef>>("AppliedOnFixedBodyParts");
	            	if ( appliedOnFixedBodyParts != null ) {
	            		if ( def.appliedOnFixedBodyParts == null ) def.appliedOnFixedBodyParts = new List<BodyPartDef>();
	            		def.appliedOnFixedBodyParts.Clear();
	            		foreach ( BodyPartDef bpd in appliedOnFixedBodyParts ) {
	            			def.appliedOnFixedBodyParts.Add(bpd);
	            		}
	            	}
					List<string> factionPrerequisiteTags = setting.get<List<string>>("FactionPrerequisiteTags");
	            	if ( factionPrerequisiteTags != null ) {
						if ( factionPrerequisiteTags.Count == 0 ) {
							def.factionPrerequisiteTags = null;
						} else {
		            		if ( def.factionPrerequisiteTags == null ) def.factionPrerequisiteTags = new List<string>();
		            		else def.factionPrerequisiteTags.Clear();
		            		foreach ( string s in factionPrerequisiteTags ) {
		            			def.factionPrerequisiteTags.Add(s);
		            		}
						}
	            	}
					if ( def.fixedIngredientFilter == null )
						def.fixedIngredientFilter = ThingFilterAdapter.AsThingFilter(setting.get<SettingsInstance>("FixedIngredientFilter"));
					else {
						ThingFilter tf = ThingFilterAdapter.AsThingFilter(setting.get<SettingsInstance>("FixedIngredientFilter"));
						if ( tf != null ) def.fixedIngredientFilter = tf;//.CopyAllowancesFrom(tf);
						else def.fixedIngredientFilter = null;
					}
					if ( def.defaultIngredientFilter == null )
						def.defaultIngredientFilter = ThingFilterAdapter.AsThingFilter(setting.get<SettingsInstance>("DefaultIngredientFilter"));
					else {
						ThingFilter tf = ThingFilterAdapter.AsThingFilter(setting.get<SettingsInstance>("DefaultIngredientFilter"));
						if ( tf != null ) {
							def.defaultIngredientFilter = tf;//.CopyAllowancesFrom(tf);
						} else def.defaultIngredientFilter = null;
					}
					List<SettingsInstance> ingredients = setting.get<List<SettingsInstance>>("Ingredients2");
					if ( ingredients != null ) {
						if ( def.ingredients == null ) def.ingredients = new List<IngredientCount>();
						else def.ingredients.Clear();
						foreach ( SettingsInstance si in ingredients ) {
							IngredientCount ic = new IngredientCount();
							ic.SetBaseCount(si.get<float>("Count"));
							ic.filter.CopyAllowancesFrom(ThingFilterAdapter.AsThingFilter(si.get<SettingsInstance>("Filter")));
							def.ingredients.Add(ic);
	            		}
					}
					
					/*
					if ( def.ingredients != null ) {
						foreach ( IngredientCount ic in def.ingredients ) {
							if ( ic.filter != null ) ic.filter.DisplayRootCategory = null;
						}
					}
					if ( def.fixedIngredientFilter != null ) def.fixedIngredientFilter.DisplayRootCategory = null;
					if ( def.defaultIngredientFilter != null ) def.defaultIngredientFilter.DisplayRootCategory = null;*/
					RecipeDefPremultipliedSmallIngredientsSet(def,null);
					RecipeDefIngredientValueGetterIntSet(def,null);
					RecipeDefWorkerCounterIntSet(def,null);
					RecipeDefWorkerIntSet(def,null);
                    RecipeDefIsSurgeryCachedSet(def, null);

                    //For all recipe user defs we have to reset the private field allRecipesCached to null
                    if ( !oldRecipeUsers.NullOrEmpty() ) {
						foreach ( ThingDef t in oldRecipeUsers ) {
							ThingDefAllRecipesCachedSet(t,null);
						}
					}
					if ( !def.recipeUsers.NullOrEmpty() ) {
						foreach ( ThingDef t in def.recipeUsers ) {
							ThingDefAllRecipesCachedSet(t,null);
						}
					}
				}});
			
			QOLMod.addPreApplySettingsListener(mod=>{
				if ( applyCounter == 1 ) {
                    //Recipe filters in generated entries must be fixed since vanilla and modders tend to forget to configure them.
                    foreach (RecipeDef recipe in DefDatabase<RecipeDef>.AllDefsListForReading) {
						if ( !recipe.generated ) continue;

                        if ( recipe.fileName == "ImpliedDefs" )
                        {
                            //Fixing vanilla recipes
                            if (!recipe.products.NullOrEmpty())
                            {
                                //make xxx recipes
                                ThingDef td = recipe.products[0].thingDef;

                                if (td.MadeFromStuff)
                                {
                                    ThingFilterAdapter.ThingFilterAllowAllWhoCanMakeSet(recipe.ingredients[0].filter, new List<ThingDef>(new[] { td }));
                                    ThingFilterAdapter.ThingFilterAllowAllWhoCanMakeSet(recipe.fixedIngredientFilter, new List<ThingDef>(new[] { td }));
                                }

                                if (!td.costList.NullOrEmpty())
                                {
                                    for (int i = td.MadeFromStuff ? 1 : 0, j = 0; i < recipe.ingredients.Count && j < td.costList.Count; i++, j++)
                                    {
                                        ThingFilterAdapter.ThingFilterThingDefsSet(recipe.ingredients[i].filter, new List<ThingDef>(new[] { td.costList[j].thingDef }));
                                    }
                                }
                            }
                            else if (!recipe.anesthetize)
                            {
                                //administer xxx recipes
                                if (!recipe.ingredients.NullOrEmpty() && recipe.ingredients.Count == 1)
                                {
                                    //sanity check, some recipes might be broken or mods tempered with it so that nothing is allowed
                                    if (recipe.ingredients[0].filter.AllowedThingDefs.Count() == 1)
                                    {
                                        ThingDef td = recipe.ingredients[0].filter.AllowedThingDefs.First();
                                        ThingFilterAdapter.ThingFilterThingDefsSet(recipe.ingredients[0].filter, new List<ThingDef>(new[] { td }));
                                        ThingFilterAdapter.ThingFilterThingDefsSet(recipe.fixedIngredientFilter, new List<ThingDef>(new[] { td }));
                                    }
                                }
                            }
                        }
                        else
                        {
                            //Fixing recipies from mods
                            if ( !recipe.ingredients.NullOrEmpty() )
                            {
                                foreach ( IngredientCount ic in recipe.ingredients )
                                {
                                    FixThingFilter(ic.filter);
                                }
                            }
                            FixThingFilter(recipe.fixedIngredientFilter);
                            FixThingFilter(recipe.defaultIngredientFilter);
                        }
                    }
				}
			});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<RecipeDef>("Recipes","Select Recipe",()=>DefDatabase<RecipeDef>.AllDefsListForReading,
				def => def.defName, DefLabelProducerGeneric<RecipeDef>.LabelProducerNotNull,key => DefDatabase<RecipeDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}

        private static void FixThingFilter(ThingFilter filter)
        {
            //remember allowed things
            List<ThingDef> allowedDefs = new List<ThingDef>(filter.AllowedThingDefs);
            //Forbid all and reload based on existing rules
            filter.SetDisallowAll();
            filter.ResolveReferences();
            List<ThingDef> missingDefs = new List<ThingDef>(allowedDefs.Where(td=>!filter.Allows(td)));
            if ( missingDefs.Count == 0 )
            {
                //all previously existing allowances are reflected in the rules
                return;
            }
            //Possible to add allow all who can make but that can be done by the authors since that rule is publicly accessible
            //All things still missing are added to the list of thingDefs(might be null)
            List<ThingDef> lst = ThingFilterAdapter.ThingFilterThingDefsGet(filter);
            if ( lst == null )
            {
                ThingFilterAdapter.ThingFilterThingDefsSet(filter, missingDefs);
            }
            else
            {
                lst.AddRange(missingDefs);
            }
            filter.SetDisallowAll();
            filter.ResolveReferences();
        }

        private static void registerStatBasesProperty(SettingsProperties props, StatDef stat, int order)
        {
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<BuildableDef>(stat.defName, stat.LabelCap, order, stat.defaultBaseValue, t => t.statBases.GetStatValueFromList(stat, stat.defaultBaseValue), stat.minValue, stat.maxValue));
        }

        private static void floorsAndTerrain() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("floorsAndTerrain");
            //StatDefOf.Beauty.minValue
            //TODO replace with a method that establishes the float field for a stat and sets the default, min and max values properly.
            HashSet<StatDef> manualStats = new HashSet<StatDef>(new StatDef[] {
                StatDefOf.Beauty, StatDefOf.WorkToBuild, StatDefOf.Flammability, StatDefOf.Cleanliness, StatDefOf.FilthMultiplier });
            int firstOrderForManualStats = 10;
            foreach ( StatDef s in manualStats )
            {
                registerStatBasesProperty(props, s, firstOrderForManualStats++);
            }
            /*props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainDef>("Beauty","Beauty",10,default(float),t=>t.statBases.GetStatValueFromList(StatDef.Named("Beauty"), 0), -1000f));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainDef>("WorkToBuild","Work",20,default(float),t=>t.statBases.GetStatValueFromList(StatDef.Named("WorkToBuild"), 1), 0));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainDef>("Flammability","Flammability",30,default(float),t=>t.statBases.GetStatValueFromList(StatDef.Named("Flammability"), 0), 0f, 2f));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainDef>("Cleanliness","Cleanliness",40,default(float),t=>t.statBases.GetStatValueFromList(StatDef.Named("Cleanliness"), 0), -1000f, 1000f));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainDef>("FilthMultiplier", "Filth Chance", 45, default(float), t => t.statBases.GetStatValueFromList(StatDef.Named("FilthMultiplier"), 1f), 0f));*/
            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<TerrainDef>("PathCost","Path-Cost",50,default(int),t=>t.pathCost));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<TerrainDef>("ExtraDraftedPerceivedPathCost", "Extra Path-Cost Drafted", 51, default(int), t => t.extraDraftedPerceivedPathCost));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<TerrainDef>("ExtraNonDraftedPerceivedPathCost", "Extra Path-Cost Undrafted", 52, default(int), t => t.extraNonDraftedPerceivedPathCost));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainDef>("Fertility","Fertility",60,default(float),t=>t.fertility));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainDef>("ResourcesFractionWhenDeconstructed", "Refund Ratio", 62, 0.75f, t => t.resourcesFractionWhenDeconstructed, 0));

            props.fields.Add(new SettingsFieldPropertiesSelectable("Passability", "Passability", 63, typeof(string), Enum.GetName(typeof(Traversability), default(Traversability)),
                    t => Enum.GetName(typeof(Traversability), ((BuildableDef)t).passability),
                    f => from Traversability d in Enum.GetValues(typeof(Traversability)) orderby Enum.GetName(typeof(Traversability), d) select (object)Enum.GetName(typeof(Traversability), d),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<string>("Passability", (string)def); return true; },
                    o => (string)o, false));

            props.fields.Add(new SettingsFieldPropertiesSelectable("TerrainAffordanceNeeded", "Terrain Affordance", 64, typeof(TerrainAffordanceDef), null, 
                t => ((BuildableDef)t).terrainAffordanceNeeded, f => from object t in DefDatabase<TerrainAffordanceDef>.AllDefsListForReading select t,
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<TerrainAffordanceDef>("TerrainAffordanceNeeded", (TerrainAffordanceDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, true));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("Affordances", "Affordances", 65, typeof(List<TerrainAffordanceDef>), new List<TerrainAffordanceDef>(),
                    t => ((TerrainDef)t).affordances != null ? ((TerrainDef)t).affordances.ToList() : null,
                    f => DefDatabase<TerrainAffordanceDef>.AllDefsListForReading.Cast<object>(),
                    flow => {
                        ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                        int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                        return listEntry => {
                            flow.addPostRenderCallback(fl => { setting.get<List<TerrainAffordanceDef>>("Affordances")[listIndex] = (TerrainAffordanceDef)listEntry; setting.markAsChanged(); });
                            return true;
                        };
                    },
                    SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<TerrainAffordanceDef>.AllDefsListForReading.First(),
                    true));

            StatModifier();
            props.fields.Add(new SettingsFieldPropertiesListSettings("StatBases", "Stats", 67, typeof(List<SettingsInstance>), null,
                t => ((TerrainDef)t).statBases.NullOrEmpty() ? new List<SettingsInstance>() :
                ((TerrainDef)t).statBases.Where(tc => tc != null && !manualStats.Contains(tc.stat))
                .Select(tc => new SettingsInstance("statModifier", tc, null, null).setProp("filteredStats", manualStats)).ToList(),
                "statModifier", () => new SettingsInstance("statModifier", new StatModifier()
                    {
                        stat = DefDatabase<StatDef>.AllDefsListForReading.Where(st => !manualStats.Contains(st)).First(),
                        value = 0
                    }, null, null).setProp("filteredStats", manualStats), true));

            props.fields.Add(new SettingsFieldPropertiesListPrimitive("Tags", "Tags", 68, typeof(List<string>), new List<string>(),
                                                                      t => ((TerrainDef)t).tags != null ? ((TerrainDef)t).tags.ToList() : new List<string>(),
                                                                      null, () => "", true));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("FilthAcceptanceMask","Filth",70,typeof(List<string>),new List<string>(),
					t=> GetEnumFlagsNames<FilthSourceFlags>(((TerrainDef)t).filthAcceptanceMask),
					f => Enum.GetNames(typeof(FilthSourceFlags)).Where(e=>e != "None").Cast<object>(),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
							flow.addPostRenderCallback(fl=>{setting.get<List<string>>("FilthAcceptanceMask")[listIndex] = (string)listEntry; setting.markAsChanged();});
							return true;
						};},
					entry => (string)entry,
					()=>Enum.GetName(typeof(FilthSourceFlags),FilthSourceFlags.None),
					true));

            ThingDefCountClass();
			props.fields.Add(new SettingsFieldPropertiesListSettings("CostList","Cost List",100,typeof(List<SettingsInstance>),null,
                                     t=> ((TerrainDef)t).costList == null ? new List<SettingsInstance>() : ((TerrainDef)t).costList.Select(tc=>new SettingsInstance("thingDefCountClass",tc,null,null)).ToList(),
                                     "thingDefCountClass", ()=>new SettingsInstance("thingDefCountClass",new ThingDefCountClass(DefDatabase<ThingDef>.AllDefsListForReading.Find(d=>d.category == ThingCategory.Item),1),null,null),true));
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("DesignationCategory","Designation Category",200,typeof(DesignationCategoryDef),null,t=>((TerrainDef)t).designationCategory,
	                                                         f=>from object d in DefDatabase<DesignationCategoryDef>.AllDefsListForReading select d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<DesignationCategoryDef>("DesignationCategory",(DesignationCategoryDef)def); return true; },
				                                             SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("DesignatorDropdown","Designator Dropdown",210,typeof(DesignatorDropdownGroupDef),null,t=>((TerrainDef)t).designatorDropdown,
	                                                         f=>from object d in DefDatabase<DesignatorDropdownGroupDef>.AllDefsListForReading select d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<DesignatorDropdownGroupDef>("DesignatorDropdown",(DesignatorDropdownGroupDef)def); return true; },
				                                             SettingsFieldPropertiesSelectable.DefLabelProducer,true));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TerrainDef>("CanGenerateDefaultDesignator", "Default Designator", 215, default(bool), t => t.canGenerateDefaultDesignator));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("ResearchPrerequisites", "Research Prerequisites", 220, typeof(List<ResearchProjectDef>), new List<ResearchProjectDef>(),
                h => ((BuildableDef)h).researchPrerequisites == null ? new List<ResearchProjectDef>() : ((BuildableDef)h).researchPrerequisites.ToList(),
                f => DefDatabase<ResearchProjectDef>.AllDefsListForReading.Cast<object>(),
                flow => {
                    ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                    int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                    return listEntry => {
                        flow.addPostRenderCallback(fl => { setting.get<List<ResearchProjectDef>>("ResearchPrerequisites")[listIndex] = (ResearchProjectDef)listEntry; setting.markAsChanged(); });
                        return true;
                    };
                },
                SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<ResearchProjectDef>.AllDefsListForReading.First(),
                true));

            props.mergers.Add(setting => {
              	TerrainDef t = DefDatabase<TerrainDef>.GetNamed(setting.getKey());
                t.pathCost = setting.get<int>("PathCost");
				t.fertility = setting.get<float>("Fertility");
                t.resourcesFractionWhenDeconstructed = setting.get<float>("ResourcesFractionWhenDeconstructed");
                t.terrainAffordanceNeeded = setting.get<TerrainAffordanceDef>("TerrainAffordanceNeeded");
                t.canGenerateDefaultDesignator = setting.get<bool>("CanGenerateDefaultDesignator");

                List<SettingsInstance> statBases = setting.get<List<SettingsInstance>>("StatBases");
                if (statBases != null)
                {
                    if (t.statBases == null) t.statBases = new List<StatModifier>();
                    t.statBases.Clear();
                    foreach (ISettingsInstance s in statBases)
                    {
                        t.statBases.Add(new StatModifier() { stat = s.get<StatDef>("Stat"), value = s.get<float>("Value") });
                    }
                }
                foreach ( StatDef s in manualStats )
                {
                    float f = setting.get<float>(s.defName);
                    if (f != s.defaultBaseValue)
                    {
                        t.SetStatBaseValue(s, f);
                    }
                    else t.statBases.RemoveAll(sm => sm.stat == s);
                }

                Traversability passability;
                try
                {
                    passability = (Traversability)Enum.Parse(typeof(Traversability), setting.get<string>("Passability"));
                }
                catch (ArgumentException)
                {
                    passability = default(Traversability);
                }
                t.passability = passability;

                FilthSourceFlags filthSource;
        		try {
        			List<string> lstFoodTypes = setting.get<List<string>>("FilthAcceptanceMask");
        			if ( lstFoodTypes.NullOrEmpty() ) {
        				filthSource = FilthSourceFlags.None;
        			} else {
            			filthSource = (FilthSourceFlags)Enum.Parse(typeof(FilthSourceFlags), lstFoodTypes[0]);
            			for ( int i = 1; i < lstFoodTypes.Count; i++ ) {
            				filthSource |= (FilthSourceFlags)Enum.Parse(typeof(FilthSourceFlags), lstFoodTypes[i]);
            			}
        			}
            	} catch ( ArgumentException ) {
            		filthSource = FilthSourceFlags.None;
            	}
				t.filthAcceptanceMask = filthSource;
				
				t.extraDraftedPerceivedPathCost = setting.get<int>("ExtraDraftedPerceivedPathCost");
				t.extraNonDraftedPerceivedPathCost = setting.get<int>("ExtraNonDraftedPerceivedPathCost");
				t.designationCategory = setting.get<DesignationCategoryDef>("DesignationCategory");
    			t.designatorDropdown = setting.get<DesignatorDropdownGroupDef>("DesignatorDropdown");
				
				List<string> tags = setting.get<List<string>>("Tags");
              	if ( tags != null ) {
              		if ( t.tags == null ) t.tags = new List<string>();
              		else t.tags.Clear();
              		t.tags.AddRange(tags);
              	}
				
				List<SettingsInstance> costList = setting.get<List<SettingsInstance>>("CostList");
				if ( costList != null ) {
					if ( t.costList == null ) t.costList = new List<ThingDefCountClass>();
					else t.costList.Clear();
					foreach ( ISettingsInstance e in costList ) {
						if ( e.get<ThingDef>("Thing") != null ) {
							t.costList.Add(new ThingDefCountClass(e.get<ThingDef>("Thing"),e.get<int>("Count")));
						}
					}
				}

                List<ResearchProjectDef> researchPrerequisites = setting.get<List<ResearchProjectDef>>("ResearchPrerequisites");
                if (researchPrerequisites != null)
                {
                    if (researchPrerequisites.Count == 0)
                    {
                        t.researchPrerequisites = null;
                    }
                    else
                    {
                        if (t.researchPrerequisites == null) t.researchPrerequisites = new List<ResearchProjectDef>();
                        else t.researchPrerequisites.Clear();
                        t.researchPrerequisites.AddRange(researchPrerequisites);
                    }
                }

                if ( t.BuildableByPlayer && t.blueprintDef == null ) {
            		//when making a non buildable thing buildable it won't have the blueprint that is implied during game start
            		//ThingDefGenerator_Buildings.NewBlueprintDef_Thing(t, false, null);
            		ThingDef blueprint = (ThingDef)typeof(ThingDefGenerator_Buildings).GetMethod("NewBlueprintDef_Terrain", BindingFlags.Static|BindingFlags.NonPublic|BindingFlags.Public).Invoke(null,new object[]{t});
            		if ( DefDatabase<ThingDef>.GetNamedSilentFail(blueprint.defName) == null ) DefGenerator.AddImpliedDef<ThingDef>(blueprint);
            	}

                List<TerrainAffordanceDef> affordances = setting.get<List<TerrainAffordanceDef>>("Affordances");
                if (affordances != null)
                {
                    if (t.affordances == null) t.affordances = new List<TerrainAffordanceDef>();
                    modifyList(t.affordances, affordances, out List<TerrainAffordanceDef> added, out List<TerrainAffordanceDef> removed);
                    if ( added.Contains(TerrainAffordanceDefOf.MovingFluid) || removed.Contains(TerrainAffordanceDefOf.MovingFluid) )
                    {
                        updateWatermills = true;
                    }
                }

                t.ClearCachedData();
				});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<TerrainDef>("Floors and Terrain","Select Floor or Terrain",()=>DefDatabase<TerrainDef>.AllDefsListForReading,
				def => def.defName, DefLabelProducerGeneric<TerrainDef>.LabelProducerNotNull,key => DefDatabase<TerrainDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}
		
		private static bool HasVerbs(this ThingDef td) {
			return td.Verbs != null && td.Verbs.Count > 0;
		}
		
		private static void extraDamage() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("extraDamage");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("Def","Damage type",10,typeof(DamageDef),null,
                                                       e=>((ExtraDamage)e).def,
			                                           f=>DefDatabase<DamageDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<DamageDef>("Def", (DamageDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ExtraDamage>("Amount","Amount",20,default(float),e=>e.amount));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ExtraDamage>("ArmorPenetration","Armor Penetration",30,-1f,e=>e.armorPenetration));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ExtraDamage>("Chance","Chance",40,-1f,e=>e.chance));

				props.dynamicLabel = settings => settings.get<float>("Amount")+"x"+settings.get<DamageDef>("Def").label+" "+settings.get<float>("Chance")+"%";
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void explosionProperties() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("explosionProperties");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ProjectileProperties>("ExplosionRadius","Radius",10,default(float),p=>p.explosionRadius));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ProjectileProperties>("ExplosionDelay","Delay",20,default(int),p=>p.explosionDelay));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ProjectileProperties>("ApplyDamageToExplosionCellsNeighbors","Damage Outset",30,default(bool),p=>p.applyDamageToExplosionCellsNeighbors));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ProjectileProperties>("ExplosionChanceToStartFire","Fire Chance",40,default(float),p=>p.explosionChanceToStartFire));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ProjectileProperties>("ExplosionDamageFalloff","Damage Falloff",50,default(bool),p=>p.explosionDamageFalloff));
				props.fields.Add(new SettingsFieldPropertiesSelectable("ExplosionEffect","Effect",60,typeof(EffecterDef),null,
                                                       p=>((ProjectileProperties)p).explosionEffect,
			                                           f=>DefDatabase<EffecterDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<EffecterDef>("ExplosionEffect",(EffecterDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ProjectileProperties>("PreExplosionSpawnChance","PRES Chance",100,1f,p=>p.preExplosionSpawnChance));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ProjectileProperties>("PreExplosionSpawnThingCount","PRES Count",110,1,p=>p.preExplosionSpawnThingCount));
				props.fields.Add(new SettingsFieldPropertiesSelectable("PreExplosionSpawnThingDef","PRES Thing",120,typeof(ThingDef),null,
                                                       p=>((ProjectileProperties)p).preExplosionSpawnThingDef,
                                                       f=>DefDatabase<ThingDef>.AllDefsListForReading.Where(t=>!t.MadeFromStuff && t.category != ThingCategory.Pawn).OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("PreExplosionSpawnThingDef",(ThingDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ProjectileProperties>("PostExplosionSpawnChance","POES Chance",200,1f,p=>p.postExplosionSpawnChance));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ProjectileProperties>("PostExplosionSpawnThingCount","POES Count",210,1,p=>p.postExplosionSpawnThingCount));
				props.fields.Add(new SettingsFieldPropertiesSelectable("PostExplosionSpawnThingDef","POES Thing",220,typeof(ThingDef),null,
                                                       p=>((ProjectileProperties)p).postExplosionSpawnThingDef,
                                                       f=>DefDatabase<ThingDef>.AllDefsListForReading.Where(t=>!t.MadeFromStuff && t.category != ThingCategory.Pawn).OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("PostExplosionSpawnThingDef",(ThingDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				
				props.dynamicLabel = settings => "Properties";
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void projectiles() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("projectiles");
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("AlwaysFreeIntercept","Hits Anything",9,default(bool),t=>t.projectile.alwaysFreeIntercept));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("FlyOverhead","Fly overhead",10,default(bool),t=>t.projectile.flyOverhead));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("DamageAmountBase","Damage amount base",20,-1,
                           t=>(int)UtilReflection.searchFieldInfo(t.projectile.GetType(),"damageAmountBase",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public).GetValue(t.projectile)));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("StoppingPower","Stopping power",30,0f,t=>t.projectile.stoppingPower));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("ArmorPenetrationBase","Armor penetration base",40,-1f,
                           t=>(float)UtilReflection.searchFieldInfo(t.projectile.GetType(),"armorPenetrationBase",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public).GetValue(t.projectile)));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("Speed","Speed",50,5f,t=>t.projectile.speed));
			props.fields.Add(new SettingsFieldPropertiesSelectable("DamageDef","Damage type",60,typeof(DamageDef),null,
                                                       t=>((ThingDef)t).projectile.damageDef,
			                                           f=>DefDatabase<DamageDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<DamageDef>("DamageDef",(DamageDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			extraDamage();
			props.fields.Add(new SettingsFieldPropertiesListSettings("ExtraDamages","Extra Damage",70,typeof(List<SettingsInstance>),null,
                                     t=> ((ThingDef)t).projectile.extraDamages == null ? new List<SettingsInstance>() : ((ThingDef)t).projectile.extraDamages.Select(ed=>new SettingsInstance("extraDamage",ed,null,null)).ToList(),
                                     "extraDamage", ()=>new SettingsInstance("extraDamage",new ExtraDamage() {def = DefDatabase<DamageDef>.AllDefsListForReading.First()},null,null),true));
			//hint for the ai whether a pawns primary weapon is incendiary. E.g. prefers targeting non burning enemies in order to spread the burning effect to as many targets as possible. 
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("Ai_IsIncendiary","AI Hint Incendiary",80,default(bool),t=>t.projectile.ai_IsIncendiary));
			
			explosionProperties();
			props.props.Add(new SettingsPropertyProperties("HasExplosion",t=>((ThingDef)t).projectile.explosionRadius > 0f));
			props.fields.Add(new SettingsFieldPropertiesSettings("Explosion","Explosion",90,typeof(SettingsInstance),null,
			                                                     t=> new SettingsInstance("explosionProperties", ((ThingDef)t).projectile, null, null),"explosionProperties")
			                                                     .setHidden(f => !(((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasExplosion") || 
			                                                                       ((ISettingsInstance)f.flowScope["edit"]).isFieldConfigured("Explosion"))));
			
			//arc height is the amplitude of the shooting arcs parabel, paired with shadowsize it determines how big the projectiles shadow is.
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("ArcHeightFactor","Arc Height",100,default(float),t=>t.projectile.arcHeightFactor));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("ShadowSize","Shadow Size",110,default(float),t=>t.projectile.shadowSize));
			
			
			props.mergers.Add(setting => {
              	ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
              	setting.resolveProps(props,t);
              	t.projectile.alwaysFreeIntercept = setting.get<bool>("AlwaysFreeIntercept");
              	t.projectile.flyOverhead = setting.get<bool>("FlyOverhead");
              	t.projectile.stoppingPower = setting.get<float>("StoppingPower");
              	t.projectile.speed = setting.get<float>("Speed");
              	t.projectile.damageDef = setting.get<DamageDef>("DamageDef");
              	
              	t.projectile.arcHeightFactor = setting.get<float>("ArcHeightFactor");
              	t.projectile.shadowSize = setting.get<float>("ShadowSize");
              	t.projectile.ai_IsIncendiary = setting.get<bool>("Ai_IsIncendiary");
              	
              	List<SettingsInstance> extraDamages = setting.get<List<SettingsInstance>>("ExtraDamages");
            	if ( extraDamages != null ) {
            		if ( t.projectile.extraDamages == null ) t.projectile.extraDamages = new List<ExtraDamage>();
            		t.projectile.extraDamages.Clear();
            		foreach ( ISettingsInstance s in extraDamages ) {
            			t.projectile.extraDamages.Add(new ExtraDamage() {def = s.get<DamageDef>("Def"), amount = s.get<float>("Amount"), 
            			                              	armorPenetration = s.get<float>("ArmorPenetration"), chance = s.get<float>("Chance")});
            		}
            	}
              	
              	if ( setting.isFieldConfigured("Explosion") ) {
              		SettingsInstance siExplosion = setting.get<SettingsInstance>("Explosion");
              		if ( siExplosion != null ) {
              			t.projectile.explosionRadius = siExplosion.get<float>("ExplosionRadius");
              			t.projectile.explosionDelay = siExplosion.get<int>("ExplosionDelay");
              			t.projectile.applyDamageToExplosionCellsNeighbors = siExplosion.get<bool>("ApplyDamageToExplosionCellsNeighbors");
              			t.projectile.explosionChanceToStartFire = siExplosion.get<float>("ExplosionChanceToStartFire");
              			t.projectile.explosionDamageFalloff = siExplosion.get<bool>("ExplosionDamageFalloff");
              			t.projectile.explosionEffect = siExplosion.get<EffecterDef>("ExplosionEffect");
              			
              			t.projectile.preExplosionSpawnChance = siExplosion.get<float>("PreExplosionSpawnChance");
              			t.projectile.preExplosionSpawnThingCount = siExplosion.get<int>("PreExplosionSpawnThingCount");
              			t.projectile.preExplosionSpawnThingDef = siExplosion.get<ThingDef>("PreExplosionSpawnThingDef");
              			
              			t.projectile.postExplosionSpawnChance = siExplosion.get<float>("PostExplosionSpawnChance");
              			t.projectile.postExplosionSpawnThingCount = siExplosion.get<int>("PostExplosionSpawnThingCount");
              			t.projectile.postExplosionSpawnThingDef = siExplosion.get<ThingDef>("PostExplosionSpawnThingDef");
              		}
              	}
              	
              	UtilReflection.searchFieldInfo(t.projectile.GetType(),"damageAmountBase",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public)
              		.SetValue(t.projectile,setting.get<int>("DamageAmountBase"));
              	UtilReflection.searchFieldInfo(t.projectile.GetType(),"armorPenetrationBase",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public)
              		.SetValue(t.projectile,setting.get<float>("ArmorPenetrationBase"));
              });
			props.validators.Add(setting => {
                 	ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey(), false);
                 	return t != null && t.projectile != null;
                 });
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ThingDef>("Projectiles","Select Projectile",()=>
                DefDatabase<ThingDef>.AllDefsListForReading.Where(d=>d.category == ThingCategory.Projectile).ToList(),
                def => def.defName, DefLabelProducerGeneric<ThingDef>.LabelProducerNotNullUnique,key => DefDatabase<ThingDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}

        private static void toolPropertiesCE()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("toolPropertiesCE");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<Tool>("ArmorPenetrationSharp", "AP Sharp", 10, default(float), CombatExtended_ToolCE.GetArmorPenetrationSharp));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<Tool>("ArmorPenetrationBlunt", "AP Blunt", 20, default(float), CombatExtended_ToolCE.GetArmorPenetrationBlunt));
                props.fields.Add(new SettingsFieldPropertiesSelectable("RestrictedGender", "Restricted Gender", 30, typeof(string), Enum.GetName(typeof(Gender), Gender.None),
                    t => Enum.GetName(typeof(Gender), CombatExtended_ToolCE.GetRestrictedGender((Tool)t)),
                    f => from Gender d in Enum.GetValues(typeof(Gender)) orderby Enum.GetName(typeof(Gender), d) select (object)Enum.GetName(typeof(Gender), d),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<string>("RestrictedGender", (string)def); return true; },
                    o => (string)o, false));
                props.fields.Add(new SettingsFieldPropertiesSelectable("RequiredAttachment", "Required Attachment", 40, typeof(Def), null, t => CombatExtended_ToolCE.GetRequiredAttachment((Tool)t),
                                                       f => CombatExtended_ToolCE.ListAttachmentDef().OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)),
                                                        f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<Def>("RequiredAttachment", (Def)def); return true; },
                                                       SettingsFieldPropertiesSelectable.DefLabelProducer, true));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Tool CE Properties");
            }
        }

        private static void ToolSettings() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("toolSettings");
			if ( props.fields.NullOrEmpty() ) {
                /*
                 * public string id; - problematic since it is the index in the list and used for loading
                 * public SoundDef soundMeleeMiss;
                 * public SoundDef soundMeleeHit;
                 * public string untranslatedLabel; - set to label
                 * public SurpriseAttackProps surpriseAttack;
                    - public string label;
                    - public bool labelUsedInLogging; - no problem
                    - public bool ensureLinkedBodyPartsGroupAlwaysUsable;
                    - public BodyPartGroupDef linkedBodyPartsGroup;
                    - public List<ExtraDamage> extraMeleeDamages;
                    - public bool alwaysTreatAsWeapon;
                    - public float chanceFactor;
                    - public HediffDef hediff;
                    - public float cooldownTime;
                    - public float armorPenetration;
                    - public float power;
                    - public List<ToolCapacityDef> capacities;
                 */
                props.fields.Add(new SettingsFieldPropertiesPrimitiveString<Tool>("Label", "Label", 1, "", t => t.label) { EmptyIsNull = true });
                props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<Tool>("LabelUsedInLogging", "Use Label in Logging", 2, true, t => t.labelUsedInLogging));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<Tool>("Power","Power",10,default(float),t=>t.power));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<Tool>("Cooldown","Cooldown",20,default(float),t=>t.cooldownTime));
				props.fields.Add(new SettingsFieldPropertiesListSelectable("Capacities","Capacities",30,typeof(List<ToolCapacityDef>),new List<ToolCapacityDef>(),
			                                                          tool=>((Tool)tool).capacities.ToList(),
			                                                          f=>DefDatabase<ToolCapacityDef>.AllDefsListForReading.Cast<object>(),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<ToolCapacityDef>>("Capacities")[listIndex] = (ToolCapacityDef)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<ToolCapacityDef>.AllDefsListForReading.First(),
			                                                          true));
				//Overrides meleeArmorPenetrationBase from verb
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<Tool>("ArmorPenetration","Armor Penetration",40,-1,t=>t.armorPenetration));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<Tool>("ChanceFactor","Chance to Pick Factor",50,1f,t=>t.chanceFactor));
				//Treats an auxillary melee attack as a normal weapon attack otherwise the chance to execute this attack is multiplied by 0.3 
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<Tool>("AlwaysTreatAsWeapon","Always Treat as Weapon",60,default(bool),t=>t.alwaysTreatAsWeapon));
				//Less efficent bodyparts deal less damage if used for melee attacks. This treats the bodyparts efficency to be at least 40%. Overrides the same setting from the weapons properties.
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<Tool>("EnsureLinkedBodyPartsGroupAlwaysUsable","Linked Body Parts Group Always Usable",70,default(bool),t=>t.ensureLinkedBodyPartsGroupAlwaysUsable));
				props.fields.Add(new SettingsFieldPropertiesSelectable("LinkedBodyPartsGroup","Linked Body Parts Group",80,typeof(BodyPartGroupDef),default(BodyPartGroupDef),t=>((Tool)t).linkedBodyPartsGroup,
                                                       f=>DefDatabase<BodyPartGroupDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<BodyPartGroupDef>("LinkedBodyPartsGroup",(BodyPartGroupDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				//Wound caused by the melee attack.
				props.fields.Add(new SettingsFieldPropertiesSelectable("Hediff","Hediff",90,typeof(HediffDef),default(HediffDef),t=>((Tool)t).hediff,
                                                       f=>DefDatabase<HediffDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<HediffDef>("Hediff",(HediffDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));

                extraDamage();
                props.fields.Add(new SettingsFieldPropertiesListSettings("ExtraMeleeDamages", "Extra Damage", 100, typeof(List<SettingsInstance>), null,
                                         t => ((Tool)t).extraMeleeDamages == null ? new List<SettingsInstance>() : ((Tool)t).extraMeleeDamages.Select(ed => new SettingsInstance("extraDamage", ed, null, null)).ToList(),
                                         "extraDamage", () => new SettingsInstance("extraDamage", new ExtraDamage() { def = DefDatabase<DamageDef>.AllDefsListForReading.First() }, null, null), true));

                //ToolCE properties
                toolPropertiesCE();
                props.fields.Add(new SettingsFieldPropertiesSettings("ToolPropertiesCE", "CE Tool Properties", 1000, typeof(SettingsInstance), null,
                                                                     t => CombatExtended_ToolCE.IsToolCE((Tool)t) ? new SettingsInstance("toolPropertiesCE", t, null, null) : null, "toolPropertiesCE")
                                                                    .setHidden(f => ((ISettingsInstance)f.flowScope["edit"]).get<SettingsInstance>("ToolPropertiesCE") == null));


                props.dynamicLabel = settings => settings.get<string>("Label");

                props.visualization = new SettingsVisualizationEdit(f => "EditEntry".Translate(((ISettingsInstance)f.flowScope["edit"]).getLabel()));
			}
		}

        private static void mergeTools(ThingDef t, ISettingsInstance setting)
        {
            List<SettingsInstance> tools = setting.get<List<SettingsInstance>>("Tools");
            if (tools != null && t.tools != null)
            {
                if (tools.Count != t.tools.Count)
                {
                    SettingsInit.restartRequiredMessages.Add("Adding/Removing tools on \""+ SettingsFieldPropertiesSelectable.DefLabelProducer(t)+ "\".");
                }
                t.tools = SettingsStorage.getInstancesForMerging(setting, "Tools", t.tools);
                bool foundDirt = false;
                for (int i = 0; i < t.tools.Count && i < tools.Count; i++)
                {
                    SettingsInstance tool = tools[i];
                    Tool tt = t.tools[i];
                    tt.id = i.ToString();
                    mergeToolSettings(tt, tool);
                    foundDirt = foundDirt | tool.getLocalDirty();
                }
                if (foundDirt)
                {
                    SettingsInit.restartRequiredMessages.Add("Changing tool properties on \"" + SettingsFieldPropertiesSelectable.DefLabelProducer(t) + "\".");
                }
            }
        }

        private static void mergeToolSettings(Tool tt, ISettingsInstance tool)
        {
            tt.label = tool.get<string>("Label");
            tt.untranslatedLabel = tt.label;
            try
            {
                typeof(Tool).GetField("cachedLabelCap", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public).SetValue(tt, null);
            }
            catch (Exception ex)
            {
                Log.Error("Failed to reset cachedLabelCap! \n" + ex.StackTrace);
            }
            tt.power = tool.get<float>("Power");
            tt.cooldownTime = tool.get<float>("Cooldown");
            List<ToolCapacityDef> capacities = tool.get<List<ToolCapacityDef>>("Capacities");
            if (capacities != null)
            {
                if (tt.capacities.NullOrEmpty()) tt.capacities = new List<ToolCapacityDef>();
                tt.capacities.Clear();
                tt.capacities.AddRange(capacities);
            }

            tt.armorPenetration = tool.get<float>("ArmorPenetration");
            tt.chanceFactor = tool.get<float>("ChanceFactor");
            tt.alwaysTreatAsWeapon = tool.get<bool>("AlwaysTreatAsWeapon");
            tt.ensureLinkedBodyPartsGroupAlwaysUsable = tool.get<bool>("EnsureLinkedBodyPartsGroupAlwaysUsable");
            tt.linkedBodyPartsGroup = tool.get<BodyPartGroupDef>("LinkedBodyPartsGroup");
            tt.hediff = tool.get<HediffDef>("Hediff");

            List<SettingsInstance> extraMeleeDamages = tool.get<List<SettingsInstance>>("ExtraMeleeDamages");
            if (extraMeleeDamages != null)
            {
                if (extraMeleeDamages.Count > 0)
                {
                    if (tt.extraMeleeDamages == null) tt.extraMeleeDamages = new List<ExtraDamage>();
                    tt.extraMeleeDamages.Clear();
                    foreach (ISettingsInstance s in extraMeleeDamages)
                    {
                        tt.extraMeleeDamages.Add(new ExtraDamage()
                        {
                            def = s.get<DamageDef>("Def"),
                            amount = s.get<float>("Amount"),
                            armorPenetration = s.get<float>("ArmorPenetration"),
                            chance = s.get<float>("Chance")
                        });
                    }
                }
                else
                {
                    tt.extraMeleeDamages = null;
                }
            }

            if (CombatExtended_ToolCE.IsToolCE(tt))
            {
                SettingsInstance siToolCE = tool.get<SettingsInstance>("ToolPropertiesCE");
                if ( siToolCE == null )
                {
                    //initialize CE properties if they weren't initialized before(edits were made before CE was active)
                    SettingsFieldPropertiesSettings field = (SettingsFieldPropertiesSettings)SettingsStorage.getSettingsProperties("toolSettings").fields.Find(f => f.id == "ToolPropertiesCE");
                    tool.set<SettingsInstance>("ToolPropertiesCE", (SettingsInstance)field.getFromBaseObject(tt));
                }
                else
                {
                    CombatExtended_ToolCE.SetArmorPenetrationSharp(tt, siToolCE.get<float>("ArmorPenetrationSharp"));
                    CombatExtended_ToolCE.SetArmorPenetrationBlunt(tt, siToolCE.get<float>("ArmorPenetrationBlunt"));
                    Gender gender;
                    try
                    {
                        gender = (Gender)Enum.Parse(typeof(Gender), siToolCE.get<string>("RestrictedGender"));
                    }
                    catch (ArgumentException)
                    {
                        gender = Gender.None;
                    }
                    CombatExtended_ToolCE.SetRestrictedGender(tt, gender);
                    CombatExtended_ToolCE.SetRequiredAttachment(tt, siToolCE.get<Def>("RequiredAttachment"));
                }
            }
        }

        private static void compAmmoUser() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compAmmoUser");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<CompProperties>("MagazineSize", "Magazine Size", 10, default(int), CombatExtended_CompProperties_AmmoUser.GetMagazineSize, 0));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties>("ReloadTime", "Reload Time", 20, 1f, CombatExtended_CompProperties_AmmoUser.GetReloadTime, 0f));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties>("ReloadOneAtATime", "Single-Round Reload", 30, default(bool), CombatExtended_CompProperties_AmmoUser.GetReloadOneAtATime));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties>("ThrowMote", "Throw Mote", 40, true, CombatExtended_CompProperties_AmmoUser.GetThrowMote));
				//props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties>("SpawnUnloaded", "Spawn Unloaded", 50, default(bool), CombatExtended_CompProperties_AmmoUser.GetSpawnUnloaded));
				props.fields.Add(new SettingsFieldPropertiesSelectable("AmmoSet","Ammo Set",60,typeof(Def),default(Def),
                                                       cp=>CombatExtended_CompProperties_AmmoUser.GetAmmoSet((CompProperties)cp),
                                                       f=>CombatExtended_CompProperties_AmmoUser.ListAmmoSetDef(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<Def>("AmmoSet",(Def)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Ammo User Properties");
			}
		}
		
		private static void compFireModes() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compFireModes");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<CompProperties>("AimedBurstShotCount", "Aimed Burst Shot Count", 10, -1, CombatExtended_CompProperties_FireModes.GetAimedBurstShotCount, -1));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties>("AiUseBurstMode", "AI Uses Burst", 20, default(bool), CombatExtended_CompProperties_FireModes.GetAiUseBurstMode));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties>("NoSingleShot", "Disable Single Shot", 30, default(bool), CombatExtended_CompProperties_FireModes.GetNoSingleShot));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties>("NoSnapshot", "Disable Snapshot", 40, default(bool), CombatExtended_CompProperties_FireModes.GetNoSnapshot));
				props.fields.Add(new SettingsFieldPropertiesSelectable("AiAimMode","AI Aim Mode",50,typeof(string),"Snapshot",
			                                                       cp=>CombatExtended_CompProperties_FireModes.GetAiAimModeAsString((CompProperties)cp),
			                                                       f=>CombatExtended_CompProperties_FireModes.ListAiAimModeAsString(),
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("AiAimMode",(string)def); return true; },
				                                             o=>(string)o,false));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Fire Modes Properties");
			}
		}
		
		private static void verbPropertiesCE() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("verbPropertiesCE");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<VerbProperties>("RecoilAmount","Recoil Amount",10,default(float), CombatExtended_VerbPropertiesCE.GetRecoilAmount));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<VerbProperties>("IndirectFirePenalty","Indirect Fire Penalty",20,default(float), CombatExtended_VerbPropertiesCE.GetIndirectFirePenalty));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<VerbProperties>("CircularError","Circular Error",30,default(float), CombatExtended_VerbPropertiesCE.GetCircularError));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<VerbProperties>("MeleeArmorPenetration","Melee Armor Penetration",40,default(float), CombatExtended_VerbPropertiesCE.GetMeleeArmorPenetration));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<VerbProperties>("EjectsCasings","Ejects Casings",50,true, CombatExtended_VerbPropertiesCE.GetEjectsCasings));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<VerbProperties>("IgnorePartialLoSBlocker","Ignore Partial LoS Blocker",60,default(bool), CombatExtended_VerbPropertiesCE.GetIgnorePartialLoSBlocker));
				props.fields.Add(new SettingsFieldPropertiesSelectable("RecoilPattern","Recoil Pattern",70,typeof(string),"None",
			                                                       cp=>CombatExtended_VerbPropertiesCE.GetRecoilPatternAsString((VerbProperties)cp),
			                                                       f=>CombatExtended_VerbPropertiesCE.ListRecoilPatternAsString(),
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("RecoilPattern",(string)def); return true; },
				                                             o=>(string)o,false));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Verb CE Properties");
			}
		}

        private static void renderingParametersConfig()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("renderingParametersConfig");
            if (props.fields.NullOrEmpty())
            {
                renderingParametersConfigEntry();
                props.fields.Add(new SettingsFieldPropertiesListSettings("Entries", "Entries", 10, typeof(List<SettingsInstance>), null,
                        rp => ((QOLDefModExtension_RenderingParametersConfig)rp).entries.NullOrEmpty() ? new List<SettingsInstance>() : 
                        ((QOLDefModExtension_RenderingParametersConfig)rp).entries.
                        Select(tc => new SettingsInstance("renderingParametersConfigEntry", tc, null, null)).ToList(),
                        "renderingParametersConfigEntry",
                        () => new SettingsInstance("renderingParametersConfigEntry", new QOLDefModExtension_RenderingParametersConfig_Entry(), null, null), true));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Rendering Properties");
            }
        }

        private static void mergeRenderingParametersConfig(Def t, ISettingsInstance settings)
        {
            List<SettingsInstance> entries = settings?.get<List<SettingsInstance>>("Entries");
            if (entries.NullOrEmpty())
            {
                QOLDefModExtension_RenderingParameters rp = t.GetModExtension<QOLDefModExtension_RenderingParameters>();
                if (rp != null)
                {
                    t.modExtensions.Remove(rp);
                    if (!t.modExtensions.Any()) t.modExtensions = null;
                }
                return;
            }
            else
            {
                QOLDefModExtension_RenderingParameters rp = t.GetModExtension<QOLDefModExtension_RenderingParameters>();
                if (rp == null)
                {
                    if (t.modExtensions == null)
                    {
                        t.modExtensions = new List<DefModExtension>();
                    }
                    rp = new QOLDefModExtension_RenderingParameters();
                    t.modExtensions.Add(rp);
                }
                rp.Config.entries.Clear();
                foreach (ISettingsInstance si in entries)
                {
                    QOLDefModExtension_RenderingParametersConfig_Entry entry = new QOLDefModExtension_RenderingParametersConfig_Entry();
                    entry.bodyType = si.get<BodyTypeDef>("BodyType");
                    entry.facing = si.get<int>("Facing");
                    entry.offsetX = si.get<float>("OffsetX");
                    entry.offsetY = si.get<float>("OffsetY");
                    entry.scaleX = si.get<float>("ScaleX");
                    entry.scaleY = si.get<float>("ScaleY");
                    rp.Config.entries.Add(entry);
                }
                rp.Reload();
            }

        }

        private static void renderingParametersConfigEntry()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("renderingParametersConfigEntry");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesSelectable("BodyType", "Body Type", 10, typeof(BodyTypeDef), default(BodyTypeDef), 
                    t => ((QOLDefModExtension_RenderingParametersConfig_Entry)t).bodyType,
                    f => DefDatabase<BodyTypeDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<BodyTypeDef>("BodyType", (BodyTypeDef)def); return true; },
                    SettingsFieldPropertiesSelectable.DefLabelProducer, true));

                props.fields.Add(new SettingsFieldPropertiesSelectable("Facing", "Facing", 20, typeof(int), 4,
                    t => ((QOLDefModExtension_RenderingParametersConfig_Entry)t).facing,
                    f => new int[] { 0, 1, 2, 3, 4 }.Cast<object>(),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<int>("Facing", (int)def); return true; },
                    o => {
                        int i = (int)o;
                        if (i >= 0 && i <= 3) return new Rot4(i).ToStringHuman();
                        return "NoDrugUseRequirement".Translate(); //Any
                    }, false));

                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLDefModExtension_RenderingParametersConfig_Entry>(
                    "OffsetX", "Offset X", 50, default(float), e => e.offsetX));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLDefModExtension_RenderingParametersConfig_Entry>(
                    "OffsetY", "Offset Y", 60, default(float), e => e.offsetY));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLDefModExtension_RenderingParametersConfig_Entry>(
                    "ScaleX", "Scale X", 70, default(float), e => e.scaleX, 0f));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<QOLDefModExtension_RenderingParametersConfig_Entry>(
                    "ScaleY", "Scale Y", 80, default(float), e => e.scaleY, 0f));

                props.dynamicLabel = si => SettingsFieldPropertiesSelectable.DefLabelProducer(si.get<BodyTypeDef>("BodyType")) + " " + si.get<int>("Facing") + " (" + si.get<float>("OffsetX") + "," + si.get<float>("OffsetY") + ") (" + si.get<float>("ScaleX") + "," + si.get<float>("ScaleY") + ")";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Properties");
            }
        }

        private static FieldInfo fiForcedMissRadius = typeof(VerbProperties).GetField("forcedMissRadius", BindingFlags.NonPublic | BindingFlags.Instance);

        private static void weapons() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("weapons");
            
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("Smeltable","Smeltable",10,default(bool),t=>t.smeltable));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("WarmupTime","Warmup time",20,default(float),t=>t.HasVerbs() ? t.Verbs[0].warmupTime : 0f));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("MinRange","Min range",30,default(float),t=>t.HasVerbs() ? t.Verbs[0].minRange : 0f));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("Range","Range",40,default(float),t=>t.HasVerbs() ? t.Verbs[0].range : 0f));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("ForcedMissRadius", "Forced miss radius", 50, default(float), t => t.HasVerbs() ? (float)fiForcedMissRadius.GetValue(t.Verbs[0]) : 0f));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("BurstShotCount","Burst shots",60,default(int),t=>t.HasVerbs() ? t.Verbs[0].burstShotCount : 0));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("TicksBetweenBurstShots","Ticks between burst shots",61,15,t=>t.HasVerbs() ? t.Verbs[0].ticksBetweenBurstShots : 0));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("MuzzleFlashScale","Muzzle flash scale",70,default(float),t=>t.HasVerbs() ? t.Verbs[0].muzzleFlashScale : 0f));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("GenerateAllowChance", "Spawn Chance", 71, 1f, t => t.generateAllowChance));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("MeleeArmorPenetrationBase","Melee Armor Penetration-Base",75,default(float),t=>t.HasVerbs() ? t.Verbs[0].meleeArmorPenetrationBase : -1f));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("EnsureLinkedBodyPartsGroupAlwaysUsable","Linked Body Parts Group Always Usable",76,default(bool),t=>t.HasVerbs() ? t.Verbs[0].ensureLinkedBodyPartsGroupAlwaysUsable : default(bool)));
			props.fields.Add(new SettingsFieldPropertiesSelectable("LinkedBodyPartsGroup","Linked Body Parts Group",77,typeof(BodyPartGroupDef),default(BodyPartGroupDef),t=>((ThingDef)t).HasVerbs() ? ((ThingDef)t).Verbs[0].linkedBodyPartsGroup : default(BodyPartGroupDef),
                                                       f=>DefDatabase<BodyPartGroupDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<BodyPartGroupDef>("LinkedBodyPartsGroup",(BodyPartGroupDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("DefaultProjectile","Projectile",80,typeof(ThingDef),default(ThingDef),
                                                       t=>((ThingDef)t).HasVerbs() ? ((ThingDef)t).Verbs[0].defaultProjectile : null,
			                                           f=>DefDatabase<ThingDef>.AllDefsListForReading.Where(d=>d.category == ThingCategory.Projectile)
			                                            	.OrderBy(d=>DefLabelProducerGeneric<ThingDef>.LabelProducerUnique(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("DefaultProjectile",(ThingDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducerUnique,false));
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("TechLevel","Tech-Level",85,typeof(string),Enum.GetName(typeof(TechLevel),(TechLevel)0),
					t=>Enum.GetName(typeof(TechLevel),((ThingDef)t).techLevel),
					f=>from TechLevel d in Enum.GetValues(typeof(TechLevel)) orderby Enum.GetName(typeof(TechLevel),d) select (object)Enum.GetName(typeof(TechLevel),d),
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("TechLevel",(string)def); return true; },
					o=>(string)o,false));

            props.fields.Add(new SettingsFieldPropertiesSelectable("Tradeability", "Tradeability", 87, typeof(string), Enum.GetName(typeof(Tradeability), Tradeability.All),
                    t => Enum.GetName(typeof(Tradeability), ((ThingDef)t).tradeability),
                    f => from Tradeability d in Enum.GetValues(typeof(Tradeability)) orderby Enum.GetName(typeof(Tradeability), d) select (object)Enum.GetName(typeof(Tradeability), d),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<string>("Tradeability", (string)def); return true; },
                    o => (string)o, false));

            StatModifier();
            props.fields.Add(new SettingsFieldPropertiesListSettings("equippedStatOffsets", "Wearer Stats", 89, typeof(List<SettingsInstance>), null,
                        t => ((ThingDef)t).equippedStatOffsets.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).equippedStatOffsets.Where(tc => tc != null).Select(tc => new SettingsInstance("statModifier", tc, null, null)).ToList(),
                        "statModifier", () => new SettingsInstance("statModifier", new StatModifier() { stat = DefDatabase<StatDef>.AllDefsListForReading.First(), value = 0 }, null, null), true));
            props.fields.Add(new SettingsFieldPropertiesListSettings("StatBases","Item Stats",90,typeof(List<SettingsInstance>),null,
                        t=> ((ThingDef)t).statBases.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).statBases.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
			
			props.fields.Add(new SettingsFieldPropertiesListPrimitive("WeaponTags","Weapon Tags",95,typeof(List<string>),new List<string>(),
			                                                          t=> ((ThingDef)t).weaponTags != null ? ((ThingDef)t).weaponTags.ToList() : new List<string>(),
			                                                          null,()=>"",true));
			
			props.fields.Add(new SettingsFieldPropertiesListPrimitive("TradeTags","Trade Tags",97,typeof(List<string>),new List<string>(),
			                                                          t=> ((ThingDef)t).tradeTags != null ? ((ThingDef)t).tradeTags.ToList() : new List<string>(),
			                                                          null,()=>"",true));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("ThingCategories", "Categories", 98, typeof(List<ThingCategoryDef>),
                   new List<ThingCategoryDef>(),
                   h => ((ThingDef)h).thingCategories == null ? new List<ThingCategoryDef>() : ((ThingDef)h).thingCategories.ToList(),
                   f => DefDatabase<ThingCategoryDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).ToList<object>(),
                flow => {
                    ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                    int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                    return listEntry => {
                        flow.addPostRenderCallback(fl => {
                            setting.get<List<ThingCategoryDef>>("ThingCategories")[listIndex] = (ThingCategoryDef)listEntry; setting.markAsChanged();
                        });
                        return true;
                    };
                }, SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<ThingCategoryDef>.AllDefsListForReading.First(), true));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("StuffCategories", "Stuff Categories", 105, typeof(List<StuffCategoryDef>), new List<StuffCategoryDef>(),
                h => ((ThingDef)h).stuffCategories == null ? new List<StuffCategoryDef>() : ((ThingDef)h).stuffCategories.ToList(),
                f => DefDatabase<StuffCategoryDef>.AllDefsListForReading.Cast<object>(),
                flow => {
                    ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                    int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                    return listEntry => {
                        flow.addPostRenderCallback(fl => { setting.get<List<StuffCategoryDef>>("StuffCategories")[listIndex] = (StuffCategoryDef)listEntry; setting.markAsChanged(); });
                        return true;
                    };
                },
                SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<StuffCategoryDef>.AllDefsListForReading.First(),
                true));

            ToolSettings();
            props.fields.Add(new SettingsFieldPropertiesListSettingsReferences("Tools", "Tools", 110, typeof(List<SettingsInstance>), null,
                        t =>
                        {
                            if (((ThingDef)t).tools == null) return new List<SettingsInstance>();
                            else
                            {
                                int i = 0;
                                return ((ThingDef)t).tools.Select(tc => new SettingsInstance("toolSettings", tc, null, tc.label)
                                {
                                    BaseObjectReferenceKey = i++,
                                    BaseObjectReference = tc
                                }).ToList();
                            }
                        }, "toolSettings", () => {
                            Tool t = CombatExtended_ToolCE.createNewTool();
                            if (t == null)
                            {
                                t = new Tool() { label = "tool" };
                            } else
                            {
                                t.label = "CE tool";
                            }
                            t.cooldownTime = 2;
                            t.power = 10;
                            t.capacities.Add(DefDatabase<ToolCapacityDef>.AllDefsListForReading.First<ToolCapacityDef>());
                            return t;
                        }, true));

            //ModExtension to scale and offset rendering
            renderingParametersConfig();
            props.fields.Add(new SettingsFieldPropertiesSettings("RenderingParameters", "Rendering Parameters", 120, typeof(SettingsInstance), null,
                t => ((ThingDef)t).GetModExtension<QOLDefModExtension_RenderingParameters>() != null ?
                new SettingsInstance("renderingParametersConfig", ((ThingDef)t).GetModExtension<QOLDefModExtension_RenderingParameters>().Config, null, null) :
                null,
                "renderingParametersConfig")
            { createNewEntry = () => {
                    return new SettingsInstance("renderingParametersConfig", new QOLDefModExtension_RenderingParameters().Config);
                }, allowRemoval = true });

            //Combat Extended properties
            compAmmoUser();
			props.props.Add(new SettingsPropertyProperties("HasCompAmmoUser",o=>CombatExtended_CompProperties_AmmoUser.HasCompProperty((ThingDef)o)));
			props.fields.Add(new SettingsFieldPropertiesSettings("CompAmmoUser","Ammo User Properties",1010,typeof(SettingsInstance),null,
			                                                     t=> CombatExtended_CompProperties_AmmoUser.HasCompProperty((ThingDef)t) ? new SettingsInstance("compAmmoUser", CombatExtended_CompProperties_AmmoUser.GetCompProperty((ThingDef)t), null, null) : null,"compAmmoUser")
			                                                    .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompAmmoUser")));
			compFireModes();
			props.props.Add(new SettingsPropertyProperties("HasCompFireModes",o=>CombatExtended_CompProperties_FireModes.HasCompProperty((ThingDef)o)));
			props.fields.Add(new SettingsFieldPropertiesSettings("CompFireModes","Fire Modes Properties",1020,typeof(SettingsInstance),null,
			                                                     t=> CombatExtended_CompProperties_FireModes.HasCompProperty((ThingDef)t) ? new SettingsInstance("compFireModes", CombatExtended_CompProperties_FireModes.GetCompProperty((ThingDef)t), null, null) : null,"compFireModes")
			                                                    .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompFireModes")));
			verbPropertiesCE();
			props.props.Add(new SettingsPropertyProperties("HasCEVerb",o=>CombatExtended_VerbPropertiesCE.HasVerbProperties((ThingDef)o)));
			props.fields.Add(new SettingsFieldPropertiesSettings("VerbPropertiesCE","CE Verb Properties",1000,typeof(SettingsInstance),null,
			                                                     t=> CombatExtended_VerbPropertiesCE.HasVerbProperties((ThingDef)t) ? new SettingsInstance("verbPropertiesCE", ((ThingDef)t).Verbs[0], null, null) : null,"verbPropertiesCE")
			                                                    .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCEVerb")));

            props.mergers.Add(setting => {
	            ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
	            setting.resolveProps(props,t);
	            t.smeltable = setting.get<bool>("Smeltable");
	            List<string> weaponTags = setting.get<List<string>>("WeaponTags");
	            if ( weaponTags != null ) {
	                if ( t.weaponTags == null ) t.weaponTags = new List<string>();
	                else t.weaponTags.Clear();
	                t.weaponTags.AddRange(weaponTags);
	            }
	                  	
	            TechLevel tl;
		        try {
			        tl = (TechLevel)Enum.Parse(typeof(TechLevel),setting.get<string>("TechLevel"));
		        } catch ( ArgumentException ) {
					tl = (TechLevel)0;
		        }
				t.techLevel = tl;
                t.generateAllowChance = setting.get<float>("GenerateAllowChance");
                Tradeability tradeability;
                try
                {
                    tradeability = (Tradeability)Enum.Parse(typeof(Tradeability), setting.get<string>("Tradeability"));
                }
                catch (ArgumentException)
                {
                    tradeability = Tradeability.All;
                }
                t.tradeability = tradeability;


                if ( t.HasVerbs() ) {
		            VerbProperties vp = t.Verbs[0];
		                  	
		            vp.warmupTime = setting.get<float>("WarmupTime");
		            vp.minRange = setting.get<float>("MinRange");
		            vp.range = setting.get<float>("Range");
                    fiForcedMissRadius.SetValue(vp, setting.get<float>("ForcedMissRadius"));
		            vp.burstShotCount = setting.get<int>("BurstShotCount");
		            vp.ticksBetweenBurstShots = setting.get<int>("TicksBetweenBurstShots");
		            vp.muzzleFlashScale = setting.get<float>("MuzzleFlashScale");
		            vp.meleeArmorPenetrationBase = setting.get<float>("MeleeArmorPenetrationBase");
		            vp.ensureLinkedBodyPartsGroupAlwaysUsable = setting.get<bool>("EnsureLinkedBodyPartsGroupAlwaysUsable");
		            vp.linkedBodyPartsGroup = setting.get<BodyPartGroupDef>("LinkedBodyPartsGroup");
		                  	
		            vp.defaultProjectile = setting.get<ThingDef>("DefaultProjectile");
	            }

                List<SettingsInstance> equippedStatOffsets = setting.get<List<SettingsInstance>>("equippedStatOffsets");
                if (equippedStatOffsets != null)
                {
                    if (t.equippedStatOffsets == null) t.equippedStatOffsets = new List<StatModifier>();
                    t.equippedStatOffsets.Clear();
                    foreach (ISettingsInstance s in equippedStatOffsets)
                    {
                        t.equippedStatOffsets.Add(new StatModifier() { stat = s.get<StatDef>("Stat"), value = s.get<float>("Value") });
                    }
                }
                List<SettingsInstance> statBases = setting.get<List<SettingsInstance>>("StatBases");
		        if ( statBases != null ) {
		            if ( t.statBases == null ) t.statBases = new List<StatModifier>();
		            t.statBases.Clear();
		            foreach ( ISettingsInstance s in statBases ) {
		            	t.statBases.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
		            }
		        }

                List<StuffCategoryDef> stuffCategories = setting.get<List<StuffCategoryDef>>("StuffCategories");
                if (stuffCategories != null)
                {
                    if (t.stuffCategories == null) t.stuffCategories = new List<StuffCategoryDef>();
                    else t.stuffCategories.Clear();
                    t.stuffCategories.AddRange(stuffCategories);
                }

                mergeThingCategories(t, setting);

                mergeTools(t, setting);
	                  	
	            List<string> tradeTags = setting.get<List<string>>("TradeTags");
		        if ( tradeTags != null ) {
		            if ( t.tradeTags == null ) t.tradeTags = new List<string>();
		            else t.tradeTags.Clear();
		            t.tradeTags.AddRange(tradeTags);
		        }
	                  	
	            CompProperties cp = CombatExtended_CompProperties_AmmoUser.GetCompProperty(t);
	            if ( cp != null ) {
	                SettingsInstance si = setting.get<SettingsInstance>("CompAmmoUser");
	                if ( si != null ) {
	                  	CombatExtended_CompProperties_AmmoUser.SetMagazineSize(cp,si.get<int>("MagazineSize"));
	                  	CombatExtended_CompProperties_AmmoUser.SetReloadTime(cp,si.get<float>("ReloadTime"));
	                  	CombatExtended_CompProperties_AmmoUser.SetReloadOneAtATime(cp,si.get<bool>("ReloadOneAtATime"));
	                  	CombatExtended_CompProperties_AmmoUser.SetThrowMote(cp,si.get<bool>("ThrowMote"));
	                  	//CombatExtended_CompProperties_AmmoUser.SetSpawnUnloaded(cp,si.get<bool>("SpawnUnloaded"));
	                  	CombatExtended_CompProperties_AmmoUser.SetAmmoSet(cp,si.get<Def>("AmmoSet"));
	                }
	            }
                CompProperties cpCECompPropertiesFireModes = CombatExtended_CompProperties_FireModes.GetCompProperty(t);
	            if (cpCECompPropertiesFireModes != null ) {
	                SettingsInstance si = setting.get<SettingsInstance>("CompFireModes");
	                if ( si != null ) {
	                  	CombatExtended_CompProperties_FireModes.SetAimedBurstShotCount(cpCECompPropertiesFireModes, si.get<int>("AimedBurstShotCount"));
	                  	CombatExtended_CompProperties_FireModes.SetAiUseBurstMode(cpCECompPropertiesFireModes, si.get<bool>("AiUseBurstMode"));
	                  	CombatExtended_CompProperties_FireModes.SetNoSingleShot(cpCECompPropertiesFireModes, si.get<bool>("NoSingleShot"));
	                  	CombatExtended_CompProperties_FireModes.SetNoSnapshot(cpCECompPropertiesFireModes, si.get<bool>("NoSnapshot"));
	                  	CombatExtended_CompProperties_FireModes.SetAiAimModeAsString(cpCECompPropertiesFireModes, si.get<string>("AiAimMode"));
                    }
                }
	            if ( CombatExtended_VerbPropertiesCE.HasVerbProperties(t) ) {
	                SettingsInstance si = setting.get<SettingsInstance>("VerbPropertiesCE");
	                if ( si != null ) {
	                  	VerbProperties vp = t.Verbs[0];
	                  	CombatExtended_VerbPropertiesCE.SetRecoilAmount(vp,si.get<float>("RecoilAmount"));
	                  	CombatExtended_VerbPropertiesCE.SetIndirectFirePenalty(vp,si.get<float>("IndirectFirePenalty"));
	                  	CombatExtended_VerbPropertiesCE.SetCircularError(vp,si.get<float>("CircularError"));
	                  	CombatExtended_VerbPropertiesCE.SetMeleeArmorPenetration(vp,si.get<float>("MeleeArmorPenetration"));
	                  	CombatExtended_VerbPropertiesCE.SetEjectsCasings(vp,si.get<bool>("EjectsCasings"));
	                  	CombatExtended_VerbPropertiesCE.SetIgnorePartialLoSBlocker(vp,si.get<bool>("IgnorePartialLoSBlocker"));
	                  	CombatExtended_VerbPropertiesCE.SetRecoilPatternAsString(vp,si.get<string>("RecoilPattern"));
	                }
	            }
                
                mergeRenderingParametersConfig(t, setting.get<SettingsInstance>("RenderingParameters"));

                //all things of this def on all maps should invoke CombatExtended.CompFireModes.InitAvailableFireModes on their fire nodes implementation
                if (Current.Game != null && cpCECompPropertiesFireModes != null)
                {
                    foreach (Map m in Find.Maps)
                    {
                        List<ThingWithComps> things = new List<ThingWithComps>();
                        ThingOwnerUtility.GetAllThingsRecursively<ThingWithComps>(m, new ThingRequest() { singleDef = t }, things, true, null, true);
                        things.ForEach(tc => {
                            ThingComp fireMode = tc.GetCompByDef(cpCECompPropertiesFireModes);
                            if ( fireMode != null )
                            {
                                MethodInfo mi = fireMode.GetType().GetMethod("InitAvailableFireModes", BindingFlags.Instance|BindingFlags.Public|BindingFlags.NonPublic);
                                if (mi != null)
                                {
                                    mi.Invoke(fireMode, null);
                                }
                            }
                        });
                    }
                }
			});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ThingDef>("Weapons","Select Weapon",()=>
                DefDatabase<ThingDef>.AllDefsListForReading.Where(t=>t.IsWeapon/*t.IsWithinCategory(ThingCategoryDefOf.Weapons) || (t.weaponTags != null && t.weaponTags.Contains("TurretGun"))*/).ToList(),
				def => def.defName, DefLabelProducerGeneric<ThingDef>.LabelProducerNotNull,key => DefDatabase<ThingDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}

        private static void damageDefAdditionalHediff()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("damageDefAdditionalHediff");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesSelectable("Hediff", "Hediff", 10, typeof(HediffDef), default(HediffDef),
                    t => ((DamageDefAdditionalHediff)t).hediff,
                    f => DefDatabase<HediffDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<HediffDef>("Hediff", (HediffDef)def); return true; },
                    SettingsFieldPropertiesSelectable.DefLabelProducer, false));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDefAdditionalHediff>("SeverityPerDamageDealt", "Severity Per Damage", 20, 0.1f, t=>t.severityPerDamageDealt));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDefAdditionalHediff>("SeverityFixed", "Severity Fixed", 30, default(float), t => t.severityFixed, 0f));
                props.fields.Add(new SettingsFieldPropertiesSelectable("VictimSeverityScaling", "Severity Scaling", 40, typeof(StatDef), default(StatDef),
                    t => ((DamageDefAdditionalHediff)t).victimSeverityScaling,
                    f => DefDatabase<StatDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<StatDef>("VictimSeverityScaling", (StatDef)def); return true; },
                    SettingsFieldPropertiesSelectable.DefLabelProducer, true));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDefAdditionalHediff>("VictimSeverityScalingByInvBodySize", "Severity Scales by Body-Size", 50, default(bool), t => t.victimSeverityScalingByInvBodySize));

                props.dynamicLabel = si => "Additional Hediff";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Additional Hediff");
            }
        }

        private static void damage()
        {
            FieldInfo fiExternalViolence = typeof(DamageDef).GetField("externalViolence", BindingFlags.NonPublic | BindingFlags.Instance);
            FieldInfo fiExternalViolenceForMechanoids = typeof(DamageDef).GetField("externalViolenceForMechanoids", BindingFlags.NonPublic | BindingFlags.Instance);

            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("damage");

            /*public RulePackDef combatLogRules;*/
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("ExternalViolence", "External Violence", 20, default(bool), t => fiExternalViolence != null ? (bool)fiExternalViolence.GetValue(t) : default(bool)));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("ExternalViolenceForMechanoids", "External Violence Mechanoids", 30, default(bool), t => fiExternalViolenceForMechanoids != null ? (bool)fiExternalViolenceForMechanoids.GetValue(t) : default(bool)));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("IsExplosive", "Explosive", 40, default(bool), t => t.isExplosive));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("IsRanged", "Ranged", 45, default(bool), t => t.isRanged));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("HasForcefulImpact", "Forceful Impact", 50, true, t => t.hasForcefulImpact));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("HarmsHealth", "Harms Health", 60, true, t => t.harmsHealth));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("MakesBlood", "Makes Blood", 70, true, t => t.makesBlood));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("CanInterruptJobs", "Interrupts Jobs", 80, true, t => t.canInterruptJobs));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("MakesAnimalsFlee", "Scares Animals", 100, default(bool), t => t.makesAnimalsFlee));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("Execution", "Execution", 110, default(bool), t => t.execution));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("CanUseDeflectMetalEffect", "Deflect Metal Effect", 120, true, t => t.canUseDeflectMetalEffect));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("ApplyAdditionalHediffsIfHuntingForFood", "Hediff on Hunting", 130, true, t => t.applyAdditionalHediffsIfHuntingForFood));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("HarmAllLayersUntilOutside", "Penetrate Layers", 140, default(bool), t => t.harmAllLayersUntilOutside));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<DamageDef>("ExplosionAffectOutsidePartsOnly", "External Explosion", 150, true, t => t.explosionAffectOutsidePartsOnly));
            
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("BuildingDamageFactor", "Building DF", 200, 1f, t => t.buildingDamageFactor));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("BuildingDamageFactorPassable", "Passable Building DF", 210, 1f, t => t.buildingDamageFactorPassable));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("BuildingDamageFactorImpassable", "Impassable Building DF", 220, 1f, t => t.buildingDamageFactorImpassable));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("PlantDamageFactor", "Plant Damage Factor", 230, 1f, t => t.plantDamageFactor));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("DefaultArmorPenetration", "Default AP", 240, -1f, t => t.defaultArmorPenetration));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("DefaultStoppingPower", "Default Stopping Power", 250, default(float), t => t.defaultStoppingPower));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("ExplosionSnowMeltAmount", "Snow Melt", 260, 1f, t => t.explosionSnowMeltAmount, 0f));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("ExplosionHeatEnergyPerCell", "Heat Energy", 270, default(float), t => t.explosionHeatEnergyPerCell, 0f));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("StabChanceOfForcedInternal", "Internal Stab Chance", 280, default(float), t => t.stabChanceOfForcedInternal, 0f, 1f));
            //unused
            //props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("StabPierceBonus", "Stab Pierce Bonus", 290, default(float), t => t.stabPierceBonus));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("CutCleaveBonus", "Cut Cleave Bonus", 300, default(float), t => t.cutCleaveBonus));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("BluntInnerHitChance", "Blunt Hit Chance", 310, default(float), t => t.bluntInnerHitChance));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("BluntStunDuration", "Blunt Stun Duration", 320, 1f, t => t.bluntStunDuration));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("ScratchSplitPercentage", "Scratch Split Percentage", 330, 0.5f, t => t.scratchSplitPercentage));
            //unused
            //props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("BiteDamageMultiplier", "Bite Damage Multiplier", 340, 1f, t => t.biteDamageMultiplier));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<DamageDef>("DefaultDamage", "Default Damage", 400, -1, t => t.defaultDamage));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<DamageDef>("MinDamageToFragment", "Min Damage to Fragment", 410, 99999, t => t.minDamageToFragment));
            
            props.fields.Add(new SettingsFieldPropertiesSelectable("ImpactSoundType", "Impact Sound", 500, typeof(ImpactSoundTypeDef), default(ImpactSoundTypeDef), 
                t => ((DamageDef)t).impactSoundType,
                f => DefDatabase<ImpactSoundTypeDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<ImpactSoundTypeDef>("ImpactSoundType", (ImpactSoundTypeDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, true));

            props.fields.Add(new SettingsFieldPropertiesSelectable("DamageEffecter", "Effecter", 510, typeof(EffecterDef), default(EffecterDef),
                t => ((DamageDef)t).damageEffecter,
                f => DefDatabase<EffecterDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<EffecterDef>("DamageEffecter", (EffecterDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, true));

            props.fields.Add(new SettingsFieldPropertiesSelectable("ArmorCategory", "Armor Category", 520, typeof(DamageArmorCategoryDef), default(DamageArmorCategoryDef),
                t => ((DamageDef)t).armorCategory,
                f => DefDatabase<DamageArmorCategoryDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<DamageArmorCategoryDef>("ArmorCategory", (DamageArmorCategoryDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, true));

            props.fields.Add(new SettingsFieldPropertiesSelectable("Hediff", "Hediff", 530, typeof(HediffDef), default(HediffDef),
                t => ((DamageDef)t).hediff,
                f => DefDatabase<HediffDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<HediffDef>("Hediff", (HediffDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, false));

            props.fields.Add(new SettingsFieldPropertiesSelectable("HediffSkin", "Hediff Skin", 540, typeof(HediffDef), default(HediffDef),
                t => ((DamageDef)t).hediffSkin,
                f => DefDatabase<HediffDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<HediffDef>("HediffSkin", (HediffDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, true));

            props.fields.Add(new SettingsFieldPropertiesSelectable("HediffSolid", "Hediff Solid", 550, typeof(HediffDef), default(HediffDef),
                t => ((DamageDef)t).hediffSolid,
                f => DefDatabase<HediffDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<HediffDef>("HediffSolid", (HediffDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, true));

            props.fields.Add(new SettingsFieldPropertiesSelectable("ExplosionCellMote", "Explosion Cell Mote", 560, typeof(ThingDef), default(ThingDef),
                t => ((DamageDef)t).explosionCellMote,
                f => DefDatabase<ThingDef>.AllDefsListForReading.Where(d => typeof(Mote).IsAssignableFrom(d.thingClass)).OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("ExplosionCellMote", (ThingDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, true));

            props.fields.Add(new SettingsFieldPropertiesSelectable("ExplosionInteriorMote", "Explosion Interior Mote", 570, typeof(ThingDef), default(ThingDef),
                t => ((DamageDef)t).explosionInteriorMote,
                f => DefDatabase<ThingDef>.AllDefsListForReading.Where(d => typeof(Mote).IsAssignableFrom(d.thingClass)).OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("ExplosionInteriorMote", (ThingDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, true));

            props.fields.Add(new SettingsFieldPropertiesSelectable("ExplosionCellFleck", "Explosion Cell Fleck", 580, typeof(FleckDef), default(FleckDef),
                t => ((DamageDef)t).explosionCellFleck,
                f => DefDatabase<FleckDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<FleckDef>("ExplosionCellFleck", (FleckDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, true));

            props.fields.Add(new SettingsFieldPropertiesSelectable("ExplosionInteriorFleck", "Explosion Interior Fleck", 590, typeof(FleckDef), default(FleckDef),
                t => ((DamageDef)t).explosionInteriorFleck,
                f => DefDatabase<FleckDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<FleckDef>("ExplosionInteriorFleck", (FleckDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, true));

            props.fields.Add(new SettingsFieldPropertiesSelectable("SoundExplosion", "Sound", 600, typeof(SoundDef), default(SoundDef),
                t => ((DamageDef)t).soundExplosion,
                f => DefDatabase<SoundDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<SoundDef>("SoundExplosion", (SoundDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, true));
            
            props.fields.Add(new SettingsFieldPropertiesPrimitiveColor<DamageDef>("ExplosionColorCenter", "Color Center", 700, Color.white, t => t.explosionColorCenter));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveColor<DamageDef>("ExplosionColorEdge", "Color Edge", 710, Color.white, t => t.explosionColorEdge));
            
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("OverkillPctToDestroyPartMin", "Overkill Part Min", 800, 0f, t => t.overkillPctToDestroyPart.min));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("OverkillPctToDestroyPartMax", "Overkill Part Max", 810, 0.7f, t => t.overkillPctToDestroyPart.max));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("BluntInnerHitDamageFractionToConvertMin", "Blunt Damage Conversion Min", 820, default(float), t => t.bluntInnerHitDamageFractionToConvert.min));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("BluntInnerHitDamageFractionToConvertMax", "Blunt Damage Conversion Max", 830, default(float), t => t.bluntInnerHitDamageFractionToConvert.max));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("BluntInnerHitDamageFractionToAddMin", "Blunt Damage Add Min", 840, default(float), t => t.bluntInnerHitDamageFractionToAdd.min));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<DamageDef>("BluntInnerHitDamageFractionToAddMax", "Blunt Damage Add Max", 850, default(float), t => t.bluntInnerHitDamageFractionToAdd.max));

            CurvePoint();
            props.fields.Add(new SettingsFieldPropertiesListSettings("CutExtraTargetsCurve", "Cut Extra Targets Curve", 900, typeof(List<SettingsInstance>), null,
                    t => ((DamageDef)t).cutExtraTargetsCurve == null ? new List<SettingsInstance>() : ((DamageDef)t).cutExtraTargetsCurve.Points.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                    "curvePoint", () => new SettingsInstance("curvePoint", new CurvePoint(0f, 0f), null, null), true)
                    { sortable = true });
            props.fields.Add(new SettingsFieldPropertiesListSettings("BluntStunChancePerDamagePctOfCorePartToHeadCurve", "Blunt(Head) Stun Chance Curve", 910, typeof(List<SettingsInstance>), null,
                    t => ((DamageDef)t).bluntStunChancePerDamagePctOfCorePartToHeadCurve == null ? new List<SettingsInstance>() : ((DamageDef)t).bluntStunChancePerDamagePctOfCorePartToHeadCurve.Points.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                    "curvePoint", () => new SettingsInstance("curvePoint", new CurvePoint(0f, 0f), null, null), true)
                    { sortable = true });
            props.fields.Add(new SettingsFieldPropertiesListSettings("BluntStunChancePerDamagePctOfCorePartToBodyCurve", "Blunt(Body) Stun Chance Curve", 920, typeof(List<SettingsInstance>), null,
                    t => ((DamageDef)t).bluntStunChancePerDamagePctOfCorePartToBodyCurve == null ? new List<SettingsInstance>() : ((DamageDef)t).bluntStunChancePerDamagePctOfCorePartToBodyCurve.Points.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                    "curvePoint", () => new SettingsInstance("curvePoint", new CurvePoint(0f, 0f), null, null), true)
                    { sortable = true });

            damageDefAdditionalHediff();
            props.fields.Add(new SettingsFieldPropertiesListSettings("AdditionalHediffs", "Additional Hediffs", 1000, typeof(List<SettingsInstance>), null,
                        t => ((DamageDef)t).additionalHediffs.NullOrEmpty() ? new List<SettingsInstance>() : ((DamageDef)t).additionalHediffs.Where(tc => tc != null).Select(tc => new SettingsInstance("damageDefAdditionalHediff", tc, null, null)).ToList(),
                        "damageDefAdditionalHediff", () => new SettingsInstance("damageDefAdditionalHediff", new DamageDefAdditionalHediff() { hediff = DefDatabase<HediffDef>.AllDefsListForReading.First() }, null, null), true));

            props.mergers.Add(setting => {
                DamageDef t = DefDatabase<DamageDef>.GetNamed(setting.getKey());

                setting.resolveProps(props, t);

                if (fiExternalViolence != null) fiExternalViolence.SetValue(t, setting.get<bool>("ExternalViolence"));
                if (fiExternalViolenceForMechanoids != null ) fiExternalViolenceForMechanoids.SetValue(t, setting.get<bool>("ExternalViolenceForMechanoids"));
                t.isExplosive = setting.get<bool>("IsExplosive");
                t.hasForcefulImpact = setting.get<bool>("HasForcefulImpact");
                t.harmsHealth = setting.get<bool>("HarmsHealth");
                t.makesBlood = setting.get<bool>("MakesBlood");
                t.canInterruptJobs = setting.get<bool>("CanInterruptJobs");
                t.isRanged = setting.get<bool>("IsRanged");
                t.makesAnimalsFlee = setting.get<bool>("MakesAnimalsFlee");
                t.execution = setting.get<bool>("Execution");
                t.canUseDeflectMetalEffect = setting.get<bool>("CanUseDeflectMetalEffect");
                t.applyAdditionalHediffsIfHuntingForFood = setting.get<bool>("ApplyAdditionalHediffsIfHuntingForFood");
                t.harmAllLayersUntilOutside = setting.get<bool>("HarmAllLayersUntilOutside");
                t.explosionAffectOutsidePartsOnly = setting.get<bool>("ExplosionAffectOutsidePartsOnly");

                t.buildingDamageFactor = setting.get<float>("BuildingDamageFactor");
                t.buildingDamageFactorPassable = setting.get<float>("BuildingDamageFactorPassable");
                t.buildingDamageFactorImpassable = setting.get<float>("BuildingDamageFactorImpassable");
                t.plantDamageFactor = setting.get<float>("PlantDamageFactor");
                t.defaultArmorPenetration = setting.get<float>("DefaultArmorPenetration");
                t.defaultStoppingPower = setting.get<float>("DefaultStoppingPower");
                t.explosionSnowMeltAmount = setting.get<float>("ExplosionSnowMeltAmount");
                t.explosionHeatEnergyPerCell = setting.get<float>("ExplosionHeatEnergyPerCell");
                t.stabChanceOfForcedInternal = setting.get<float>("StabChanceOfForcedInternal");
                //t.stabPierceBonus = setting.get<float>("StabPierceBonus");
                t.cutCleaveBonus = setting.get<float>("CutCleaveBonus");
                t.bluntInnerHitChance = setting.get<float>("BluntInnerHitChance");
                t.bluntStunDuration = setting.get<float>("BluntStunDuration");
                t.scratchSplitPercentage = setting.get<float>("ScratchSplitPercentage");
                //t.biteDamageMultiplier = setting.get<float>("BiteDamageMultiplier");

                t.defaultDamage = setting.get<int>("DefaultDamage");
                t.minDamageToFragment = setting.get<int>("MinDamageToFragment");
                
                t.impactSoundType = setting.get<ImpactSoundTypeDef>("ImpactSoundType");
                t.damageEffecter = setting.get<EffecterDef>("DamageEffecter");
                t.armorCategory = setting.get<DamageArmorCategoryDef>("ArmorCategory");
                t.hediff = setting.get<HediffDef>("Hediff");
                t.hediffSkin = setting.get<HediffDef>("HediffSkin");
                t.hediffSolid = setting.get<HediffDef>("HediffSolid");
                t.explosionCellMote = setting.get<ThingDef>("ExplosionCellMote");
                t.explosionInteriorMote = setting.get<ThingDef>("ExplosionInteriorMote");
                t.explosionCellFleck = setting.get<FleckDef>("ExplosionCellFleck");
                t.explosionInteriorFleck = setting.get<FleckDef>("ExplosionInteriorFleck");
                t.soundExplosion = setting.get<SoundDef>("SoundExplosion");

                t.explosionColorCenter = setting.get<Color>("ExplosionColorCenter");
                t.explosionColorEdge = setting.get<Color>("ExplosionColorEdge");
                
                t.overkillPctToDestroyPart = new FloatRange(setting.get<float>("OverkillPctToDestroyPartMin"), 
                    setting.get<float>("OverkillPctToDestroyPartMax"));
                t.bluntInnerHitDamageFractionToConvert = new FloatRange(setting.get<float>("BluntInnerHitDamageFractionToConvertMin"), 
                    setting.get<float>("BluntInnerHitDamageFractionToConvertMax"));
                t.bluntInnerHitDamageFractionToAdd = new FloatRange(setting.get<float>("BluntInnerHitDamageFractionToAddMin"), 
                    setting.get<float>("BluntInnerHitDamageFractionToAddMax"));

                List<SettingsInstance> cutExtraTargetsCurve = setting.get<List<SettingsInstance>>("CutExtraTargetsCurve");
                if (!cutExtraTargetsCurve.NullOrEmpty())
                {
                    if (t.cutExtraTargetsCurve == null) t.cutExtraTargetsCurve = new SimpleCurve();
                    SimpleCurve sc = t.cutExtraTargetsCurve;
                    sc.Points.Clear();
                    sc.Points.AddRange(cutExtraTargetsCurve.Select(si => new CurvePoint(si.get<float>("X"), si.get<float>("Y"))));
                    sc.SortPoints();
                }
                else t.cutExtraTargetsCurve = null;

                List <SettingsInstance> bluntStunChancePerDamagePctOfCorePartToHeadCurve = setting.get<List<SettingsInstance>>("BluntStunChancePerDamagePctOfCorePartToHeadCurve");
                if (!bluntStunChancePerDamagePctOfCorePartToHeadCurve.NullOrEmpty())
                {
                    if (t.bluntStunChancePerDamagePctOfCorePartToHeadCurve == null) t.bluntStunChancePerDamagePctOfCorePartToHeadCurve = new SimpleCurve();
                    SimpleCurve sc = t.bluntStunChancePerDamagePctOfCorePartToHeadCurve;
                    sc.Points.Clear();
                    sc.Points.AddRange(bluntStunChancePerDamagePctOfCorePartToHeadCurve.Select(si => new CurvePoint(si.get<float>("X"), si.get<float>("Y"))));
                    sc.SortPoints();
                }
                else t.bluntStunChancePerDamagePctOfCorePartToHeadCurve = null;
                List<SettingsInstance> bluntStunChancePerDamagePctOfCorePartToBodyCurve = setting.get<List<SettingsInstance>>("BluntStunChancePerDamagePctOfCorePartToBodyCurve");
                if (!bluntStunChancePerDamagePctOfCorePartToBodyCurve.NullOrEmpty())
                {
                    if (t.bluntStunChancePerDamagePctOfCorePartToBodyCurve == null) t.bluntStunChancePerDamagePctOfCorePartToBodyCurve = new SimpleCurve();
                    SimpleCurve sc = t.bluntStunChancePerDamagePctOfCorePartToBodyCurve;
                    sc.Points.Clear();
                    sc.Points.AddRange(bluntStunChancePerDamagePctOfCorePartToBodyCurve.Select(si => new CurvePoint(si.get<float>("X"), si.get<float>("Y"))));
                    sc.SortPoints();
                }
                else t.bluntStunChancePerDamagePctOfCorePartToBodyCurve = null;

                List<SettingsInstance> additionalHediffs = setting.get<List<SettingsInstance>>("AdditionalHediffs");
                if (additionalHediffs != null)
                {
                    if (t.additionalHediffs == null) t.additionalHediffs = new List<DamageDefAdditionalHediff>();
                    t.additionalHediffs.Clear();
                    foreach (ISettingsInstance s in additionalHediffs)
                    {
                        if (s.get<HediffDef>("Hediff") == null) continue;
                        t.additionalHediffs.Add(new DamageDefAdditionalHediff() {
                            hediff = s.get<HediffDef>("Hediff"),
                            severityFixed = s.get<float>("SeverityFixed"),
                            severityPerDamageDealt = s.get<float>("SeverityPerDamageDealt"),
                            victimSeverityScalingByInvBodySize = s.get<bool>("VictimSeverityScalingByInvBodySize"),
                            victimSeverityScaling = s.get<StatDef>("VictimSeverityScaling")
                        });
                    }
                }

                t.ClearCachedData();
            });

            props.visualization = new SettingsVisualizationMenuSelectEdit<DamageDef>("Damage Types", "Select Damage Type", () =>
                  DefDatabase<DamageDef>.AllDefsListForReading.Where(t => t != null).ToList(),
                def => def.defName, DefLabelProducerGeneric<DamageDef>.LabelProducerNotNull, key => DefDatabase<DamageDef>.GetNamed(key),
                f => "Edit \"" + ((ISettingsInstance)f.flowScope["edit"]).getLabel() + "\"");
        }

        private static void ThoughtStage() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("thoughtStage");
			if ( props.fields.NullOrEmpty() ) {
                props.fields.Add(new SettingsFieldPropertiesPrimitiveString<ThoughtStage>("Label", "Label", 5, null, t => t.label) { EmptyIsNull = true });
                props.fields.Add(new SettingsFieldPropertiesPrimitiveString<ThoughtStage>("LabelSocial", "Label Social", 6, null, t => t.labelSocial) { EmptyIsNull = true });
                props.fields.Add(new SettingsFieldPropertiesPrimitiveText<ThoughtStage>("Description", "Description", 9, null, t => t.description) { EmptyIsNull = true });
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThoughtStage>("BaseMoodEffect","Base Mood Effect",10,default(float),t=>t.baseMoodEffect));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThoughtStage>("BaseOpinionOffset","Base Opinion Offset",20,default(float),t=>t.baseOpinionOffset));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThoughtStage>("Visible","Visible",30,default(bool),t=>t.visible));
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void thoughts() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("thoughts");
            props.fields.Add(new SettingsFieldPropertiesPrimitiveString<ThoughtDef>("Label", "Label", 5, null, t => t.label) { EmptyIsNull = true });
            props.fields.Add(new SettingsFieldPropertiesPrimitiveText<ThoughtDef>("Description", "Description", 9, null, t => t.description) { EmptyIsNull = true });
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThoughtDef>("Duration","Duration(Days)",10,default(float),t=>t.durationDays));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThoughtDef>("OnlyForColonists","Only For Colonists",20,default(bool),t=>t.nullifiedIfNotColonist));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThoughtDef>("ShowBubble","Show Bubble",30,default(bool),t=>t.showBubble));
			props.fields.Add(new SettingsFieldPropertiesSelectable("MultiplyingStat","Multiplying Stat",40,typeof(StatDef),default(StatDef),t=>((ThoughtDef)t).effectMultiplyingStat,
                                                       f=>DefDatabase<StatDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<StatDef>("MultiplyingStat",(StatDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("NextThought","Next Thought",50,typeof(ThoughtDef),default(ThoughtDef),t=>((ThoughtDef)t).nextThought,
			                                            f=>DefDatabase<ThoughtDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThoughtDef>("NextThought",(ThoughtDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThoughtDef>("StackLimit","Stack Limit",60,1,t=>t.stackLimit));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThoughtDef>("StackLimitPerPawn","Stack Limit Per Pawn",70,-1,t=>t.stackLimitForSameOtherPawn));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThoughtDef>("StackedEffectMultiplier","Stacked Effect Multiplier",80,0.75f,t=>t.stackedEffectMultiplier));
			//The default points to float max which is a value that can not be parsed with "float.parse" which is used by the numeric-text widget. 
			//Therefore the value would trigger a change whenever it is displayed. We convert the value to string and parse it to ensure that this wont happen.
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThoughtDef>("MaximumCumulatedOpinion","Maximum Cumulated Opinion",90,float.MaxValue,t=>float.Parse(t.maxCumulatedOpinionOffset.ToString())));
			
			//TODO maybe lerpOpinionToZero as "Age Factor" property. Regulates the decay based on age and Duration.
			//TODO a help page for each settings page that can be configured.
			
			ThoughtStage();
			props.fields.Add(new SettingsFieldPropertiesListSettings("Stages","Stages",100,typeof(List<SettingsInstance>),null,
                        t=> ((ThoughtDef)t).stages.NullOrEmpty() ? new List<SettingsInstance>() : ((ThoughtDef)t).stages.Where(tc => tc != null).Select(tc=>new SettingsInstance("thoughtStage",tc,null,tc.label)).ToList(),
						"thoughtStage",null,false));
			
			props.mergers.Add(setting => {
			    ThoughtDef t = DefDatabase<ThoughtDef>.GetNamed(setting.getKey());
                t.label = setting.get<string>("Label");
                t.description = setting.get<string>("Description");
                t.durationDays = setting.get<float>("Duration");
            	t.nullifiedIfNotColonist = setting.get<bool>("OnlyForColonists");
            	t.showBubble = setting.get<bool>("ShowBubble");
            	t.effectMultiplyingStat = setting.get<StatDef>("MultiplyingStat");
            	t.nextThought = setting.get<ThoughtDef>("NextThought");
            	t.stackLimit = setting.get<int>("StackLimit");
            	t.stackLimitForSameOtherPawn = setting.get<int>("StackLimitPerPawn");
            	t.stackedEffectMultiplier = setting.get<float>("StackedEffectMultiplier");
            	t.maxCumulatedOpinionOffset = setting.get<float>("MaximumCumulatedOpinion");
            	List<SettingsInstance> stages = setting.get<List<SettingsInstance>>("Stages");
            	if ( t.stages != null && stages != null ) {
            		//we need to skip over the null values in t.stages without skipping values in stages
            		for ( int i = 0, j = 0; i < t.stages.Count && j < stages.Count; i++ ) {
            			if ( t.stages[i] != null ) {
                            t.stages[i].label = stages[j].get<string>("Label");
                            typeof(ThoughtStage).GetField("cachedLabelCap", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public).SetValue(t.stages[i], null);
                            t.stages[i].labelSocial = stages[j].get<string>("LabelSocial");
                            typeof(ThoughtStage).GetField("cachedLabelSocialCap", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public).SetValue(t.stages[i], null);
                            t.stages[i].description = stages[j].get<string>("Description");

                            t.stages[i].baseMoodEffect = stages[j].get<float>("BaseMoodEffect");
	            			t.stages[i].baseOpinionOffset = stages[j].get<float>("BaseOpinionOffset");
	            			t.stages[i].visible = stages[j].get<bool>("Visible");
	            			j++;
            			}
            		}
            	}
                t.ClearCachedData();
            });
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ThoughtDef>("Thoughts","Select Thought (group)",()=>DefDatabase<ThoughtDef>.AllDefsListForReading,
				def => def.defName, DefLabelProducerGeneric<ThoughtDef>.LabelProducerNotNull,key => DefDatabase<ThoughtDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}
		
		private static void StatModifier() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("statModifier");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("Stat","Stat",10,typeof(StatDef),default(StatDef),t=>((StatModifier)t).stat,
				    f=>
                    {
                        HashSet<StatDef> filteredStats = ((ISettingsInstance)f.flowScope["edit"]).prop<HashSet<StatDef>>("filteredStats");
                        if (filteredStats == null) return from object d in DefDatabase<StatDef>.AllDefsListForReading select d;
                        return from StatDef s in DefDatabase<StatDef>.AllDefsListForReading where !filteredStats.Contains(s) select (object)s;
                    },
				    f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<StatDef>("Stat",(StatDef)def); return true; },
				    SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<StatModifier>("Value","Value",20,default(float),t=>t.value));

				props.dynamicLabel = settings => settings.get<float>("Value")+"x"+SettingsFieldPropertiesSelectable.DefLabelProducer(settings.get<StatDef>("Stat"));
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}

        private static void GraphicData()
        {
            /*
            public string texPath;
            public Type graphicClass;
            public ShaderTypeDef shaderType;
            public List<ShaderParameter> shaderParameters;

            public Vector3? drawOffsetNorth;
            public Vector3? drawOffsetEast;
            public Vector3? drawOffsetSouth;
            public Vector3? drawOffsetWest;
            
            public int renderQueue;
            public ShadowData shadowData;
            public DamageGraphicData damageData;
            public LinkDrawerType linkType;
            public LinkFlags linkFlags;
            public AsymmetricLinkData asymmetricLink;
             */
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("graphicData");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<GraphicData>("DrawRotated", "Draw Rotated", 10, true, t => t.drawRotated));
                //Not practical since this relies on image data being present
                //props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<GraphicData>("AllowFlip", "Allow Flip", 20, true, t => t.allowFlip));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<GraphicData>("RenderInstanced", "Render Instanced", 30, default(bool), t => t.renderInstanced));
                //unused
                //props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<GraphicData>("AllowAtlasing", "Allow Atlasing", 40, true, t => t.allowAtlasing));

                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<GraphicData>("DrawSizeX", "Draw Size X", 100, 1f, t => t.drawSize.x, 0f));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<GraphicData>("DrawSizeY", "Draw Size Y", 110, 1f, t => t.drawSize.y, 0f));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<GraphicData>("DrawOffsetX", "Draw Offset X", 120, 0f, t => t.drawOffset.x));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<GraphicData>("DrawOffsetY", "Draw Offset Y", 130, 0f, t => t.drawOffset.y));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<GraphicData>("DrawOffsetZ", "Draw Offset Z", 140, 0f, t => t.drawOffset.z));

                //unused
                //props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<GraphicData>("OnGroundRandomRotateAngle", "onGroundRandomRotateAngle", 200, default(float), t => t.onGroundRandomRotateAngle));
                //props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<GraphicData>("FlipExtraRotation", "flipExtraRotation", 210, default(float), t => t.flipExtraRotation));

                props.fields.Add(new SettingsFieldPropertiesPrimitiveColor<GraphicData>("Color", "Color", 200, Color.white, t => t.color));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveColor<GraphicData>("ColorTwo", "Color Two", 210, Color.white, t => t.colorTwo));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Graphic");
            }
        }

        private static void mergeGraphicData(GraphicData g, ISettingsInstance setting)
        {
            if (g == null || setting == null) return;

            g.drawRotated = setting.get<bool>("DrawRotated");
            //g.allowFlip = setting.get<bool>("AllowFlip");
            g.renderInstanced = setting.get<bool>("RenderInstanced");
            g.color = setting.get<Color>("Color");
            g.colorTwo = setting.get<Color>("ColorTwo");
            g.drawSize.x = setting.get<float>("DrawSizeX");
            g.drawSize.y = setting.get<float>("DrawSizeY");
            g.drawOffset.x = setting.get<float>("DrawOffsetX");
            g.drawOffset.y = setting.get<float>("DrawOffsetY");
            g.drawOffset.z = setting.get<float>("DrawOffsetZ");

            FieldInfo fi = typeof(GraphicData).GetField("cachedGraphic", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public);
            if (fi != null) fi.SetValue(g, null);
        }

        private static void WornGraphicData()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("wornGraphicData");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<WornGraphicData>("RenderUtilityAsPack", "Pack", 10, true, t => t.renderUtilityAsPack));
                WornGraphicDirectionData();
                props.fields.Add(new SettingsFieldPropertiesSettings("North", "North", 20, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicDirectionData", ((WornGraphicData)t).north, null, null), "wornGraphicDirectionData"));
                props.fields.Add(new SettingsFieldPropertiesSettings("South", "South", 30, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicDirectionData", ((WornGraphicData)t).south, null, null), "wornGraphicDirectionData"));
                props.fields.Add(new SettingsFieldPropertiesSettings("East", "East", 40, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicDirectionData", ((WornGraphicData)t).east, null, null), "wornGraphicDirectionData"));
                props.fields.Add(new SettingsFieldPropertiesSettings("West", "West", 50, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicDirectionData", ((WornGraphicData)t).west, null, null), "wornGraphicDirectionData"));
                WornGraphicBodyTypeDataScaleOnly();
                props.fields.Add(new SettingsFieldPropertiesSettings("Male", "Male", 60, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicBodyTypeDataScaleOnly", ((WornGraphicData)t).male, null, null), "wornGraphicBodyTypeDataScaleOnly"));
                props.fields.Add(new SettingsFieldPropertiesSettings("Female", "Female", 70, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicBodyTypeDataScaleOnly", ((WornGraphicData)t).female, null, null), "wornGraphicBodyTypeDataScaleOnly"));
                props.fields.Add(new SettingsFieldPropertiesSettings("Thin", "Thin", 80, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicBodyTypeDataScaleOnly", ((WornGraphicData)t).thin, null, null), "wornGraphicBodyTypeDataScaleOnly"));
                props.fields.Add(new SettingsFieldPropertiesSettings("Hulk", "Hulk", 90, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicBodyTypeDataScaleOnly", ((WornGraphicData)t).hulk, null, null), "wornGraphicBodyTypeDataScaleOnly"));
                props.fields.Add(new SettingsFieldPropertiesSettings("Fat", "Fat", 100, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicBodyTypeDataScaleOnly", ((WornGraphicData)t).fat, null, null), "wornGraphicBodyTypeDataScaleOnly"));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Worn Graphic Data");
            }
        }

        private static void mergeWornGraphicData(WornGraphicData g, ISettingsInstance setting)
        {
            if (g == null || setting == null) return;

            g.renderUtilityAsPack = setting.get<bool>("RenderUtilityAsPack");

            mergeWornGraphicDirectionData(ref g.north, setting.get<ISettingsInstance>("North"));
            mergeWornGraphicDirectionData(ref g.south, setting.get<ISettingsInstance>("South"));
            mergeWornGraphicDirectionData(ref g.east, setting.get<ISettingsInstance>("East"));
            mergeWornGraphicDirectionData(ref g.west, setting.get<ISettingsInstance>("West"));

            mergeWornGraphicBodyTypeDataScaleOnly(ref g.male, setting.get<ISettingsInstance>("Male"));
            mergeWornGraphicBodyTypeDataScaleOnly(ref g.female, setting.get<ISettingsInstance>("Female"));
            mergeWornGraphicBodyTypeDataScaleOnly(ref g.thin, setting.get<ISettingsInstance>("Thin"));
            mergeWornGraphicBodyTypeDataScaleOnly(ref g.hulk, setting.get<ISettingsInstance>("Hulk"));
            mergeWornGraphicBodyTypeDataScaleOnly(ref g.fat, setting.get<ISettingsInstance>("Fat"));
        }

        private static void WornGraphicDirectionData()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("wornGraphicDirectionData");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<WornGraphicDirectionData>("OffsetX", "Offset X", 10, 0f, t => t.offset.x));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<WornGraphicDirectionData>("OffsetY", "Offset Y", 20, 0f, t => t.offset.y));
                WornGraphicBodyTypeDataOffsetOnly();
                props.fields.Add(new SettingsFieldPropertiesSettings("Male", "Male", 30, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicBodyTypeDataOffsetOnly", ((WornGraphicDirectionData)t).male, null, null), "wornGraphicBodyTypeDataOffsetOnly"));
                props.fields.Add(new SettingsFieldPropertiesSettings("Female", "Female", 40, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicBodyTypeDataOffsetOnly", ((WornGraphicDirectionData)t).female, null, null), "wornGraphicBodyTypeDataOffsetOnly"));
                props.fields.Add(new SettingsFieldPropertiesSettings("Thin", "Thin", 50, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicBodyTypeDataOffsetOnly", ((WornGraphicDirectionData)t).thin, null, null), "wornGraphicBodyTypeDataOffsetOnly"));
                props.fields.Add(new SettingsFieldPropertiesSettings("Hulk", "Hulk", 60, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicBodyTypeDataOffsetOnly", ((WornGraphicDirectionData)t).hulk, null, null), "wornGraphicBodyTypeDataOffsetOnly"));
                props.fields.Add(new SettingsFieldPropertiesSettings("Fat", "Fat", 70, typeof(SettingsInstance), null,
                    t => new SettingsInstance("wornGraphicBodyTypeDataOffsetOnly", ((WornGraphicDirectionData)t).fat, null, null), "wornGraphicBodyTypeDataOffsetOnly"));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Worn Graphic Direction Data");
            }
        }

        private static void mergeWornGraphicDirectionData(ref WornGraphicDirectionData g, ISettingsInstance setting)
        {
            if (setting == null) return;

            g.offset.x = setting.get<float>("OffsetX");
            g.offset.y = setting.get<float>("OffsetY");
            mergeWornGraphicBodyTypeDataOffsetOnly(ref g.male, setting.get<ISettingsInstance>("Male"));
            mergeWornGraphicBodyTypeDataOffsetOnly(ref g.female, setting.get<ISettingsInstance>("Female"));
            mergeWornGraphicBodyTypeDataOffsetOnly(ref g.thin, setting.get<ISettingsInstance>("Thin"));
            mergeWornGraphicBodyTypeDataOffsetOnly(ref g.hulk, setting.get<ISettingsInstance>("Hulk"));
            mergeWornGraphicBodyTypeDataOffsetOnly(ref g.fat, setting.get<ISettingsInstance>("Fat"));
        }

        private static void WornGraphicBodyTypeData()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("wornGraphicBodyTypeData");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<WornGraphicBodyTypeData>("OffsetX", "Offset X", 10, default(float), t => t.offset.x));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<WornGraphicBodyTypeData>("OffsetY", "Offset Y", 20, default(float), t => t.offset.y));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<WornGraphicBodyTypeData>("ScaleX", "Scale X", 30, 1f, t => t.Scale.x));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<WornGraphicBodyTypeData>("ScaleY", "Scale Y", 40, 1f, t => t.Scale.y));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Worn Graphic Body Type Data");
            }
        }

        private static void mergeWornGraphicBodyTypeData(ref WornGraphicBodyTypeData g, ISettingsInstance setting)
        {
            if (setting == null) return;

            g.offset.x = setting.get<float>("OffsetX");
            g.offset.y = setting.get<float>("OffsetY");
            float x = setting.get<float>("ScaleX"), y = setting.get<float>("ScaleY");
            if (x != 1f || y != 1f)
                g.scale = new Vector2(setting.get<float>("ScaleX"), setting.get<float>("ScaleY"));
            else g.scale = null;
        }

        private static void WornGraphicBodyTypeDataOffsetOnly()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("wornGraphicBodyTypeDataOffsetOnly");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<WornGraphicBodyTypeData>("OffsetX", "Offset X", 10, default(float), t => t.offset.x));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<WornGraphicBodyTypeData>("OffsetY", "Offset Y", 20, default(float), t => t.offset.y));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Worn Graphic Body Type Data");
            }
        }

        private static void mergeWornGraphicBodyTypeDataOffsetOnly(ref WornGraphicBodyTypeData g, ISettingsInstance setting)
        {
            if (setting == null) return;

            g.offset.x = setting.get<float>("OffsetX");
            g.offset.y = setting.get<float>("OffsetY");
        }

        private static void WornGraphicBodyTypeDataScaleOnly()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("wornGraphicBodyTypeDataScaleOnly");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<WornGraphicBodyTypeData>("ScaleX", "Scale X", 30, 1f, t => t.Scale.x));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<WornGraphicBodyTypeData>("ScaleY", "Scale Y", 40, 1f, t => t.Scale.y));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Worn Graphic Body Type Data");
            }
        }

        private static void mergeWornGraphicBodyTypeDataScaleOnly(ref WornGraphicBodyTypeData g, ISettingsInstance setting)
        {
            if (setting == null) return;
            
            float x = setting.get<float>("ScaleX"), y = setting.get<float>("ScaleY");
            if (x != 1f || y != 1f)
                g.scale = new Vector2(setting.get<float>("ScaleX"), setting.get<float>("ScaleY"));
            else g.scale = null;
        }

        private static void apparel() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("apparel");
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("wearPerDay","WearPerDay",10,default(float),t=>t.apparel.wearPerDay));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("GenerateAllowChance", "Spawn Chance", 12, 1f, t => t.generateAllowChance));

            //SelectMany field is needed for these

            props.fields.Add(new SettingsFieldPropertiesListSelectable("layers","Layers",20,typeof(List<string>),new List<string>(),
			                                                          apparel=>((ThingDef)apparel).apparel.layers.Select(al=>al.defName).ToList(),
			                                                          f=>DefDatabase<ApparelLayerDef>.AllDefsListForReading.Select(d=>(object)d.defName),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<string>>("layers")[listIndex] = (string)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          entry=>(string)entry,()=>ApparelLayerDefOf.Belt.defName,
			                                                          true));
			props.fields.Add(new SettingsFieldPropertiesListSelectable("bodyPartGroups","Body Parts Covered",30,typeof(List<string>),new List<string>(),
			                                                          apparel=>((ThingDef)apparel).apparel.bodyPartGroups.Select(bpg=>bpg.defName).ToList(),
			                                                          f=>DefDatabase<BodyPartGroupDef>.AllDefsListForReading.Select(bpg=>(object)bpg.defName),
			                                                          flow=>{
			                                                          	ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                                   		int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
			                                                          	return listEntry=>{
	                                                                   	flow.addPostRenderCallback(fl=>{setting.get<List<string>>("bodyPartGroups")[listIndex] = (string)listEntry; setting.markAsChanged();});
			                                                          	return true;
                                                                   		};},
			                                                          entry=>DefDatabase<BodyPartGroupDef>.GetNamed((string)entry).label,()=>DefDatabase<BodyPartGroupDef>.AllDefsListForReading.First().defName,
			                                                          true));
			props.fields.Add(new SettingsFieldPropertiesListPrimitive("tags","Tags",35,typeof(List<string>),new List<string>(),
			                                                          t=> ((ThingDef)t).apparel.tags != null ? ((ThingDef)t).apparel.tags.ToList() : new List<string>(),
			                                                          null,()=>"",true));

            props.fields.Add(new SettingsFieldPropertiesSelectable("Tradeability", "Tradeability", 36, typeof(string), Enum.GetName(typeof(Tradeability), Tradeability.All),
                    t => Enum.GetName(typeof(Tradeability), ((ThingDef)t).tradeability),
                    f => from Tradeability d in Enum.GetValues(typeof(Tradeability)) orderby Enum.GetName(typeof(Tradeability), d) select (object)Enum.GetName(typeof(Tradeability), d),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<string>("Tradeability", (string)def); return true; },
                    o => (string)o, false));

            props.fields.Add(new SettingsFieldPropertiesListPrimitive("TradeTags","Trade Tags",37,typeof(List<string>),new List<string>(),
			                                                          t=> ((ThingDef)t).tradeTags != null ? ((ThingDef)t).tradeTags.ToList() : new List<string>(),
			                                                          null,()=>"",true));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("StuffCategories", "Stuff Categories", 50, typeof(List<StuffCategoryDef>), new List<StuffCategoryDef>(),
                                            h => ((ThingDef)h).stuffCategories == null ? new List<StuffCategoryDef>() : ((ThingDef)h).stuffCategories.ToList(),
                                            f => DefDatabase<StuffCategoryDef>.AllDefsListForReading.Cast<object>(),
                                            flow => {
                                                ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                                                return listEntry => {
                                                    flow.addPostRenderCallback(fl => { setting.get<List<StuffCategoryDef>>("StuffCategories")[listIndex] = (StuffCategoryDef)listEntry; setting.markAsChanged(); });
                                                    return true;
                                                };
                                            },
                                            SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<StuffCategoryDef>.AllDefsListForReading.First(),
                                            true));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("ThingCategories", "Categories", 60, typeof(List<ThingCategoryDef>),
                   new List<ThingCategoryDef>(),
                   h => ((ThingDef)h).thingCategories == null ? new List<ThingCategoryDef>() : ((ThingDef)h).thingCategories.ToList(),
                   f => DefDatabase<ThingCategoryDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).ToList<object>(),
                flow => {
                    ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                    int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                    return listEntry => {
                        flow.addPostRenderCallback(fl => {
                            setting.get<List<ThingCategoryDef>>("ThingCategories")[listIndex] = (ThingCategoryDef)listEntry; setting.markAsChanged();
                        });
                        return true;
                    };
                }, SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<ThingCategoryDef>.AllDefsListForReading.First(), true));

            GraphicData();
            props.fields.Add(new SettingsFieldPropertiesSettings("GraphicData", "Graphic Data", 90, typeof(SettingsInstance), null,
                t => ((ThingDef)t).graphicData != null ? new SettingsInstance("graphicData", ((ThingDef)t).graphicData, null, null) : null, "graphicData"));

            WornGraphicData();
            props.fields.Add(new SettingsFieldPropertiesSettings("WornGraphicData", "Worn Graphic Data", 91, typeof(SettingsInstance), null,
                t => ((ThingDef)t).apparel.wornGraphicData != null ? new SettingsInstance("wornGraphicData", ((ThingDef)t).apparel.wornGraphicData, null, null) : null, "wornGraphicData")
            { createNewEntry = () => new SettingsInstance("wornGraphicData", new WornGraphicData(), null, null), allowRemoval = true });

            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("ShellRenderedBehindHead", "Shell Rendered Behind Head", 92, default(bool), t => t.apparel.shellRenderedBehindHead));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("ShellCoversHead", "Shell Covers Head", 93, default(bool), t => t.apparel.shellCoversHead));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("ForceRenderUnderHair", "Force Render Under Hair", 94, default(bool), t => t.apparel.forceRenderUnderHair));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("HatRenderedFrontOfFace", "Hat Rendered Front Of Face", 95, default(bool), t => t.apparel.hatRenderedFrontOfFace));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("HatRenderedBehindHead", "Hat Rendered Behind Head", 96, default(bool), t => t.apparel.hatRenderedBehindHead));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("HatRenderedAboveBody", "Hat Rendered Above Body", 97, default(bool), t => t.apparel.hatRenderedAboveBody));

            StatModifier();
			props.fields.Add(new SettingsFieldPropertiesListSettings("equippedStatOffsets","Wearer Stats",100,typeof(List<SettingsInstance>),null,
                        t=> ((ThingDef)t).equippedStatOffsets.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).equippedStatOffsets.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
						"statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
			props.fields.Add(new SettingsFieldPropertiesListSettings("statBases","Item Stats",110,typeof(List<SettingsInstance>),null,
                        t=> ((ThingDef)t).statBases.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).statBases.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
			
			props.mergers.Add(setting => {
              	ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
            	
            	t.apparel.wearPerDay = setting.get<float>("wearPerDay");
                t.generateAllowChance = setting.get<float>("GenerateAllowChance");
                t.apparel.shellRenderedBehindHead = setting.get<bool>("ShellRenderedBehindHead");
                t.apparel.shellCoversHead = setting.get<bool>("ShellCoversHead");
                t.apparel.forceRenderUnderHair = setting.get<bool>("ForceRenderUnderHair");
                t.apparel.hatRenderedFrontOfFace = setting.get<bool>("HatRenderedFrontOfFace");
                t.apparel.hatRenderedBehindHead = setting.get<bool>("HatRenderedBehindHead");
                t.apparel.hatRenderedAboveBody = setting.get<bool>("HatRenderedAboveBody");
                Tradeability tradeability;
                try
                {
                    tradeability = (Tradeability)Enum.Parse(typeof(Tradeability), setting.get<string>("Tradeability"));
                }
                catch (ArgumentException)
                {
                    tradeability = Tradeability.All;
                }
                t.tradeability = tradeability;
                List<string> lsDefNames = setting.get<List<string>>("layers");
                List<ApparelLayerDef> layers = lsDefNames.Where(s => DefDatabase<ApparelLayerDef>.GetNamedSilentFail(s) != null).Select(s=> DefDatabase<ApparelLayerDef>.GetNamed(s)).ToList();
            	if ( layers.Any() ) {
            		if ( t.apparel.layers == null ) t.apparel.layers = new List<ApparelLayerDef>();
        			t.apparel.layers.Clear();
            		t.apparel.layers.AddRange(layers);
            	}
            	List<string> bpgsDefNames = setting.get<List<string>>("bodyPartGroups");
            	IEnumerable<BodyPartGroupDef> bpgs = DefDatabase<BodyPartGroupDef>.AllDefsListForReading.Where(bpg=>bpgsDefNames.Contains(bpg.defName));
            	if ( bpgs != null ) {
            		if ( t.apparel.bodyPartGroups == null ) t.apparel.bodyPartGroups = new List<BodyPartGroupDef>();
            		t.apparel.bodyPartGroups.Clear();
            		t.apparel.bodyPartGroups.AddRange(bpgs);
            	}
            	List<string> tags = setting.get<List<string>>("tags");
              	if ( tags != null ) {
              		if ( t.apparel.tags == null ) t.apparel.tags = new List<string>();
              		else t.apparel.tags.Clear();
              		t.apparel.tags.AddRange(tags);
              	}
                List<StuffCategoryDef> stuffCategories = setting.get<List<StuffCategoryDef>>("StuffCategories");
                if (stuffCategories != null)
                {
                    if (t.stuffCategories == null) t.stuffCategories = new List<StuffCategoryDef>();
                    else t.stuffCategories.Clear();
                    t.stuffCategories.AddRange(stuffCategories);
                }
                mergeThingCategories(t, setting);
                List<SettingsInstance> equippedStatOffsets = setting.get<List<SettingsInstance>>("equippedStatOffsets");
            	if ( equippedStatOffsets != null ) {
            		if ( t.equippedStatOffsets == null ) t.equippedStatOffsets = new List<StatModifier>();
            		t.equippedStatOffsets.Clear();
            		foreach ( ISettingsInstance s in equippedStatOffsets ) {
            			t.equippedStatOffsets.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
            		}
            	}
            	List<SettingsInstance> statBases = setting.get<List<SettingsInstance>>("statBases");
            	if ( statBases != null ) {
            		if ( t.statBases == null ) t.statBases = new List<StatModifier>();
            		t.statBases.Clear();
            		foreach ( ISettingsInstance s in statBases ) {
            			t.statBases.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
            		}
            	}
            	List<string> tradeTags = setting.get<List<string>>("TradeTags");
              	if ( tradeTags != null ) {
              		if ( t.tradeTags == null ) t.tradeTags = new List<string>();
              		else t.tradeTags.Clear();
              		t.tradeTags.AddRange(tradeTags);
              	}

                mergeGraphicData(t.graphicData, setting.get<ISettingsInstance>("GraphicData"));
                ISettingsInstance wornGraphicData = setting.get<ISettingsInstance>("WornGraphicData");
                if (t.apparel.wornGraphicData == null && wornGraphicData != null) t.apparel.wornGraphicData = new WornGraphicData();
                if (wornGraphicData != null)
                    mergeWornGraphicData(t.apparel.wornGraphicData, wornGraphicData);
                else t.apparel.wornGraphicData = null;

                updatePawns = true;

                t.ClearCachedData();
            });
			
			QOLMod.addApplySettingsListener(mod => PawnApparelGenerator.Reset());
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ThingDef>("Apparel","Select Apparel",()=>DefDatabase<ThingDef>.AllDefsListForReading.Where(t=>t.IsApparel).ToList(),
				def => def.defName, DefLabelProducerGeneric<ThingDef>.LabelProducerNotNull,key => DefDatabase<ThingDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}
		
		private static void items() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("items");
            props.props.Add(new SettingsPropertyProperties("FilledPartial", o => ((ThingDef)o).Fillage == FillCategory.Partial));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveString<ThingDef>("Label", "Label", 5, null, t => t.label));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("StackLimit", "Stack Limit", 9, 1, t => t.stackLimit));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("PathCost","Path Cost",10,0,t=>t.pathCost));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("PathCostIgnoreRepeat", "Path-Cost ignore repeat", 20, default(bool), t => t.pathCostIgnoreRepeat));
            
            props.fields.Add(new SettingsFieldPropertiesSelectable("Passability", "Passability", 25, typeof(string), Enum.GetName(typeof(Traversability), default(Traversability)),
                    t => Enum.GetName(typeof(Traversability), ((ThingDef)t).passability),
                    f => from Traversability d in Enum.GetValues(typeof(Traversability)) orderby Enum.GetName(typeof(Traversability), d) select (object)Enum.GetName(typeof(Traversability), d),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<string>("Passability", (string)def); return true; },
                    o => (string)o, false));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("DrawGUIOverlay", "GUI Overlay", 27, default(bool), t => t.drawGUIOverlay));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("DrawGUIOverlayQuality", "GUI Overlay Quality", 28, default(bool), t => t.drawGUIOverlayQuality));

            props.fields.Add(new SettingsFieldPropertiesListPrimitive("TradeTags","Trade Tags",30,typeof(List<string>),new List<string>(),
																	  t=> ((ThingDef)t).tradeTags != null ? ((ThingDef)t).tradeTags.ToList() : new List<string>(),
																	  null,()=>"",true));
            props.fields.Add(new SettingsFieldPropertiesSelectable("Tradeability", "Tradeability", 40, typeof(string), Enum.GetName(typeof(Tradeability), Tradeability.All),
                    t => Enum.GetName(typeof(Tradeability), ((ThingDef)t).tradeability),
                    f => from Tradeability d in Enum.GetValues(typeof(Tradeability)) orderby Enum.GetName(typeof(Tradeability), d) select (object)Enum.GetName(typeof(Tradeability), d),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<string>("Tradeability", (string)def); return true; },
                    o => (string)o, false));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("FillPercent", "Fillage", 50, default(float), t => t.fillPercent, 0.01f, 0.99f)
                .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("FilledPartial")));

            StatModifier();
			props.fields.Add(new SettingsFieldPropertiesListSettings("statBases","Item Stats",110,typeof(List<SettingsInstance>),null,
						t=> ((ThingDef)t).statBases.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).statBases.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
						"statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("ThingCategories", "Categories", 120, typeof(List<ThingCategoryDef>),
                   new List<ThingCategoryDef>(),
                   h => ((ThingDef)h).thingCategories == null ? new List<ThingCategoryDef>() : ((ThingDef)h).thingCategories.ToList(),
                   f => DefDatabase<ThingCategoryDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).ToList<object>(),
                flow => {
                    ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                    int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                    return listEntry => {
                        flow.addPostRenderCallback(fl => {
                            setting.get<List<ThingCategoryDef>>("ThingCategories")[listIndex] = (ThingCategoryDef)listEntry; setting.markAsChanged();
                        });
                        return true;
                    };
                }, SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<ThingCategoryDef>.AllDefsListForReading.First(), true));

            props.mergers.Add(setting => {
				ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
                setting.resolveProps(props, t);

                string configuredLabel = setting.get<string>("Label");
                if (!configuredLabel.NullOrEmpty() && configuredLabel.Trim().Length != 0)
                {
                    t.label = configuredLabel;
                    DefCachedLabelCapSet(t, null);
                }
                t.stackLimit = setting.get<int>("StackLimit");
                t.drawGUIOverlay = setting.get<bool>("DrawGUIOverlay");
                t.drawGUIOverlayQuality = setting.get<bool>("DrawGUIOverlayQuality");
                t.pathCost = setting.get<int>("PathCost");
              	t.pathCostIgnoreRepeat = setting.get<bool>("PathCostIgnoreRepeat");
                t.fillPercent = setting.get<float>("FillPercent");

                Traversability passability;
                try
                {
                    passability = (Traversability)Enum.Parse(typeof(Traversability), setting.get<string>("Passability"));
                }
                catch (ArgumentException)
                {
                    passability = default(Traversability);
                }
                t.passability = passability;

                Tradeability tradeability;
                try
                {
                    tradeability = (Tradeability)Enum.Parse(typeof(Tradeability), setting.get<string>("Tradeability"));
                }
                catch (ArgumentException)
                {
                    tradeability = Tradeability.All;
                }
                t.tradeability = tradeability;

                List<SettingsInstance> statBases = setting.get<List<SettingsInstance>>("statBases");
				if ( statBases != null ) {
					if ( t.statBases == null ) t.statBases = new List<StatModifier>();
					t.statBases.Clear();
					foreach ( ISettingsInstance s in statBases ) {
						t.statBases.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
					}
				}

                mergeThingCategories(t, setting);

                List<string> tradeTags = setting.get<List<string>>("TradeTags");
				if ( tradeTags != null ) {
					if ( t.tradeTags == null ) t.tradeTags = new List<string>();
					else t.tradeTags.Clear();
					t.tradeTags.AddRange(tradeTags);
				}

                t.ClearCachedData();
            });
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ThingDef>("Items","Select Item",()=>DefDatabase<ThingDef>.AllDefsListForReading.Where(
				t=>!t.IsApparel && !t.IsWeapon && !t.IsStuff && !t.IsIngestible && !t.IsBlueprint && t.plant == null && t.building == null && 
				t.category == ThingCategory.Item 
				).ToList(),
				def => def.defName, DefLabelProducerGeneric<ThingDef>.LabelProducerNotNull,key => DefDatabase<ThingDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}

        private static void filth()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("filth");

            props.fields.Add(new SettingsFieldPropertiesPrimitiveString<ThingDef>("Label", "Label", 5, null, t => t.label));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("PathCost", "Path Cost", 10, 0, t => t.pathCost));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("PathCostIgnoreRepeat", "Path-Cost ignore repeat", 20, default(bool), t => t.pathCostIgnoreRepeat));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("UseHitPoints", "Label", 30, true, t => t.useHitPoints));

            StatModifier();
            props.fields.Add(new SettingsFieldPropertiesListSettings("statBases", "Item Stats", 100, typeof(List<SettingsInstance>), null,
                        t => ((ThingDef)t).statBases.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).statBases.Where(tc => tc != null).Select(tc => new SettingsInstance("statModifier", tc, null, null)).ToList(),
                        "statModifier", () => new SettingsInstance("statModifier", new StatModifier() { stat = DefDatabase<StatDef>.AllDefsListForReading.First(), value = 0 }, null, null), true));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("CleaningWorkToReduceThickness", "Cleaning Work", 200, 35f, t => t.filth.cleaningWorkToReduceThickness));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("CanFilthAttach", "Can Attach", 210, default(bool), t => t.filth.canFilthAttach));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("RainWashes", "Cleaned by Rain", 220, default(bool), t => t.filth.rainWashes));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("AllowsFire", "Allows Fire", 230, true, t => t.filth.allowsFire));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("IgnoreFilthMultiplierStat", "Ignore Filth", 230, default(bool), t => t.filth.ignoreFilthMultiplierStat));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("MaxThickness", "Max Thickness", 230, 100, t => t.filth.maxThickness));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("DisappearsInDaysMin", "Duration Min", 240, 0f, t => t.filth.disappearsInDays.min));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("DisappearsInDaysMax", "Duration Max", 250, 0f, t => t.filth.disappearsInDays.max));
            props.fields.Add(new SettingsFieldPropertiesListSelectable("PlacementMask", "Placement", 260, typeof(List<string>), new List<string>(),
                    t => GetEnumFlagsNames<FilthSourceFlags>(((ThingDef)t).filth.placementMask),
                    f => Enum.GetNames(typeof(FilthSourceFlags)).Where(e => e != "None").Cast<object>(),
                    flow => {
                        ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                        int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                        return listEntry => {
                            flow.addPostRenderCallback(fl => { setting.get<List<string>>("PlacementMask")[listIndex] = (string)listEntry; setting.markAsChanged(); });
                            return true;
                        };
                    },
                    entry => (string)entry,
                    () => Enum.GetName(typeof(FilthSourceFlags), FilthSourceFlags.None),
                    true));

            props.mergers.Add(setting => {
                ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
                setting.resolveProps(props, t);

                string configuredLabel = setting.get<string>("Label");
                if (!configuredLabel.NullOrEmpty() && configuredLabel.Trim().Length != 0)
                {
                    t.label = configuredLabel;
                    DefCachedLabelCapSet(t, null);
                }
                t.pathCost = setting.get<int>("PathCost");
                t.pathCostIgnoreRepeat = setting.get<bool>("PathCostIgnoreRepeat");
                t.useHitPoints = setting.get<bool>("UseHitPoints");
                t.filth.cleaningWorkToReduceThickness = setting.get<float>("CleaningWorkToReduceThickness");
                t.filth.canFilthAttach = setting.get<bool>("CanFilthAttach");
                t.filth.rainWashes = setting.get<bool>("RainWashes");
                t.filth.allowsFire = setting.get<bool>("AllowsFire");
                t.filth.ignoreFilthMultiplierStat = setting.get<bool>("IgnoreFilthMultiplierStat");
                t.filth.maxThickness = setting.get<int>("MaxThickness");
                t.filth.disappearsInDays.min = setting.get<float>("DisappearsInDaysMin");
                t.filth.disappearsInDays.max = setting.get<float>("DisappearsInDaysMax");

                List<SettingsInstance> statBases = setting.get<List<SettingsInstance>>("statBases");
                if (statBases != null)
                {
                    if (t.statBases == null) t.statBases = new List<StatModifier>();
                    t.statBases.Clear();
                    foreach (ISettingsInstance s in statBases)
                    {
                        t.statBases.Add(new StatModifier() { stat = s.get<StatDef>("Stat"), value = s.get<float>("Value") });
                    }
                }

                FilthSourceFlags filthSource;
                try
                {
                    List<string> lstFilthSources = setting.get<List<string>>("PlacementMask");
                    if (lstFilthSources.NullOrEmpty())
                    {
                        filthSource = FilthSourceFlags.None;
                    }
                    else
                    {
                        filthSource = (FilthSourceFlags)Enum.Parse(typeof(FilthSourceFlags), lstFilthSources[0]);
                        for (int i = 1; i < lstFilthSources.Count; i++)
                        {
                            filthSource |= (FilthSourceFlags)Enum.Parse(typeof(FilthSourceFlags), lstFilthSources[i]);
                        }
                    }
                }
                catch (ArgumentException)
                {
                    filthSource = FilthSourceFlags.None;
                }
                t.filth.placementMask = filthSource;
            });

            props.visualization = new SettingsVisualizationMenuSelectEdit<ThingDef>("Filth", "Select Filth", () => DefDatabase<ThingDef>.AllDefsListForReading.Where(
                  t => t.IsFilth).ToList(),
                def => def.defName, DefLabelProducerGeneric<ThingDef>.LabelProducerNotNull, key => DefDatabase<ThingDef>.GetNamed(key),
                f => "Edit \"" + ((ISettingsInstance)f.flowScope["edit"]).getLabel() + "\"");
        }

        private static void abilityEffect() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("abilityEffect");
			if ( props.fields.NullOrEmpty() ) {
				/*
				public SoundDef sound;
				public string customLetterLabel;
				public string customLetterText;
				public string message;
				public MessageTypeDef messageType;
				 */
				props.fields.Add(new SettingsFieldPropertiesPrimitiveString<CompProperties_AbilityEffect>("label","Label",0,"Effect", ae => ae.GetType().Name)
				                 .setHidden(f=>true));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<CompProperties_AbilityEffect>("goodwillImpact","Goodwill",10,default(int), ae => ae.goodwillImpact));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_AbilityEffect>("psychic","Psychic",20,default(bool), ae => ae.psychic));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_AbilityEffect>("applicableToMechs","Applicable To Mechs",30,true, ae => ae.applicableToMechs));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_AbilityEffect>("applyGoodwillImpactToLodgers","Apply GW To Lodgers",40,true, ae => ae.applyGoodwillImpactToLodgers));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<CompProperties_AbilityEffect>("clamorRadius","Clamor Radius",50,default(int), ae => ae.clamorRadius));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_AbilityEffect>("screenShakeIntensity","Screen Shake Intensity",60,default(float), ae => ae.screenShakeIntensity, 0f));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_AbilityEffect>("sendLetter","Send Letter",70,true, ae => ae.sendLetter));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_AbilityEffect>("availableWhenTargetIsWounded","Available When Target is Wounded",80,true, ae => ae.availableWhenTargetIsWounded));
				props.fields.Add(new SettingsFieldPropertiesSelectable("clamorType","Clamor Type",90,typeof(ClamorDef),default(ClamorDef),ae => ((CompProperties_AbilityEffect)ae).clamorType,
                                                       f=>DefDatabase<ClamorDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ClamorDef>("clamorType",(ClamorDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				
				props.dynamicLabel = si => si.get<string>("label");
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Ability Effect");
			}
		}
		
		private static void targetingParameters() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("targetingParameters");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("onlyTargetControlledPawns","Only Controlled Pawns",10,default(bool), tp => tp.onlyTargetControlledPawns));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("canTargetSelf","Target Self",20,default(bool), tp => tp.canTargetSelf));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("canTargetMechs","Target Mechs",30,true, tp => tp.canTargetMechs));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("canTargetAnimals","Target Animals",40,true, tp => tp.canTargetAnimals));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("canTargetPawns","Target Pawns",50,true, tp => tp.canTargetPawns));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("canTargetLocations","Target Locations",60,default(bool), tp => tp.canTargetLocations));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("canTargetFires","Target Fires",70,default(bool), tp => tp.canTargetFires));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("canTargetBuildings","Target Buildings",80,true, tp => tp.canTargetBuildings));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("canTargetItems","Target Items",90,default(bool), tp => tp.canTargetItems));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("canTargetHumans","Target Humans",100,true, tp => tp.canTargetHumans));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("onlyTargetFlammables","Only Flammables",110,default(bool), tp => tp.onlyTargetFlammables));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("mustBeSelectable","Only Selectables",120,default(bool), tp => tp.mustBeSelectable));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("neverTargetDoors","Never Doors",130,default(bool), tp => tp.neverTargetDoors));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("neverTargetIncapacitated","Never Incapacitated",140,default(bool), tp => tp.neverTargetIncapacitated));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("onlyTargetThingsAffectingRegions","Only Regionals",150,default(bool), tp => tp.onlyTargetThingsAffectingRegions));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("onlyTargetDamagedThings","Only Damaged",160,default(bool), tp => tp.onlyTargetDamagedThings));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("mapObjectTargetsMustBeAutoAttackable","Must be Autoattackable",170,true, tp => tp.mapObjectTargetsMustBeAutoAttackable));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("onlyTargetIncapacitatedPawns","Only Incapacitated",180,default(bool), tp => tp.onlyTargetIncapacitatedPawns));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TargetingParameters>("onlyTargetColonists","Only Colonists",190,default(bool), tp => tp.onlyTargetColonists));
				props.fields.Add(new SettingsFieldPropertiesSelectable("thingCategory","Thing Category",200,typeof(string),ThingCategory.None.ToString(),
			                                                       t=>Enum.GetName(typeof(ThingCategory),((TargetingParameters)t).thingCategory),
			                                                       f=>from ThingCategory d in Enum.GetValues(typeof(ThingCategory)) orderby d.ToString() select (object)d.ToString(),
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("thingCategory",(string)def); return true; },
				                                             o=>(string)o,false));
				
				props.dynamicLabel = si => "Parameter";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Targeting Parameter");
			}
		}
		
		private static void verbProperties() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("verbProperties");

            if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<VerbProperties>("WarmupTime","Warmup Time",10,default(float), vp => vp.warmupTime));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<VerbProperties>("MinRange","Min Range",20,default(float), vp => vp.minRange));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<VerbProperties>("Range","Range",30,default(float), vp => vp.range));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<VerbProperties>("ForcedMissRadius","Forced Miss Radius",40,default(float), vp => (float)fiForcedMissRadius.GetValue(vp)));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<VerbProperties>("BurstShotCount","Burst Shots",50,default(int), vp => vp.burstShotCount));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<VerbProperties>("TicksBetweenBurstShots","Ticks between burst shots",60,15, vp => vp.ticksBetweenBurstShots));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<VerbProperties>("MuzzleFlashScale","Muzzle flash scale",70,default(float), vp => vp.muzzleFlashScale));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<VerbProperties>("MeleeArmorPenetrationBase","Melee Armor Penetration-Base",80,default(float), vp => vp.meleeArmorPenetrationBase));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<VerbProperties>("EnsureLinkedBodyPartsGroupAlwaysUsable","Linked Body Parts Group Always Usable",90,default(bool), vp => vp.ensureLinkedBodyPartsGroupAlwaysUsable));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<VerbProperties>("RequiresLineOfSight","Requires LOS",100,default(bool), vp => vp.requireLineOfSight));
				
				props.fields.Add(new SettingsFieldPropertiesSelectable("LinkedBodyPartsGroup","Linked Body Parts Group",110,typeof(BodyPartGroupDef),default(BodyPartGroupDef),vp=>((VerbProperties)vp).linkedBodyPartsGroup,
                                                       f=>DefDatabase<BodyPartGroupDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<BodyPartGroupDef>("LinkedBodyPartsGroup",(BodyPartGroupDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			
				props.fields.Add(new SettingsFieldPropertiesSelectable("DefaultProjectile","Projectile",120,typeof(ThingDef),default(ThingDef),
                                                       vp=>((VerbProperties)vp).defaultProjectile,
			                                           f=>DefDatabase<ThingDef>.AllDefsListForReading.Where(d=>d.category == ThingCategory.Projectile)
			                                            	.OrderBy(d=>DefLabelProducerGeneric<ThingDef>.LabelProducerUnique(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("DefaultProjectile",(ThingDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducerUnique,false));
				targetingParameters();
				props.fields.Add(new SettingsFieldPropertiesSettings("TargetParams","Target Params",200,typeof(SettingsInstance),null,
			                                                     vp=>((VerbProperties)vp).targetParams != null ? new SettingsInstance("targetingParameters", ((VerbProperties)vp).targetParams, null, null) : null,"targetingParameters"));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Verb Properties");
			}
		}
		
		private static void abilities() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("abilities");

			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<AbilityDef>("level","Level",10,0,a=>a.level));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<AbilityDef>("detectionChanceOverride","Detection Chance Override",20,-1f,a=>a.detectionChanceOverride));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<AbilityDef>("canUseAoeToGetTargets","Use AOE",30,true,a=>a.canUseAoeToGetTargets));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<AbilityDef>("targetRequired","Target Required",40,true,a=>a.targetRequired));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<AbilityDef>("cooldownTicksRangeMin","Cooldown Min",50,0,a=>a.cooldownTicksRange.min));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<AbilityDef>("cooldownTicksRangeMax","Cooldown Max",60,0,a=>a.cooldownTicksRange.max));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<AbilityDef>("sendLetterOnCooldownComplete","Send Letter",70,default(bool),a=>a.sendLetterOnCooldownComplete));
			
			StatModifier();
			props.fields.Add(new SettingsFieldPropertiesListSettings("statBases","Stats",100,typeof(List<SettingsInstance>),null,
						a=> ((AbilityDef)a).statBases.NullOrEmpty() ? new List<SettingsInstance>() : ((AbilityDef)a).statBases.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
						"statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
			
			abilityEffect();
			props.fields.Add(new SettingsFieldPropertiesListSettings("comps","Components",110,typeof(List<SettingsInstance>),null,
                         a=> ((AbilityDef)a).comps.NullOrEmpty() ? new List<SettingsInstance>() : 
                         ((AbilityDef)a).comps.OfType<CompProperties_AbilityEffect>().Select(tc=>new SettingsInstance("abilityEffect",tc,null,null)).ToList(),
                        "abilityEffect",null,false));
			
			verbProperties();
			props.fields.Add(new SettingsFieldPropertiesSettings("VerbProperties","Verb Properties",200,typeof(SettingsInstance),null,
                         a=> ((AbilityDef)a).verbProperties != null ? new SettingsInstance("verbProperties", ((AbilityDef)a).verbProperties, null, null) : null,"verbProperties"));
			
			props.mergers.Add(setting => {
				AbilityDef a = DefDatabase<AbilityDef>.GetNamed(setting.getKey());
				a.level = setting.get<int>("level");
				a.detectionChanceOverride = setting.get<float>("detectionChanceOverride");
				a.canUseAoeToGetTargets = setting.get<bool>("canUseAoeToGetTargets");
				a.targetRequired = setting.get<bool>("targetRequired");
				a.cooldownTicksRange = new IntRange(setting.get<int>("cooldownTicksRangeMin"),setting.get<int>("cooldownTicksRangeMax"));
				a.sendLetterOnCooldownComplete = setting.get<bool>("sendLetterOnCooldownComplete");
				
				List<SettingsInstance> statBases = setting.get<List<SettingsInstance>>("statBases");
				if ( statBases != null ) {
					if ( a.statBases == null ) a.statBases = new List<StatModifier>();
					a.statBases.Clear();
					foreach ( ISettingsInstance s in statBases ) {
						a.statBases.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
					}
				}
				
				if ( a.verbProperties != null ) {
              		SettingsInstance si = setting.get<SettingsInstance>("VerbProperties");
              		if ( si != null ) {
              			VerbProperties vp = a.verbProperties;
              			vp.warmupTime = si.get<float>("WarmupTime");
              			vp.minRange = si.get<float>("MinRange");
              			vp.range = si.get<float>("Range");
                        fiForcedMissRadius.SetValue(vp, si.get<float>("ForcedMissRadius"));
              			vp.burstShotCount = si.get<int>("BurstShotCount");
              			vp.ticksBetweenBurstShots = si.get<int>("TicksBetweenBurstShots");
              			vp.muzzleFlashScale = si.get<float>("MuzzleFlashScale");
              			vp.meleeArmorPenetrationBase = si.get<float>("MeleeArmorPenetrationBase");
              			vp.ensureLinkedBodyPartsGroupAlwaysUsable = si.get<bool>("EnsureLinkedBodyPartsGroupAlwaysUsable");
              			vp.requireLineOfSight = si.get<bool>("RequiresLineOfSight");
              			vp.linkedBodyPartsGroup = si.get<BodyPartGroupDef>("LinkedBodyPartsGroup");
              			vp.defaultProjectile = si.get<ThingDef>("DefaultProjectile");
              			
              			if ( vp.targetParams != null ) {
              				SettingsInstance siTargetParams = si.get<SettingsInstance>("TargetParams");
              				TargetingParameters tp = vp.targetParams;
              				
              				tp.onlyTargetControlledPawns = siTargetParams.get<bool>("onlyTargetControlledPawns");
              				tp.canTargetSelf = siTargetParams.get<bool>("canTargetSelf");
              				tp.canTargetMechs = siTargetParams.get<bool>("canTargetMechs");
              				tp.canTargetAnimals = siTargetParams.get<bool>("canTargetAnimals");
              				tp.canTargetPawns = siTargetParams.get<bool>("canTargetPawns");
              				tp.canTargetLocations = siTargetParams.get<bool>("canTargetLocations");
              				tp.canTargetFires = siTargetParams.get<bool>("canTargetFires");
              				tp.canTargetBuildings = siTargetParams.get<bool>("canTargetBuildings");
              				tp.canTargetItems = siTargetParams.get<bool>("canTargetItems");
              				tp.canTargetHumans = siTargetParams.get<bool>("canTargetHumans");
              				tp.onlyTargetFlammables = siTargetParams.get<bool>("onlyTargetFlammables");
              				tp.mustBeSelectable = siTargetParams.get<bool>("mustBeSelectable");
              				tp.neverTargetDoors = siTargetParams.get<bool>("neverTargetDoors");
              				tp.neverTargetIncapacitated = siTargetParams.get<bool>("neverTargetIncapacitated");
              				tp.onlyTargetThingsAffectingRegions = siTargetParams.get<bool>("onlyTargetThingsAffectingRegions");
              				tp.onlyTargetDamagedThings = siTargetParams.get<bool>("onlyTargetDamagedThings");
              				tp.mapObjectTargetsMustBeAutoAttackable = siTargetParams.get<bool>("mapObjectTargetsMustBeAutoAttackable");
              				tp.onlyTargetIncapacitatedPawns = siTargetParams.get<bool>("onlyTargetIncapacitatedPawns");
              				tp.onlyTargetColonists = siTargetParams.get<bool>("onlyTargetColonists");
              				
              				ThingCategory thingCategory;
              				try {
				            	thingCategory = (ThingCategory)Enum.Parse(typeof(ThingCategory),siTargetParams.get<string>("thingCategory"));
			            	} catch ( ArgumentException ) {
			            		thingCategory = ThingCategory.None;
			            	}
              				tp.thingCategory = thingCategory;
              			}
              		}
              	}
				
				//processing the compatible comps
				if ( !a.comps.NullOrEmpty() ) {
					List<CompProperties_AbilityEffect> comps = a.comps.OfType<CompProperties_AbilityEffect>().ToList<CompProperties_AbilityEffect>();
					List<SettingsInstance> compsConfiguration = setting.get<List<SettingsInstance>>("comps");
					if ( compsConfiguration != null ) {
						if ( comps.Count == compsConfiguration.Count ) {
							for ( int i = 0; i < comps.Count; i++) {
								CompProperties_AbilityEffect ae = comps[i];
								SettingsInstance siAbilityEffect = compsConfiguration[i];
								ae.goodwillImpact = siAbilityEffect.get<int>("goodwillImpact");
								ae.psychic = siAbilityEffect.get<bool>("psychic");
								ae.applicableToMechs = siAbilityEffect.get<bool>("applicableToMechs");
								ae.applyGoodwillImpactToLodgers = siAbilityEffect.get<bool>("applyGoodwillImpactToLodgers");
								ae.clamorRadius = siAbilityEffect.get<int>("clamorRadius");
								ae.screenShakeIntensity = siAbilityEffect.get<float>("screenShakeIntensity");
								ae.sendLetter = siAbilityEffect.get<bool>("sendLetter");
								ae.availableWhenTargetIsWounded = siAbilityEffect.get<bool>("availableWhenTargetIsWounded");
								ae.clamorType = siAbilityEffect.get<ClamorDef>("clamorType");
							}
						} else {
							Log.Warning("Removing comps configuation for AbilityDef "+a.defName+"(\""+a.label+"\") since a comp was added or removed.");
							setting.markFieldAsNotConfigured("comps", false);
							setting.resetUnconfiguredFields();
						}
					}
				}
				
				//resetting the caches
				typeof(AbilityDef).GetField("requiredPsyfocusBandCached", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public).SetValue(a,-1);
				typeof(AbilityDef).GetField("cachedTooltip", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public).SetValue(a,null);
				typeof(AbilityDef).GetField("cachedTargets", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public).SetValue(a,null);
			});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<AbilityDef>("Abilities","Select Ability",()=>DefDatabase<AbilityDef>.AllDefsListForReading.ToList(),
				def => def.defName, DefLabelProducerGeneric<AbilityDef>.LabelProducerNotNull,key => DefDatabase<AbilityDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}
		
		private static void materials() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("material");
			
			StatModifier();
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("StackLimit","Stack Limit",10,1,t=>t.stackLimit));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("PathCost","Path Cost",20,0,t=>t.pathCost));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("UseHitPoints","Use Hit Points",30,default(bool),t=>t.useHitPoints));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("SmallVolume", "Small Volume", 31, default(bool), t => t.smallVolume));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("DeepCommonality","Deep Commonality",40,default(float),t=>t.deepCommonality));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("DeepCountPerPortion","Deep Count Per Portion",50,-1,t=>t.deepCountPerPortion));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("DeepCountPerCell","Deep Count Per Cell",60,300,t=>t.deepCountPerCell,0,65535));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("DeepLumpSizeMin","Deep Lump Size Min",70,default(int),t=>t.deepLumpSizeRange.min));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("DeepLumpSizeMax","Deep Lump Size Max",80,default(int),t=>t.deepLumpSizeRange.max));
			
			props.fields.Add(new SettingsFieldPropertiesListSettings("statBases","Item Stats",90,typeof(List<SettingsInstance>),null,
                        t=> ((ThingDef)t).statBases.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).statBases.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("ThingCategories", "Categories", 95, typeof(List<ThingCategoryDef>),
                   new List<ThingCategoryDef>(),
                   h => ((ThingDef)h).thingCategories == null ? new List<ThingCategoryDef>() : ((ThingDef)h).thingCategories.ToList(),
                   f => DefDatabase<ThingCategoryDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).ToList<object>(),
                flow => {
                    ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                    int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                    return listEntry => {
                        flow.addPostRenderCallback(fl => {
                            setting.get<List<ThingCategoryDef>>("ThingCategories")[listIndex] = (ThingCategoryDef)listEntry; setting.markAsChanged();
                        });
                        return true;
                    };
                }, SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<ThingCategoryDef>.AllDefsListForReading.First(), true));

            //stuff props
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("Commonality","Commonality",110,default(float),t=>t.stuffProps.commonality));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveColor<ThingDef>("Color","Color",120,new UnityEngine.Color(0.8f,0.8f,0.8f),t=>t.stuffProps.color));
            GraphicData();
            props.fields.Add(new SettingsFieldPropertiesSettings("GraphicData", "Graphic Data", 121, typeof(SettingsInstance), null,
                t => ((ThingDef)t).graphicData != null ? new SettingsInstance("graphicData", ((ThingDef)t).graphicData, null, null) : null, "graphicData"));
            props.fields.Add(new SettingsFieldPropertiesListSettings("StatFactors","Material Stat Factors",130,typeof(List<SettingsInstance>),null,
                        t=> ((ThingDef)t).stuffProps.statFactors.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).stuffProps.statFactors.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
			props.fields.Add(new SettingsFieldPropertiesListSettings("StatOffsets","Material Stat Offsets",140,typeof(List<SettingsInstance>),null,
                        t=> ((ThingDef)t).stuffProps.statOffsets.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).stuffProps.statOffsets.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
			
			props.fields.Add(new SettingsFieldPropertiesListSelectable("Categories","Material Categories",89,typeof(List<StuffCategoryDef>),new List<StuffCategoryDef>(),
						t=>((ThingDef)t).stuffProps.categories == null ? new List<StuffCategoryDef>() : ((ThingDef)t).stuffProps.categories.ToList(),
						f=>DefDatabase<StuffCategoryDef>.AllDefsListForReading.Cast<object>(),
						flow=>{
							ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
							int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
							return listEntry=>{
							flow.addPostRenderCallback(fl=>{setting.get<List<StuffCategoryDef>>("Categories")[listIndex] = (StuffCategoryDef)listEntry; setting.markAsChanged();});
							return true;
						};},
						SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<StuffCategoryDef>.AllDefsListForReading.First(),
						true));
			
			props.fields.Add(new SettingsFieldPropertiesListPrimitive("TradeTags","Trade Tags",100,typeof(List<string>),new List<string>(),
                      t=> ((ThingDef)t).tradeTags != null ? ((ThingDef)t).tradeTags.ToList() : new List<string>(),
                      null,()=>"",true));

            props.fields.Add(new SettingsFieldPropertiesSelectable("Tradeability", "Tradeability", 105, typeof(string), Enum.GetName(typeof(Tradeability), Tradeability.All),
                    t => Enum.GetName(typeof(Tradeability), ((ThingDef)t).tradeability),
                    f => from Tradeability d in Enum.GetValues(typeof(Tradeability)) orderby Enum.GetName(typeof(Tradeability), d) select (object)Enum.GetName(typeof(Tradeability), d),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<string>("Tradeability", (string)def); return true; },
                    o => (string)o, false));

            props.mergers.Add(setting => {
              	ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
            	
              	t.stackLimit = setting.get<int>("StackLimit");
              	t.pathCost = setting.get<int>("PathCost");
              	t.useHitPoints = setting.get<bool>("UseHitPoints");
              	t.deepCommonality = setting.get<float>("DeepCommonality");
              	t.deepCountPerPortion = setting.get<int>("DeepCountPerPortion");
              	t.deepCountPerCell = setting.get<int>("DeepCountPerCell");
              	t.deepLumpSizeRange.min = setting.get<int>("DeepLumpSizeMin");
              	t.deepLumpSizeRange.max = setting.get<int>("DeepLumpSizeMax");
              	t.stuffProps.commonality = setting.get<float>("Commonality");
              	t.stuffProps.color = setting.get<UnityEngine.Color>("Color");
                if (t.smallVolume != setting.get<bool>("SmallVolume"))
                {
                    t.smallVolume = setting.get<bool>("SmallVolume");
                    updateThingFilters = true;
                }


                  List<SettingsInstance> stats = setting.get<List<SettingsInstance>>("StatFactors");
            	if ( stats != null ) {
            		if ( t.stuffProps.statFactors == null ) t.stuffProps.statFactors = new List<StatModifier>();
            		t.stuffProps.statFactors.Clear();
            		foreach ( ISettingsInstance s in stats ) {
            			t.stuffProps.statFactors.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
            		}
            	}
              	stats = setting.get<List<SettingsInstance>>("StatOffsets");
            	if ( stats != null ) {
            		if ( t.stuffProps.statOffsets == null ) t.stuffProps.statOffsets = new List<StatModifier>();
            		t.stuffProps.statOffsets.Clear();
            		foreach ( ISettingsInstance s in stats ) {
            			t.stuffProps.statOffsets.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
            		}
            	}
              	List<StuffCategoryDef> stuffCategories = setting.get<List<StuffCategoryDef>>("Categories");
            	if ( stuffCategories != null ) {
            		if ( t.stuffProps.categories == null ) t.stuffProps.categories = new List<StuffCategoryDef>();
            		else t.stuffProps.categories.Clear();
            		t.stuffProps.categories.AddRange(stuffCategories);
            		
            		SettingsInit.updateThingFilters = true;
            	}
                mergeThingCategories(t, setting);
                List<string> tradeTags = setting.get<List<string>>("TradeTags");
              	if ( tradeTags != null ) {
              		if ( t.tradeTags == null ) t.tradeTags = new List<string>();
              		else t.tradeTags.Clear();
              		t.tradeTags.AddRange(tradeTags);
              	}

                Tradeability tradeability;
                try
                {
                    tradeability = (Tradeability)Enum.Parse(typeof(Tradeability), setting.get<string>("Tradeability"));
                }
                catch (ArgumentException)
                {
                    tradeability = Tradeability.All;
                }
                t.tradeability = tradeability;

                List<SettingsInstance> statBases = setting.get<List<SettingsInstance>>("statBases");
            	if ( statBases != null ) {
            		if ( t.statBases == null ) t.statBases = new List<StatModifier>();
            		t.statBases.Clear();
            		foreach ( ISettingsInstance s in statBases ) {
            			t.statBases.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
            		}
            	}

                mergeGraphicData(t.graphicData, setting.get<ISettingsInstance>("GraphicData"));

                if (!Find.Maps.NullOrEmpty())
                {
                    Find.Maps.ForEach(map => {
                        //update graphics for things placed.
                        map.listerThings.ThingsOfDef(t).ForEach(thingToUpdate => thingToUpdate.Notify_ColorChanged());
                    });
                }

                t.ClearCachedData();
              });
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ThingDef>("Materials","Select Material",()=>DefDatabase<ThingDef>.AllDefsListForReading.Where(t=>t.IsStuff).ToList(),
				def => def.defName, DefLabelProducerGeneric<ThingDef>.LabelProducerNotNull,key => DefDatabase<ThingDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			
			//After changing the color of the material all things made from this stuff should set "Thing.graphicInt" to null so the graphic is rebuilt.
		}
		
		private static void compDrug() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compDrug");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Drug>("Addictiveness","Addictiveness",10,default(float),t=>t.addictiveness, 0f));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Drug>("MinToleranceToAddict","Min Tolerance to Addiction",20,default(float),t=>t.minToleranceToAddict, 0f));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Drug>("ExistingAddictionSeverityOffset","Existing Addiction Severity Offset",30,0.1f,t=>t.existingAddictionSeverityOffset));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Drug>("NeedLevelOffset","Need Level Offset",40,1f,t=>t.needLevelOffset, 0f, 1f));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Drug>("LargeOverdoseChance","LargeOverdoseChance",50,default(float),t=>t.largeOverdoseChance,0f,1f));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Drug>("ListOrder","List Order",60,default(float),t=>t.listOrder));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Drug>("IsCombatEnhancingDrug","Combat Enhancing Drug",70,default(bool),t=>t.isCombatEnhancingDrug));
				
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Drug>("OverdoseSeverityOffsetMin","Min Overdose Severity Offset",80,default(float),t=>t.overdoseSeverityOffset.min));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Drug>("OverdoseSeverityOffsetMax","Max Overdose Severity Offset",90,default(float),t=>t.overdoseSeverityOffset.max));
				
				props.fields.Add(new SettingsFieldPropertiesSelectable("Chemical","Chemical",100,typeof(ChemicalDef),default(ChemicalDef),t=>((CompProperties_Drug)t).chemical,
			                                            f=>DefDatabase<ChemicalDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ChemicalDef>("Chemical",(ChemicalDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Drug Properties");
			}
		}
		
		private static void compHatcher() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compHatcher");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("HatcherPawn","Race",10,typeof(PawnKindDef),default(PawnKindDef),t=>((CompProperties_Hatcher)t).hatcherPawn,
			                                            f=>DefDatabase<PawnKindDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<PawnKindDef>("HatcherPawn",(PawnKindDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Hatcher>("HatcherDaystoHatch","Days to Hatch",20,1f,t=>t.hatcherDaystoHatch, 0.0001f));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Hatcher Properties");
			}
		}
		
		private static void foods() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("food");
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("Version","version",0,2,t=>2).setHidden(f=>true));
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveString<ThingDef>("Label","Label",5,null,t=>t.label));
			StatModifier();
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("StackLimit","Stack Limit",10,1,t=>t.stackLimit));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("PathCost","Path Cost",20,0,t=>t.pathCost));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("UseHitPoints","Use Hit Points",30,default(bool),t=>t.useHitPoints));
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("DeepCommonality","Deep Commonality",40,default(float),t=>t.deepCommonality));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("DeepCountPerPortion","Deep Count Per Portion",50,-1,t=>t.deepCountPerPortion));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("DeepCountPerCell","Deep Count Per Cell",60,300,t=>t.deepCountPerCell,0,65535));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("DeepLumpSizeMin","Deep Lump Size Min",70,default(int),t=>t.deepLumpSizeRange.min));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("DeepLumpSizeMax","Deep Lump Size Max",80,default(int),t=>t.deepLumpSizeRange.max));
			
			props.fields.Add(new SettingsFieldPropertiesListSettings("statBases","Item Stats",90,typeof(List<SettingsInstance>),null,
                        t=> ((ThingDef)t).statBases.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).statBases.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
			
			/*props.fields.Add(new SettingsFieldPropertiesSelectable("FoodType","Food-Type",100,typeof(string),Enum.GetName(typeof(FoodTypeFlags),FoodTypeFlags.None),
			                                                       t=>Enum.GetName(typeof(FoodTypeFlags),((ThingDef)t).ingestible.foodType),
	                                                       f=>from FoodTypeFlags d in Enum.GetValues(typeof(FoodTypeFlags)) orderby Enum.GetName(typeof(FoodTypeFlags),d) select (object)Enum.GetName(typeof(FoodTypeFlags),d),
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("FoodType",(string)def); return true; },
				                                             o=>(string)o,false));*/
			props.fields.Add(new SettingsFieldPropertiesListSelectable("FoodType","foodType",100,typeof(List<string>),new List<string>(),
					t=> GetEnumFlagsNames<FoodTypeFlags>(((ThingDef)t).ingestible.foodType),
					f => Enum.GetNames(typeof(FoodTypeFlags)).Where(e=>e != "None").Cast<object>(),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
							flow.addPostRenderCallback(fl=>{setting.get<List<string>>("FoodType")[listIndex] = (string)listEntry; setting.markAsChanged();});
							return true;
						};},
					entry => (string)entry,
					()=>Enum.GetName(typeof(FoodTypeFlags),FoodTypeFlags.None),
					true));
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("FoodPreferability","Food-Preferability",110,typeof(string),FoodPreferability.Undefined.ToString(),
			                                                       t=>Enum.GetName(typeof(FoodPreferability),((ThingDef)t).ingestible.preferability),
			                                                       f=>from FoodPreferability d in Enum.GetValues(typeof(FoodPreferability)) orderby d.ToString() select (object)d.ToString(),
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("FoodPreferability",(string)def); return true; },
				                                             o=>(string)o,false));

            props.fields.Add(new SettingsFieldPropertiesSelectable("Tradeability", "Tradeability", 115, typeof(string), Enum.GetName(typeof(Tradeability), Tradeability.All),
                    t => Enum.GetName(typeof(Tradeability), ((ThingDef)t).tradeability),
                    f => from Tradeability d in Enum.GetValues(typeof(Tradeability)) orderby Enum.GetName(typeof(Tradeability), d) select (object)Enum.GetName(typeof(Tradeability), d),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<string>("Tradeability", (string)def); return true; },
                    o => (string)o, false));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("MaxNumToIngestAtOnce","maxNumToIngestAtOnce",120,20,t=>t.ingestible.maxNumToIngestAtOnce,1));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("BaseIngestTicks","baseIngestTicks",130,500,t=>t.ingestible.baseIngestTicks,0));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("ChairSearchRadius","chairSearchRadius",140,32f,t=>t.ingestible.chairSearchRadius,0));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("UseEatingSpeedStat","useEatingSpeedStat",150,true,t=>t.ingestible.useEatingSpeedStat));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("IngestHoldUsesTable","ingestHoldUsesTable",160,true,t=>t.ingestible.ingestHoldUsesTable));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("Joy","joy",170,default(float),t=>t.ingestible.joy));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("Nurseable","nurseable",180,default(bool),t=>t.ingestible.nurseable));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("OptimalityOffsetHumanlikes","optimalityOffsetHumanlikes",190,default(float),t=>t.ingestible.optimalityOffsetHumanlikes));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("OptimalityOffsetFeedingAnimals","optimalityOffsetFeedingAnimals",200,default(float),t=>t.ingestible.optimalityOffsetFeedingAnimals));
			props.fields.Add(new SettingsFieldPropertiesSelectable("TasteThought","tasteThought",210,typeof(ThoughtDef),default(ThoughtDef),t=>((ThingDef)t).ingestible.tasteThought,
			                                            f=>DefDatabase<ThoughtDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThoughtDef>("TasteThought",(ThoughtDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("SpecialThoughtDirect","specialThoughtDirect",220,typeof(ThoughtDef),default(ThoughtDef),t=>((ThingDef)t).ingestible.specialThoughtDirect,
			                                            f=>DefDatabase<ThoughtDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThoughtDef>("SpecialThoughtDirect",(ThoughtDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("SpecialThoughtAsIngredient","specialThoughtAsIngredient",230,typeof(ThoughtDef),default(ThoughtDef),t=>((ThingDef)t).ingestible.specialThoughtAsIngredient,
			                                            f=>DefDatabase<ThoughtDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThoughtDef>("SpecialThoughtAsIngredient",(ThoughtDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("JoyKind","joyKind",240,typeof(JoyKindDef),default(JoyKindDef),t=>((ThingDef)t).ingestible.joyKind,
			                                            f=>DefDatabase<JoyKindDef>.AllDefsListForReading.OrderBy(d=>SettingsFieldPropertiesSelectable.DefLabelProducer(d)).Cast<object>(),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<JoyKindDef>("JoyKind",(JoyKindDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("DrugCategory","drugCategory",250,typeof(string),Enum.GetName(typeof(DrugCategory),DrugCategory.None),
			                                                       t=>Enum.GetName(typeof(DrugCategory),((ThingDef)t).ingestible.drugCategory),
	                                                       f=>from DrugCategory d in Enum.GetValues(typeof(DrugCategory)) orderby Enum.GetName(typeof(DrugCategory),d) select (object)Enum.GetName(typeof(DrugCategory),d),
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("DrugCategory",(string)def); return true; },
				                                             o=>(string)o,false));
			
			compDrug();
			props.props.Add(new SettingsPropertyProperties("HasCompDrug",o=>((ThingDef)o).GetCompProperties<CompProperties_Drug>() != null));
			props.fields.Add(new SettingsFieldPropertiesSettings("CompDrug","Drug Properties",260,typeof(SettingsInstance),null,
			                                                     t=> ((ThingDef)t).GetCompProperties<CompProperties_Drug>() != null ? new SettingsInstance("compDrug", ((ThingDef)t).GetCompProperties<CompProperties_Drug>(), null, null) : null,"compDrug")
						.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompDrug")));
			
			compHatcher();
			props.props.Add(new SettingsPropertyProperties("HasCompHatcher",o=>((ThingDef)o).GetCompProperties<CompProperties_Hatcher>() != null));
			props.fields.Add(new SettingsFieldPropertiesSettings("CompHatcher","Hatcher",270,typeof(SettingsInstance),null,
			                                                     t=> ((ThingDef)t).GetCompProperties<CompProperties_Hatcher>() != null ? new SettingsInstance("compHatcher", ((ThingDef)t).GetCompProperties<CompProperties_Hatcher>(), null, null) : null,"compHatcher")
						.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompHatcher")));
			
			props.mergers.Add(setting => {
              	ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
              	setting.resolveProps(props,t);
              	
              	string configuredLabel = setting.get<string>("Label");
				if ( !configuredLabel.NullOrEmpty() && configuredLabel.Trim().Length != 0 ) {
					t.label = configuredLabel;
                    DefCachedLabelCapSet(t, null);
                }
            	
              	t.stackLimit = setting.get<int>("StackLimit");
              	t.pathCost = setting.get<int>("PathCost");
              	t.useHitPoints = setting.get<bool>("UseHitPoints");
              	t.deepCommonality = setting.get<float>("DeepCommonality");
              	t.deepCountPerPortion = setting.get<int>("DeepCountPerPortion");
              	t.deepCountPerCell = setting.get<int>("DeepCountPerCell");
              	t.deepLumpSizeRange.min = setting.get<int>("DeepLumpSizeMin");
              	t.deepLumpSizeRange.max = setting.get<int>("DeepLumpSizeMax");
              	
              	FoodPreferability foodPreferability;
              	/*FoodTypeFlags foodType;
            	try {
	            	foodType = (FoodTypeFlags)Enum.Parse(typeof(FoodTypeFlags),setting.get<string>("FoodType"));
            	} catch ( ArgumentException ) {
            		foodType = FoodTypeFlags.None;
            	}*/
              	FoodTypeFlags foodType;
        		try {
        			List<string> lstFoodTypes = setting.get<List<string>>("FoodType");
        			if ( lstFoodTypes.NullOrEmpty() ) {
        				foodType = FoodTypeFlags.None;
        			} else {
            			foodType = (FoodTypeFlags)Enum.Parse(typeof(FoodTypeFlags), lstFoodTypes[0]);
            			for ( int i = 1; i < lstFoodTypes.Count; i++ ) {
            				foodType |= (FoodTypeFlags)Enum.Parse(typeof(FoodTypeFlags), lstFoodTypes[i]);
            			}
        			}
            	} catch ( ArgumentException ) {
            		foodType = FoodTypeFlags.None;
            	}
            	try {
	            	foodPreferability = (FoodPreferability)Enum.Parse(typeof(FoodPreferability),setting.get<string>("FoodPreferability"));
            	} catch ( ArgumentException ) {
            		foodPreferability = FoodPreferability.Undefined;
            	}
        		t.ingestible.foodType = foodType;
        		t.ingestible.preferability = foodPreferability;
                Tradeability tradeability;
                try
                {
                    tradeability = (Tradeability)Enum.Parse(typeof(Tradeability), setting.get<string>("Tradeability"));
                }
                catch (ArgumentException)
                {
                    tradeability = Tradeability.All;
                }
                t.tradeability = tradeability;

                t.ingestible.maxNumToIngestAtOnce = setting.get<int>("MaxNumToIngestAtOnce");
        		t.ingestible.baseIngestTicks = setting.get<int>("BaseIngestTicks");
				t.ingestible.chairSearchRadius = setting.get<float>("ChairSearchRadius");
				t.ingestible.useEatingSpeedStat = setting.get<bool>("UseEatingSpeedStat");
				t.ingestible.ingestHoldUsesTable = setting.get<bool>("IngestHoldUsesTable");
				t.ingestible.joy = setting.get<float>("Joy");
				t.ingestible.nurseable = setting.get<bool>("Nurseable");
				t.ingestible.optimalityOffsetHumanlikes = setting.get<float>("OptimalityOffsetHumanlikes");
				t.ingestible.optimalityOffsetFeedingAnimals = setting.get<float>("OptimalityOffsetFeedingAnimals");
				t.ingestible.tasteThought = setting.get<ThoughtDef>("TasteThought");
				t.ingestible.specialThoughtDirect = setting.get<ThoughtDef>("SpecialThoughtDirect");
				t.ingestible.specialThoughtAsIngredient = setting.get<ThoughtDef>("SpecialThoughtAsIngredient");
				t.ingestible.joyKind = setting.get<JoyKindDef>("JoyKind");
				
				DrugCategory drugCategory;
            	try {
	            	drugCategory = (DrugCategory)Enum.Parse(typeof(DrugCategory),setting.get<string>("DrugCategory"));
            	} catch ( ArgumentException ) {
            		drugCategory = DrugCategory.None;
            	}
				t.ingestible.drugCategory = drugCategory;
              	
            	List<SettingsInstance> statBases = setting.get<List<SettingsInstance>>("statBases");
            	if ( statBases != null ) {
            		if ( t.statBases == null ) t.statBases = new List<StatModifier>();
            		t.statBases.Clear();
            		foreach ( ISettingsInstance s in statBases ) {
            			t.statBases.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
            		}
            	}
            	
            	CompProperties_Drug cd = t.GetCompProperties<CompProperties_Drug>();
            	if ( cd != null ) {
            		SettingsInstance si = setting.get<SettingsInstance>("CompDrug");
            		if ( si != null ) {
            			cd.addictiveness = si.get<float>("Addictiveness");
            			cd.minToleranceToAddict = si.get<float>("MinToleranceToAddict");
            			cd.existingAddictionSeverityOffset = si.get<float>("ExistingAddictionSeverityOffset");
            			cd.needLevelOffset = si.get<float>("NeedLevelOffset");
            			cd.largeOverdoseChance = si.get<float>("LargeOverdoseChance");
            			cd.listOrder = si.get<float>("ListOrder");
            			cd.isCombatEnhancingDrug = si.get<bool>("IsCombatEnhancingDrug");
            			float min = si.get<float>("OverdoseSeverityOffsetMin"), max = si.get<float>("OverdoseSeverityOffsetMax");
            			cd.overdoseSeverityOffset = new FloatRange(min, max);
            			cd.chemical = si.get<ChemicalDef>("Chemical");
            		}
            	}
            	
            	CompProperties_Hatcher ch = t.GetCompProperties<CompProperties_Hatcher>();
            	if ( ch != null ) {
            		SettingsInstance si = setting.get<SettingsInstance>("CompHatcher");
            		if ( si != null ) {
            			ch.hatcherDaystoHatch = si.get<float>("HatcherDaystoHatch");
            			ch.hatcherPawn = si.get<PawnKindDef>("HatcherPawn");
            		}
            	}
            	
            	
            	typeof(IngestibleProperties).GetField("cachedNutrition", BindingFlags.Instance | BindingFlags.NonPublic).SetValue(t.ingestible,-1f);
          	});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ThingDef>("Foods","Select Food",()=>DefDatabase<ThingDef>.AllDefsListForReading.Where(t=>t.IsIngestible && t.plant == null).ToList(),
				def => def.defName, DefLabelProducerGeneric<ThingDef>.LabelProducerNotNull,key => DefDatabase<ThingDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			
			props.validators.Add(setting => {
             	ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
             	if ( t == null ) return false;
             	int version = setting.get<int>("Version");
             	if ( version < 2 ) {
					setting.set<List<string>>("FoodType",GetEnumFlagsNames<FoodTypeFlags>(t.ingestible.foodType));
					Log.Warning("Reset food types on "+t.label);
             	}
             	
             	setting.set<int>("Version",2);
             	return true;
             });
		}
		
		private static void plants() {
            /*
             * public List<PlantBiomeRecord> wildBiomes;
                public bool wildEqualLocalDistribution = true;
                public bool cavePlant;
                public float cavePlantWeight = 1f;
                public string harvestTag;
                public float harvestAfterGrowth;
                public bool autoHarvestable = true;
                public bool dieIfLeafless;
                public bool neverBlightable;
                public bool dieIfNoSunlight = true;
                public bool dieFromToxicFallout = true;
                public PlantPurpose purpose = PlantPurpose.Misc;
                public bool allowAutoCut = true;
                public TreeCategory treeCategory;
                public ThingDef burnedThingDef;
                public bool showGrowthInInspectPane = true;
                public float minSpacingBetweenSamePlant;
                public bool dropLeaves;

                public int maxMeshCount = 1;
                private string leaflessGraphicPath;
                public Graphic leaflessGraphic;
                private string immatureGraphicPath;
                public Graphic immatureGraphic;
             */

            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("plants");
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("Version","version",0,2,t=>2).setHidden(f=>true));
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("PathCost","Path-Cost",10,0,t=>t.pathCost));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("PathCostIgnoreRepeat","Path-Cost ignore repeat",11,default(bool),t=>t.pathCostIgnoreRepeat));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("BlockWind","Block wind",20,default(bool),t=>t.blockWind));
			
			/*props.fields.Add(new SettingsFieldPropertiesSelectable("FoodType","Food-Type",30,typeof(string),Enum.GetName(typeof(FoodTypeFlags),FoodTypeFlags.None),
			                                                       t=>((ThingDef)t).ingestible != null ? Enum.GetName(typeof(FoodTypeFlags),((ThingDef)t).ingestible.foodType) : Enum.GetName(typeof(FoodTypeFlags),FoodTypeFlags.None),
	                                                       f=>from FoodTypeFlags d in Enum.GetValues(typeof(FoodTypeFlags)) orderby Enum.GetName(typeof(FoodTypeFlags),d) select (object)Enum.GetName(typeof(FoodTypeFlags),d),
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("FoodType",(string)def); return true; },
				                                             o=>(string)o,false));*/
			props.fields.Add(new SettingsFieldPropertiesListSelectable("FoodType","foodType",30,typeof(List<string>),new List<string>(),
                   t=> ((ThingDef)t).ingestible != null ? GetEnumFlagsNames<FoodTypeFlags>(((ThingDef)t).ingestible.foodType) : new List<string>(),
					f => Enum.GetNames(typeof(FoodTypeFlags)).Where(e=>e != "None").Cast<object>(),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
							flow.addPostRenderCallback(fl=>{setting.get<List<string>>("FoodType")[listIndex] = (string)listEntry; setting.markAsChanged();});
							return true;
						};},
					entry => (string)entry,
					()=>Enum.GetName(typeof(FoodTypeFlags),FoodTypeFlags.None),
					true));
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("FoodPreferability","Food-Preferability",40,typeof(string),FoodPreferability.Undefined.ToString(),
			                                                       t=>((ThingDef)t).ingestible != null ? Enum.GetName(typeof(FoodPreferability),((ThingDef)t).ingestible.preferability) : Enum.GetName(typeof(FoodPreferability),FoodPreferability.Undefined),
			                                                       f=>from FoodPreferability d in Enum.GetValues(typeof(FoodPreferability)) orderby d.ToString() select (object)d.ToString(),
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("FoodPreferability",(string)def); return true; },
				                                             o=>(string)o,false));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("GrowDays","Grow Days", 47, 2f, t => t.plant.growDays));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("LifespanDaysPerGrowDays", "Lifespan Days per Grow Days", 48, 8f, t => t.plant.lifespanDaysPerGrowDays));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("FertilityMin","Fertility-Min",49,0.9f,t=>t.plant.fertilityMin));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("FertilitySensitivity","Fertility-Sensitivity",50,0.5f,t=>t.plant.fertilitySensitivity));
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("GrowMinGlow","Grow Glow-Min",55,0.51f,t=>t.plant.growMinGlow,0f,1f));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("GrowOptimalGlow","Grow Glow-Optimal",56,1f,t=>t.plant.growOptimalGlow,0f,1f));


            props.fields.Add(new SettingsFieldPropertiesListSelectable("SowResearchPrerequisites", "Sow Research Prerequisite", 59, typeof(List<ResearchProjectDef>), new List<ResearchProjectDef>(),
                    t => ((ThingDef)t).plant.sowResearchPrerequisites == null ? new List<ResearchProjectDef>() : ((ThingDef)t).plant.sowResearchPrerequisites.ToList(),
                    f => DefDatabase<ResearchProjectDef>.AllDefsListForReading.Cast<object>(),
                    flow => {
                        ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                        int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                        return listEntry => {
                            flow.addPostRenderCallback(fl => { setting.get<List<ResearchProjectDef>>("SowResearchPrerequisites")[listIndex] = (ResearchProjectDef)listEntry; setting.markAsChanged(); });
                            return true;
                        };
                    },
                    SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<ResearchProjectDef>.AllDefsListForReading.First(),
                    true));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("SowWork","Sowing-Work",60,default(float),t=>t.plant.sowWork, 0));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("SowMinSkill","Skill requirement",70,0,t=>t.plant.sowMinSkill, 0));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("MustBeWildToSow","Restrict by Biome",80,default(bool),t=>t.plant.mustBeWildToSow));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("HarvestWork","Harvest-Work",90,default(float),t=>t.plant.harvestWork, 0));
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("HarvestedThingDef","Harvest-Product",100,typeof(ThingDef),null,t=>((ThingDef)t).plant.harvestedThingDef,
	                                                         f=>from object d in DefDatabase<ThingDef>.AllDefsListForReading select d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("HarvestedThingDef",(ThingDef)def); return true; },
				                                             SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("HarvestYield","Yield",110,default(float),t=>t.plant.harvestYield));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("HarvestMinGrowth","Min-Growth",120,default(float),t=>t.plant.harvestMinGrowth));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("HarvestFailable","Harvest can fail",130,default(bool),t=>t.plant.harvestFailable));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("BlockAdjacentSow","Block adjacent sowing",140,default(bool),t=>t.plant.blockAdjacentSow));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("VisualSizeRangeMin","Min-Visual-Size",150,default(float),t=>t.plant.visualSizeRange.min));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("VisualSizeRangeMax","Max-Visual-Size",160,default(float),t=>t.plant.visualSizeRange.max));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("TopWindExposure","Top-Wind-Exposure",170,default(float),t=>t.plant.topWindExposure));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("WildClusterRadius","Wild growth radius",180,-1,t=>t.plant.wildClusterRadius, -1));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("WildClusterWeight","Wild growth weight",190,15f,t=>t.plant.wildClusterWeight, 0));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("WildOrder","Wild order",200,2f,t=>t.plant.wildOrder, 0));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("InterferesWithRoof","Blocks roof",210,default(bool),t=>t.plant.interferesWithRoof));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("HumanFoodPlant", "Human Food", 211, default(bool), t => t.plant.humanFoodPlant));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("TreeLoversCareIfChopped", "Tree Lovers Care", 212, true, t => t.plant.treeLoversCareIfChopped));

            props.fields.Add(new SettingsFieldPropertiesListPrimitive("SowTags","Sow Tags",220,typeof(List<string>),new List<string>(),
                      t=> ((ThingDef)t).plant.sowTags != null ? ((ThingDef)t).plant.sowTags.ToList() : new List<string>(),
                      null,()=>"",true));
			
			StatModifier();
			props.fields.Add(new SettingsFieldPropertiesListSettings("statBases","Item Stats",230,typeof(List<SettingsInstance>),null,
                        t=> ((ThingDef)t).statBases.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).statBases.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
			
			props.mergers.Add(setting => {
              	ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
            	
            	t.pathCost = setting.get<int>("PathCost");
            	t.pathCostIgnoreRepeat = setting.get<bool>("PathCostIgnoreRepeat");
            	t.blockWind = setting.get<bool>("BlockWind");
            	
            	FoodPreferability foodPreferability;
            	/*FoodTypeFlags foodType;
            	try {
	            	foodType = (FoodTypeFlags)Enum.Parse(typeof(FoodTypeFlags),setting.get<string>("FoodType"));
            	} catch ( ArgumentException ) {
            		foodType = FoodTypeFlags.None;
            	}*/
            	FoodTypeFlags foodType;
        		try {
        			List<string> lstFoodTypes = setting.get<List<string>>("FoodType");
        			if ( lstFoodTypes.NullOrEmpty() ) {
        				foodType = FoodTypeFlags.None;
        			} else {
            			foodType = (FoodTypeFlags)Enum.Parse(typeof(FoodTypeFlags), lstFoodTypes[0]);
            			for ( int i = 1; i < lstFoodTypes.Count; i++ ) {
            				foodType |= (FoodTypeFlags)Enum.Parse(typeof(FoodTypeFlags), lstFoodTypes[i]);
            			}
        			}
            	} catch ( ArgumentException ) {
            		foodType = FoodTypeFlags.None;
            	}
            	
            	try {
	            	foodPreferability = (FoodPreferability)Enum.Parse(typeof(FoodPreferability),setting.get<string>("FoodPreferability"));
            	} catch ( ArgumentException ) {
            		foodPreferability = FoodPreferability.Undefined;
            	}
            	if ( t.ingestible != null ) {
            		t.ingestible.foodType = foodType;
            		t.ingestible.preferability = foodPreferability;
            	} else {
            		if ( foodType != FoodTypeFlags.None ) setting.set<List<string>>("FoodType", new List<string>());
            		if ( foodPreferability != FoodPreferability.Undefined ) setting.set<string>("FoodPreferability", Enum.GetName(typeof(FoodPreferability),FoodPreferability.Undefined));
            	}
            	
            	t.plant.growDays = setting.get<float>("GrowDays");
        		t.plant.fertilityMin = setting.get<float>("FertilityMin");
            	t.plant.fertilitySensitivity = setting.get<float>("FertilitySensitivity");
            	t.plant.sowWork = setting.get<float>("SowWork");
            	t.plant.sowMinSkill = setting.get<int>("SowMinSkill");
            	t.plant.mustBeWildToSow = setting.get<bool>("MustBeWildToSow");
            	t.plant.harvestWork = setting.get<float>("HarvestWork");
            	t.plant.harvestedThingDef = setting.get<ThingDef>("HarvestedThingDef");
            	t.plant.harvestYield = setting.get<float>("HarvestYield");
            	t.plant.harvestMinGrowth = setting.get<float>("HarvestMinGrowth");
            	t.plant.harvestFailable = setting.get<bool>("HarvestFailable");
            	t.plant.blockAdjacentSow = setting.get<bool>("BlockAdjacentSow");
            	t.plant.visualSizeRange.min = setting.get<float>("VisualSizeRangeMin");
            	t.plant.visualSizeRange.max = setting.get<float>("VisualSizeRangeMax");
            	t.plant.topWindExposure = setting.get<float>("TopWindExposure");
            	t.plant.wildClusterRadius = setting.get<int>("WildClusterRadius");
            	t.plant.wildClusterWeight = setting.get<float>("WildClusterWeight");
            	t.plant.wildOrder = setting.get<float>("WildOrder");
            	t.plant.interferesWithRoof = setting.get<bool>("InterferesWithRoof");
            	t.plant.growMinGlow = setting.get<float>("GrowMinGlow");
            	t.plant.growOptimalGlow = setting.get<float>("GrowOptimalGlow");
                t.plant.humanFoodPlant = setting.get<bool>("HumanFoodPlant");
                t.plant.treeLoversCareIfChopped = setting.get<bool>("TreeLoversCareIfChopped");
                t.plant.lifespanDaysPerGrowDays = setting.get<float>("LifespanDaysPerGrowDays");

                List<string> sowTags = setting.get<List<string>>("SowTags");
              	if ( sowTags != null ) {
              		if ( t.plant.sowTags == null ) t.plant.sowTags = new List<string>();
              		else t.plant.sowTags.Clear();
              		t.plant.sowTags.AddRange(sowTags);
              	}

                List<ResearchProjectDef> sowResearchPrerequisites = setting.get<List<ResearchProjectDef>>("SowResearchPrerequisites");
                if (sowResearchPrerequisites != null)
                {
                    if (sowResearchPrerequisites.Any())
                    {
                        if (t.plant.sowResearchPrerequisites == null) t.plant.sowResearchPrerequisites = new List<ResearchProjectDef>();
                        else t.plant.sowResearchPrerequisites.Clear();
                        t.plant.sowResearchPrerequisites.AddRange(sowResearchPrerequisites);
                    }
                    else
                    {
                        if (t.plant.sowResearchPrerequisites != null) t.plant.sowResearchPrerequisites.Clear();
                        t.plant.sowResearchPrerequisites = null;
                    }
                }

                List<SettingsInstance> statBases = setting.get<List<SettingsInstance>>("statBases");
            	if ( statBases != null ) {
            		if ( t.statBases == null ) t.statBases = new List<StatModifier>();
            		t.statBases.Clear();
            		foreach ( ISettingsInstance s in statBases ) {
            			t.statBases.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
            		}
            	}});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ThingDef>("Plants","Select Plant",
			                                                                        ()=>DefDatabase<ThingDef>.AllDefsListForReading.Where(t=>t.plant != null).ToList(),
				def => def.defName, DefLabelProducerGeneric<ThingDef>.LabelProducerNotNull,key => DefDatabase<ThingDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			
			props.validators.Add(setting => {
             	ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
             	if ( t == null ) return false;
             	int version = setting.get<int>("Version");
             	if ( version < 2 ) {
             		if ( t.ingestible != null ) {
						setting.set<List<string>>("FoodType",GetEnumFlagsNames<FoodTypeFlags>(t.ingestible.foodType));
						Log.Warning("Reset food types on "+t.label);
             		} else {
             			setting.set<List<string>>("FoodType",new List<string>());
             		}
             	}
             	
             	setting.set<int>("Version",2);
             	return true;
             });
		}
		
		private static void resourceRocks() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("resourceRock");
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("MineableThingDef","Minining-Product",10,typeof(ThingDef),null,t=>((ThingDef)t).building.mineableThing,
	                                                         f=>from object d in DefDatabase<ThingDef>.AllDefsListForReading select d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("MineableThingDef",(ThingDef)def); return true; },
				                                             SettingsFieldPropertiesSelectable.DefLabelProducer,false));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("MineableYield","Yield",20,default(int),t=>t.building.mineableYield));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("MineableYieldWasteable","Yield Wasteable",30,true,t=>t.building.mineableYieldWasteable));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("MineableScatterCommonality","Scatter Commonality",40,0f,t=>t.building.mineableScatterCommonality));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("MineableScatterLumpSizeRangeMin","Min-Lump-Size",50,0,t=>t.building.mineableScatterLumpSizeRange.min, 0));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("MineableScatterLumpSizeRangeMax","Max-Lump-Size",60,0,t=>t.building.mineableScatterLumpSizeRange.max, 0));

            StatModifier();
			props.fields.Add(new SettingsFieldPropertiesListSettings("statBases","Item Stats",70,typeof(List<SettingsInstance>),null,
                        t=> ((ThingDef)t).statBases.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).statBases.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
			
			props.mergers.Add(setting => {
              	ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
            	
              	t.building.mineableThing = setting.get<ThingDef>("MineableThingDef");
              	t.building.mineableYield = setting.get<int>("MineableYield");
              	t.building.mineableYieldWasteable = setting.get<bool>("MineableYieldWasteable");
              	t.building.mineableScatterCommonality = setting.get<float>("MineableScatterCommonality");
              	t.building.mineableScatterLumpSizeRange.min = setting.get<int>("MineableScatterLumpSizeRangeMin");
              	t.building.mineableScatterLumpSizeRange.max = setting.get<int>("MineableScatterLumpSizeRangeMax");

                List<SettingsInstance> statBases = setting.get<List<SettingsInstance>>("statBases");
            	if ( statBases != null ) {
            		if ( t.statBases == null ) t.statBases = new List<StatModifier>();
            		t.statBases.Clear();
            		foreach ( ISettingsInstance s in statBases ) {
            			t.statBases.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
            		}
            	}
            });
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ThingDef>("Resource Rocks","Select Resource Rock",
			                                                                        ()=>DefDatabase<ThingDef>.AllDefsListForReading.Where(t=>t.building != null && t.building.isResourceRock).ToList(),
				def => def.defName, DefLabelProducerGeneric<ThingDef>.LabelProducerNotNull,key => DefDatabase<ThingDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}
		
		private static Func<CompProperties_Refuelable,float> CompProperties_RefuelableFuelMultiplierGet = 
				UtilReflection.CreateGetter<CompProperties_Refuelable,float>(typeof(CompProperties_Refuelable).GetField("fuelMultiplier", UtilReflection.BindingInstanceFlags));
		private static Action<CompProperties_Refuelable,float> CompProperties_RefuelableFuelMultiplierSet = 
				UtilReflection.CreateSetter<CompProperties_Refuelable,float>(typeof(CompProperties_Refuelable).GetField("fuelMultiplier", UtilReflection.BindingInstanceFlags));
		
		private static void compTransporter() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compTransporter");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Transporter>("MassCapacity","Mass Capacity",10,150f,t=>t.massCapacity));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Transporter>("RestEffectiveness","Rest Effectiveness",20,default(float),t=>t.restEffectiveness,0f));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Transporter Properties");
			}
		}
		
		private static void compRefuelable() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compRefuelable");
			if ( props.fields.NullOrEmpty() ) {
				/*
				public string fuelLabel;
				public string fuelGizmoLabel;
				public string outOfFuelMessage;
				public string fuelIconPath;
				private Texture2D fuelIcon;
				*/
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Refuelable>("FuelConsumptionRate","Fuel Consumption Rate",10,1f,t=>t.fuelConsumptionRate));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Refuelable>("FuelCapacity","Fuel Capacity",20,2f,t=>t.fuelCapacity));
				//Do not set this below 0 if destroyOnNoFuel is true
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Refuelable>("InitialFuelPercent","Initial Fuel Percent",30,default(float),t=>t.initialFuelPercent));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Refuelable>("AutoRefuelPercent","Auto Refuel Percent",40,0.3f,t=>t.autoRefuelPercent));
				//if sufficient rain and unroofed is in addition to normal fuel consumption.
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Refuelable>("FuelConsumptionPerTickInRain","Fuel Consumption per Tick in Rain",50,default(float),t=>t.fuelConsumptionPerTickInRain,0f));
				//If the target level is configurable this value is used unless a target fuel level was manually set. If it isn't configurable and no value was configured the fuel capacity is used.
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Refuelable>("InitialConfigurableTargetFuelLevel","Initial Configurable Target Fuel Level",60,default(float),t=>t.initialConfigurableTargetFuelLevel));
				//If current fuel level is below this level the buildings is treated as out of fuel 
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Refuelable>("MinimumFueledThreshold","Minimum Fueled Threshold",70,default(float),t=>t.minimumFueledThreshold));
				//if above 1 it consumes more fuel and below 1 less. If factor by difficulty is true this value is modified by  the games difficulty setting "maintenance cost factor".
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Refuelable>("FuelMultiplier", "Fuel Multiplier", 80, 1f, CompProperties_RefuelableFuelMultiplierGet));
				//This removes the building if it has no more fuel - different from "out of fuel" this requires all fuel to be consumed.
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Refuelable>("DestroyOnNoFuel","Destroy on no Fuel",100,default(bool),t=>t.destroyOnNoFuel));
				//If false the building consumes additional fuel while being flicked on(or always if the building can not be flicked). 
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Refuelable>("ConsumeFuelOnlyWhenUsed","ConsumeFuelOnlyWhenUsed",110,default(bool),t=>t.consumeFuelOnlyWhenUsed));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Refuelable>("ShowFuelGizmo","Show Fuel Gizmo",120,default(bool),t=>t.showFuelGizmo));
				//displays gui for setting the target level
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Refuelable>("TargetFuelLevelConfigurable","Target Fuel Level Configurable",130,default(bool),t=>t.targetFuelLevelConfigurable));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Refuelable>("DrawOutOfFuelOverlay","DrawOutOfFuelOverlay",140,true,t=>t.drawOutOfFuelOverlay));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Refuelable>("DrawFuelGaugeInMap","Draw Fuel Gauge in Map",150,default(bool),t=>t.drawFuelGaugeInMap));
				//???
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Refuelable>("AtomicFueling","Atomic Fueling",160,default(bool),t=>t.atomicFueling));
				//If true applies the difficulty setting "maintenance cost factor"
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Refuelable>("FactorByDifficulty","Factor by Difficulty",170,default(bool),t=>t.factorByDifficulty));
				
				ThingFilter();
				//Fuel filter must have at least one entry in the allowed thing defs
				props.fields.Add(new SettingsFieldPropertiesSettings("FuelFilter","Fuel Filter",200,typeof(SettingsInstance),new SettingsInstance("thingFilter",new ThingFilter(),null,null),
                                         t=>new SettingsInstance("thingFilter",((CompProperties_Refuelable)t).fuelFilter??new ThingFilter(),null,null),"thingFilter"));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Refuelable Properties");
			}
		}
		
		private static void compBattery() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compBattery");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Battery>("Efficiency","Efficiency",10,0.5f,t=>t.efficiency));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Battery>("StoredEnergyMax","Max Energy",20,1000f,t=>t.storedEnergyMax));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Battery Properties");
			}
		}
		
		private static void compPower() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compPower");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Power>("BasePowerConsumption","Base Power",10,default(float),t=>t.basePowerConsumption));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Power>("ShortCircuitInRain","Short Circuits",20,default(bool),t=>t.shortCircuitInRain));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Power Properties");
			}
		}
		
		private static void compHeatPusher() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compHeatPusher");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_HeatPusher>("HeatPerSecond","Heat",10,default(float),t=>t.heatPerSecond));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_HeatPusher>("HeatPushMaxTemperature","Max Temperature",20,99999f,t=>t.heatPushMaxTemperature));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_HeatPusher>("HeatPushMinTemperature","Min Temperature",30,-99999f,t=>t.heatPushMinTemperature));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Heat Pusher Properties");
			}
		}

        private static void compGlower()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compGlower");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Glower>("GlowRadius", "Glow Radius", 10, 14f, t => t.glowRadius));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Glower>("OverlightRadius", "Overlight Radius", 20, default(float), t => t.overlightRadius));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveColor<CompProperties_Glower>("GlowColor", "Color", 30, (new ColorInt(255, 255, 255, 0) * 1.45f).ToColor, t => t.glowColor.ToColor));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Glower Properties");
            }
        }

        private static void mergeCompGlower(ThingDef t, ISettingsInstance mergeProperties)
        {
            CompProperties_Glower comp = t.GetCompProperties<CompProperties_Glower>();
            if (comp != null && mergeProperties != null)
            {
                bool hasChanged = false;
                float f = mergeProperties.get<float>("GlowRadius");
                if (comp.glowRadius != f)
                {
                    comp.glowRadius = f;
                    hasChanged = true;
                }
                f = mergeProperties.get<float>("OverlightRadius");
                if (comp.overlightRadius != f)
                {
                    comp.overlightRadius = f;
                    hasChanged = true;
                }
                ColorInt ci = new ColorInt(mergeProperties.get<Color>("GlowColor"));
                if (comp.glowColor != ci)
                {
                    comp.glowColor = ci;
                    hasChanged = true;
                }

                if (hasChanged)
                {
                    if (Current.Game != null)
                    {
                        foreach (Map m in Find.Maps)
                        {
                            foreach (Thing thing in m.listerThings.ThingsOfDef(t))
                            {
                                CompGlower cg = thing.TryGetComp<CompGlower>();
                                //if any of these buildings are currently glowing we toggle them to recalculate their glow.
                                if (cg.Glows)
                                {
                                    m.mapDrawer.MapMeshDirty(cg.parent.Position, MapMeshFlag.Things);
                                    m.glowGrid.DeRegisterGlower(cg);
                                    m.glowGrid.RegisterGlower(cg);
                                }
                            }
                        }
                    }
                }
            }
        }

        private static void compBiosculpterPod_HealingCycle()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compBiosculpterPod_HealingCycle");
            if (props.fields.NullOrEmpty())
            {
                /*
                 * public float durationDays;
public Color operatingColor = new Color(0.5f, 0.7f, 0.5f);
public ThoughtDef gainThoughtOnCompletion;
public List<ResearchProjectDef> requiredResearch;
public List<ThingDefCountClass> extraRequiredIngredients;
                 */
                props.fields.Add(new SettingsFieldPropertiesListSelectable("BodyPartsToRestore", "Restorable Body Parts", 60, typeof(List<BodyPartDef>), new List<BodyPartDef>(),
                    h => ((CompProperties_BiosculpterPod_HealingCycle)h).bodyPartsToRestore == null ? new List<BodyPartDef>() : ((CompProperties_BiosculpterPod_HealingCycle)h).bodyPartsToRestore.ToList(),
                    f => DefDatabase<BodyPartDef>.AllDefsListForReading.Cast<object>(),
                    flow => {
                        ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                        int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                        return listEntry => {
                            flow.addPostRenderCallback(fl => { setting.get<List<BodyPartDef>>("BodyPartsToRestore")[listIndex] = (BodyPartDef)listEntry; setting.markAsChanged(); });
                            return true;
                        };
                    },
                    SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<BodyPartDef>.AllDefsListForReading.First(),
                    true));
                props.fields.Add(new SettingsFieldPropertiesListSelectable("ConditionsToPossiblyCure", "Possibly Curable", 70, typeof(List<HediffDef>), new List<HediffDef>(),
                    h => ((CompProperties_BiosculpterPod_HealingCycle)h).conditionsToPossiblyCure == null ? new List<HediffDef>() : ((CompProperties_BiosculpterPod_HealingCycle)h).conditionsToPossiblyCure.ToList(),
                    f => DefDatabase<HediffDef>.AllDefsListForReading.Where(hd => hd.everCurableByItem && !hd.countsAsAddedPartOrImplant).Cast<object>(),
                    flow => {
                        ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                        int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                        return listEntry => {
                            flow.addPostRenderCallback(fl => { setting.get<List<HediffDef>>("ConditionsToPossiblyCure")[listIndex] = (HediffDef)listEntry; setting.markAsChanged(); });
                            return true;
                        };
                    },
                    SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<HediffDef>.AllDefsListForReading.Where(hd => hd.everCurableByItem && !hd.countsAsAddedPartOrImplant).First(),
                    true));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Healing Cycle Properties");
            }
        }

        private static void mergeCompBiosculpterPod_HealingCycle(ThingDef t, ISettingsInstance mergeProperties)
        {
            CompProperties_BiosculpterPod_HealingCycle healingCycle = t.GetCompProperties<CompProperties_BiosculpterPod_HealingCycle>();
            if (healingCycle != null && mergeProperties != null)
            {
                List<BodyPartDef> bodyPartsToRestore = mergeProperties.get<List<BodyPartDef>>("BodyPartsToRestore");
                if (bodyPartsToRestore != null)
                {
                    if (bodyPartsToRestore.Any())
                    {
                        if (healingCycle.bodyPartsToRestore == null) healingCycle.bodyPartsToRestore = new List<BodyPartDef>();
                        else healingCycle.bodyPartsToRestore.Clear();
                        healingCycle.bodyPartsToRestore.AddRange(bodyPartsToRestore);
                    } else {
                        if (healingCycle.bodyPartsToRestore != null) healingCycle.bodyPartsToRestore.Clear();
                        healingCycle.bodyPartsToRestore = null;
                    }
                }
                List<HediffDef> conditionsToPossiblyCure = mergeProperties.get<List<HediffDef>>("ConditionsToPossiblyCure");
                if (conditionsToPossiblyCure != null)
                {
                    if (conditionsToPossiblyCure.Any())
                    {
                        if (healingCycle.conditionsToPossiblyCure == null) healingCycle.conditionsToPossiblyCure = new List<HediffDef>();
                        else healingCycle.conditionsToPossiblyCure.Clear();
                        healingCycle.conditionsToPossiblyCure.AddRange(conditionsToPossiblyCure);
                    }
                    else
                    {
                        if (healingCycle.conditionsToPossiblyCure != null) healingCycle.conditionsToPossiblyCure.Clear();
                        healingCycle.conditionsToPossiblyCure = null;
                    }
                }
            }

        }

        private static void compFacility()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compFacility");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<CompProperties_Facility>("MaxSimultaneous", "Max Simultaneous", 10, 1, t => t.maxSimultaneous, 0));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_Facility>("MaxDistance", "Max Distance", 20, 8f, t => t.maxDistance, 0f));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Facility>("MustBePlacedAdjacent", "Adjacency", 30, default(bool), t => t.mustBePlacedAdjacent));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Facility>("MustBePlacedAdjacentCardinalToBedHead", "Cardinal Bed Head", 40, default(bool), t => t.mustBePlacedAdjacentCardinalToBedHead));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Facility>("MustBePlacedAdjacentCardinalToAndFacingBedHead", "Facing Bed Head", 50, default(bool), t => t.mustBePlacedAdjacentCardinalToAndFacingBedHead));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Facility>("CanLinkToMedBedsOnly", "Med Beds Only", 60, default(bool), t => t.canLinkToMedBedsOnly));

                StatModifier();
                props.fields.Add(new SettingsFieldPropertiesListSettings("StatOffsets", "Provided Stats", 100, typeof(List<SettingsInstance>), null,
                    cp => ((CompProperties_Facility)cp).statOffsets.NullOrEmpty() ? new List<SettingsInstance>() : 
                    ((CompProperties_Facility)cp).statOffsets.Where(tc => tc != null).Select(tc => new SettingsInstance("statModifier", tc, null, null)).ToList(),
                    "statModifier", () => new SettingsInstance("statModifier", new StatModifier() {
                        stat = DefDatabase<StatDef>.AllDefsListForReading.First(), value = 0
                    }, null, null), true));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Facility Properties");
            }
        }

        private static void buildings() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("buildings");
			props.props.Add(new SettingsPropertyProperties("HasCompRefuelable",o=>((ThingDef)o).GetCompProperties<CompProperties_Refuelable>() != null));
			props.props.Add(new SettingsPropertyProperties("HasCompTransporter",o=>((ThingDef)o).GetCompProperties<CompProperties_Transporter>() != null));
			props.props.Add(new SettingsPropertyProperties("HasBuildingProperties",o=>((ThingDef)o).building != null));
			props.props.Add(new SettingsPropertyProperties("HasCompAffectedByFacilities",o=>((ThingDef)o).GetCompProperties<CompProperties_AffectedByFacilities>() != null));
            props.props.Add(new SettingsPropertyProperties("HasCompFacility", o => ((ThingDef)o).GetCompProperties<CompProperties_Facility>() != null));
            props.props.Add(new SettingsPropertyProperties("HasCompBattery",o=>((ThingDef)o).GetCompProperties<CompProperties_Battery>() != null));
			props.props.Add(new SettingsPropertyProperties("HasCompPower",o=>((ThingDef)o).GetCompProperties<CompProperties_Power>() != null));
			props.props.Add(new SettingsPropertyProperties("HasCompHeatPusher",o=>((ThingDef)o).GetCompProperties<CompProperties_HeatPusher>() != null));
            props.props.Add(new SettingsPropertyProperties("HasCompBiosculpterPod_HealingCycle", o => ((ThingDef)o).GetCompProperties<CompProperties_BiosculpterPod_HealingCycle>() != null));
            props.props.Add(new SettingsPropertyProperties("FilledPartial", o => ((ThingDef)o).Fillage == FillCategory.Partial));
            props.props.Add(new SettingsPropertyProperties("HasCompGlower", o => ((ThingDef)o).GetCompProperties<CompProperties_Glower>() != null));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("Version","version",0,5,t=>5).setHidden(f=>true));
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveString<ThingDef>("Label","Label",5,null,t=>t.label));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("PathCost","Path-Cost",7,0,t=>t.pathCost));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("PathCostIgnoreRepeat","Path-Cost ignore repeat",8,default(bool),t=>t.pathCostIgnoreRepeat));
            props.fields.Add(new SettingsFieldPropertiesSelectable("Passability", "Passability", 9, typeof(string), Enum.GetName(typeof(Traversability), default(Traversability)),
                    t => Enum.GetName(typeof(Traversability), ((BuildableDef)t).passability),
                    f => from Traversability d in Enum.GetValues(typeof(Traversability)) orderby Enum.GetName(typeof(Traversability), d) select (object)Enum.GetName(typeof(Traversability), d),
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<string>("Passability", (string)def); return true; },
                    o => (string)o, false));
            props.fields.Add(new SettingsFieldPropertiesSelectable("TerrainAffordanceNeeded", "Terrain Affordance", 10, typeof(TerrainAffordanceDef), null,
                t => ((ThingDef)t).terrainAffordanceNeeded, f => from object t in DefDatabase<TerrainAffordanceDef>.AllDefsListForReading select t,
                f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<TerrainAffordanceDef>("TerrainAffordanceNeeded", (TerrainAffordanceDef)def); return true; },
                SettingsFieldPropertiesSelectable.DefLabelProducer, true));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("Selectable","Selectable",15,default(bool),t=>t.selectable));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("NeverMultiSelect","No Multiselect",20,default(bool),t=>t.neverMultiSelect));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("BlockWind","Blocks wind",30,default(bool),t=>t.blockWind));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("HoldsRoof","Supports roof",40,default(bool),t=>t.holdsRoof));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("FillPercent", "Fillage", 50, default(float), t => t.fillPercent, 0.01f, 0.99f)
                .setHidden(f=> !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("FilledPartial")));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("CoversFloor","Covers floor",60,default(bool),t=>t.coversFloor));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("CastEdgeShadows","Cast shadows",70,default(bool),t=>t.castEdgeShadows));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("StaticSunShadowHeight","Shadow height",80,0f,t=>t.staticSunShadowHeight));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("Fertility","Fertility",85,-1f,t=>t.fertility));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("Minifiable", "Minifiable", 86, default(bool), t => t.Minifiable));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("Stealable", "Stealable", 87, default(bool), t => t.stealable));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("StuffCategories","Stuff Categories",89,typeof(List<StuffCategoryDef>),new List<StuffCategoryDef>(),
											h=>((ThingDef)h).stuffCategories == null ? new List<StuffCategoryDef>() : ((ThingDef)h).stuffCategories.ToList(),
											f=>DefDatabase<StuffCategoryDef>.AllDefsListForReading.Cast<object>(),
											flow=>{
												ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
												int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
												return listEntry=>{
												flow.addPostRenderCallback(fl=>{setting.get<List<StuffCategoryDef>>("StuffCategories")[listIndex] = (StuffCategoryDef)listEntry; setting.markAsChanged();});
												return true;
											};},
											SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<StuffCategoryDef>.AllDefsListForReading.First(),
											true));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("CostStuffCount","Material costs",90,0,t=>t.costStuffCount));
			ThingDefCountClass();
			props.fields.Add(new SettingsFieldPropertiesListSettings("CostList","Cost List",91,typeof(List<SettingsInstance>),null,
                                     t=> ((ThingDef)t).costList == null ? new List<SettingsInstance>() : ((ThingDef)t).costList.Select(tc=>new SettingsInstance("thingDefCountClass",tc,null,null)).ToList(),
                                     "thingDefCountClass", ()=>new SettingsInstance("thingDefCountClass",new ThingDefCountClass(DefDatabase<ThingDef>.AllDefsListForReading.Find(d=>d.category == ThingCategory.Item),1),null,null),true));
			props.fields.Add(new SettingsFieldPropertiesListSettings("KilledLeavings","Killed Leavings",92,typeof(List<SettingsInstance>),null,
                                     t=> ((ThingDef)t).killedLeavings == null ? new List<SettingsInstance>() : ((ThingDef)t).killedLeavings.Select(tc=>new SettingsInstance("thingDefCountClass",tc,null,null)).ToList(),
                                     "thingDefCountClass", ()=>new SettingsInstance("thingDefCountClass",new ThingDefCountClass(DefDatabase<ThingDef>.AllDefsListForReading.Find(d=>d.category == ThingCategory.Item),1),null,null),true));
			
			StatModifier();
			props.fields.Add(new SettingsFieldPropertiesListSettings("statBases","Item Stats",70,typeof(List<SettingsInstance>),null,
                        t=> ((ThingDef)t).statBases.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).statBases.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("DesignationCategory","Designation Category",200,typeof(DesignationCategoryDef),null,t=>((ThingDef)t).designationCategory,
	                                                         f=>from object d in DefDatabase<DesignationCategoryDef>.AllDefsListForReading select d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<DesignationCategoryDef>("DesignationCategory",(DesignationCategoryDef)def); return true; },
				                                             SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("DesignatorDropdown","Designator Dropdown",210,typeof(DesignatorDropdownGroupDef),null,t=>((ThingDef)t).designatorDropdown,
	                                                         f=>from object d in DefDatabase<DesignatorDropdownGroupDef>.AllDefsListForReading select d,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<DesignatorDropdownGroupDef>("DesignatorDropdown",(DesignatorDropdownGroupDef)def); return true; },
				                                             SettingsFieldPropertiesSelectable.DefLabelProducer,true));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("CanGenerateDefaultDesignator", "Default Designator", 215, default(bool), t => t.canGenerateDefaultDesignator));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("ResearchPrerequisites","Research Prerequisites",220,typeof(List<ResearchProjectDef>),new List<ResearchProjectDef>(),
											h=>((ThingDef)h).researchPrerequisites == null ? new List<ResearchProjectDef>() : ((ThingDef)h).researchPrerequisites.ToList(),
											f=>DefDatabase<ResearchProjectDef>.AllDefsListForReading.Cast<object>(),
											flow=>{
												ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
												int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
												return listEntry=>{
												flow.addPostRenderCallback(fl=>{setting.get<List<ResearchProjectDef>>("ResearchPrerequisites")[listIndex] = (ResearchProjectDef)listEntry; setting.markAsChanged();});
												return true;
											};},
											SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<ResearchProjectDef>.AllDefsListForReading.First(),
											true));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("Recipes", "Recipes", 225, typeof(List<RecipeDef>), new List<RecipeDef>(),
                                            h => ((ThingDef)h).recipes == null ? new List<RecipeDef>() : ((ThingDef)h).recipes.ToList(),
                                            f => DefDatabase<RecipeDef>.AllDefsListForReading.Cast<object>(),
                                            flow => {
                                                ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                                                int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                                                return listEntry => {
                                                    flow.addPostRenderCallback(fl => { setting.get<List<RecipeDef>>("Recipes")[listIndex] = (RecipeDef)listEntry; setting.markAsChanged(); });
                                                    return true;
                                                };
                                            },
                                            SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<RecipeDef>.AllDefsListForReading.First(),
                                            true));

            props.fields.Add(new SettingsFieldPropertiesSelectable("TurretGunDef","Turret Gun",230,typeof(ThingDef),null,t=>((ThingDef)t).building != null ? ((ThingDef)t).building.turretGunDef : null,
			                                                       f=>from ThingDef t in DefDatabase<ThingDef>.AllDefsListForReading where t.IsWeapon select (object)t,
				                                             f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("TurretGunDef",(ThingDef)def); return true; },
				                                             SettingsFieldPropertiesSelectable.DefLabelProducer,false)
                                     .setHidden(f=>!((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasBuildingProperties")));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("TurretBurstCooldownTime", "Turret Cooldown", 231, -1f, t => t.building != null ? t.building.turretBurstCooldownTime : -1)
                .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasBuildingProperties")));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("TurretBurstWarmupTime", "Turret Warmup", 232, default(float), t => t.building != null ? t.building.turretBurstWarmupTime : default(float))
                .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasBuildingProperties")));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("TurretInitialCooldownTime", "Turret Spawn Cooldown", 233, default(float), t => t.building != null ? t.building.turretInitialCooldownTime : default(float))
                .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasBuildingProperties")));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveString<ThingDef>("SowTag","Sow Tag",240,"",t=>((ThingDef)t).building != null ? (!string.IsNullOrEmpty(t.building.sowTag) ? t.building.sowTag : "") : "")
			                 .setHidden(f=>!((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasBuildingProperties")));

            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("CombatPower", "Combat Power", 250, default(float), t => ((ThingDef)t).building != null ? t.building.combatPower : default(float), 0)
                             .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasBuildingProperties")));
            props.fields.Add(new SettingsFieldPropertiesListPrimitive("BuildingTags", "Tags", 260, typeof(List<string>), new List<string>(),
                      t => ((ThingDef)t).building != null ? ((ThingDef)t).building.buildingTags != null ? ((ThingDef)t).building.buildingTags.ToList() : 
                        new List<string>() : new List<string>(),
                      null, () => "", true)
                      .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasBuildingProperties")));


            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("UseIdeoColor", "Use Ideo Color", 270, default(bool), t => t.building != null ? t.building.useIdeoColor : default(bool))
                .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasBuildingProperties")));
            props.fields.Add(new SettingsFieldPropertiesSelectable("DominantStyleCategory", "Style Category", 271, typeof(StyleCategoryDef), null, t => ((BuildableDef)t).dominantStyleCategory,
                                                             f => from object d in DefDatabase<StyleCategoryDef>.AllDefsListForReading select d,
                                                             f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<StyleCategoryDef>("DominantStyleCategory", (StyleCategoryDef)def); return true; },
                                                             SettingsFieldPropertiesSelectable.DefLabelProducer, true)
                                                             .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasBuildingProperties")));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("ArtificialForMeditationPurposes", "Artificial for Meditation", 275, true, t => t.building != null ? t.building.artificialForMeditationPurposes : true)
                .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasBuildingProperties")));

            //optional comps
            compRefuelable();
			props.fields.Add(new SettingsFieldPropertiesSettings("CompRefuelable","Refuelable Properties",300,typeof(SettingsInstance),null,
			                                                     t=> ((ThingDef)t).GetCompProperties<CompProperties_Refuelable>() != null ? new SettingsInstance("compRefuelable", ((ThingDef)t).GetCompProperties<CompProperties_Refuelable>(), null, null) : null,"compRefuelable")
						.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompRefuelable")));
			
			compTransporter();
			props.fields.Add(new SettingsFieldPropertiesSettings("CompTransporter","Transporter Properties",310,typeof(SettingsInstance),null,
			                                                     t=> ((ThingDef)t).GetCompProperties<CompProperties_Transporter>() != null ? new SettingsInstance("compTransporter", ((ThingDef)t).GetCompProperties<CompProperties_Transporter>(), null, null) : null,"compTransporter")
						.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompTransporter")));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("LinkableFacilities","Linkable Facilities",320,typeof(List<ThingDef>),
               	new List<ThingDef>(),
               	h=> {
               		CompProperties_AffectedByFacilities cp = ((ThingDef)h).GetCompProperties<CompProperties_AffectedByFacilities>();
               		if ( cp != null ) {
               			return cp.linkableFacilities ?? new List<ThingDef>();
               		}
               		return new List<ThingDef>();
               	},
               	f=>DefDatabase<ThingDef>.AllDefsListForReading.Where(t=>t.GetCompProperties<CompProperties_Facility>() != null).Select(t=>(object)t),
				flow=>{
					ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
					int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
					return listEntry=>{
					flow.addPostRenderCallback(fl=>{setting.get<List<ThingDef>>("LinkableFacilities")[listIndex] = 
						                           		(ThingDef)listEntry; setting.markAsChanged();});
					return true;
				};},
				SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<ThingDef>.AllDefsListForReading.First(t => t.GetCompProperties<CompProperties_Facility>() != null),
				true).setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompAffectedByFacilities")));

            compFacility();
            props.fields.Add(new SettingsFieldPropertiesSettings("CompFacility", "Facility Properties", 321, typeof(SettingsInstance), null,
                 t => ((ThingDef)t).GetCompProperties<CompProperties_Facility>() != null ?
                     new SettingsInstance("compFacility", ((ThingDef)t).GetCompProperties<CompProperties_Facility>(), null, null) : null, "compFacility")
                .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompFacility")));

            props.props.Add(new SettingsPropertyProperties("HasCompGasStorage",o=>VanillaFurnitureExpandedConnectors.HasCompPropertyGasStorage((ThingDef)o)));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("GasStorageCapacity", "Gas Storage Capacity", 500, 100, VanillaFurnitureExpandedConnectors.GetGasStorageCapacity)
			                 .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompGasStorage")));
			props.props.Add(new SettingsPropertyProperties("HasCompGasTrader",o=>VanillaFurnitureExpandedConnectors.HasCompPropertyGasTrader((ThingDef)o)));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("GasTraderConsumption","Gas Consumption",510,0f,VanillaFurnitureExpandedConnectors.GetGasTraderGasConsumption)
			                 .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompGasTrader")));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ThingDef>("GasTraderConsumptionWhenUsed","Gas Consumption in Use",520,0f,VanillaFurnitureExpandedConnectors.GetGasTraderGasConsumptionWhenUsed)
			                 .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompGasTrader")));
			
			compBattery();
			props.fields.Add(new SettingsFieldPropertiesSettings("CompBattery","Battery Properties",600,typeof(SettingsInstance),null,
                 t=> ((ThingDef)t).GetCompProperties<CompProperties_Battery>() != null ? 
                 	new SettingsInstance("compBattery", ((ThingDef)t).GetCompProperties<CompProperties_Battery>(), null, null) : null,"compBattery")
				.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompBattery")));
			compPower();
			props.fields.Add(new SettingsFieldPropertiesSettings("CompPower","Power Properties",610,typeof(SettingsInstance),null,
                 t=> ((ThingDef)t).GetCompProperties<CompProperties_Power>() != null ? 
                 	new SettingsInstance("compPower", ((ThingDef)t).GetCompProperties<CompProperties_Power>(), null, null) : null,"compPower")
				.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompPower")));
			compHeatPusher();
			props.fields.Add(new SettingsFieldPropertiesSettings("CompHeatPusher","Heat Pusher Properties",620,typeof(SettingsInstance),null,
                 t=> ((ThingDef)t).GetCompProperties<CompProperties_HeatPusher>() != null ? 
                 	new SettingsInstance("compHeatPusher", ((ThingDef)t).GetCompProperties<CompProperties_HeatPusher>(), null, null) : null,"compHeatPusher")
				.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompHeatPusher")));

            compGlower();
            props.fields.Add(new SettingsFieldPropertiesSettings("CompGlower", "Glower Properties", 625, typeof(SettingsInstance), null,
                 t => ((ThingDef)t).GetCompProperties<CompProperties_Glower>() != null ?
                     new SettingsInstance("compGlower", ((ThingDef)t).GetCompProperties<CompProperties_Glower>(), null, null) : null, "compGlower")
                .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompGlower")));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("ThingCategories", "Categories", 630, typeof(List<ThingCategoryDef>),
                   new List<ThingCategoryDef>(),
                   h => ((ThingDef)h).thingCategories == null ? new List<ThingCategoryDef>() : ((ThingDef)h).thingCategories.ToList(),
                   f => DefDatabase<ThingCategoryDef>.AllDefsListForReading.OrderBy(d => SettingsFieldPropertiesSelectable.DefLabelProducer(d)).ToList<object>(),
                flow => {
                    ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                    int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                    return listEntry => {
                        flow.addPostRenderCallback(fl => {
                            setting.get<List<ThingCategoryDef>>("ThingCategories")[listIndex] = (ThingCategoryDef)listEntry; setting.markAsChanged();
                        });
                        return true;
                    };
                }, SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<ThingCategoryDef>.AllDefsListForReading.First(), true));

            compBiosculpterPod_HealingCycle();
            props.fields.Add(new SettingsFieldPropertiesSettings("CompBiosculpterPod_HealingCycle", "Healing Cycle", 800, typeof(SettingsInstance), null,
                                                                 t => ((ThingDef)t).GetCompProperties<CompProperties_BiosculpterPod_HealingCycle>() != null ? new SettingsInstance("compBiosculpterPod_HealingCycle", ((ThingDef)t).GetCompProperties<CompProperties_BiosculpterPod_HealingCycle>(), null, null) : null, "compBiosculpterPod_HealingCycle")
                        .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompBiosculpterPod_HealingCycle")));

            props.mergers.Add(setting => {
                ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
                int version = setting.get<int>("Version");
                setting.resolveProps(props, t);

                string configuredLabel = setting.get<string>("Label");
                if (!configuredLabel.NullOrEmpty() && configuredLabel.Trim().Length != 0) {
                    t.label = configuredLabel;
                    DefCachedLabelCapSet(t, null);
                }
                t.pathCost = setting.get<int>("PathCost");
                t.pathCostIgnoreRepeat = setting.get<bool>("PathCostIgnoreRepeat");
                t.selectable = setting.get<bool>("Selectable");
                t.neverMultiSelect = setting.get<bool>("NeverMultiSelect");
                t.blockWind = setting.get<bool>("BlockWind");
                t.holdsRoof = setting.get<bool>("HoldsRoof");
                t.coversFloor = setting.get<bool>("CoversFloor");
                t.castEdgeShadows = setting.get<bool>("CastEdgeShadows");
                t.staticSunShadowHeight = setting.get<float>("StaticSunShadowHeight");
                t.costStuffCount = setting.get<int>("CostStuffCount");
                t.fertility = setting.get<float>("Fertility");
                t.stealable = setting.get<bool>("Stealable");
                t.terrainAffordanceNeeded = setting.get<TerrainAffordanceDef>("TerrainAffordanceNeeded");
                t.fillPercent = setting.get<float>("FillPercent");
                t.dominantStyleCategory = setting.get<StyleCategoryDef>("DominantStyleCategory");
                t.canGenerateDefaultDesignator = setting.get<bool>("CanGenerateDefaultDesignator");

                Traversability passability;
                try
                {
                    passability = (Traversability)Enum.Parse(typeof(Traversability), setting.get<string>("Passability"));
                }
                catch (ArgumentException)
                {
                    passability = default(Traversability);
                }
                t.passability = passability;

                bool minifiable = setting.get<bool>("Minifiable");
                if (minifiable && !t.Minifiable)
                {
                    //find existing blueprint definition
                    t.minifiedDef = DefDatabase<ThingDef>.GetNamed("MinifiedThing");
                    //generate install blueprint if missing
                    if (t.installBlueprintDef == null)
                    {
                        MethodInfo mi = typeof(ThingDefGenerator_Buildings).GetMethod("NewBlueprintDef_Thing", BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic);
                        ThingDef tdInstallBlueprint = (ThingDef)mi.Invoke(null, new object[] { t, true, t.blueprintDef });
                        tdInstallBlueprint.ResolveReferences();
                        bool hasError = false;
                        foreach (string error in tdInstallBlueprint.ConfigErrors())
                        {
                            hasError = true;
                            Log.Error(string.Concat(new object[]
                            {
                                "Config error in ",
                                tdInstallBlueprint,
                                ": ",
                                error
                            }));
                        }
                        if (!hasError)
                        {
                            tdInstallBlueprint.modContentPack = LoadedModManager.GetMod<QOLMod>().Content;
                            MethodInfo miShortHashGiver = typeof(ShortHashGiver).GetMethod("GiveShortHash", BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic);
                            miShortHashGiver.Invoke(null, new object[] { tdInstallBlueprint, typeof(ThingDef) });
                            DefGenerator.AddImpliedDef<ThingDef>(tdInstallBlueprint);
                        }
                        else
                        {
                            t.installBlueprintDef = null;
                        }
                        tdInstallBlueprint.ClearCachedData();
                    }
                    minifiableHasChanged = true;
                    updateThingFilters = true;
                }
                else if (!minifiable && t.Minifiable)
                {
                    //mark as non minifiable but leave blueprint alone
                    //TODO possible to map a custom minifiable defName in case modders use different defs than MinifiedThing
                    t.minifiedDef = null;
                    minifiableHasChanged = true;
                    updateThingFilters = true;
                }
                
                mergeThingCategories(t, setting);

                ThingCategoryDef cat = DefDatabase<ThingCategoryDef>.GetNamed("BuildingsMinified");
                if (t.Minifiable && t.thingCategories.NullOrEmpty())
                {
                    if (t.thingCategories == null) t.thingCategories = new List<ThingCategoryDef>();
                    t.thingCategories.Add(cat);
                    if (!cat.childThingDefs.Contains(t)) cat.childThingDefs.Add(t);
                    updateThingFilters = true;
                }
                else if (!t.Minifiable && !t.thingCategories.NullOrEmpty() && t.thingCategories.Count == 1 && t.thingCategories[0] == cat)
                {
                    t.thingCategories.Clear();
                    cat.childThingDefs.Remove(t);
                    updateStockpiles(t);
                    updateThingFilters = true;
                }

                if (t.building != null)
                {
                    t.building.turretGunDef = setting.get<ThingDef>("TurretGunDef");
                    string configuredSowTag = setting.get<string>("SowTag");
                    if (configuredSowTag != null && configuredSowTag.Length == 0) configuredSowTag = null;
                    t.building.sowTag = configuredSowTag;

                    t.building.turretBurstCooldownTime = setting.get<float>("TurretBurstCooldownTime");
                    t.building.turretBurstWarmupTime = setting.get<float>("TurretBurstWarmupTime");
                    t.building.turretInitialCooldownTime = setting.get<float>("TurretInitialCooldownTime");
                    t.building.combatPower = setting.get<float>("CombatPower");
                    t.building.useIdeoColor = setting.get<bool>("UseIdeoColor");
                    t.building.artificialForMeditationPurposes = setting.get<bool>("ArtificialForMeditationPurposes");

                    List<string> buildingTags = setting.get<List<string>>("BuildingTags");
                    if (buildingTags != null)
                    {
                        if (t.building.buildingTags == null) t.building.buildingTags = new List<string>();
                        else t.building.buildingTags.Clear();
                        t.building.buildingTags.AddRange(buildingTags);
                    }
                }
				
    			t.designationCategory = setting.get<DesignationCategoryDef>("DesignationCategory");
    			t.designatorDropdown = setting.get<DesignatorDropdownGroupDef>("DesignatorDropdown");
    			
    			List<ResearchProjectDef> researchPrerequisites = setting.get<List<ResearchProjectDef>>("ResearchPrerequisites");
            	if ( researchPrerequisites != null ) {
    				if ( researchPrerequisites.Count == 0 ) {
    					t.researchPrerequisites = null;
    				} else {
	            		if ( t.researchPrerequisites == null ) t.researchPrerequisites = new List<ResearchProjectDef>();
	            		else t.researchPrerequisites.Clear();
	            		t.researchPrerequisites.AddRange(researchPrerequisites);
    				}
            	}
    			
    			List<StuffCategoryDef> stuffCategories = setting.get<List<StuffCategoryDef>>("StuffCategories");
            	if ( stuffCategories != null ) {
            		if ( t.stuffCategories == null ) t.stuffCategories = new List<StuffCategoryDef>();
            		else t.stuffCategories.Clear();
            		t.stuffCategories.AddRange(stuffCategories);
            	}
    			
         		List<SettingsInstance> costList = setting.get<List<SettingsInstance>>("CostList");
				if ( costList != null ) {
					if ( t.costList == null ) t.costList = new List<ThingDefCountClass>();
					else t.costList.Clear();
					foreach ( ISettingsInstance e in costList ) {
						if ( e.get<ThingDef>("Thing") != null ) {
							t.costList.Add(new ThingDefCountClass(e.get<ThingDef>("Thing"),e.get<int>("Count")));
						}
					}
				}
         		
         		List<SettingsInstance> killedLeavings = setting.get<List<SettingsInstance>>("KilledLeavings");
         		if ( !killedLeavings.NullOrEmpty() ) {
					if ( t.killedLeavings == null ) t.killedLeavings = new List<ThingDefCountClass>();
					else t.killedLeavings.Clear();
					foreach ( ISettingsInstance e in killedLeavings ) {
						if ( e.get<ThingDef>("Thing") != null ) {
							t.killedLeavings.Add(new ThingDefCountClass(e.get<ThingDef>("Thing"),e.get<int>("Count")));
						}
					}
         		} else if ( killedLeavings != null ) {
         			if ( t.killedLeavings != null ) t.killedLeavings.Clear();
         			t.killedLeavings = null;
         		}
              	
            	List<SettingsInstance> statBases = setting.get<List<SettingsInstance>>("statBases");
            	if ( statBases != null ) {
            		if ( t.statBases == null ) t.statBases = new List<StatModifier>();
            		t.statBases.Clear();
            		foreach ( ISettingsInstance s in statBases ) {
            			t.statBases.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
            		}
            	}

                List<RecipeDef> recipes = setting.get<List<RecipeDef>>("Recipes");
                if ( recipes != null )
                {
                    if (t.recipes == null) t.recipes = new List<RecipeDef>();
                    modifyList(t.recipes, recipes, out List<RecipeDef> added, out List<RecipeDef> removed);
                    foreach ( RecipeDef rd in added )
                    {
                        RecipeDefIsSurgeryCachedSet(rd, null);
                    }
                    foreach (RecipeDef rd in removed)
                    {
                        RecipeDefIsSurgeryCachedSet(rd, null);
                    }
                    ThingDefAllRecipesCachedSet(t, null);
                }

                CompProperties_AffectedByFacilities cabf = t.GetCompProperties<CompProperties_AffectedByFacilities>();
            	if ( cabf != null ) {
            		List<ThingDef> linkables = setting.get<List<ThingDef>>("LinkableFacilities");
            		if ( linkables != null ) {
            			if ( cabf.linkableFacilities == null ) cabf.linkableFacilities = new List<ThingDef>();
	            		else cabf.linkableFacilities.Clear();
	            		cabf.linkableFacilities.AddRange(linkables);
            		}
            	}
            	
            	CompProperties_Refuelable cr = t.GetCompProperties<CompProperties_Refuelable>();
            	if ( cr != null ) {
            		SettingsInstance si = setting.get<SettingsInstance>("CompRefuelable");
            		if ( si != null ) {
            			cr.fuelConsumptionRate = si.get<float>("FuelConsumptionRate");
            			cr.fuelCapacity = si.get<float>("FuelCapacity");
            			cr.initialFuelPercent = si.get<float>("InitialFuelPercent");
            			cr.autoRefuelPercent = si.get<float>("AutoRefuelPercent");
            			cr.fuelConsumptionPerTickInRain = si.get<float>("FuelConsumptionPerTickInRain");
            			cr.initialConfigurableTargetFuelLevel = si.get<float>("InitialConfigurableTargetFuelLevel");
            			cr.minimumFueledThreshold = si.get<float>("MinimumFueledThreshold");
            			CompProperties_RefuelableFuelMultiplierSet(cr,si.get<float>("FuelMultiplier"));
            			
            			cr.destroyOnNoFuel = si.get<bool>("DestroyOnNoFuel");
            			cr.consumeFuelOnlyWhenUsed = si.get<bool>("ConsumeFuelOnlyWhenUsed");
            			cr.showFuelGizmo = si.get<bool>("ShowFuelGizmo");
            			cr.targetFuelLevelConfigurable = si.get<bool>("TargetFuelLevelConfigurable");
            			cr.drawOutOfFuelOverlay = si.get<bool>("DrawOutOfFuelOverlay");
            			cr.drawFuelGaugeInMap = si.get<bool>("DrawFuelGaugeInMap");
            			cr.atomicFueling = si.get<bool>("AtomicFueling");
            			cr.factorByDifficulty = si.get<bool>("FactorByDifficulty");
            			
            			cr.fuelFilter = ThingFilterAdapter.AsThingFilter(si.get<ISettingsInstance>("FuelFilter"));
            		}
            	}
            	
            	CompProperties_Transporter ct = t.GetCompProperties<CompProperties_Transporter>();
            	if ( ct != null ) {
            		SettingsInstance si = setting.get<SettingsInstance>("CompTransporter");
            		if ( si != null ) {
            			ct.massCapacity = si.get<float>("MassCapacity");
            			ct.restEffectiveness = si.get<float>("RestEffectiveness");
            		}
            	}

                CompProperties_Facility cFacility = t.GetCompProperties<CompProperties_Facility>();
                if (cFacility != null)
                {
                    SettingsInstance si = setting.get<SettingsInstance>("CompFacility");
                    if (si != null)
                    {
                        cFacility.maxSimultaneous = si.get<int>("MaxSimultaneous");
                        cFacility.maxDistance = si.get<float>("MaxDistance");
                        cFacility.mustBePlacedAdjacent = si.get<bool>("MustBePlacedAdjacent");
                        cFacility.mustBePlacedAdjacentCardinalToAndFacingBedHead = si.get<bool>("MustBePlacedAdjacentCardinalToAndFacingBedHead");
                        cFacility.mustBePlacedAdjacentCardinalToBedHead = si.get<bool>("MustBePlacedAdjacentCardinalToBedHead");
                        cFacility.canLinkToMedBedsOnly = si.get<bool>("CanLinkToMedBedsOnly");

                        List<SettingsInstance> statOffsets = si.get<List<SettingsInstance>>("StatOffsets");
                        if (statOffsets != null)
                        {
                            if (cFacility.statOffsets == null) cFacility.statOffsets = new List<StatModifier>();
                            cFacility.statOffsets.Clear();
                            foreach (ISettingsInstance s in statOffsets)
                            {
                                cFacility.statOffsets.Add(new StatModifier() { stat = s.get<StatDef>("Stat"), value = s.get<float>("Value") });
                            }
                        }
                        else cFacility.statOffsets = null;
                    }
                }

                float oldBasePowerConsumption = 0;
            	CompProperties_Battery cbattery = t.GetCompProperties<CompProperties_Battery>();
            	if ( cbattery != null ) {
            		SettingsInstance si = setting.get<SettingsInstance>("CompBattery");
            		if ( si != null ) {
            			cbattery.efficiency = si.get<float>("Efficiency");
            			cbattery.storedEnergyMax = si.get<float>("StoredEnergyMax");
            		}
            	}
            	
            	CompProperties_Power cpower = t.GetCompProperties<CompProperties_Power>();
            	if ( cpower != null ) {
            		SettingsInstance si = setting.get<SettingsInstance>("CompPower");
            		if ( si != null ) {
            			oldBasePowerConsumption = cpower.basePowerConsumption;
            			cpower.basePowerConsumption = si.get<float>("BasePowerConsumption");
            			cpower.shortCircuitInRain = si.get<bool>("ShortCircuitInRain");
            		}
            	}
            	
            	CompProperties_HeatPusher cheatp = t.GetCompProperties<CompProperties_HeatPusher>();
            	if ( cheatp != null ) {
            		SettingsInstance si = setting.get<SettingsInstance>("CompHeatPusher");
            		if ( si != null ) {
            			cheatp.heatPerSecond = si.get<float>("HeatPerSecond");
            			cheatp.heatPushMaxTemperature = si.get<float>("HeatPushMaxTemperature");
            			cheatp.heatPushMinTemperature = si.get<float>("HeatPushMinTemperature");
            		}
            	}

                mergeCompGlower(t, setting.get<SettingsInstance>("CompGlower"));
                mergeCompBiosculpterPod_HealingCycle(t, setting.get<SettingsInstance>("CompBiosculpterPod_HealingCycle"));

                CompProperties cp = VanillaFurnitureExpandedConnectors.GetCompPropertyGasStorage(t);
				if ( cp != null ) {
					VanillaFurnitureExpandedConnectors.SetGasStorageCapacity(cp,setting.get<int>("GasStorageCapacity"));
				}
            	
            	cp = VanillaFurnitureExpandedConnectors.GetCompPropertyGasTrader(t);
				if ( cp != null ) {
					VanillaFurnitureExpandedConnectors.SetGasTraderGasConsumption(cp,setting.get<float>("GasTraderConsumption"));
					VanillaFurnitureExpandedConnectors.SetGasTraderGasConsumptionWhenUsed(cp,setting.get<float>("GasTraderConsumptionWhenUsed"));
				}
            	
            	if ( t.BuildableByPlayer && t.blueprintDef == null ) {
            		//when making a non buildable thing buildable it won't have the blueprint that is implied during game start
            		//ThingDefGenerator_Buildings.NewBlueprintDef_Thing(t, false, null);
            		ThingDef blueprint = (ThingDef)typeof(ThingDefGenerator_Buildings).GetMethod("NewBlueprintDef_Thing", BindingFlags.Static|BindingFlags.NonPublic|BindingFlags.Public).Invoke(null,new object[]{t,false,null});
            		if ( DefDatabase<ThingDef>.GetNamedSilentFail(blueprint.defName) == null ) DefGenerator.AddImpliedDef<ThingDef>(blueprint);
            	}
            	
            	//updates for power properties
            	if ( Current.Game != null && ( cbattery != null || cpower != null ) ) {
					HarmonyPatchNamespaceProperties hpnsp;
					HarmonyPatchNamespace.namespaces.TryGetValue("PowerTraderZero", out hpnsp);
					foreach ( Map m in Find.Maps ) {
						m.listerThings.ThingsOfDef(t).ForEach(thingOnMap=>{
                          	CompPowerTrader pt = thingOnMap.TryGetComp<CompPowerTrader>();
                          	if ( pt != null ) {
                          		if ( hpnsp != null && hpnsp.active ) {
                              		if (Math.Abs(oldBasePowerConsumption) < float.Epsilon && Math.Abs(pt.Props.basePowerConsumption) > float.Epsilon ) pt.ResetPowerVars();
                          			if (Math.Abs(pt.Props.basePowerConsumption) < float.Epsilon && Math.Abs(oldBasePowerConsumption) > float.Epsilon ) pt.PowerOn = true;
                          		}
                          		pt.SetUpPowerVars();
                          	} else {
                          		CompPowerBattery pb = thingOnMap.TryGetComp<CompPowerBattery>();
                          		if ( hpnsp != null && hpnsp.active ) {
                              		if (Math.Abs(oldBasePowerConsumption) < float.Epsilon && Math.Abs(pb.Props.basePowerConsumption) > float.Epsilon ) pb.ResetPowerVars();
                          		}
                          		pb.SetUpPowerVars();
                          	}
                          });
                    }
				}

                t.ClearCachedData();
          	});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ThingDef>("Buildings","Select Building",
			                                                                        ()=>DefDatabase<ThingDef>.AllDefsListForReading.Where(t=>t.category == ThingCategory.Building).ToList(),
				def => def.defName, DefLabelProducerGeneric<ThingDef>.LabelProducerNotNull,key => DefDatabase<ThingDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			
			props.validators.Add(setting => {
                 	ThingDef dc = DefDatabase<ThingDef>.GetNamed(setting.getKey());
                 	if ( dc == null ) return false;
                 	int version = setting.get<int>("Version");
                 	if ( version < 2 ) {
                 		setting.set<DesignationCategoryDef>("DesignationCategory", dc.designationCategory);
                 		setting.set<DesignatorDropdownGroupDef>("DesignatorDropdown", dc.designatorDropdown);
                 	}
                 	if ( version < 3 ) {
                 		setting.resetFields(dc, new[]{"StuffCategories","CostList"});
                 	}
                 	if ( version < 4 ) {
                 		setting.resetFields(dc, new[]{"CompRefuelable"});
                 	}
                 	if ( version < 5 ) {
                 		setting.resetFields(dc, new[]{"CompTransporter"});
                 	}
                 	
                 	setting.set<int>("Version",5);
                 	return true;
                 });
			
			QOLMod.addApplySettingsListener(mod=>{
                //updating linkable buildings in facilities
                DefDatabase<ThingDef>.AllDefsListForReading.ForEach(
                    t => {
                        CompProperties_Facility cp = t.GetCompProperties<CompProperties_Facility>();
                        if (cp != null)
                        {
                            if (cp.linkableBuildings == null) cp.linkableBuildings = new List<ThingDef>();
                            else cp.linkableBuildings.Clear();
                            List<ThingDef> allDefsListForReading = DefDatabase<ThingDef>.AllDefsListForReading;
                            for (int i = 0; i < allDefsListForReading.Count; i++)
                            {
                                CompProperties_AffectedByFacilities compProperties = allDefsListForReading[i].GetCompProperties<CompProperties_AffectedByFacilities>();
                                if (compProperties != null && compProperties.linkableFacilities != null)
                                {
                                    for (int j = 0; j < compProperties.linkableFacilities.Count; j++)
                                    {
                                        if (compProperties.linkableFacilities[j] == t)
                                        {
                                            cp.linkableBuildings.Add(allDefsListForReading[i]);
                                            break;
                                        }
                                    }
                                }
                            }
                        }
                    });
                if ( !Find.Maps.NullOrEmpty() ) {
                	Find.Maps.ForEach(map=>{
		                if ( map.mapDrawer != null ) map.mapDrawer.WholeMapChanged(MapMeshFlag.Buildings);
		                //triggering a relinking of buildings
		                map.listerThings.ThingsInGroup(ThingRequestGroup.AffectedByFacilities).ForEach(
		                  	b=> {
		                  		CompAffectedByFacilities cp = b.TryGetComp<CompAffectedByFacilities>();
		                  		if ( cp != null ) {
		                  			cp.Notify_LOSBlockerSpawnedOrDespawned();
		                  		}
		                  	}
		                );
	                });
                }
                if ( !QOLMod.getSettings().avoidDesignatorRefresh || applyCounter == 1 ) {
	            	foreach ( DesignationCategoryDef c in DefDatabase<DesignationCategoryDef>.AllDefsListForReading ) {
	                	typeof(DesignationCategoryDef).GetMethod("ResolveDesignators", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public).Invoke(c,null);
	            	}
                }
            });
		}

        private static void mergeThingCategories(ThingDef t, ISettingsInstance setting)
        {
            List<ThingCategoryDef> thingCategories = setting.get<List<ThingCategoryDef>>("ThingCategories");
            if (thingCategories != null)
            {
                if (thingCategories.Count == 0)
                {
                    if (!t.thingCategories.NullOrEmpty())
                    {
                        foreach (ThingCategoryDef tcd in t.thingCategories)
                        {
                            tcd.childThingDefs.Remove(t);
                        }
                        updateThingFilters = true;
                    }
                    t.thingCategories = null;

                    updateStockpiles(t);
                }
                else
                {
                    if (t.thingCategories == null) t.thingCategories = new List<ThingCategoryDef>();
                    modifyList(t.thingCategories, thingCategories, out List<ThingCategoryDef> added, out List<ThingCategoryDef> removed);
                    foreach (ThingCategoryDef tcd in added)
                    {
                        if (!tcd.childThingDefs.Contains(t)) tcd.childThingDefs.Add(t);
                    }
                    foreach (ThingCategoryDef tcd in removed)
                    {
                        tcd.childThingDefs.Remove(t);
                    }

                    if (!added.NullOrEmpty() || !removed.NullOrEmpty())
                    {
                        updateThingFilters = true;
                    }
                }
            }
        }

        private static void updateStockpiles(ThingDef t)
        {
            if (Current.Game != null)
            {
                //running through available maps and all zones to update stockpile filters
                foreach (Map m in Find.Maps)
                {
                    if (m.zoneManager != null)
                    {
                        List<Zone> zones = m.zoneManager.AllZones;
                        if (zones != null)
                        {
                            foreach (Zone z in zones)
                            {
                                Zone_Stockpile stockpile = z as Zone_Stockpile;
                                if (stockpile != null)
                                {
                                    if (stockpile.settings != null && stockpile.settings.filter != null && stockpile.settings.filter.Allows(t))
                                    {
                                        stockpile.settings.filter.SetAllow(t, false);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        private static void modifyList<T>(List<T> source, List<T> target, out List<T> added, out List<T> removed)
        {
            added = new List<T>(target.Where(e=>!source.Contains(e)));
            removed = new List<T>(source.Where(e => !target.Contains(e)));
            source.Clear();
            source.AddRange(target);
        }

        private static void MentalStateGivers() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("mentalStateGiverModifier");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("MentalState","mentalState",10,typeof(MentalStateDef),null,pc=>((MentalStateGiver)pc).mentalState,
					f=>from object d in DefDatabase<MentalStateDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<MentalStateDef>("MentalState",(MentalStateDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<MentalStateGiver>("MtbDays","Mtb Days",20,default(float),pc=>pc.mtbDays));
				
				props.dynamicLabel = settings => settings.get<MentalStateDef>("MentalState").label+"("+settings.get<float>("MtbDays")+")";
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void PawnCapacityModifiers() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("pawnCapacityModifier");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("Capacity","Capacity",10,typeof(PawnCapacityDef),null,pc=>((PawnCapacityModifier)pc).capacity,
					f=>from object d in DefDatabase<PawnCapacityDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<PawnCapacityDef>("Capacity",(PawnCapacityDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<PawnCapacityModifier>("Offset","Offset",20,default(float),pc=>pc.offset));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<PawnCapacityModifier>("SetMax","Max",30,999f,pc=>pc.setMax));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<PawnCapacityModifier>("PostFactor","Post Factor",40,1f,pc=>pc.postFactor));
				
				props.dynamicLabel = settings => settings.get<PawnCapacityDef>("Capacity").label;
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void HediffStages() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("hediffStages");
			if ( props.fields.NullOrEmpty() ) {
                /*
                 * All properties must be available for editing since we replace the entire entry
                minSeverity,becomeVisible,lifeThreatening,tale,vomitMtbDays,deathMtbDays,painFactor,painOffset,totalBleedFactor,naturalHealingFactor,
                forgetMemoryThoughtMtbDays,pctConditionalThoughtsNullified,opinionOfOthersFactor,hungerRateFactor,hungerRateFactorOffset,restFallFactor,
                restFallFactorOffset,socialFightChanceFactor,foodPoisoningChanceFactor,mentalBreakMtbDays,allowedMentalBreakIntensities,makeImmuneTo,
                capMods,mentalStateGivers,statOffsets,statFactors,statOffsetEffectMultiplier,statFactorEffectMultiplier,capacityFactorEffectMultiplier,
                disabledWorkTags,partEfficiencyOffset,partIgnoreMissingHP,destroyPart

		        public string label;
		        public string overrideLabel;
		        public string untranslatedLabel;***

		        public string mentalBreakExplanation;
		        public string overrideTooltip;
		        public string extraTooltip;

                public List<HediffGiver> hediffGivers;***

                We cannot replace the entity, we must make it editable if at all possible. Since hediffGivers is an abstract construct that would require tons of custom cases
                Retaining those entries is crucial.
                Possibilities: Maintaining a lookup with custom id like HediffStage_[Hediff.defName]_[IndexInStages] then when merging we grab the reference?
                    Would require to persist the referred id in the settings instance in case we delete and then add entries.
                    The referred objects would be cached before the first merge and based on which references were requested so that there is no bloat.
                Simpler and more stable is that we do not allow adding or removing stages and always merge into existing instances.
                 */
                props.fields.Add(new SettingsFieldPropertiesPrimitiveString<HediffStage>("Label", "Label", 1, "", h => h.label));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveString<HediffStage>("OverrideLabel", "Over. Label", 5, null, h => h.overrideLabel) { EmptyIsNull = true });
                props.fields.Add(new SettingsFieldPropertiesPrimitiveText<HediffStage>("OverrideTooltip", "Over. Tooltip", 6, null, h => h.overrideTooltip) { EmptyIsNull = true });
                props.fields.Add(new SettingsFieldPropertiesPrimitiveText<HediffStage>("ExtraTooltip", "Extra Tooltip", 7, null, h => h.extraTooltip) { EmptyIsNull = true });

                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("MinSeverity","Min Severity",10,default(float),h=>h.minSeverity));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffStage>("BecomeVisible","Become Visible",20,default(bool),h=>h.becomeVisible));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffStage>("LifeThreatening","Life Threatening",30,default(bool),h=>h.lifeThreatening));
				props.fields.Add(new SettingsFieldPropertiesSelectable("Tale","Tale",40,typeof(TaleDef),null,h=>((HediffStage)h).tale,
					f=>from object d in DefDatabase<TaleDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<TaleDef>("Tale",(TaleDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("VomitMtbDays","Vomit Mtb Days",50,-1f,h=>h.vomitMtbDays));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("DeathMtbDays","Death Mtb Days",60,-1f,h=>h.deathMtbDays));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("PainFactor","Pain Factor",70,1f,h=>h.painFactor));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("PainOffset","Pain Offset",80,default(float),h=>h.painOffset));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("ForgetMemoryThoughtMtbDays","Forget Memory Thought Mtb Days",90,-1f,h=>h.forgetMemoryThoughtMtbDays));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("PctConditionalThoughtsNullified","Pct Conditional Thoughts Nullified",100,default(float),h=>h.pctConditionalThoughtsNullified));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("OpinionOfOthersFactor","Opinion of others Factor",110,1f,h=>h.opinionOfOthersFactor));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("HungerRateFactor","Hunger Rate Factor",120,1f,h=>h.hungerRateFactor));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("HungerRateFactorOffset","Hunger rate factor Offset",130,default(float),h=>h.hungerRateFactorOffset));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("RestFallFactor","Rest fall Factor",140,1f,h=>h.restFallFactor));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("RestFallFactorOffset","Rest fall Factor Offset",150,default(float),h=>h.restFallFactorOffset));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("SocialFightChanceFactor","Social Fight Chance Factor",160,1f,h=>h.socialFightChanceFactor));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("MentalBreakMtbDays","Mental Break Mtb Days",170,default(float),h=>h.mentalBreakMtbDays));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("PartEfficiencyOffset","Part Efficiency Offset",180,default(float),h=>h.partEfficiencyOffset));
				
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("TotalBleedFactor","Bleed Factor",181,1f,h=>h.totalBleedFactor));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("NaturalHealingFactor","Healing Factor",182,-1,h=>h.naturalHealingFactor));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffStage>("FoodPoisoningChanceFactor","Food Poisoning Chance",183,1f,h=>h.foodPoisoningChanceFactor));
				
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffStage>("PartIgnoreMissingHP","Part Ignore Missing HP",190,default(bool),h=>h.partIgnoreMissingHP));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffStage>("DestroyPart","Destroy Part",200,default(bool),h=>h.destroyPart));
				
				StatModifier();
				props.fields.Add(new SettingsFieldPropertiesListSettings("StatOffsets","Stat Offsets",210,typeof(List<SettingsInstance>),null,
                        t=> ((HediffStage)t).statOffsets.NullOrEmpty() ? new List<SettingsInstance>() : ((HediffStage)t).statOffsets.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
				props.fields.Add(new SettingsFieldPropertiesSelectable("StatOffsetEffectMultiplier","Stat Offset Multiplier",211,typeof(StatDef),null,h=>((HediffStage)h).statOffsetEffectMultiplier,
					f=>from object d in DefDatabase<StatDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<StatDef>("StatOffsetEffectMultiplier",(StatDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				
				props.fields.Add(new SettingsFieldPropertiesSelectable("StatFactorEffectMultiplier","Stat Factor Multiplier",212,typeof(StatDef),null,h=>((HediffStage)h).statFactorEffectMultiplier,
					f=>from object d in DefDatabase<StatDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<StatDef>("StatFactorEffectMultiplier",(StatDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				props.fields.Add(new SettingsFieldPropertiesListSettings("StatFactors","Stat Factors",213,typeof(List<SettingsInstance>),null,
                        t=> ((HediffStage)t).statFactors.NullOrEmpty() ? new List<SettingsInstance>() : ((HediffStage)t).statFactors.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
				
				props.fields.Add(new SettingsFieldPropertiesListSelectable("MakeImmuneTo","Make Immune To",220,typeof(List<HediffDef>),new List<HediffDef>(),
											h=>((HediffStage)h).makeImmuneTo == null ? new List<HediffDef>() : ((HediffStage)h).makeImmuneTo.ToList(),
											f=>DefDatabase<HediffDef>.AllDefsListForReading.Cast<object>(),
											flow=>{
												ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
												int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
												return listEntry=>{
												flow.addPostRenderCallback(fl=>{setting.get<List<HediffDef>>("MakeImmuneTo")[listIndex] = (HediffDef)listEntry; setting.markAsChanged();});
												return true;
											};},
											SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<HediffDef>.AllDefsListForReading.First(),
											true));
				
				props.fields.Add(new SettingsFieldPropertiesSelectable("CapacityFactorEffectMultiplier","Capacity Multiplier",229,typeof(StatDef),null,h=>((HediffStage)h).capacityFactorEffectMultiplier,
					f=>from object d in DefDatabase<StatDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<StatDef>("CapacityFactorEffectMultiplier",(StatDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				PawnCapacityModifiers();
				props.fields.Add(new SettingsFieldPropertiesListSettings("CapMods","Capacity Modifiers",230,typeof(List<SettingsInstance>),null,
                        t=> ((HediffStage)t).capMods.NullOrEmpty() ? new List<SettingsInstance>() : ((HediffStage)t).capMods.Where(tc => tc != null).Select(tc=>new SettingsInstance("pawnCapacityModifier",tc,null,null)).ToList(),
                        "pawnCapacityModifier",()=>new SettingsInstance("pawnCapacityModifier",new PawnCapacityModifier(){capacity=DefDatabase<PawnCapacityDef>.AllDefsListForReading.First()},null,null),true));
				
				props.fields.Add(new SettingsFieldPropertiesListSelectable("AllowedMentalBreakIntensities","Allowed MentalBreak Intensities",239,typeof(List<string>),new List<string>(),
			        t=> ((HediffStage)t).allowedMentalBreakIntensities != null ? (from MentalBreakIntensity spt in ((HediffStage)t).allowedMentalBreakIntensities select Enum.GetName(typeof(MentalBreakIntensity),spt)).ToList() : new List<string>(),
			        f => Enum.GetNames(typeof(MentalBreakIntensity)).Cast<object>(),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
							flow.addPostRenderCallback(fl=>{setting.get<List<string>>("AllowedMentalBreakIntensities")[listIndex] = (string)listEntry; setting.markAsChanged();});
							return true;
						};},
					entry => (string)entry,
					()=>Enum.GetName(typeof(MentalBreakIntensity),MentalBreakIntensity.None),
					true));
				
				MentalStateGivers();
				props.fields.Add(new SettingsFieldPropertiesListSettings("MentalStateGivers","Mental State Givers",240,typeof(List<SettingsInstance>),null,
                        t=> ((HediffStage)t).mentalStateGivers.NullOrEmpty() ? new List<SettingsInstance>() : 
                                 ((HediffStage)t).mentalStateGivers.Where(tc => tc != null).Select(tc=>new SettingsInstance("mentalStateGiverModifier",tc,null,null)).ToList(),
                        "mentalStateGiverModifier",
                        ()=>new SettingsInstance("mentalStateGiverModifier",new MentalStateGiver(){mentalState=DefDatabase<MentalStateDef>.AllDefsListForReading.First()}
                                                 ,null,null),true));

                props.fields.Add(new SettingsFieldPropertiesPrimitiveText<HediffStage>("MentalBreakExplanation", "Mental Break Explanation", 245, null, h => h.mentalBreakExplanation) { EmptyIsNull = true });

                props.fields.Add(new SettingsFieldPropertiesListSelectable("DisabledWorkTags","Disabled Work Tags",250,typeof(List<string>),new List<string>(),
					t=> GetEnumFlagsNames<WorkTags>(((HediffStage)t).disabledWorkTags),
					f => Enum.GetNames(typeof(WorkTags)).Where(e=>e != "None").Cast<object>(),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
							flow.addPostRenderCallback(fl=>{setting.get<List<string>>("DisabledWorkTags")[listIndex] = (string)listEntry; setting.markAsChanged();});
							return true;
						};},
					entry => (string)entry,
					()=>Enum.GetName(typeof(WorkTags),WorkTags.None),
					true));
				
				props.dynamicLabel = settings => settings.get<string>("Label");
				
                props.visualization = new SettingsVisualizationEdit(f => "EditEntry".Translate(((ISettingsInstance)f.flowScope["edit"]).getLabel()));
            }
		}

        private static void compReactOnDamage()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compReactOnDamage");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesSelectable("DamageDefIncoming", "Incoming Damage", 10, typeof(DamageDef), null, 
                    p => ((HediffCompProperties_ReactOnDamage)p).damageDefIncoming,
                    f => from object d in DefDatabase<DamageDef>.AllDefsListForReading select d,
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set("DamageDefIncoming", (DamageDef)def); return true; },
                    SettingsFieldPropertiesSelectable.DefLabelProducer, true));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffCompProperties_ReactOnDamage>("Vomit", "Vomit", 20, false, p => p.vomit));
                props.fields.Add(new SettingsFieldPropertiesSelectable("CreateHediff", "Hediff", 30, typeof(HediffDef), null,
                    p => ((HediffCompProperties_ReactOnDamage)p).createHediff,
                    f => from object d in DefDatabase<HediffDef>.AllDefsListForReading select d,
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set("CreateHediff", (HediffDef)def); return true; },
                    SettingsFieldPropertiesSelectable.DefLabelProducer, true));
                props.fields.Add(new SettingsFieldPropertiesSelectable("CreateHediffOn", "Hediff On", 40, typeof(BodyPartDef), null,
                    p => ((HediffCompProperties_ReactOnDamage)p).createHediffOn,
                    f => from object d in DefDatabase<BodyPartDef>.AllDefsListForReading select d,
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set("CreateHediffOn", (BodyPartDef)def); return true; },
                    SettingsFieldPropertiesSelectable.DefLabelProducer, true));

                props.dynamicLabel = si => "Properties";
                props.visualization = new SettingsVisualizationEdit(f => "Edit Properties");
            }
        }

        private static void hediffs() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("hediffs");
			/*
				public float initialSeverity = 0.5f;
				public float lethalSeverity = -1f;
				public bool tendable;
				public bool isBad = true;
				public ThingDef spawnThingOnRemoved;
				public float chanceToCauseNoPain;
				public bool makesSickThought;
				public bool makesAlert = true;
				public NeedDef causesNeed; <-- AddOrRemoveNeedsAsAppropriate() must be called on all pawns
				public float minSeverity;
				public float maxSeverity = 3.40282347E+38f;
				public bool scenarioCanAdd;
				public bool cureAllAtOnceIfCuredByItem;
				public TaleDef taleOnVisible;
				public bool everCurableByItem = true;
				public bool displayWound;
				public Color defaultLabelColor = Color.white;
				
				public InjuryProps injuryProps;
				public AddedBodyPartProps addedPartProps;
				public List<HediffStage> stages;
				public List<HediffGiver> hediffGivers; <-- abstract class ignore
			 */
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffDef>("InitialSeverity","Initial Severity",10,0.5f,h=>h.initialSeverity));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffDef>("LethalSeverity","Lethal Severity",20,-1.0f,h=>h.lethalSeverity));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("Tendable","Tendable",30,default(bool),h=>h.tendable));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("IsBad","IsBad",40,true,h=>h.isBad));
			props.fields.Add(new SettingsFieldPropertiesSelectable("SpawnThingOnRemoved","Spawn on Removal",50,typeof(ThingDef),null,h=>((HediffDef)h).spawnThingOnRemoved,
				f=>from object d in DefDatabase<ThingDef>.AllDefsListForReading where !((ThingDef)d).MadeFromStuff select d,
				f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("SpawnThingOnRemoved",(ThingDef)def); return true;},
				SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffDef>("ChanceToCauseNoPain","Chance of no Pain",60,default(float),h=>h.chanceToCauseNoPain));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("MakesSickThought","Sick Thought",70,default(bool),h=>h.makesSickThought));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("MakesAlert","Makes Alert",80,true,h=>h.makesAlert));
			props.fields.Add(new SettingsFieldPropertiesSelectable("CausesNeed","Causes Need",90,typeof(NeedDef),null,h=>((HediffDef)h).causesNeed,
                           f=>from NeedDef d in DefDatabase<NeedDef>.AllDefsListForReading where typeof(Need_Chemical).IsAssignableFrom(d.needClass) select (object)d,
				f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<NeedDef>("CausesNeed",(NeedDef)def); return true;},
				SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffDef>("MinSeverity","Min Severity",100,default(float),h=>h.minSeverity));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffDef>("MaxSeverity","Max Severity",110,float.MaxValue,h=>float.Parse(h.maxSeverity.ToString())));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("ScenarioCanAdd","Scenario Can Add",120,default(bool),h=>h.scenarioCanAdd));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("CureAllAtOnceIfCuredByItem","Cure All with Item",130,default(bool),h=>h.cureAllAtOnceIfCuredByItem));
			props.fields.Add(new SettingsFieldPropertiesSelectable("TaleOnVisible","Tale on becoming visible",140,typeof(TaleDef),null,h=>((HediffDef)h).taleOnVisible,
				f=>from object d in DefDatabase<TaleDef>.AllDefsListForReading select d,
				f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<TaleDef>("TaleOnVisible",(TaleDef)def); return true;},
				SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("EverCurableByItem","Curable by Item",150,true,h=>h.everCurableByItem));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("DisplayWound","Display Wound",160,default(bool),h=>h.displayWound));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveColor<HediffDef>("DefaultLabelColor","Default Label Color",170,UnityEngine.Color.white,h=>h.defaultLabelColor));
			
			//Injury props - Only useful if the hediff class is already defined as Hediff_AddedPart so never create an entry
			props.props.Add(new SettingsPropertyProperties("IsInjury",o=>((HediffDef)o).injuryProps != null));
			//props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("IsInjury","",0,true,h => h.injuryProps != null).setHidden(f => true));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffDef>("PainPerSeverity","Pain per Severity",310,1.0f,
			                                                                      h=>h.injuryProps != null ? h.injuryProps.painPerSeverity : -1f)
							.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("IsInjury")));
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffDef>("AveragePainPerSeverityPermanent","Average Pain per Severity Permanent",320,0.5f,
			                                                                      h=>h.injuryProps != null ? h.injuryProps.averagePainPerSeverityPermanent : -1f)
							.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("IsInjury")));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffDef>("BleedRate","Bleed rate",330,default(float),
			                                                                      h=>h.injuryProps != null ? h.injuryProps.bleedRate : -1f)
							.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("IsInjury")));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("CanMerge","Can Merge",340,default(bool),
			                                                                     h => h.injuryProps != null && h.injuryProps.canMerge)
							.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("IsInjury")));
			//props.fields.Add(new SettingsFieldPropertiesPrimitiveString<HediffDef>("DestroyedLabel","Destroyed Label",350,default(string),h => h.injuryProps != null ? h.injuryProps.destroyedLabel : "").setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("IsInjury")));
			//props.fields.Add(new SettingsFieldPropertiesPrimitiveString<HediffDef>("DestroyedOutLabel","Destroyed out Label",360,default(string),h => h.injuryProps != null ? h.injuryProps.destroyedOutLabel : "").setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("IsInjury")));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("UseRemovedLabel","Use removed Label",370,default(bool),
			                                                                     h => h.injuryProps != null && h.injuryProps.useRemovedLabel)
							.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("IsInjury")));

			//AddedBodyPartProps - Only useful if the hediff class is defined as Hediff_MissingPart or Hediff_Injury so never create an entry 
			props.props.Add(new SettingsPropertyProperties("HasAddedPart",o=>((HediffDef)o).addedPartProps != null));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffDef>("PartEfficiency","Part Efficiency",510,1.0f,
			                                                                      h=>h.addedPartProps != null ? h.addedPartProps.partEfficiency : -1f)
							.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasAddedPart")));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("Solid","Solid",520,default(bool),
			                                                                     h => h.addedPartProps != null && h.addedPartProps.solid)
							.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasAddedPart")));
			//not in use
			/*props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("IsGoodWeapon","Is Good Weapon",530,default(bool),
			                                                                     h => h.addedPartProps != null && h.addedPartProps.isGoodWeapon).setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasAddedPart")));*/
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<HediffDef>("BetterThanNatural","Better Than Natural",540,default(bool),
			                                                                     h => h.addedPartProps != null && h.addedPartProps.betterThanNatural)
							.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasAddedPart")));
			
			HediffStages();
			props.fields.Add(new SettingsFieldPropertiesListSettingsReferences("Stages","Stages",700,typeof(List<SettingsInstance>),null,
                 t=>
                 {
                     if (((HediffDef)t).stages == null) return new List<SettingsInstance>();
                     else
                     {
                         int i = 0;
                         return ((HediffDef)t).stages.Select(tc => new SettingsInstance("hediffStages", tc, null, tc.label) { BaseObjectReferenceKey = i++, BaseObjectReference = tc }).ToList();
                     }
                 },
                 "hediffStages",()=>new HediffStage() { label = "New Entry" },true));

            props.props.Add(new SettingsPropertyProperties("HasCompSeverityPerDay",o=>((HediffDef)o).CompProps<HediffCompProperties_SeverityPerDay>() != null));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<HediffDef>("SeverityPerDay","Severity per Day",550,default(float),
			                                                                      h=>{
			                                                                      	HediffCompProperties_SeverityPerDay p = h.CompProps<HediffCompProperties_SeverityPerDay>(); 
			                                                                      	return p != null ? p.severityPerDay : default(float);
			                                                                      })
							.setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompSeverityPerDay")));

            compReactOnDamage();
            props.props.Add(new SettingsPropertyProperties("HasCompReactOnDamage", o => ((HediffDef)o).CompProps<HediffCompProperties_ReactOnDamage>() != null));
            props.fields.Add(new SettingsFieldPropertiesSettings("CompReactOnDamage", "React On Dmg Properties", 560, typeof(SettingsInstance), null,
                h => ((HediffDef)h).CompProps<HediffCompProperties_ReactOnDamage>() != null ? 
                    new SettingsInstance("compReactOnDamage", ((HediffDef)h).CompProps<HediffCompProperties_ReactOnDamage>(), null, null) : null, "compReactOnDamage")
                .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompReactOnDamage")));

            props.mergers.Add(setting => {
              	HediffDef h = DefDatabase<HediffDef>.GetNamed(setting.getKey());

              	setting.resolveProps(props,h);
              	h.initialSeverity = setting.get<float>("InitialSeverity");
				h.lethalSeverity = setting.get<float>("LethalSeverity");
				h.tendable = setting.get<bool>("Tendable");
				h.isBad = setting.get<bool>("IsBad");
				h.spawnThingOnRemoved = setting.get<ThingDef>("SpawnThingOnRemoved");
				h.chanceToCauseNoPain = setting.get<float>("ChanceToCauseNoPain");
				h.makesSickThought = setting.get<bool>("MakesSickThought");
				h.makesAlert = setting.get<bool>("MakesAlert");
				h.causesNeed = setting.get<NeedDef>("CausesNeed");
				h.minSeverity = setting.get<float>("MinSeverity");
				h.maxSeverity = setting.get<float>("MaxSeverity");
				h.scenarioCanAdd = setting.get<bool>("ScenarioCanAdd");
				h.cureAllAtOnceIfCuredByItem = setting.get<bool>("CureAllAtOnceIfCuredByItem");
				h.taleOnVisible = setting.get<TaleDef>("TaleOnVisible");
				h.everCurableByItem = setting.get<bool>("EverCurableByItem");
				h.displayWound = setting.get<bool>("DisplayWound");
				h.defaultLabelColor = setting.get<UnityEngine.Color>("DefaultLabelColor");
				if ( h.injuryProps != null ) {
					h.injuryProps.painPerSeverity = setting.get<float>("PainPerSeverity");
					h.injuryProps.averagePainPerSeverityPermanent = setting.get<float>("AveragePainPerSeverityPermanent");
					h.injuryProps.bleedRate = setting.get<float>("BleedRate");
					h.injuryProps.canMerge = setting.get<bool>("CanMerge");
					//h.injuryProps.destroyedLabel = setting.get<string>("DestroyedLabel");
					//h.injuryProps.destroyedOutLabel = setting.get<string>("DestroyedOutLabel");
					h.injuryProps.useRemovedLabel = setting.get<bool>("UseRemovedLabel");
				}
				if ( h.addedPartProps != null ) {
					h.addedPartProps.partEfficiency = setting.get<float>("PartEfficiency");
					h.addedPartProps.solid = setting.get<bool>("Solid");
					h.addedPartProps.isGoodWeapon = setting.get<bool>("IsGoodWeapon");
					h.addedPartProps.betterThanNatural = setting.get<bool>("BetterThanNatural");
				}
				HediffCompProperties_SeverityPerDay compSeverityPerDay = h.CompProps<HediffCompProperties_SeverityPerDay>();
				if ( compSeverityPerDay != null ) {
					compSeverityPerDay.severityPerDay = setting.get<float>("SeverityPerDay");
				}

                HediffCompProperties_ReactOnDamage compReactOnDamage = h.CompProps<HediffCompProperties_ReactOnDamage>();
                if (compReactOnDamage != null)
                {
                    SettingsInstance si = setting.get<SettingsInstance>("CompReactOnDamage");
                    if (si != null)
                    {
                        compReactOnDamage.vomit = si.get<bool>("Vomit");
                        compReactOnDamage.damageDefIncoming = si.get<DamageDef>("DamageDefIncoming");
                        compReactOnDamage.createHediff = si.get<HediffDef>("CreateHediff");
                        compReactOnDamage.createHediffOn = si.get<BodyPartDef>("CreateHediffOn");
                    }
                }

                List<SettingsInstance> stagesConfigured = setting.get<List<SettingsInstance>>("Stages");
            	if ( stagesConfigured != null ) {
                    h.stages = SettingsStorage.getInstancesForMerging(setting, "Stages", h.stages);
                    for (int j = 0; j < h.stages.Count && j < stagesConfigured.Count; j++)
                    {
                        SettingsInstance stageSettings = stagesConfigured[j];
                        HediffStage newStage = h.stages[j];

                        newStage.statOffsetEffectMultiplier = stageSettings.get<StatDef>("StatOffsetEffectMultiplier");
            			newStage.statFactorEffectMultiplier = stageSettings.get<StatDef>("StatFactorEffectMultiplier");
            			newStage.capacityFactorEffectMultiplier = stageSettings.get<StatDef>("CapacityFactorEffectMultiplier");
            			newStage.totalBleedFactor = stageSettings.get<float>("TotalBleedFactor");
            			newStage.naturalHealingFactor = stageSettings.get<float>("NaturalHealingFactor");
            			newStage.foodPoisoningChanceFactor = stageSettings.get<float>("FoodPoisoningChanceFactor");
            			newStage.minSeverity = stageSettings.get<float>("MinSeverity");
						newStage.becomeVisible = stageSettings.get<bool>("BecomeVisible");
						newStage.lifeThreatening = stageSettings.get<bool>("LifeThreatening");
						newStage.tale = stageSettings.get<TaleDef>("Tale");
						newStage.vomitMtbDays = stageSettings.get<float>("VomitMtbDays");
						newStage.deathMtbDays = stageSettings.get<float>("DeathMtbDays");
						newStage.painFactor = stageSettings.get<float>("PainFactor");
						newStage.painOffset = stageSettings.get<float>("PainOffset");
						newStage.forgetMemoryThoughtMtbDays = stageSettings.get<float>("ForgetMemoryThoughtMtbDays");
						newStage.pctConditionalThoughtsNullified = stageSettings.get<float>("PctConditionalThoughtsNullified");
						newStage.opinionOfOthersFactor = stageSettings.get<float>("OpinionOfOthersFactor");
						newStage.hungerRateFactor = stageSettings.get<float>("HungerRateFactor");
						newStage.hungerRateFactorOffset = stageSettings.get<float>("HungerRateFactorOffset");
						newStage.restFallFactor = stageSettings.get<float>("RestFallFactor");
						newStage.restFallFactorOffset = stageSettings.get<float>("RestFallFactorOffset");
						newStage.socialFightChanceFactor = stageSettings.get<float>("SocialFightChanceFactor");
						newStage.mentalBreakMtbDays = stageSettings.get<float>("MentalBreakMtbDays");
						newStage.partEfficiencyOffset = stageSettings.get<float>("PartEfficiencyOffset");
						newStage.partIgnoreMissingHP = stageSettings.get<bool>("PartIgnoreMissingHP");
						newStage.destroyPart = stageSettings.get<bool>("DestroyPart");
                        newStage.label = stageSettings.get<string>("Label");
                        newStage.overrideLabel = stageSettings.get<string>("OverrideLabel");
                        newStage.mentalBreakExplanation = stageSettings.get<string>("MentalBreakExplanation");
                        newStage.overrideTooltip = stageSettings.get<string>("OverrideTooltip");
                        newStage.extraTooltip = stageSettings.get<string>("ExtraTooltip");

                        List<SettingsInstance> statOffsets = stageSettings.get<List<SettingsInstance>>("StatOffsets");
		            	if ( statOffsets != null ) {
		            		if ( newStage.statOffsets == null ) newStage.statOffsets = new List<StatModifier>();
		            		newStage.statOffsets.Clear();
		            		foreach ( ISettingsInstance s in statOffsets ) {
		            			newStage.statOffsets.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
		            		}
		            	}
						List<HediffDef> makeImmuneTo = stageSettings.get<List<HediffDef>>("MakeImmuneTo");
		            	if ( makeImmuneTo != null ) {
							//explicitly expects null if the list is empty
							if ( makeImmuneTo.NullOrEmpty() ) {
								if ( newStage.makeImmuneTo != null ) newStage.makeImmuneTo.Clear();
								newStage.makeImmuneTo = null;
							} else {
			            		if ( newStage.makeImmuneTo == null ) newStage.makeImmuneTo = new List<HediffDef>();
			            		newStage.makeImmuneTo.Clear();
			            		newStage.makeImmuneTo.AddRange(makeImmuneTo);
							}
		            	}
						List<SettingsInstance> capMods = stageSettings.get<List<SettingsInstance>>("CapMods");
		            	if ( capMods != null ) {
		            		if ( newStage.capMods == null ) newStage.capMods = new List<PawnCapacityModifier>();
		            		newStage.capMods.Clear();
		            		foreach ( ISettingsInstance s in capMods ) {
		            			newStage.capMods.Add(new PawnCapacityModifier() {capacity = s.get<PawnCapacityDef>("Capacity"), offset = s.get<float>("Offset"), 
		                     		setMax = s.get<float>("SetMax"), postFactor = s.get<float>("PostFactor")});
		            		}
		            	}
						List<SettingsInstance> mentalStateGivers = stageSettings.get<List<SettingsInstance>>("MentalStateGivers");
		            	if ( mentalStateGivers != null ) {
		            		if ( newStage.mentalStateGivers == null ) newStage.mentalStateGivers = new List<MentalStateGiver>();
		            		newStage.mentalStateGivers.Clear();
		            		foreach ( ISettingsInstance s in mentalStateGivers ) {
		            			newStage.mentalStateGivers.Add(new MentalStateGiver() {mentalState = s.get<MentalStateDef>("MentalState"), mtbDays = s.get<float>("MtbDays")});
		            		}
		            	}
						
						List<string> allowedMentalBreakIntensities = stageSettings.get<List<string>>("AllowedMentalBreakIntensities");
		            	if ( allowedMentalBreakIntensities != null ) {
							if ( allowedMentalBreakIntensities.Count == 0 ) {
								newStage.allowedMentalBreakIntensities = null;
							} else {
			            		if ( newStage.allowedMentalBreakIntensities == null ) newStage.allowedMentalBreakIntensities = new List<MentalBreakIntensity>();
			            		else newStage.allowedMentalBreakIntensities.Clear();
			            		foreach ( string s in allowedMentalBreakIntensities ) {
									newStage.allowedMentalBreakIntensities.Add((MentalBreakIntensity)Enum.Parse(typeof(MentalBreakIntensity), s));
			            		}
							}
		            	}
						List<SettingsInstance> statBases = stageSettings.get<List<SettingsInstance>>("StatFactors");
		            	if ( statBases != null ) {
		            		if ( newStage.statFactors == null ) newStage.statFactors = new List<StatModifier>();
		            		newStage.statFactors.Clear();
		            		foreach ( ISettingsInstance s in statBases ) {
		            			newStage.statFactors.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
		            		}
		            	}
						WorkTags workTags;
						try {
		        			List<string> lstWorkTags = stageSettings.get<List<string>>("DisabledWorkTags");
		        			if ( lstWorkTags.NullOrEmpty() ) {
		        				workTags = WorkTags.None;
		        			} else {
		            			workTags = (WorkTags)Enum.Parse(typeof(WorkTags), lstWorkTags[0]);
		            			for ( int i = 1; i < lstWorkTags.Count; i++ ) {
		            				workTags |= (WorkTags)Enum.Parse(typeof(WorkTags), lstWorkTags[i]);
		            			}
		        			}
		            	} catch ( ArgumentException ) {
		            		workTags = WorkTags.None;
		            	}
					    newStage.disabledWorkTags = workTags;
            		}
            	}
              	
              	//public NeedDef causesNeed; <-- AddOrRemoveNeedsAsAppropriate() must be called on all pawns
              	//private bool alwaysAllowMothballCached; <-- controls caching must be set to false
				//private Hediff concreteExampleInt; <-- cache must be set to null
				typeof(HediffDef).GetField("alwaysAllowMothballCached", BindingFlags.NonPublic|BindingFlags.Instance).SetValue(h,false);
				typeof(HediffDef).GetField("concreteExampleInt", BindingFlags.NonPublic|BindingFlags.Instance).SetValue(h,null);
              	});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<HediffDef>("Hediff","Select Hediff",
			                                                                        ()=>DefDatabase<HediffDef>.AllDefsListForReading.ToList(),
				def => def.defName, DefLabelProducerGeneric<HediffDef>.LabelProducerNotNull,key => DefDatabase<HediffDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			
			QOLMod.addApplySettingsListener(mod=>{
					if ( Find.World != null && Find.WorldPawns != null ) {
            			FieldInfo cachedPain = typeof(HediffSet).GetField("cachedPain", BindingFlags.NonPublic|BindingFlags.Instance);
            			Find.Maps.ForEach(m=> {
		                  	foreach ( Pawn p in m.mapPawns.AllPawns ) {
		                  		if ( p.needs != null ) p.needs.AddOrRemoveNeedsAsAppropriate();
		                  		if ( p.health != null && p.health.hediffSet != null )
		            				cachedPain.SetValue(p.health.hediffSet,-1);
		                  	}
		                });
		            	foreach ( Pawn p in Find.WorldPawns.AllPawnsAliveOrDead ) {
		            		if ( p.needs != null ) p.needs.AddOrRemoveNeedsAsAppropriate();
		            		if ( p.health != null && p.health.hediffSet != null )
		            			cachedPain.SetValue(p.health.hediffSet,-1);
		            	}
                	}
                });
		}
		
		private static void LifeStageAge() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("lifeStageAge");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("Def","def",10,typeof(LifeStageDef),null,rp=>((LifeStageAge)rp).def,
					f=>from object d in DefDatabase<LifeStageDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<LifeStageDef>("Def",(LifeStageDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<LifeStageAge>("MinAge","minAge",20,default(float),lsa=>lsa.minAge));
				
				props.dynamicLabel = settings => settings.get<LifeStageDef>("Def").label;
				props.visualization = new SettingsVisualizationEdit(f=>"EditEntry".Translate(((ISettingsInstance)f.flowScope["edit"]).getLabel()));
			}
		}

        private static void animalBiomeRecord()
        {
            SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("animalBiomeRecord");
            if (props.fields.NullOrEmpty())
            {
                props.fields.Add(new SettingsFieldPropertiesSelectable("Biome", "Biome", 10, typeof(BiomeDef), default(BiomeDef), wcr => ((AnimalBiomeRecord)wcr).biome,
                                                       f => from object t in DefDatabase<BiomeDef>.AllDefsListForReading select t,
                                                        f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<BiomeDef>("Biome", (BiomeDef)def); return true; },
                                                       SettingsFieldPropertiesSelectable.DefLabelProducer, false));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<AnimalBiomeRecord>("Commonality", "Commonality", 20, default(float), wcr => wcr.commonality));

                props.dynamicLabel = settings => settings.get<BiomeDef>("Biome").label + " " + settings.get<float>("Commonality");

                props.visualization = new SettingsVisualizationEdit(f => "Edit \"" + ((ISettingsInstance)f.flowScope["edit"]).getLabel() + "\"");
            }
        }

        private static void RaceProperties() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("raceProperties");
			if ( props.fields.NullOrEmpty() ) {
				/*
				--- unsure if useful ---
				public List<string> untrainableTags;
				public List<string> trainableTags;
				public SimpleCurve litterSizeCurve;
				public List<HediffGiverSetDef> hediffGiverSets;
				public SimpleCurve ageGenerationCurve;
				public List<AnimalBiomeRecord> wildBiomes; (all BiomeDefs set cachedAnimalCommonalities = null)
				public ShadowData specialShadowData;
					- public Vector3 volume = Vector3.one;
					- public Vector3 offset = Vector3.zero;
				*/
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RaceProperties>("HasGenders","hasGenders",1,true,rp=>rp.hasGenders));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RaceProperties>("NeedsRest","needsRest",2,true,rp=>rp.needsRest));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RaceProperties>("MakesFootprints","makesFootprints",3,default(bool),rp=>rp.makesFootprints));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("ManhunterOnDamageChance","manhunterOnDamageChance",10,default(float),rp=>rp.manhunterOnDamageChance));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("ManhunterOnTameFailChance","manhunterOnTameFailChance",20,default(float),rp=>rp.manhunterOnTameFailChance));
				//props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("NameOnTameChance","nameOnTameChance",21,default(float),rp=>rp.nameOnTameChance));
				//props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("NameOnNuzzleChance","nameOnNuzzleChance",30,default(float),rp=>rp.nameOnNuzzleChance));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("NuzzleMtbHours","nuzzleMtbHours",31,-1f,rp=>rp.nuzzleMtbHours));
				props.fields.Add(new SettingsFieldPropertiesSelectable("Intelligence","intelligence",32,typeof(string),Enum.GetName(typeof(Intelligence),(Intelligence)0),
					t=>Enum.GetName(typeof(Intelligence),((RaceProperties)t).intelligence),
					f=>from Intelligence d in Enum.GetValues(typeof(Intelligence)) orderby Enum.GetName(typeof(Intelligence),d) select (object)Enum.GetName(typeof(Intelligence),d),
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("Intelligence",(string)def); return true; },
					o=>(string)o,false));
				props.fields.Add(new SettingsFieldPropertiesSelectable("Trainability","trainability",40,typeof(TrainabilityDef),null,rp=>((RaceProperties)rp).trainability,
					f=>from object d in DefDatabase<TrainabilityDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<TrainabilityDef>("Trainability",(TrainabilityDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RaceProperties>("HerdAnimal","herdAnimal",50,default(bool),rp=>rp.herdAnimal));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RaceProperties>("HerdMigrationAllowed","herdMigrationAllowed",51,true,rp=>rp.herdMigrationAllowed));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RaceProperties>("Predator","predator",52,default(bool),rp=>rp.predator));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("MaxPreyBodySize","maxPreyBodySize",53,99999f,rp=>rp.maxPreyBodySize));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RaceProperties>("CanBePredatorPrey","canBePredatorPrey",54,true,rp=>rp.canBePredatorPrey));
				props.fields.Add(new SettingsFieldPropertiesSelectable("Body","body",60,typeof(BodyDef),null,rp=>((RaceProperties)rp).body,
					f=>from object d in DefDatabase<BodyDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<BodyDef>("Body",(BodyDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("BaseBodySize","baseBodySize",70,1f,rp=>rp.baseBodySize));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("BaseHealthScale","baseHealthScale",80,1f,rp=>rp.baseHealthScale));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("BaseHungerRate","baseHungerRate",90,1f,rp=>rp.baseHungerRate));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("RoamMtbDays", "Roam Chance", 95, 0f, rp => rp.roamMtbDays.HasValue?rp.roamMtbDays.Value:0f, 0f));

                props.fields.Add(new SettingsFieldPropertiesListSelectable("FoodType","foodType",100,typeof(List<string>),new List<string>(),
					t=> GetEnumFlagsNames<FoodTypeFlags>(((RaceProperties)t).foodType),
					f => Enum.GetNames(typeof(FoodTypeFlags)).Where(e=>e != "None").Cast<object>(),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
							flow.addPostRenderCallback(fl=>{setting.get<List<string>>("FoodType")[listIndex] = (string)listEntry; setting.markAsChanged();});
							return true;
						};},
					entry => (string)entry,
					()=>Enum.GetName(typeof(FoodTypeFlags),FoodTypeFlags.None),
					true));
				
				props.fields.Add(new SettingsFieldPropertiesSelectable("LeatherDef","leatherDef",110,typeof(ThingDef),null,rp=>((RaceProperties)rp).leatherDef,
					f=>from ThingDef d in DefDatabase<ThingDef>.AllDefsListForReading where !d.MadeFromStuff select (object)d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("LeatherDef",(ThingDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("GestationPeriodDays","gestationPeriodDays",120,10f,rp=>rp.gestationPeriodDays));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("Wildness","wildness",130,default(float),rp=>rp.wildness));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("Petness","petness",131,default(float),rp=>rp.petness));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<RaceProperties>("PackAnimal","packAnimal",140,default(bool),rp=>rp.packAnimal));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("LifeExpectancy","lifeExpectancy",150,10f,rp=>rp.lifeExpectancy));
				LifeStageAge();
				props.fields.Add(new SettingsFieldPropertiesListSettings("LifeStageAges","lifeStageAges",160,typeof(List<SettingsInstance>),null,
                        t=> ((RaceProperties)t).lifeStageAges.NullOrEmpty() ? new List<SettingsInstance>() : ((RaceProperties)t).lifeStageAges.Where(tc => tc != null)
                        	.Select(tc=>new SettingsInstance("lifeStageAge",tc,null,null)).ToList(),
                        "lifeStageAge",()=>new SettingsInstance("lifeStageAge",new Verse.LifeStageAge(){def=DefDatabase<LifeStageDef>.AllDefsListForReading.First(),minAge=0},
                        	                                        null,null),true));
				props.fields.Add(new SettingsFieldPropertiesSelectable("FleshType","fleshType",170,typeof(FleshTypeDef),null,rp=>((RaceProperties)rp).FleshType,
					f=>from object d in DefDatabase<FleshTypeDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<FleshTypeDef>("FleshType",(FleshTypeDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				props.fields.Add(new SettingsFieldPropertiesSelectable("BloodDef","bloodDef",180,typeof(ThingDef),null,rp=>((RaceProperties)rp).BloodDef,
                    f=>from ThingDef d in DefDatabase<ThingDef>.AllDefsListForReading where d.IsFilth select (object)d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("BloodDef",(ThingDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<RaceProperties>("ExecutionRange","executionRange",190,2,rp=>rp.executionRange));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("MateMtbHours","mateMtbHours",210,12f,rp=>rp.mateMtbHours));
				//Used on the implied meat only - which is edited in the food category
				/*props.fields.Add(new SettingsFieldPropertiesPrimitiveColor<RaceProperties>("MeatColor","meatColor",220,Color.white,rp=>rp.meatColor));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<RaceProperties>("MeatMarketValue","meatMarketValue",230,2f,rp=>rp.meatMarketValue));*/
				props.fields.Add(new SettingsFieldPropertiesSelectable("UseMeatFrom","useMeatFrom",240,typeof(ThingDef),null,rp=>((RaceProperties)rp).useMeatFrom,
                    f=>from ThingDef d in DefDatabase<ThingDef>.AllDefsListForReading where d.race != null && d.race.meatDef != null select (object)d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("UseMeatFrom",(ThingDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));
				props.fields.Add(new SettingsFieldPropertiesSelectable("UseLeatherFrom","useLeatherFrom",250,typeof(ThingDef),null,rp=>((RaceProperties)rp).useLeatherFrom,
                    f=>from ThingDef d in DefDatabase<ThingDef>.AllDefsListForReading where d.race != null && d.race.leatherDef != null select (object)d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("UseLeatherFrom",(ThingDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));

                animalBiomeRecord();
                props.fields.Add(new SettingsFieldPropertiesListSettings("WildBiomes", "Wild Biomes", 260, typeof(List<SettingsInstance>), null,
                         rp => ((RaceProperties)rp).wildBiomes == null ? new List<SettingsInstance>() : ((RaceProperties)rp).wildBiomes.Select(tc => new SettingsInstance("animalBiomeRecord", tc, null, null)).ToList(),
                        "animalBiomeRecord", () => new SettingsInstance("animalBiomeRecord", new AnimalBiomeRecord() { biome = DefDatabase<BiomeDef>.AllDefsListForReading.FirstOrDefault(t => t != null), commonality = default(float) }, null, null), true));

                props.dynamicLabel = settings => "RaceProperties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit RaceProperties");
			}
		}
		
		delegate void RecalculateLifeStageIndex(Pawn_AgeTracker p);
		
		private static void compEggLayer() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compEggLayer");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_EggLayer>("EggLayIntervalDays", "Lay Interval", 10, 1f, p => p.eggLayIntervalDays, 0.001f));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<CompProperties_EggLayer>("EggFertilizationCountMax", "Fertilization Count", 20, 1, p => p.eggFertilizationCountMax, 0));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_EggLayer>("EggLayFemaleOnly", "Female Only", 30, true, p => p.eggLayFemaleOnly));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CompProperties_EggLayer>("EggProgressUnfertilizedMax", "Progress Unfertilized Max", 40, 1f, p => p.eggProgressUnfertilizedMax, 0f, 1f));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<CompProperties_EggLayer>("EggCountRangeMin", "Count Min", 50, 1, p => p.eggCountRange.min, 0));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<CompProperties_EggLayer>("EggCountRangeMax", "Count Max", 60, 1, p => p.eggCountRange.max, 0));
				
				props.fields.Add(new SettingsFieldPropertiesSelectable("EggFertilizedDef","Fertilized",70,typeof(ThingDef),default(ThingDef),
                                                       cp=>((CompProperties_EggLayer)cp).eggFertilizedDef,
                                                       f=>DefDatabase<ThingDef>.AllDefsListForReading.Where(d=>d != null && !d.MadeFromStuff && !d.IsBlueprint && d.category != ThingCategory.Mote && d.category != ThingCategory.Building),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("EggFertilizedDef",(ThingDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesSelectable("EggUnfertilizedDef","Unfertilized",80,typeof(ThingDef),default(ThingDef),
                                                       cp=>((CompProperties_EggLayer)cp).eggUnfertilizedDef,
                                                       f=>DefDatabase<ThingDef>.AllDefsListForReading.Where(d=>d != null && !d.MadeFromStuff && !d.IsBlueprint && d.category != ThingCategory.Mote && d.category != ThingCategory.Building),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("EggUnfertilizedDef",(ThingDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Egg Layer Properties");
			}
		}
		
		private static void compShearable() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compShearable");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<CompProperties_Shearable>("ShearIntervalDays", "Shear Interval", 10, default(int), p => p.shearIntervalDays, 1));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<CompProperties_Shearable>("WoolAmount", "Amount", 20, 1, p => p.woolAmount, 0));
				props.fields.Add(new SettingsFieldPropertiesSelectable("WoolDef","Product",30,typeof(ThingDef),default(ThingDef),
                                                       cp=>((CompProperties_Shearable)cp).woolDef,
                                                       f=>DefDatabase<ThingDef>.AllDefsListForReading.Where(d=>d != null && !d.MadeFromStuff && !d.IsBlueprint && d.category != ThingCategory.Mote && d.category != ThingCategory.Building),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("WoolDef",(ThingDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Shearable Properties");
			}
		}
		
		private static void compMilkable() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("compMilkable");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<CompProperties_Milkable>("MilkIntervalDays", "Milk Interval", 10, default(int), p => p.milkIntervalDays, 1));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<CompProperties_Milkable>("MilkAmount", "Amount", 20, 1, p => p.milkAmount, 0));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<CompProperties_Milkable>("MilkFemaleOnly", "Female Only", 30, true, p => p.milkFemaleOnly));
				props.fields.Add(new SettingsFieldPropertiesSelectable("MilkDef","Product",40,typeof(ThingDef),default(ThingDef),
                                                       cp=>((CompProperties_Milkable)cp).milkDef,
                                                       f=>DefDatabase<ThingDef>.AllDefsListForReading.Where(d=>d != null && !d.MadeFromStuff && !d.IsBlueprint && d.category != ThingCategory.Mote && d.category != ThingCategory.Building),
			                                           f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("MilkDef",(ThingDef)def); return true; },
			                                          SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				
				props.dynamicLabel = si => "Properties";
				props.visualization = new SettingsVisualizationEdit(f=>"Edit Milkable Properties");
			}
		}
		
		private static void pawns() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("pawns");
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ThingDef>("Version","version",0,2,t=>2).setHidden(f=>true));
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("Selectable","Selectable",10,default(bool),t=>t.selectable));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ThingDef>("NeverMultiSelect","No Multiselect",20,default(bool),t=>t.neverMultiSelect));
			RaceProperties();
			props.fields.Add(new SettingsFieldPropertiesSettings("Race","Race Properties",30,typeof(SettingsInstance),null,t=>new SettingsInstance("raceProperties",((ThingDef)t).race,null,null),"raceProperties"));
			
			StatModifier();
			props.fields.Add(new SettingsFieldPropertiesListSettings("statBases","Race Stats",70,typeof(List<SettingsInstance>),null,
                        t=> ((ThingDef)t).statBases.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).statBases.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
			
			ThingDefCountClass();
			props.fields.Add(new SettingsFieldPropertiesListSettings("ButcherProducts","Butcher Products",75,typeof(List<SettingsInstance>),null,
                                     t=> ((ThingDef)t).butcherProducts == null ? new List<SettingsInstance>() : ((ThingDef)t).butcherProducts.Select(tc=>new SettingsInstance("thingDefCountClass",tc,null,null)).ToList(),
                                     "thingDefCountClass", ()=>new SettingsInstance("thingDefCountClass",new ThingDefCountClass(DefDatabase<ThingDef>.AllDefsListForReading.Find(d=>d.category == ThingCategory.Item),1),null,null),true));
			
			ToolSettings();
            props.fields.Add(new SettingsFieldPropertiesListSettingsReferences("Tools", "Tools", 100, typeof(List<SettingsInstance>), null,
                        t =>
                        {
                            if (((ThingDef)t).tools == null) return new List<SettingsInstance>();
                            else
                            {
                                int i = 0;
                                return ((ThingDef)t).tools.Select(tc => new SettingsInstance("toolSettings", tc, null, tc.label)
                                {
                                    BaseObjectReferenceKey = i++,
                                    BaseObjectReference = tc
                                }).ToList();
                            }
                        }, "toolSettings", () => {
                            Tool t = CombatExtended_ToolCE.createNewTool();
                            if (t == null)
                            {
                                t = new Tool() { label = "tool" };
                            }
                            else
                            {
                                t.label = "CE tool";
                            }
                            t.cooldownTime = 2;
                            t.power = 10;
                            t.capacities.Add(DefDatabase<ToolCapacityDef>.AllDefsListForReading.First<ToolCapacityDef>());
                            return t;
                        }, true));
            /*props.fields.Add(new SettingsFieldPropertiesListSettings("Tools","Tools",100,typeof(List<SettingsInstance>),null,
                        t=> ((ThingDef)t).tools.NullOrEmpty() ? new List<SettingsInstance>() : ((ThingDef)t).tools.Where(tc => tc != null).Select(tc=>new SettingsInstance("toolSettings",tc,null,tc.label)).ToList(),
						"toolSettings",null,false));*/

            props.fields.Add(new SettingsFieldPropertiesListPrimitive("TradeTags","Trade Tags",110,typeof(List<string>),new List<string>(),
                      t=> ((ThingDef)t).tradeTags != null ? ((ThingDef)t).tradeTags.ToList() : new List<string>(),
                      null,()=>"",true));

            props.fields.Add(new SettingsFieldPropertiesListSelectable("Recipes", "Recipes", 120, typeof(List<RecipeDef>), new List<RecipeDef>(),
                    h => ((ThingDef)h).recipes == null ? new List<RecipeDef>() : ((ThingDef)h).recipes.ToList(),
                    f => DefDatabase<RecipeDef>.AllDefsListForReading.Cast<object>(),
                    flow => {
                        ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
                        int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
                        return listEntry => {
                            flow.addPostRenderCallback(fl => { setting.get<List<RecipeDef>>("Recipes")[listIndex] = (RecipeDef)listEntry; setting.markAsChanged(); });
                            return true;
                        };
                    },
                    SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<RecipeDef>.AllDefsListForReading.First(),
                    true));

            //individual comps
            compEggLayer();
			props.props.Add(new SettingsPropertyProperties("HasCompEggLayer",o=>((ThingDef)o).GetCompProperties<CompProperties_EggLayer>() != null));
			props.fields.Add(new SettingsFieldPropertiesSettings("CompEggLayer","Egg Layer",200,typeof(SettingsInstance),null,
			                                                     t=> ((ThingDef)t).GetCompProperties<CompProperties_EggLayer>() != null ? new SettingsInstance("compEggLayer", ((ThingDef)t).GetCompProperties<CompProperties_EggLayer>(), null, null) : null,"compEggLayer")
			                                                    .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompEggLayer")));
			compShearable();
			props.props.Add(new SettingsPropertyProperties("HasCompShearable",o=>((ThingDef)o).GetCompProperties<CompProperties_Shearable>() != null));
			props.fields.Add(new SettingsFieldPropertiesSettings("CompShearable","Shearable",210,typeof(SettingsInstance),null,
			                                                     t=> ((ThingDef)t).GetCompProperties<CompProperties_Shearable>() != null ? new SettingsInstance("compShearable", ((ThingDef)t).GetCompProperties<CompProperties_Shearable>(), null, null) : null,"compShearable")
			                                                    .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompShearable")));
			compMilkable();
			props.props.Add(new SettingsPropertyProperties("HasCompMilkable",o=>((ThingDef)o).GetCompProperties<CompProperties_Milkable>() != null));
			props.fields.Add(new SettingsFieldPropertiesSettings("CompMilkable","Milkable",220,typeof(SettingsInstance),null,
			                                                     t=> ((ThingDef)t).GetCompProperties<CompProperties_Milkable>() != null ? new SettingsInstance("compMilkable", ((ThingDef)t).GetCompProperties<CompProperties_Milkable>(), null, null) : null,"compMilkable")
			                                                    .setHidden(f => !((ISettingsInstance)f.flowScope["edit"]).prop<bool>("HasCompMilkable")));

            props.mergers.Add(setting => {
            ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());

            setting.resolveProps(props, t);

            t.selectable = setting.get<bool>("Selectable");
            t.neverMultiSelect = setting.get<bool>("NeverMultiSelect");

            List<SettingsInstance> statBases = setting.get<List<SettingsInstance>>("statBases");
            if (statBases != null) {
                if (t.statBases == null) t.statBases = new List<StatModifier>();
                t.statBases.Clear();
                foreach (ISettingsInstance s in statBases) {
                    t.statBases.Add(new StatModifier() { stat = s.get<StatDef>("Stat"), value = s.get<float>("Value") });
                }
            }

            mergeTools(t, setting);

            List<SettingsInstance> butcherProducts = setting.get<List<SettingsInstance>>("ButcherProducts");
            if (!butcherProducts.NullOrEmpty()) {
                if (t.butcherProducts == null) t.butcherProducts = new List<ThingDefCountClass>();
                else t.butcherProducts.Clear();
                foreach (ISettingsInstance e in butcherProducts) {
                    if (e.get<ThingDef>("Thing") != null) {
                        t.butcherProducts.Add(new ThingDefCountClass(e.get<ThingDef>("Thing"), e.get<int>("Count")));
                    }
                }
            } else if (butcherProducts != null) {
                if (t.butcherProducts != null) t.butcherProducts.Clear();
                t.butcherProducts = null;
            }

            List<string> tradeTags = setting.get<List<string>>("TradeTags");
            if (tradeTags != null) {
                if (t.tradeTags == null) t.tradeTags = new List<string>();
                else t.tradeTags.Clear();
                t.tradeTags.AddRange(tradeTags);
            }

            List<RecipeDef> recipes = setting.get<List<RecipeDef>>("Recipes");
            if (recipes != null)
            {
                if (t.recipes == null) t.recipes = new List<RecipeDef>();
                modifyList(t.recipes, recipes, out List<RecipeDef> added, out List<RecipeDef> removed);
                foreach (RecipeDef rd in added)
                {
                    RecipeDefIsSurgeryCachedSet(rd, null);
                }
                foreach (RecipeDef rd in removed)
                {
                    RecipeDefIsSurgeryCachedSet(rd, null);
                }
                ThingDefAllRecipesCachedSet(t, null);
            }

            //comps
            CompProperties_EggLayer cpEggLayer = t.GetCompProperties<CompProperties_EggLayer>();
            if (cpEggLayer != null) {
                SettingsInstance si = setting.get<SettingsInstance>("CompEggLayer");
                if (si != null) {
                    cpEggLayer.eggCountRange.min = si.get<int>("EggCountRangeMin");
                    cpEggLayer.eggCountRange.max = si.get<int>("EggCountRangeMax");
                    cpEggLayer.eggFertilizationCountMax = si.get<int>("EggFertilizationCountMax");
                    cpEggLayer.eggLayFemaleOnly = si.get<bool>("EggLayFemaleOnly");
                    cpEggLayer.eggLayIntervalDays = si.get<float>("EggLayIntervalDays");
                    cpEggLayer.eggProgressUnfertilizedMax = si.get<float>("EggProgressUnfertilizedMax");
                    cpEggLayer.eggFertilizedDef = si.get<ThingDef>("EggFertilizedDef");
                    cpEggLayer.eggUnfertilizedDef = si.get<ThingDef>("EggUnfertilizedDef");
                }
            }
            CompProperties_Shearable cpShearable = t.GetCompProperties<CompProperties_Shearable>();
            if (cpShearable != null) {
                SettingsInstance si = setting.get<SettingsInstance>("CompShearable");
                if (si != null) {
                    cpShearable.shearIntervalDays = si.get<int>("ShearIntervalDays");
                    cpShearable.woolAmount = si.get<int>("WoolAmount");
                    cpShearable.woolDef = si.get<ThingDef>("WoolDef");
                }
            }
            CompProperties_Milkable cpMilkable = t.GetCompProperties<CompProperties_Milkable>();
            if (cpMilkable != null) {
                SettingsInstance si = setting.get<SettingsInstance>("CompMilkable");
                if (si != null) {
                    cpMilkable.milkIntervalDays = si.get<int>("MilkIntervalDays");
                    cpMilkable.milkAmount = si.get<int>("MilkAmount");
                    cpMilkable.milkFemaleOnly = si.get<bool>("MilkFemaleOnly");
                    cpMilkable.milkDef = si.get<ThingDef>("MilkDef");
                }
            }

            //race properties
            SettingsInstance rps = setting.get<SettingsInstance>("Race");
            if (t.race != null && rps != null) {
                RaceProperties rp = t.race;
                rp.manhunterOnDamageChance = rps.get<float>("ManhunterOnDamageChance");
                rp.manhunterOnTameFailChance = rps.get<float>("ManhunterOnTameFailChance");
                //rp.nameOnNuzzleChance = rps.get<float>("NameOnNuzzleChance");
                rp.trainability = rps.get<TrainabilityDef>("Trainability");
                rp.herdAnimal = rps.get<bool>("HerdAnimal");
                rp.body = rps.get<BodyDef>("Body");
                rp.baseBodySize = rps.get<float>("BaseBodySize");
                rp.baseHealthScale = rps.get<float>("BaseHealthScale");
                rp.baseHungerRate = rps.get<float>("BaseHungerRate");
                float roamMtbDays = rps.get<float>("RoamMtbDays");
                if (roamMtbDays > 0 ) rp.roamMtbDays = roamMtbDays;
                else rp.roamMtbDays = null;
                FoodTypeFlags foodType;
                try {
                    List<string> lstFoodTypes = rps.get<List<string>>("FoodType");
                    if (lstFoodTypes.NullOrEmpty()) {
                        foodType = FoodTypeFlags.None;
                    } else {
                        foodType = (FoodTypeFlags)Enum.Parse(typeof(FoodTypeFlags), lstFoodTypes[0]);
                        for (int i = 1; i < lstFoodTypes.Count; i++) {
                            foodType |= (FoodTypeFlags)Enum.Parse(typeof(FoodTypeFlags), lstFoodTypes[i]);
                        }
                    }
                } catch (ArgumentException) {
                    foodType = FoodTypeFlags.None;
                }
                rp.foodType = foodType;
                rp.leatherDef = rps.get<ThingDef>("LeatherDef");
                rp.gestationPeriodDays = rps.get<float>("GestationPeriodDays");
                rp.wildness = rps.get<float>("Wildness");
                rp.packAnimal = rps.get<bool>("PackAnimal");
                rp.lifeExpectancy = rps.get<float>("LifeExpectancy");

                rp.nuzzleMtbHours = rps.get<float>("NuzzleMtbHours");
                //rp.nameOnTameChance = rps.get<float>("NameOnTameChance");
                Intelligence intelligence;
                try {
                    intelligence = (Intelligence)Enum.Parse(typeof(Intelligence), rps.get<string>("Intelligence"));
                } catch (ArgumentException) {
                    intelligence = (Intelligence)0;
                }
                rp.intelligence = intelligence;
                rp.predator = rps.get<bool>("Predator");
                rp.maxPreyBodySize = rps.get<float>("MaxPreyBodySize");
                rp.canBePredatorPrey = rps.get<bool>("CanBePredatorPrey");
                rp.herdMigrationAllowed = rps.get<bool>("HerdMigrationAllowed");
                rp.hasGenders = rps.get<bool>("HasGenders");
                rp.needsRest = rps.get<bool>("NeedsRest");
                rp.makesFootprints = rps.get<bool>("MakesFootprints");
                rp.executionRange = rps.get<int>("ExecutionRange");
                rp.petness = rps.get<float>("Petness");
                rp.mateMtbHours = rps.get<float>("MateMtbHours");
                //only affects implied meat definitions - which  are edited under the foods category.
                /*rp.meatMarketValue = rps.get<float>("MeatMarketValue");
                rp.meatColor = rps.get<Color>("MeatColor");
                if ( rp.meatDef != null && rp.useMeatFrom == null ) {
                    rp.meatDef.graphicData.color = rp.meatColor;
                    rp.meatDef.BaseMarketValue = rp.meatMarketValue;
                }*/
                rp.GetType().GetField("fleshType", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public).SetValue(rp, rps.get<FleshTypeDef>("FleshType"));
                rp.GetType().GetField("bloodDef", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public).SetValue(rp, rps.get<ThingDef>("BloodDef"));
                rp.useMeatFrom = rps.get<ThingDef>("UseMeatFrom");
                rp.useLeatherFrom = rps.get<ThingDef>("UseLeatherFrom");

                List<SettingsInstance> lsa = rps.get<List<SettingsInstance>>("LifeStageAges");
                if (lsa != null) {
                    //If the current list contains an entry that matches on def and on age we want to use that since it might have sounds 
                    //defined that the custom one is lacking.
                    List<LifeStageAge> lst = new List<LifeStageAge>();
                    foreach (SettingsInstance si in lsa) {
                        LifeStageDef def = si.get<LifeStageDef>("Def");
                        float minAge = si.get<float>("MinAge");
                        LifeStageAge ori = rp.lifeStageAges.FirstOrDefault(original => original.def == def && original.minAge.Equals(minAge));
                        if (ori != null) {
                            lst.Add(ori);
                        } else {
                            lst.Add(new LifeStageAge() { def = def, minAge = minAge });
                        }
                    }
                    rp.lifeStageAges.Clear();
                    rp.lifeStageAges.AddRange(lst);
                }

                lsa = rps.get<List<SettingsInstance>>("WildBiomes");
                if (lsa != null)
                {
                    List<BiomeDef> involvedBiomes = new List<BiomeDef>();
                    List<AnimalBiomeRecord> l = rp.wildBiomes;
                    if (rp.wildBiomes == null)
                    {
                        rp.wildBiomes = new List<AnimalBiomeRecord>();
                    }
                    else
                    {
                        rp.wildBiomes.ForEach(r => { if (involvedBiomes.Contains(r.biome)) involvedBiomes.Add(r.biome); });
                        rp.wildBiomes.Clear();
                    }
                    foreach (SettingsInstance si in lsa)
                    {
                        l.Add(new AnimalBiomeRecord()
                        {
                            biome = si.get<BiomeDef>("Biome"),
                            commonality = si.get<float>("Commonality")
                        });
                    }
                    rp.wildBiomes.ForEach(r => { if (involvedBiomes.Contains(r.biome)) involvedBiomes.Add(r.biome); });
                    FieldInfo f = typeof(BiomeDef).GetField("cachedAnimalCommonalities", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public);
                    foreach (BiomeDef b in involvedBiomes)
                    {
                        f.SetValue(b, null);
                    }
                }

                rp.ResolveReferencesSpecial();
            }});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ThingDef>("Races","Select Race",
			                                                                        ()=>DefDatabase<ThingDef>.AllDefsListForReading.Where(t=>t.category == ThingCategory.Pawn).ToList(),
					def => def.defName, DefLabelProducerGeneric<ThingDef>.LabelProducerNotNull,key => DefDatabase<ThingDef>.GetNamed(key),
					f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			
			props.validators.Add(setting => {
                 	ThingDef t = DefDatabase<ThingDef>.GetNamed(setting.getKey());
                 	
                 	int version = setting.get<int>("Version");
					if ( version < 2 ) {
                 		SettingsInstance rps = setting.get<SettingsInstance>("Race");
                 		rps.set<List<string>>("FoodType",GetEnumFlagsNames<FoodTypeFlags>(t.race.foodType));
                 		Log.Warning("Reset food types on race properties of "+t.label);
					}
					
					setting.set<int>("Version",2);
                 	
                 	return t != null;
                 });
			
			QOLMod.addApplySettingsListener(mod=>{
				if ( Find.World != null && Find.WorldPawns != null ) {
            		MethodInfo mi = typeof(Pawn_AgeTracker).GetMethod("RecalculateLifeStageIndex",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
            		RecalculateLifeStageIndex rclsi = (RecalculateLifeStageIndex)Delegate.CreateDelegate(typeof(RecalculateLifeStageIndex), mi);
        			Find.Maps.ForEach(m=> {
	                  	foreach ( Pawn p in m.mapPawns.AllPawns ) {
	                  		if ( p.ageTracker != null ) rclsi(p.ageTracker);
	                  	}
	                });
	            	foreach ( Pawn p in Find.WorldPawns.AllPawnsAliveOrDead ) {
	            		if ( p.ageTracker != null ) rclsi(p.ageTracker);
	            	}
            	}
            });
			//Already part of hediff's but refreshing the needs is required for NeedsRest
			/*QOLMod.addApplySettingsListener(mod=>{
					if ( Find.World != null && Find.WorldPawns != null ) {
            			Find.Maps.ForEach(m=> {
		                  	foreach ( Pawn p in m.mapPawns.AllPawns ) {
		                  		if ( p.needs != null ) p.needs.AddOrRemoveNeedsAsAppropriate();
		                  	}
		                });
		            	foreach ( Pawn p in Find.WorldPawns.AllPawnsAliveOrDead ) {
		            		if ( p.needs != null ) p.needs.AddOrRemoveNeedsAsAppropriate();
		            	}
                	}
                });*/
		}
		
		private static void SkillGain() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("skillGain");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("Skill","Skill",10,typeof(SkillDef),default(SkillDef),t=>((KeyValuePair<SkillDef, int>)t).Key,
				                                             f=>from object d in DefDatabase<SkillDef>.AllDefsListForReading select d,
				                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<SkillDef>("Skill",(SkillDef)def); return true; },
				                                           SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<KeyValuePair<SkillDef, int>>("Offset","Offset",20,default(int),t=>t.Value));

				props.dynamicLabel = settings => settings.get<SkillDef>("Skill").label+" "+settings.get<int>("Offset");
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void CurvePoint() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("curvePoint");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CurvePoint>("X","X",10,default(float),t=>t.x));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<CurvePoint>("Y","Y",20,default(float),t=>t.y));

				props.dynamicLabel = settings => "("+settings.get<float>("X")+", "+settings.get<float>("Y")+")";
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void TraitDegreeData() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("traitDegreeData");
			if ( props.fields.NullOrEmpty() ) {
				/*
					public ThinkTreeDef thinkTree; //not used in vanilla but could trigger a job when the AI does ThinkNode_TraitBehaviours
				  */
			 	props.fields.Add(new SettingsFieldPropertiesPrimitiveString<TraitDegreeData>("Label","Label",5,"",t=>t.label));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TraitDegreeData>("Commonality","Commonality",10,1f,rp=>rp.commonality,0f));
				//multiplicative factor for humanlikes
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TraitDegreeData>("SocialFightChanceFactor","Social Fight Chance Factor",11,1f,rp=>rp.socialFightChanceFactor,0f));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TraitDegreeData>("MarketValueFactorOffset","Market Value Factor Offset",12,0f,rp=>rp.marketValueFactorOffset));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TraitDegreeData>("RandomDiseaseMtbDays","Random Disease Mtb Days",13,0f,rp=>rp.randomDiseaseMtbDays));
				props.fields.Add(new SettingsFieldPropertiesSelectable("RandomMentalState","Random Mental State",20,typeof(MentalStateDef),null,
			        rp=>((TraitDegreeData)rp).randomMentalState,
					f=>from object d in DefDatabase<MentalStateDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<MentalStateDef>("RandomMentalState",(MentalStateDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));

                props.fields.Add(new SettingsFieldPropertiesSelectable("ForcedMentalState", "Forced Mental State", 21, typeof(MentalStateDef), null,
                    rp => ((TraitDegreeData)rp).forcedMentalState,
                    f => from object d in DefDatabase<MentalStateDef>.AllDefsListForReading select d,
                    f => def => { ((ISettingsInstance)f.flowScope["edit"]).set<MentalStateDef>("ForcedMentalState", (MentalStateDef)def); return true; },
                    SettingsFieldPropertiesSelectable.DefLabelProducer, true));
                props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TraitDegreeData>("ForcedMentalStateMtbDays", "Forced MS Chance", 22, -1f, rp => rp.forcedMentalStateMtbDays, -1f));

                props.fields.Add(new SettingsFieldPropertiesListSettings("StatFactors","Stat Factors",100,typeof(List<SettingsInstance>),null,
                        t=> ((TraitDegreeData)t).statFactors.NullOrEmpty() ? new List<SettingsInstance>() : ((TraitDegreeData)t).statFactors.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
				props.fields.Add(new SettingsFieldPropertiesListSettings("StatOffsets","Stat Offsets",110,typeof(List<SettingsInstance>),null,
                        t=> ((TraitDegreeData)t).statOffsets.NullOrEmpty() ? new List<SettingsInstance>() : ((TraitDegreeData)t).statOffsets.Where(tc => tc != null).Select(tc=>new SettingsInstance("statModifier",tc,null,null)).ToList(),
                        "statModifier",()=>new SettingsInstance("statModifier",new StatModifier(){stat=DefDatabase<StatDef>.AllDefsListForReading.First(),value=0},null,null),true));
				
				props.fields.Add(new SettingsFieldPropertiesListSelectable("DisallowedMentalStates","Disallowed Mental States",120,typeof(List<MentalStateDef>),new List<MentalStateDef>(),
											h=>((TraitDegreeData)h).disallowedMentalStates == null ? new List<MentalStateDef>() : ((TraitDegreeData)h).disallowedMentalStates.ToList(),
											f=>DefDatabase<MentalStateDef>.AllDefsListForReading.Cast<object>(),
											flow=>{
												ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
												int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
												return listEntry=>{
												flow.addPostRenderCallback(fl=>{setting.get<List<MentalStateDef>>("DisallowedMentalStates")[listIndex] = (MentalStateDef)listEntry; setting.markAsChanged();});
												return true;
											};},
											SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<MentalStateDef>.AllDefsListForReading.First(),
											true));
				
				props.fields.Add(new SettingsFieldPropertiesListSelectable("DisallowedInspirations","Disallowed Inspirations",130,typeof(List<InspirationDef>),new List<InspirationDef>(),
											h=>((TraitDegreeData)h).disallowedInspirations == null ? new List<InspirationDef>() : ((TraitDegreeData)h).disallowedInspirations.ToList(),
											f=>DefDatabase<InspirationDef>.AllDefsListForReading.Cast<object>(),
											flow=>{
												ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
												int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
												return listEntry=>{
												flow.addPostRenderCallback(fl=>{setting.get<List<InspirationDef>>("DisallowedInspirations")[listIndex] = (InspirationDef)listEntry; setting.markAsChanged();});
												return true;
											};},
											SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<InspirationDef>.AllDefsListForReading.First(),
											true));
				
				props.fields.Add(new SettingsFieldPropertiesListSelectable("TheOnlyAllowedMentalBreaks","Only Allowed Mental Breaks",140,typeof(List<MentalBreakDef>),new List<MentalBreakDef>(),
											h=>((TraitDegreeData)h).theOnlyAllowedMentalBreaks == null ? new List<MentalBreakDef>() : ((TraitDegreeData)h).theOnlyAllowedMentalBreaks.ToList(),
											f=>DefDatabase<MentalBreakDef>.AllDefsListForReading.Cast<object>(),
											flow=>{
												ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
												int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
												return listEntry=>{
												flow.addPostRenderCallback(fl=>{setting.get<List<MentalBreakDef>>("TheOnlyAllowedMentalBreaks")[listIndex] = (MentalBreakDef)listEntry; setting.markAsChanged();});
												return true;
											};},
											SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<MentalBreakDef>.AllDefsListForReading.First(),
											true));
				
				props.fields.Add(new SettingsFieldPropertiesListSelectable("AllowedMeditationFocusTypes","Allowed Meditation Focus",142,typeof(List<MeditationFocusDef>),new List<MeditationFocusDef>(),
											h=>((TraitDegreeData)h).allowedMeditationFocusTypes == null ? new List<MeditationFocusDef>() : ((TraitDegreeData)h).allowedMeditationFocusTypes.ToList(),
											f=>DefDatabase<MeditationFocusDef>.AllDefsListForReading.Cast<object>(),
											flow=>{
												ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
												int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
												return listEntry=>{
												flow.addPostRenderCallback(fl=>{setting.get<List<MeditationFocusDef>>("AllowedMeditationFocusTypes")[listIndex] = (MeditationFocusDef)listEntry; setting.markAsChanged();});
												return true;
											};},
											SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<MeditationFocusDef>.AllDefsListForReading.First(),
											true));
				props.fields.Add(new SettingsFieldPropertiesListSelectable("DisallowedMeditationFocusTypes","Disallowed Meditation Focus",143,typeof(List<MeditationFocusDef>),new List<MeditationFocusDef>(),
											h=>((TraitDegreeData)h).disallowedMeditationFocusTypes == null ? new List<MeditationFocusDef>() : ((TraitDegreeData)h).disallowedMeditationFocusTypes.ToList(),
											f=>DefDatabase<MeditationFocusDef>.AllDefsListForReading.Cast<object>(),
											flow=>{
												ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
												int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
												return listEntry=>{
												flow.addPostRenderCallback(fl=>{setting.get<List<MeditationFocusDef>>("DisallowedMeditationFocusTypes")[listIndex] = (MeditationFocusDef)listEntry; setting.markAsChanged();});
												return true;
											};},
											SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<MeditationFocusDef>.AllDefsListForReading.First(),
											true));
				
				SkillGain();
				props.fields.Add(new SettingsFieldPropertiesListSettings("SkillGains","Skill Gains",150,typeof(List<SettingsInstance>),null,
                        t=> ((TraitDegreeData)t).skillGains == null ? new List<SettingsInstance>() : ((TraitDegreeData)t).skillGains.Select(tc=>new SettingsInstance("skillGain",tc,null,null)).ToList(),
                        "skillGain",()=>new SettingsInstance("skillGain",new KeyValuePair<SkillDef, int>(DefDatabase<SkillDef>.AllDefsListForReading.First(),0),null,null),true));
				
				CurvePoint();
				props.fields.Add(new SettingsFieldPropertiesListSettings("RandomMentalStateMtbDaysMoodCurve", "Random Mental State Mood Curve", 160, typeof(List<SettingsInstance>), null,
                        t => ((TraitDegreeData)t).randomMentalStateMtbDaysMoodCurve == null ? new List<SettingsInstance>() : ((TraitDegreeData)t).randomMentalStateMtbDaysMoodCurve.Points.Select(tc => new SettingsInstance("curvePoint", tc, null, null)).ToList(),
                        "curvePoint", () => new SettingsInstance("curvePoint", new CurvePoint(0f, 0f), null, null), true) { sortable = true });
				
				props.dynamicLabel = si => si.get<string>("Label");
				props.visualization = new SettingsVisualizationEdit(f=>"EditEntry".Translate(((ISettingsInstance)f.flowScope["edit"]).getLabel()));
			}
		}
		
		private static void traits() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("traits");
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TraitDef>("Commonality","Commonality",10,1f,rp=>
                     (float)UtilReflection.searchFieldInfo(rp.GetType(),"commonality",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public).GetValue(rp),0f));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TraitDef>("CommonalityFemale","Commonality-Female",20,-1f,rp=>
                     (float)UtilReflection.searchFieldInfo(rp.GetType(),"commonalityFemale",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public).GetValue(rp),-1f));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<TraitDef>("AllowOnHostileSpawn","Allow on Hostile Spawn",30,true,rp=>rp.allowOnHostileSpawn));
			
			props.fields.Add(new SettingsFieldPropertiesListSelectable("ConflictingTraits","Conflicting Traits",40,typeof(List<TraitDef>),new List<TraitDef>(),
											h=>((TraitDef)h).conflictingTraits == null ? new List<TraitDef>() : ((TraitDef)h).conflictingTraits.ToList(),
											f=>DefDatabase<TraitDef>.AllDefsListForReading.Cast<object>(),
											flow=>{
												ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
												int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
												return listEntry=>{
												flow.addPostRenderCallback(fl=>{setting.get<List<TraitDef>>("ConflictingTraits")[listIndex] = (TraitDef)listEntry; setting.markAsChanged();});
												return true;
											};},
											SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<TraitDef>.AllDefsListForReading.First(),
											true));
			props.fields.Add(new SettingsFieldPropertiesListSelectable("RequiredWorkTypes","Required Work-Types",50,typeof(List<WorkTypeDef>),new List<WorkTypeDef>(),
											h=>((TraitDef)h).requiredWorkTypes == null ? new List<WorkTypeDef>() : ((TraitDef)h).requiredWorkTypes.ToList(),
											f=>DefDatabase<WorkTypeDef>.AllDefsListForReading.Cast<object>(),
											flow=>{
												ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
												int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
												return listEntry=>{
												flow.addPostRenderCallback(fl=>{setting.get<List<WorkTypeDef>>("RequiredWorkTypes")[listIndex] = (WorkTypeDef)listEntry; setting.markAsChanged();});
												return true;
											};},
											SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<WorkTypeDef>.AllDefsListForReading.First(),
											true));
			props.fields.Add(new SettingsFieldPropertiesListSelectable("RequiredWorkTags","Required Work Tags",60,typeof(List<string>),new List<string>(),
					t=> GetEnumFlagsNames<WorkTags>(((TraitDef)t).requiredWorkTags),
					f => Enum.GetNames(typeof(WorkTags)).Where(e=>e != "None").Cast<object>(),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
							flow.addPostRenderCallback(fl=>{setting.get<List<string>>("RequiredWorkTags")[listIndex] = (string)listEntry; setting.markAsChanged();});
							return true;
						};},
					entry => (string)entry,
					()=>Enum.GetName(typeof(WorkTags),WorkTags.None),
					true));
			//If added would require existing pawns to be cleansed of illegal weapons after loading a save not just after changing these settings.
			props.fields.Add(new SettingsFieldPropertiesListSelectable("DisabledWorkTypes","Disabled Work Types",70,typeof(List<WorkTypeDef>),new List<WorkTypeDef>(),
											h=>((TraitDef)h).disabledWorkTypes == null ? new List<WorkTypeDef>() : ((TraitDef)h).disabledWorkTypes.ToList(),
											f=>DefDatabase<WorkTypeDef>.AllDefsListForReading.Cast<object>(),
											flow=>{
												ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
												int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
												return listEntry=>{
												flow.addPostRenderCallback(fl=>{setting.get<List<WorkTypeDef>>("DisabledWorkTypes")[listIndex] = (WorkTypeDef)listEntry; setting.markAsChanged();});
												return true;
											};},
											SettingsFieldPropertiesSelectable.DefLabelProducer,()=>DefDatabase<WorkTypeDef>.AllDefsListForReading.First(),
											true));
			props.fields.Add(new SettingsFieldPropertiesListSelectable("DisabledWorkTags","Disabled Work Tags",80,typeof(List<string>),new List<string>(),
					t=> GetEnumFlagsNames<WorkTags>(((TraitDef)t).disabledWorkTags),
					f => Enum.GetNames(typeof(WorkTags)).Where(e=>e != "None").Cast<object>(),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
							flow.addPostRenderCallback(fl=>{setting.get<List<string>>("DisabledWorkTags")[listIndex] = (string)listEntry; setting.markAsChanged();});
							return true;
						};},
					entry => (string)entry,
					()=>Enum.GetName(typeof(WorkTags),WorkTags.None),
					true));
			
			TraitDegreeData();
			props.fields.Add(new SettingsFieldPropertiesListSettings("DegreeDatas","Degrees",100,typeof(List<SettingsInstance>),null,
                        t=> ((TraitDef)t).degreeDatas.NullOrEmpty() ? new List<SettingsInstance>() : ((TraitDef)t).degreeDatas.Where(tc => tc != null).Select(tc=>new SettingsInstance("traitDegreeData",tc,null,tc.label)).ToList(),
						"traitDegreeData",null,false));
			
			props.mergers.Add(setting => {
			    TraitDef t = DefDatabase<TraitDef>.GetNamed(setting.getKey());
			    
			    UtilReflection.searchFieldInfo(t.GetType(),"commonality",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public)
			    	.SetValue(t,setting.get<float>("Commonality"));
			    UtilReflection.searchFieldInfo(t.GetType(),"commonalityFemale",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public)
			    	.SetValue(t,setting.get<float>("CommonalityFemale"));
			    t.allowOnHostileSpawn = setting.get<bool>("AllowOnHostileSpawn");
			    List<TraitDef> conflictingTraits = setting.get<List<TraitDef>>("ConflictingTraits");
			    if ( conflictingTraits != null ) {
			    	if ( t.conflictingTraits == null ) t.conflictingTraits = new List<TraitDef>();
			    	else t.conflictingTraits.Clear();
			    	t.conflictingTraits.AddRange(conflictingTraits);
			    }
			    List<WorkTypeDef> requiredWorkTypes = setting.get<List<WorkTypeDef>>("RequiredWorkTypes");
			    if ( requiredWorkTypes != null ) {
			    	if ( t.requiredWorkTypes == null ) t.requiredWorkTypes = new List<WorkTypeDef>();
			    	else t.requiredWorkTypes.Clear();
			    	t.requiredWorkTypes.AddRange(requiredWorkTypes);
			    }
			    WorkTags workTags;
        		try {
        			List<string> lstWorkTags = setting.get<List<string>>("RequiredWorkTags");
        			if ( lstWorkTags.NullOrEmpty() ) {
        				workTags = WorkTags.None;
        			} else {
            			workTags = (WorkTags)Enum.Parse(typeof(WorkTags), lstWorkTags[0]);
            			for ( int i = 1; i < lstWorkTags.Count; i++ ) {
            				workTags |= (WorkTags)Enum.Parse(typeof(WorkTags), lstWorkTags[i]);
            			}
        			}
            	} catch ( ArgumentException ) {
            		workTags = WorkTags.None;
            	}
			    t.requiredWorkTags = workTags;
			    List<WorkTypeDef> disabledWorkTypes = setting.get<List<WorkTypeDef>>("DisabledWorkTypes");
			    if ( disabledWorkTypes != null ) {
			    	if ( t.disabledWorkTypes == null ) t.disabledWorkTypes = new List<WorkTypeDef>();
			    	else t.disabledWorkTypes.Clear();
			    	t.disabledWorkTypes.AddRange(disabledWorkTypes);
			    }
        		try {
        			List<string> lstWorkTags = setting.get<List<string>>("DisabledWorkTags");
        			if ( lstWorkTags.NullOrEmpty() ) {
        				workTags = WorkTags.None;
        			} else {
            			workTags = (WorkTags)Enum.Parse(typeof(WorkTags), lstWorkTags[0]);
            			for ( int i = 1; i < lstWorkTags.Count; i++ ) {
            				workTags |= (WorkTags)Enum.Parse(typeof(WorkTags), lstWorkTags[i]);
            			}
        			}
            	} catch ( ArgumentException ) {
            		workTags = WorkTags.None;
            	}
			    t.disabledWorkTags = workTags;
			    
			    List<SettingsInstance> degreeDatas = setting.get<List<SettingsInstance>>("DegreeDatas");
            	if ( t.degreeDatas != null && degreeDatas != null ) {
            		for ( int i = 0; i < t.degreeDatas.Count && i < degreeDatas.Count; i++ ) {
			    		TraitDegreeData tdd = t.degreeDatas[i];
			    		SettingsInstance stdd = degreeDatas[i];
			    		
			    		List<SettingsInstance> stats = stdd.get<List<SettingsInstance>>("StatFactors");
		            	if ( stats != null ) {
		            		if ( tdd.statFactors == null ) tdd.statFactors = new List<StatModifier>();
		            		tdd.statFactors.Clear();
		            		foreach ( ISettingsInstance s in stats ) {
		            			tdd.statFactors.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
		            		}
		            	}
		              	stats = stdd.get<List<SettingsInstance>>("StatOffsets");
		            	if ( stats != null ) {
		            		if ( tdd.statOffsets == null ) tdd.statOffsets = new List<StatModifier>();
		            		tdd.statOffsets.Clear();
		            		foreach ( ISettingsInstance s in stats ) {
		            			tdd.statOffsets.Add(new StatModifier() {stat = s.get<StatDef>("Stat"), value = s.get<float>("Value")});
		            		}
		            	}
		              	
		              	string configuredLabel = stdd.get<string>("Label"); 
						if ( !configuredLabel.NullOrEmpty() && configuredLabel.Trim().Length != 0 ) {
							tdd.label = configuredLabel;
                            TraitDegreeDataCachedLabelCapSet(tdd, null);
                            TraitDegreeDataCachedLabelMaleCapSet(tdd, null);
                            TraitDegreeDataCachedLabelFemaleCapSet(tdd, null);
                        }
		              	tdd.commonality = stdd.get<float>("Commonality");
		              	tdd.socialFightChanceFactor = stdd.get<float>("SocialFightChanceFactor");
		              	tdd.marketValueFactorOffset = stdd.get<float>("MarketValueFactorOffset");
		              	tdd.randomDiseaseMtbDays = stdd.get<float>("RandomDiseaseMtbDays");
		              	tdd.randomMentalState = stdd.get<MentalStateDef>("RandomMentalState");
                        tdd.forcedMentalState = stdd.get<MentalStateDef>("ForcedMentalState");
                        tdd.forcedMentalStateMtbDays = stdd.get<float>("ForcedMentalStateMtbDays");

                        List<MentalStateDef> disallowedMentalStates = stdd.get<List<MentalStateDef>>("DisallowedMentalStates");
		              	if ( disallowedMentalStates != null ) {
		              		if ( tdd.disallowedMentalStates == null ) tdd.disallowedMentalStates = new List<MentalStateDef>();
		              		else tdd.disallowedMentalStates.Clear();
		              		tdd.disallowedMentalStates.AddRange(disallowedMentalStates);
		              	}
		              	List<InspirationDef> disallowedInspirations = stdd.get<List<InspirationDef>>("DisallowedInspirations");
		              	if ( disallowedInspirations != null ) {
		              		if ( tdd.disallowedInspirations == null ) tdd.disallowedInspirations = new List<InspirationDef>();
		              		else tdd.disallowedInspirations.Clear();
		              		tdd.disallowedInspirations.AddRange(disallowedInspirations);
		              	}
		              	List<MentalBreakDef> theOnlyAllowedMentalBreaks = stdd.get<List<MentalBreakDef>>("TheOnlyAllowedMentalBreaks");
		              	if ( theOnlyAllowedMentalBreaks != null ) {
		              		if ( tdd.theOnlyAllowedMentalBreaks == null ) tdd.theOnlyAllowedMentalBreaks = new List<MentalBreakDef>();
		              		else tdd.theOnlyAllowedMentalBreaks.Clear();
		              		tdd.theOnlyAllowedMentalBreaks.AddRange(theOnlyAllowedMentalBreaks);
		              	}
		              	List<MeditationFocusDef> allowedMeditationFocusTypes = stdd.get<List<MeditationFocusDef>>("AllowedMeditationFocusTypes");
		              	if ( allowedMeditationFocusTypes != null ) {
		              		if ( tdd.allowedMeditationFocusTypes == null ) tdd.allowedMeditationFocusTypes = new List<MeditationFocusDef>();
		              		else tdd.allowedMeditationFocusTypes.Clear();
		              		tdd.allowedMeditationFocusTypes.AddRange(allowedMeditationFocusTypes);
		              	}
		              	List<MeditationFocusDef> disallowedMeditationFocusTypes = stdd.get<List<MeditationFocusDef>>("DisallowedMeditationFocusTypes");
		              	if ( disallowedMeditationFocusTypes != null ) {
		              		if ( tdd.disallowedMeditationFocusTypes == null ) tdd.disallowedMeditationFocusTypes = new List<MeditationFocusDef>();
		              		else tdd.disallowedMeditationFocusTypes.Clear();
		              		tdd.disallowedMeditationFocusTypes.AddRange(disallowedMeditationFocusTypes);
		              	}
		              	
		              	List<SettingsInstance> skillGains = stdd.get<List<SettingsInstance>>("SkillGains");
		              	if ( skillGains != null ) {
		              		if ( tdd.skillGains == null ) tdd.skillGains = new Dictionary<SkillDef, int>();
		              		tdd.skillGains.Clear();
		              		foreach ( SettingsInstance si in skillGains ) {
		              			SkillDef skill = si.get<SkillDef>("Skill");
		              			if ( !tdd.skillGains.ContainsKey(skill) ) tdd.skillGains.Add(skill, si.get<int>("Offset"));
		              			else {
		              				Log.Warning("Duplicate skill gain for skill "+SettingsFieldPropertiesSelectable.DefLabelProducer(skill)+" on trait "+SettingsFieldPropertiesSelectable.DefLabelProducer(tdd)+".");
		              			}
		              		}
		              	}
		              	List<SettingsInstance> randomMentalStateMtbDaysMoodCurve = stdd.get<List<SettingsInstance>>("RandomMentalStateMtbDaysMoodCurve");
		              	if ( randomMentalStateMtbDaysMoodCurve != null ) {
		              		if ( tdd.randomMentalStateMtbDaysMoodCurve == null ) tdd.randomMentalStateMtbDaysMoodCurve = new SimpleCurve();
		              		else tdd.randomMentalStateMtbDaysMoodCurve.Points.Clear();
		              		tdd.randomMentalStateMtbDaysMoodCurve.Points.AddRange(randomMentalStateMtbDaysMoodCurve.Select(si=>new CurvePoint(si.get<float>("X"),si.get<float>("Y"))));
		              		tdd.randomMentalStateMtbDaysMoodCurve.SortPoints();
		              	}
		              	if ( tdd.randomMentalStateMtbDaysMoodCurve == null && tdd.randomMentalState != null ) {
		              		tdd.randomMentalStateMtbDaysMoodCurve = new SimpleCurve();
		              	}
		              	if ( tdd.randomMentalStateMtbDaysMoodCurve != null && tdd.randomMentalStateMtbDaysMoodCurve.PointsCount == 0 ) {
		              		tdd.randomMentalStateMtbDaysMoodCurve.Add(0,0);
		              	}
            		}
            	}
			    
				SettingsInit.updatePawns |= true;
			});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<TraitDef>("Traits","Select Trait (group)",()=>DefDatabase<TraitDef>.AllDefsListForReading.Where(td=>!td.degreeDatas.NullOrEmpty()).ToList<TraitDef>(),
				def => def.defName, DefLabelProducerGeneric<TraitDef>.LabelProducerNotNull,key => DefDatabase<TraitDef>.GetNamed(key),
				f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}
		
		private static void researchProjects() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("researchProjects");
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveString<Def>("Label","Label",5,null,t=>t.label));
			//may not be 0
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ResearchProjectDef>("BaseCost","Base Cost",10,100f,t=>t.baseCost,1f));
			props.fields.Add(new SettingsFieldPropertiesSelectable("Tab","tab",30,typeof(ResearchTabDef),null,
			        rp=>((ResearchProjectDef)rp).tab,
					f=>from object d in DefDatabase<ResearchTabDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ResearchTabDef>("Tab",(ResearchTabDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,false));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ResearchProjectDef>("ResearchViewX", "Position X", 31, 1f, t => t.researchViewX, 0));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<ResearchProjectDef>("ResearchViewY", "Position Y", 32, 1f, t => t.researchViewY, 0, 6.5f));
            props.fields.Add(new SettingsFieldPropertiesSelectable("TechLevel","Tech-Level",40,typeof(string),Enum.GetName(typeof(TechLevel),(TechLevel)0),
					t=>Enum.GetName(typeof(TechLevel),((ResearchProjectDef)t).techLevel),
					f=>from TechLevel d in Enum.GetValues(typeof(TechLevel)) orderby Enum.GetName(typeof(TechLevel),d) select (object)Enum.GetName(typeof(TechLevel),d),
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<string>("TechLevel",(string)def); return true; },
					o=>(string)o,false));
			props.fields.Add(new SettingsFieldPropertiesSelectable("RequiredResearchBuilding","Required Research Building",50,typeof(ThingDef),null,
                    rp=>((ResearchProjectDef)rp).requiredResearchBuilding,
                    f=>from ThingDef t in DefDatabase<ThingDef>.AllDefsListForReading where typeof(Building_ResearchBench).IsAssignableFrom(t.thingClass) select (object)t,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("RequiredResearchBuilding",(ThingDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesListSelectable("RequiredResearchFacilities","Required Research Facilities",60,typeof(List<ThingDef>),new List<ThingDef>(),
					h=>((ResearchProjectDef)h).requiredResearchFacilities == null ? new List<ThingDef>() : ((ResearchProjectDef)h).requiredResearchFacilities.ToList(),
					f=>(from ThingDef t in DefDatabase<ThingDef>.AllDefsListForReading let sdf = t != null ? t.GetCompProperties<CompProperties_AffectedByFacilities>() : null where typeof(Building_ResearchBench).IsAssignableFrom(t.thingClass) &&
					    sdf != null && !sdf.linkableFacilities.NullOrEmpty() 
					    select sdf.linkableFacilities).SelectMany(list=>list).Distinct().Cast<object>(),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
						flow.addPostRenderCallback(fl=>{setting.get<List<ThingDef>>("RequiredResearchFacilities")[listIndex] = (ThingDef)listEntry; setting.markAsChanged();});
						return true;
					};},
					SettingsFieldPropertiesSelectable.DefLabelProducer, () => (from ThingDef t in DefDatabase<ThingDef>.AllDefsListForReading let sdf = t != null ? t.GetCompProperties<CompProperties_AffectedByFacilities>() : null where typeof(Building_ResearchBench).IsAssignableFrom(t.thingClass) &&
                    sdf != null && !sdf.linkableFacilities.NullOrEmpty() 
                    select sdf.linkableFacilities).SelectMany(list=>list).FirstOrDefault(),
					true));
			props.fields.Add(new SettingsFieldPropertiesListSelectable("Prerequisites","Prerequisites",60,typeof(List<ResearchProjectDef>),new List<ResearchProjectDef>(),
					h=>((ResearchProjectDef)h).prerequisites == null ? new List<ResearchProjectDef>() : ((ResearchProjectDef)h).prerequisites.ToList(),
					f=>from object t in DefDatabase<ResearchProjectDef>.AllDefsListForReading select t,
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
						flow.addPostRenderCallback(fl=>{setting.get<List<ResearchProjectDef>>("Prerequisites")[listIndex] = (ResearchProjectDef)listEntry; setting.markAsChanged();});
						return true;
					};},
					SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<ResearchProjectDef>.AllDefsListForReading.First(),
					true));
			//On starting a new world, checks the starting research tags of the player faction and finishes all research with a matching tag.
			props.fields.Add(new SettingsFieldPropertiesListSelectable("Tags","Tags",70,typeof(List<ResearchProjectTagDef>),new List<ResearchProjectTagDef>(),
					h=>((ResearchProjectDef)h).tags == null ? new List<ResearchProjectTagDef>() : ((ResearchProjectDef)h).tags.ToList(),
					f=>from object t in DefDatabase<ResearchProjectTagDef>.AllDefsListForReading select t,
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
						flow.addPostRenderCallback(fl=>{setting.get<List<ResearchProjectTagDef>>("Tags")[listIndex] = (ResearchProjectTagDef)listEntry; setting.markAsChanged();});
						return true;
					};},
					SettingsFieldPropertiesSelectable.DefLabelProducer, () => DefDatabase<ResearchProjectTagDef>.AllDefsListForReading.First(),
					true));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ResearchProjectDef>("TechprintCount","Techprints Req.",80,0, t => t.techprintCount, 0));
			
			props.mergers.Add(setting => {
			    ResearchProjectDef t = DefDatabase<ResearchProjectDef>.GetNamed(setting.getKey());
			    
			    string configuredLabel = setting.get<string>("Label");
				if ( !configuredLabel.NullOrEmpty() && configuredLabel.Trim().Length != 0 ) {
					t.label = configuredLabel;
                    DefCachedLabelCapSet(t, null);
                }
			    
                if (Math.Abs(t.researchViewX - setting.get<float>("ResearchViewX")) < float.Epsilon || Math.Abs(t.researchViewY - setting.get<float>("ResearchViewY")) < float.Epsilon)
                {
                    SettingsInit.updateResearchLayout = true;
                }
                t.researchViewX = setting.get<float>("ResearchViewX");
                t.researchViewY = setting.get<float>("ResearchViewY");
                t.baseCost = setting.get<float>("BaseCost");
			    t.tab = setting.get<ResearchTabDef>("Tab");
			    t.techprintCount = setting.get<int>("TechprintCount");
			    if ( t.techprintCount > 0 ) {
			    	ThingDef techPrint = DefDatabase<ThingDef>.AllDefs.FirstOrDefault(delegate(ThingDef x) {
							CompProperties_Techprint compProperties = x.GetCompProperties<CompProperties_Techprint>();
							return compProperties != null && compProperties.project == t;
			    		});
			    	if ( techPrint == null ) {
			    		t.techprintCount = 0;
			    	}
			    }
			    TechLevel tl;
        		try {
	            	tl = (TechLevel)Enum.Parse(typeof(TechLevel),setting.get<string>("TechLevel"));
            	} catch ( ArgumentException ) {
					tl = (TechLevel)0;
            	}
			    t.techLevel = tl;
			    t.requiredResearchBuilding = setting.get<ThingDef>("RequiredResearchBuilding");
			    List<ThingDef> rrf = setting.get<List<ThingDef>>("RequiredResearchFacilities");
            	if ( rrf != null ) {
            		if ( t.requiredResearchFacilities == null ) t.requiredResearchFacilities = new List<ThingDef>();
            		else t.requiredResearchFacilities.Clear();
            		t.requiredResearchFacilities.AddRange(rrf);
            	}
			    List<ResearchProjectDef> prerequisites = setting.get<List<ResearchProjectDef>>("Prerequisites");
            	if ( prerequisites != null ) {
            		if ( t.prerequisites == null ) t.prerequisites = new List<ResearchProjectDef>();
            		else t.prerequisites.Clear();
            		t.prerequisites.AddRange(prerequisites);
            	}
			    List<ResearchProjectTagDef> tags = setting.get<List<ResearchProjectTagDef>>("Tags");
            	if ( tags != null ) {
            		if ( t.tags == null ) t.tags = new List<ResearchProjectTagDef>();
            		else t.tags.Clear();
            		t.tags.AddRange(tags);
            	}
			    
			    //hook for research mods that wan't to react to changes
			    t.ClearCachedData();
			});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ResearchProjectDef>("Research-Projects","Select Research-Project",()=>DefDatabase<ResearchProjectDef>.AllDefsListForReading.ToList<ResearchProjectDef>(),
				def => def.defName, DefLabelProducerGeneric<ResearchProjectDef>.LabelProducerNotNull,key => DefDatabase<ResearchProjectDef>.GetNamed(key),
				f=>"SelectEntry".Translate(((ISettingsInstance)f.flowScope["edit"]).getLabel()));
		}
		
		private static void researchTabs() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("researchTabs");
			props.fields.Add(new SettingsFieldPropertiesPrimitiveString<ResearchTabDef>("Label","Label",10,"",t=>t.label));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<ResearchTabDef>("Order","Order",20,0,t=> {
                 	QOLDefModExtension_ResearchTabDef modExtension = t.GetModExtension<QOLDefModExtension_ResearchTabDef>();
                 	if ( modExtension != null ) return modExtension.order;
                 	return 0;
                 }));
            props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<ResearchTabDef>("AlwaysHidden", "Hidden", 30, false, t => {
                    QOLDefModExtension_ResearchTabDef modExtension = t.GetModExtension<QOLDefModExtension_ResearchTabDef>();
                    if (modExtension != null) return modExtension.alwaysHidden;
                    return false;
                }));

            props.mergers.Add(setting => {
              	ResearchTabDef rt = DefDatabase<ResearchTabDef>.GetNamed(setting.getKey());

              	setting.resolveProps(props,rt);
				string configuredLabel = setting.get<string>("Label"); 
				if ( !configuredLabel.NullOrEmpty() && configuredLabel.Trim().Length != 0 ) {
					rt.label = configuredLabel;
                    DefCachedLabelCapSet(rt, null);
                }
				
				QOLDefModExtension_ResearchTabDef modExtension = rt.GetModExtension<QOLDefModExtension_ResearchTabDef>();
      			if ( modExtension == null ) {
      				if ( rt.modExtensions == null ) rt.modExtensions = new List<DefModExtension>();
      				modExtension = new QOLDefModExtension_ResearchTabDef();
      				rt.modExtensions.Add(modExtension);
      			}
  				modExtension.order = setting.get<int>("Order");
                modExtension.alwaysHidden = setting.get<bool>("AlwaysHidden");

                rt.ClearCachedData();
				});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<ResearchTabDef>("Research Tabs","Select Research Tab",()=>DefDatabase<ResearchTabDef>.AllDefsListForReading,
               def => def.defName, DefLabelProducerGeneric<ResearchTabDef>.LabelProducerNotNull,key => DefDatabase<ResearchTabDef>.GetNamed(key),
               f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}
		
		private static string traitDegreeLabel(string str) {
			if ( str == null ) return null;
			string[] p = str.Split(new []{'_'},2);
			TraitDef td = DefDatabase<TraitDef>.GetNamed(p[0],false);
			if ( td != null ) {
				int degree;
				if ( int.TryParse(p[1], out degree) ) {
					return td.DataAtDegree(degree).label;
				}
			}
			return null;
		}
		
		private static string traitDegreeSerialized(TraitDef def, TraitDegreeData tdd) {
			return new StringBuilder().Append(def.defName).Append("_").Append(tdd.degree).ToString();
		}
		
		private static string traitDegreeSerialized(TraitDef def, int degree) {
			return new StringBuilder().Append(def.defName).Append("_").Append(degree).ToString();
		}
		
		private static string traitDegreeSerialized(TraitDef def) {
			return new StringBuilder().Append(def.defName).Append("_").Append(def.degreeDatas[0].degree).ToString();
		}
		
		private static void backstories() {
			/*
			//public WorkTags workDisables;
			 */
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("backstories");
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<Backstory>("Shuffleable","Shuffleable",10,true,t=>t.shuffleable));
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("BodyTypeGlobalResolved","Body Type Global",20,typeof(BodyTypeDef),null,
			        rp=>UtilReflection.searchFieldInfo(rp.GetType(),"bodyTypeGlobalResolved",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public).GetValue(rp),
					f=>from object d in DefDatabase<BodyTypeDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<BodyTypeDef>("BodyTypeGlobalResolved",(BodyTypeDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,false));
			props.fields.Add(new SettingsFieldPropertiesSelectable("BodyTypeFemaleResolved","Body Type Female",30,typeof(BodyTypeDef),null,
			        rp=>UtilReflection.searchFieldInfo(rp.GetType(),"bodyTypeFemaleResolved",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public).GetValue(rp),
					f=>from object d in DefDatabase<BodyTypeDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<BodyTypeDef>("BodyTypeFemaleResolved",(BodyTypeDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			props.fields.Add(new SettingsFieldPropertiesSelectable("BodyTypeMaleResolved","Body Type Male",40,typeof(BodyTypeDef),null,
			        rp=>UtilReflection.searchFieldInfo(rp.GetType(),"bodyTypeMaleResolved",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public).GetValue(rp),
					f=>from object d in DefDatabase<BodyTypeDef>.AllDefsListForReading select d,
					f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<BodyTypeDef>("BodyTypeMaleResolved",(BodyTypeDef)def); return true;},
					SettingsFieldPropertiesSelectable.DefLabelProducer,true));
			
			props.fields.Add(new SettingsFieldPropertiesListPrimitive("SpawnCategories","Spawn Categories",50,typeof(List<string>),new List<string>(),
                  	t=> ((Backstory)t).spawnCategories != null ? ((Backstory)t).spawnCategories.ToList() : new List<string>(),
                          			null,()=>(object)"",true));
			
			props.fields.Add(new SettingsFieldPropertiesListSelectable("RequiredWorkTags","Required Work Tags",60,typeof(List<string>),new List<string>(),
					t=> GetEnumFlagsNames<WorkTags>(((Backstory)t).requiredWorkTags),
					f => Enum.GetNames(typeof(WorkTags)).Where(e=>e != "None").Cast<object>(),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
							flow.addPostRenderCallback(fl=>{setting.get<List<string>>("RequiredWorkTags")[listIndex] = (string)listEntry; setting.markAsChanged();});
							return true;
						};},
					entry => (string)entry,
					()=>Enum.GetName(typeof(WorkTags),WorkTags.None),
					true));
			
			props.fields.Add(new SettingsFieldPropertiesListSelectable("ForcedTraits","Forced Traits",70,typeof(List<string>),new List<string>(),
	                h=>((Backstory)h).forcedTraits == null ? new List<string>() : ((Backstory)h).forcedTraits.Select(t=>traitDegreeSerialized(t.def,t.degree)).ToList(),
	                f=>DefDatabase<TraitDef>.AllDefsListForReading.SelectMany(def=>def.degreeDatas.Select(degree=>(object)(traitDegreeSerialized(def,degree.degree)))),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
						flow.addPostRenderCallback(fl=>{setting.get<List<string>>("ForcedTraits")[listIndex] = (string)listEntry; setting.markAsChanged();});
						return true;
					};},
	                str=>traitDegreeLabel((string)str), () => traitDegreeSerialized(DefDatabase<TraitDef>.AllDefsListForReading.First()),
					true));
			
			props.fields.Add(new SettingsFieldPropertiesListSelectable("DisallowedTraits","Disallowed Traits",80,typeof(List<string>),new List<string>(),
	                h=>((Backstory)h).disallowedTraits == null ? new List<string>() : ((Backstory)h).disallowedTraits.Select(t=>traitDegreeSerialized(t.def,t.degree)).ToList(),
	                f=>DefDatabase<TraitDef>.AllDefsListForReading.SelectMany(def=>def.degreeDatas.Select(degree=>(object)(traitDegreeSerialized(def,degree.degree)))),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
						flow.addPostRenderCallback(fl=>{setting.get<List<string>>("DisallowedTraits")[listIndex] = (string)listEntry; setting.markAsChanged();});
						return true;
					};},
	                str=>traitDegreeLabel((string)str), () => traitDegreeSerialized(DefDatabase<TraitDef>.AllDefsListForReading.First()),
					true));
			
			SkillGain();
			props.fields.Add(new SettingsFieldPropertiesListSettings("SkillGainsResolved","Skill Gains",90,typeof(List<SettingsInstance>),null,
                    t=> ((Backstory)t).skillGainsResolved == null ? new List<SettingsInstance>() : ((Backstory)t).skillGainsResolved.Select(tc=>new SettingsInstance("skillGain",tc,null,null)).ToList(),
                    "skillGain",()=>new SettingsInstance("skillGain",new KeyValuePair<SkillDef, int>(DefDatabase<SkillDef>.AllDefsListForReading.First(),0),null,null),true));
			
			props.fields.Add(new SettingsFieldPropertiesListSelectable("WorkDisables","Disabled Work Tags",100,typeof(List<string>),new List<string>(),
					t=> GetEnumFlagsNames<WorkTags>(((Backstory)t).workDisables),
					f => Enum.GetNames(typeof(WorkTags)).Where(e=>e != "None").Cast<object>(),
					flow=>{
						ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
						int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
						return listEntry=>{
							flow.addPostRenderCallback(fl=>{setting.get<List<string>>("WorkDisables")[listIndex] = (string)listEntry; setting.markAsChanged();});
							return true;
						};},
					entry => (string)entry,
					()=>Enum.GetName(typeof(WorkTags),WorkTags.None),
					true));
			
			props.mergers.Add(setting => {
          		Backstory bs; 
          		if ( BackstoryDatabase.allBackstories.TryGetValue(setting.getKey(), out bs) ) {
          			setting.resolveProps(props,bs);
          			
          			WorkTags workTags;
	        		try {
	        			List<string> lstWorkTags = setting.get<List<string>>("RequiredWorkTags");
	        			if ( lstWorkTags.NullOrEmpty() ) {
	        				workTags = WorkTags.None;
	        			} else {
	            			workTags = (WorkTags)Enum.Parse(typeof(WorkTags), lstWorkTags[0]);
	            			for ( int i = 1; i < lstWorkTags.Count; i++ ) {
	            				workTags |= (WorkTags)Enum.Parse(typeof(WorkTags), lstWorkTags[i]);
	            			}
	        			}
	            	} catch ( ArgumentException ) {
	            		workTags = WorkTags.None;
	            	}
				    bs.requiredWorkTags = workTags;
				    
				    bs.shuffleable = setting.get<bool>("shuffleable");
				    UtilReflection.searchFieldInfo(bs.GetType(),"bodyTypeGlobalResolved",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public)
				    	.SetValue(bs,setting.get<BodyTypeDef>("BodyTypeGlobalResolved"));
				    UtilReflection.searchFieldInfo(bs.GetType(),"bodyTypeFemaleResolved",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public)
				    	.SetValue(bs,setting.get<BodyTypeDef>("BodyTypeFemaleResolved"));
				    UtilReflection.searchFieldInfo(bs.GetType(),"bodyTypeMaleResolved",BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public)
				    	.SetValue(bs,setting.get<BodyTypeDef>("BodyTypeMaleResolved"));
				    List<string> spawnCategories = setting.get<List<string>>("SpawnCategories");
				    if ( spawnCategories != null ) {
				    	if ( bs.spawnCategories == null ) bs.spawnCategories = new List<string>();
				    	else bs.spawnCategories.Clear();
				    	bs.spawnCategories.AddRange(spawnCategories);
				    }
				    List<string> forcedTraits = setting.get<List<string>>("ForcedTraits");
				    if ( forcedTraits != null ) {
				    	if ( bs.forcedTraits == null ) bs.forcedTraits = new List<TraitEntry>();
				    	else bs.forcedTraits.Clear();
				    	
				    	foreach ( string str in forcedTraits ) {
			    			if ( str == null ) continue;
				    		string[] p = str.Split(new []{'_'},2);
				    		if ( p.Length != 2 ) continue;
							TraitDef td = DefDatabase<TraitDef>.GetNamed(p[0],false);
							if ( td != null ) {
								int degree;
								if ( int.TryParse(p[1], out degree) ) {
									bs.forcedTraits.Add(new TraitEntry(td,degree));
								}
							}
				    	}
				    }
				    List<string> disallowedTraits = setting.get<List<string>>("DisallowedTraits");
				    if ( disallowedTraits != null ) {
				    	if ( bs.disallowedTraits == null ) bs.disallowedTraits = new List<TraitEntry>();
				    	else bs.disallowedTraits.Clear();
				    	
				    	foreach ( string str in disallowedTraits ) {
			    			if ( str == null ) continue;
				    		string[] p = str.Split(new []{'_'},2);
				    		if ( p.Length != 2 ) continue;
							TraitDef td = DefDatabase<TraitDef>.GetNamed(p[0],false);
							if ( td != null ) {
								int degree;
								if ( int.TryParse(p[1], out degree) ) {
									bs.disallowedTraits.Add(new TraitEntry(td,degree));
								}
							}
				    	}
				    }
				    List<SettingsInstance> skillGainsResolved = setting.get<List<SettingsInstance>>("SkillGainsResolved");
	              	if ( skillGainsResolved != null ) {
	              		if ( bs.skillGainsResolved == null ) bs.skillGainsResolved = new Dictionary<SkillDef, int>();
	              		else bs.skillGainsResolved.Clear();
	              		foreach ( SettingsInstance si in skillGainsResolved ) {
	              			SkillDef skill = si.get<SkillDef>("Skill");
	              			if ( !bs.skillGainsResolved.ContainsKey(skill) ) bs.skillGainsResolved.Add(skill, si.get<int>("Offset"));
	              			else {
	              				Log.Warning("Duplicate skill gain for skill "+SettingsFieldPropertiesSelectable.DefLabelProducer(skill)+" on backstory "+bs.title+".");
	              			}
	              		}
	              	}
				    try {
	        			List<string> lstWorkTags = setting.get<List<string>>("WorkDisables");
	        			if ( lstWorkTags.NullOrEmpty() ) {
	        				workTags = WorkTags.None;
	        			} else {
	            			workTags = (WorkTags)Enum.Parse(typeof(WorkTags), lstWorkTags[0]);
	            			for ( int i = 1; i < lstWorkTags.Count; i++ ) {
	            				workTags |= (WorkTags)Enum.Parse(typeof(WorkTags), lstWorkTags[i]);
	            			}
	        			}
	            	} catch ( ArgumentException ) {
	            		workTags = WorkTags.None;
	            	}
				    bs.workDisables = workTags;
				    
				    SettingsInit.updatePawns |= true;
              	}
				});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<Backstory>("Backstories","Select Backstory",()=>BackstoryDatabase.allBackstories.Values.ToList(),
             	bs => bs.identifier, bs=>bs.title, key => { Backstory bs; return BackstoryDatabase.allBackstories.TryGetValue(key, out bs) ? bs : null;  },
               f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}
		
		private static void weatherCommonalityRecord() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("weatherCommonalityRecord");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("weather","Weather",10,typeof(WeatherDef),default(WeatherDef),wcr=>((WeatherCommonalityRecord)wcr).weather,
			                                             f=>from object d in DefDatabase<WeatherDef>.AllDefsListForReading select d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<WeatherDef>("weather",(WeatherDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<WeatherCommonalityRecord>("commonality","Commonality",20,default(float),wcr=>wcr.commonality));

				props.dynamicLabel = settings => settings.get<WeatherDef>("weather").label+" "+settings.get<float>("commonality");
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void terrainThreshold() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("terrainThreshold");
			if ( props.fields.NullOrEmpty() ) {
				
				props.fields.Add(new SettingsFieldPropertiesSelectable("terrain","Terrain",10,typeof(TerrainDef),default(TerrainDef),wcr=>((TerrainThreshold)wcr).terrain,
				                                                       f=>from TerrainDef d in DefDatabase<TerrainDef>.AllDefsListForReading where !d.layerable && !d.MadeFromStuff select (object)d,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<TerrainDef>("terrain",(TerrainDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainThreshold>("min","Min",20,-1000f,wcr=>wcr.min));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainThreshold>("max","Max",30,1000f,wcr=>wcr.max));

				props.dynamicLabel = settings => settings.get<TerrainDef>("terrain").label+" "+settings.get<float>("min")+" -> "+settings.get<float>("max");
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void biomePlantRecord() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("biomePlantRecord");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("plant","Plant",10,typeof(ThingDef),default(ThingDef),wcr=>((BiomePlantRecord)wcr).plant,
                                                       f=>from ThingDef t in DefDatabase<ThingDef>.AllDefsListForReading where t != null && t.plant != null select (object)t,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("plant",(ThingDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<BiomePlantRecord>("commonality","Commonality",20,default(float),wcr=>wcr.commonality));

				props.dynamicLabel = settings => settings.get<ThingDef>("plant").label+" "+settings.get<float>("commonality");
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void biomeAnimalRecord() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("biomeAnimalRecord");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("animal","Animal",10,typeof(PawnKindDef),default(PawnKindDef),wcr=>((BiomeAnimalRecord)wcr).animal,
                                                       f=>from object t in DefDatabase<PawnKindDef>.AllDefsListForReading select t,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<PawnKindDef>("animal",(PawnKindDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<BiomeAnimalRecord>("commonality","Commonality",20,default(float),wcr=>wcr.commonality));

				props.dynamicLabel = settings => settings.get<PawnKindDef>("animal").label+" "+settings.get<float>("commonality");
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void biomeDiseaseRecord() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("biomeDiseaseRecord");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesSelectable("diseaseInc","Disease",10,typeof(IncidentDef),default(IncidentDef),wcr=>((BiomeDiseaseRecord)wcr).diseaseInc,
                                                       f=>from object t in DefDatabase<IncidentDef>.AllDefsListForReading select t,
			                                            f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<IncidentDef>("diseaseInc",(IncidentDef)def); return true; },
			                                           SettingsFieldPropertiesSelectable.DefLabelProducer,false));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<BiomeDiseaseRecord>("commonality","Commonality",20,default(float),wcr=>wcr.commonality));

				props.dynamicLabel = settings => settings.get<IncidentDef>("diseaseInc").label+" "+settings.get<float>("commonality");
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void terrainPatchMaker() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("terrainPatchMaker");
			if ( props.fields.NullOrEmpty() ) {
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainPatchMaker>("minFertility","Fertility Min",10,-999f,tpm=>tpm.minFertility));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainPatchMaker>("maxFertility","Fertility Max",20,999f,tpm=>tpm.maxFertility));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<TerrainPatchMaker>("minSize","Min Size",30,default(int),tpm=>tpm.minSize));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainPatchMaker>("perlinFrequency","Perlin Frequency",40,0.01f,tpm=>tpm.perlinFrequency));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainPatchMaker>("perlinLacunarity","Perlin Lacunarity",50,2f,tpm=>tpm.perlinLacunarity));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<TerrainPatchMaker>("perlinPersistence","Perlin Persistence",60,0.5f,tpm=>tpm.perlinPersistence));
				props.fields.Add(new SettingsFieldPropertiesPrimitiveInt<TerrainPatchMaker>("perlinOctaves","Perlin Octaves",70,6,tpm=>tpm.perlinOctaves));
				
				terrainThreshold();
				props.fields.Add(new SettingsFieldPropertiesListSettings("thresholds","Thresholds",200,typeof(List<SettingsInstance>),null,
                    t=> ((TerrainPatchMaker)t).thresholds == null ? new List<SettingsInstance>() : ((TerrainPatchMaker)t).thresholds.Select(tc=>new SettingsInstance("terrainThreshold",tc,null,null)).ToList(),
                    "terrainThreshold",()=>new SettingsInstance("terrainThreshold",new TerrainThreshold() {terrain=DefDatabase<TerrainDef>.AllDefsListForReading.FirstOrDefault(t=>t != null), min=-1000f, max=1000f},null,null),true));

				props.dynamicLabel = settings => settings.get<List<SettingsInstance>>("thresholds").NullOrEmpty() ? "No Terrain" : settings.get<List<SettingsInstance>>("thresholds").Join<SettingsInstance>(si=>SettingsFieldPropertiesSelectable.DefLabelProducer(si.get<TerrainDef>("terrain")),",")+" "+settings.get<int>("minSize");
				
				props.visualization = new SettingsVisualizationEdit(f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
			}
		}
		
		private static void biomes() {
			SettingsProperties props = SettingsStorage.getOrInitSettingsProperties("biomes");
			FieldInfo fAllowedPackAnimals = typeof(BiomeDef).GetField("allowedPackAnimals", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
			FieldInfo fWildPlants = typeof(BiomeDef).GetField("wildPlants", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
			FieldInfo fWildAnimals = typeof(BiomeDef).GetField("wildAnimals", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
			FieldInfo fDiseases = typeof(BiomeDef).GetField("diseases", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<BiomeDef>("implemented","Implemented",10,true,b=>b.implemented));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<BiomeDef>("canBuildBase","Allow Base",20,true,b=>b.canBuildBase));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<BiomeDef>("canAutoChoose","Allow Auto Choose",30,true,b=>b.canAutoChoose));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<BiomeDef>("allowRoads","Allow Roads",40,true,b=>b.allowRoads));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<BiomeDef>("allowRivers","Allow Rivers",50,true,b=>b.allowRivers));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<BiomeDef>("impassable","Impassable",60,default(bool),b=>b.impassable));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<BiomeDef>("hasVirtualPlants","Virtual Plants",70,true,b=>b.hasVirtualPlants));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<BiomeDef>("wildPlantsCareAboutLocalFertility","Wild Plants use Fertility",80,true,b=>b.wildPlantsCareAboutLocalFertility));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<BiomeDef>("hasBedrock","Bedrock",90,true,b=>b.hasBedrock));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveBool<BiomeDef>("isExtremeBiome","Extreme",100,true,b=>b.isExtremeBiome));
			
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<BiomeDef>("animalDensity","Animal Density",110,default(float),b=>b.animalDensity));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<BiomeDef>("plantDensity","Plant Density",120,default(float),b=>b.plantDensity));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<BiomeDef>("diseaseMtbDays","Disease Frequency",130,60f,b=>b.diseaseMtbDays));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<BiomeDef>("settlementSelectionWeight","Settlement Selection Weight",140,1f,b=>b.settlementSelectionWeight));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<BiomeDef>("forageability","Forageability",150,default(float),b=>b.forageability));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<BiomeDef>("wildPlantRegrowDays","Wild Plant Regrowth",160,25f,b=>b.wildPlantRegrowDays));
			props.fields.Add(new SettingsFieldPropertiesPrimitiveFloat<BiomeDef>("movementDifficulty","Movement Difficulty",170,1f,b=>b.movementDifficulty));
			
			props.fields.Add(new SettingsFieldPropertiesSelectable("foragedFood","Foraged Food",200,typeof(ThingDef),default(ThingDef),
                               b=>((BiomeDef)b).foragedFood,
                               f=>DefDatabase<ThingDef>.AllDefsListForReading.Where(d=>d.IsIngestible),
                               f=>def=>{ ((ISettingsInstance)f.flowScope["edit"]).set<ThingDef>("foragedFood",(ThingDef)def); return true; },
                              SettingsFieldPropertiesSelectable.DefLabelProducer,false));
			props.fields.Add(new SettingsFieldPropertiesListSelectable("allowedPackAnimals","Pack Animals",210,typeof(List<ThingDef>),new List<ThingDef>(),
				b=>fAllowedPackAnimals.GetValue(b) == null ? null : ((List<ThingDef>)fAllowedPackAnimals.GetValue(b)).ToList(),
				f=>DefDatabase<ThingDef>.AllDefsListForReading.Where(t=>t != null && t.race != null && t.race.packAnimal).Cast<object>(),
				flow=>{
					ISettingsInstance setting = (ISettingsInstance)flow.flowScope["edit"];
					int listIndex = ((IterationItem)flow.pageScope["curItem"]).index;
					return listEntry=>{
					flow.addPostRenderCallback(fl=>{setting.get<List<ThingDef>>("allowedPackAnimals")[listIndex] = (ThingDef)listEntry; setting.markAsChanged();});
					return true;
					};},
				SettingsFieldPropertiesSelectable.DefLabelProducer,
				()=>DefDatabase<ThingDef>.AllDefsListForReading.FirstOrDefault(t=>t != null && t.race != null && t.race.packAnimal),
				true));
			weatherCommonalityRecord();
			props.fields.Add(new SettingsFieldPropertiesListSettings("baseWeatherCommonalities","Weather Commonalities",220,typeof(List<SettingsInstance>),null,
                    t=> ((BiomeDef)t).baseWeatherCommonalities == null ? new List<SettingsInstance>() : ((BiomeDef)t).baseWeatherCommonalities.Select(tc=>new SettingsInstance("weatherCommonalityRecord",tc,null,null)).ToList(),
                    "weatherCommonalityRecord",()=>new SettingsInstance("weatherCommonalityRecord",new WeatherCommonalityRecord() {weather=DefDatabase<WeatherDef>.AllDefsListForReading.FirstOrDefault(t=>t != null), commonality=default(float)},null,null),true));
			terrainThreshold();
			props.fields.Add(new SettingsFieldPropertiesListSettings("terrainsByFertility","Terrain Fertility",230,typeof(List<SettingsInstance>),null,
                    t=> ((BiomeDef)t).terrainsByFertility == null ? new List<SettingsInstance>() : ((BiomeDef)t).terrainsByFertility.Select(tc=>new SettingsInstance("terrainThreshold",tc,null,null)).ToList(),
                    "terrainThreshold",()=>new SettingsInstance("terrainThreshold",new TerrainThreshold() {terrain=DefDatabase<TerrainDef>.AllDefsListForReading.FirstOrDefault(t=>t != null), min=-1000f, max=1000f},null,null),true));
			biomePlantRecord();
			props.fields.Add(new SettingsFieldPropertiesListSettings("wildPlants","Wild Plants",240,typeof(List<SettingsInstance>),null,
                     t=> fWildPlants.GetValue(t) == null ? new List<SettingsInstance>() : ((List<BiomePlantRecord>)fWildPlants.GetValue(t)).Select(tc=>new SettingsInstance("biomePlantRecord",tc,null,null)).ToList(),
                    "biomePlantRecord",()=>new SettingsInstance("biomePlantRecord",new BiomePlantRecord() {plant=DefDatabase<ThingDef>.AllDefsListForReading.FirstOrDefault(t=>t.plant != null), commonality=default(float)},null,null),true));
			biomeAnimalRecord();
			props.fields.Add(new SettingsFieldPropertiesListSettings("wildAnimals","Wild Animals",250,typeof(List<SettingsInstance>),null,
                     t=> fWildAnimals.GetValue(t) == null ? new List<SettingsInstance>() : ((List<BiomeAnimalRecord>)fWildAnimals.GetValue(t)).Select(tc=>new SettingsInstance("biomeAnimalRecord",tc,null,null)).ToList(),
                    "biomeAnimalRecord",()=>new SettingsInstance("biomeAnimalRecord",new BiomeAnimalRecord() {animal=DefDatabase<PawnKindDef>.AllDefsListForReading.FirstOrDefault(t=>t != null), commonality=default(float)},null,null),true));
			biomeDiseaseRecord();
			props.fields.Add(new SettingsFieldPropertiesListSettings("diseases","Diseases",260,typeof(List<SettingsInstance>),null,
                     t=> fDiseases.GetValue(t) == null ? new List<SettingsInstance>() : ((List<BiomeDiseaseRecord>)fDiseases.GetValue(t)).Select(tc=>new SettingsInstance("biomeDiseaseRecord",tc,null,null)).ToList(),
                    "biomeDiseaseRecord",()=>new SettingsInstance("biomeDiseaseRecord",new BiomeDiseaseRecord() {diseaseInc=DefDatabase<IncidentDef>.AllDefsListForReading.FirstOrDefault(t=>t != null), commonality=999999f},null,null),true));
			terrainPatchMaker();
			props.fields.Add(new SettingsFieldPropertiesListSettings("terrainPatchMakers","Terrain Patches",270,typeof(List<SettingsInstance>),null,
			        t=> ((BiomeDef)t).terrainPatchMakers == null ? new List<SettingsInstance>() : ((BiomeDef)t).terrainPatchMakers.Select(tc=>new SettingsInstance("terrainPatchMaker",tc,null,null)).ToList(),
                    "terrainPatchMaker",()=>new SettingsInstance("terrainPatchMaker",new TerrainPatchMaker(),null,null),true));
			
			props.mergers.Add(setting => {
          		BiomeDef bd = DefDatabase<BiomeDef>.GetNamed(setting.getKey());

              	setting.resolveProps(props,bd);
              	
              	bd.implemented = setting.get<bool>("implemented");
              	bd.canBuildBase = setting.get<bool>("canBuildBase");
				bd.canAutoChoose = setting.get<bool>("canAutoChoose");
				bd.allowRoads = setting.get<bool>("allowRoads");
				bd.allowRivers = setting.get<bool>("allowRivers");
				bd.impassable = setting.get<bool>("impassable");
				bd.hasVirtualPlants = setting.get<bool>("hasVirtualPlants");
				bd.wildPlantsCareAboutLocalFertility = setting.get<bool>("wildPlantsCareAboutLocalFertility");
				bd.hasBedrock = setting.get<bool>("hasBedrock");
				bd.isExtremeBiome = setting.get<bool>("isExtremeBiome");
				
				bd.animalDensity = setting.get<float>("animalDensity");
				bd.plantDensity = setting.get<float>("plantDensity");
				bd.diseaseMtbDays = setting.get<float>("diseaseMtbDays");
				bd.settlementSelectionWeight = setting.get<float>("settlementSelectionWeight");
				bd.forageability = setting.get<float>("forageability");
				bd.wildPlantRegrowDays = setting.get<float>("wildPlantRegrowDays");
				bd.movementDifficulty = setting.get<float>("movementDifficulty");
				
				bd.foragedFood = setting.get<ThingDef>("foragedFood");
				//allowedPackAnimals,baseWeatherCommonalities,terrainsByFertility,wildPlants,wildAnimals,diseases,terrainPatchMakers
				List<ThingDef> allowedPackAnimals = setting.get<List<ThingDef>>("allowedPackAnimals");
            	if ( allowedPackAnimals != null ) {
					List<ThingDef> l = (List<ThingDef>)fAllowedPackAnimals.GetValue(bd);
					if ( l == null ) {
						l = new List<ThingDef>();
						fAllowedPackAnimals.SetValue(bd,l);
					} else l.Clear();
            		l.AddRange(allowedPackAnimals);
            	}
				
				List<SettingsInstance> ss = setting.get<List<SettingsInstance>>("baseWeatherCommonalities");
            	if ( ss != null ) {
            		if ( bd.baseWeatherCommonalities == null ) bd.baseWeatherCommonalities = new List<WeatherCommonalityRecord>();
            		else bd.baseWeatherCommonalities.Clear();
            		foreach ( SettingsInstance si in ss ) {
            			bd.baseWeatherCommonalities.Add(new WeatherCommonalityRecord() { 
                        	weather = si.get<WeatherDef>("weather"), 
                        	commonality = si.get<float>("commonality") });
            		}
            	}
				
				ss = setting.get<List<SettingsInstance>>("terrainsByFertility");
            	if ( ss != null ) {
            		if ( bd.terrainsByFertility == null ) bd.terrainsByFertility = new List<TerrainThreshold>();
            		else bd.terrainsByFertility.Clear();
            		foreach ( SettingsInstance si in ss ) {
            			bd.terrainsByFertility.Add(new TerrainThreshold() { 
                        	terrain = si.get<TerrainDef>("terrain"), 
                        	min = si.get<float>("min"),
							max = si.get<float>("max") });
            		}
            	}
				
				ss = setting.get<List<SettingsInstance>>("wildPlants");
            	if ( ss != null ) {
					List<BiomePlantRecord> l = (List<BiomePlantRecord>)fWildPlants.GetValue(bd);
					if ( l == null ) {
						l = new List<BiomePlantRecord>();
						fWildPlants.SetValue(bd,l);
					} else l.Clear();
            		foreach ( SettingsInstance si in ss ) {
            			l.Add(new BiomePlantRecord() { 
                        	plant = si.get<ThingDef>("plant"), 
                        	commonality = si.get<float>("commonality")});
            		}
            	}
              	
				ss = setting.get<List<SettingsInstance>>("wildAnimals");
            	if ( ss != null ) {
					List<BiomeAnimalRecord> l = (List<BiomeAnimalRecord>)fWildAnimals.GetValue(bd);
					if ( l == null ) {
						l = new List<BiomeAnimalRecord>();
						fWildAnimals.SetValue(bd,l);
					} else l.Clear();
            		foreach ( SettingsInstance si in ss ) {
            			l.Add(new BiomeAnimalRecord() {
                        	animal = si.get<PawnKindDef>("animal"), 
                        	commonality = si.get<float>("commonality")});
            		}
            	}
				
				ss = setting.get<List<SettingsInstance>>("diseases");
            	if ( ss != null ) {
					List<BiomeDiseaseRecord> l = (List<BiomeDiseaseRecord>)fDiseases.GetValue(bd);
					if ( l == null ) {
						l = new List<BiomeDiseaseRecord>();
						fDiseases.SetValue(bd,l);
					} else l.Clear();
            		foreach ( SettingsInstance si in ss ) {
            			l.Add(new BiomeDiseaseRecord() {
                        	diseaseInc = si.get<IncidentDef>("diseaseInc"), 
                        	commonality = si.get<float>("commonality")});
            		}
            	}
				
				ss = setting.get<List<SettingsInstance>>("terrainPatchMakers");
            	if ( ss != null ) {
            		if ( bd.terrainPatchMakers == null ) bd.terrainPatchMakers = new List<TerrainPatchMaker>();
            		else bd.terrainPatchMakers.Clear();
            		foreach ( SettingsInstance si in ss ) {
            			TerrainPatchMaker tpm = new TerrainPatchMaker() {
            				perlinFrequency = si.get<float>("perlinFrequency"),
							perlinLacunarity = si.get<float>("perlinLacunarity"),
							perlinPersistence = si.get<float>("perlinPersistence"),
							minFertility = si.get<float>("minFertility"),
							maxFertility = si.get<float>("maxFertility"),
							perlinOctaves = si.get<int>("perlinOctaves"),
							minSize = si.get<int>("minSize")
            			};
            			List<SettingsInstance> sss = si.get<List<SettingsInstance>>("thresholds");
            			if ( sss != null ) {
        					foreach ( SettingsInstance sis in sss ) {
		            			tpm.thresholds.Add(new TerrainThreshold() { 
		                        	terrain = sis.get<TerrainDef>("terrain"), 
		                        	min = sis.get<float>("min"),
									max = sis.get<float>("max") });
		            		}
            			}
            			bd.terrainPatchMakers.Add(tpm);
            		}
            	}
				
              	FieldInfo f = typeof(BiomeDef).GetField("cachedAnimalCommonalities", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
              	f.SetValue(bd,null);
              	f = typeof(BiomeDef).GetField("cachedDiseaseCommonalities", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
              	f.SetValue(bd,null);
              	f = typeof(BiomeDef).GetField("cachedLowestWildPlantOrder", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
              	f.SetValue(bd,null);
              	f = typeof(BiomeDef).GetField("cachedMat", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
              	f.SetValue(bd,null);
              	f = typeof(BiomeDef).GetField("cachedMaxWildPlantsClusterRadius", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
              	f.SetValue(bd,null);
              	f = typeof(BiomeDef).GetField("cachedPlantCommonalitiesSum", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
              	f.SetValue(bd,null);
              	f = typeof(BiomeDef).GetField("cachedPlantCommonalities", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
              	f.SetValue(bd,null);
              	f = typeof(BiomeDef).GetField("cachedWildPlants", BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public);
              	f.SetValue(bd,null);
              	
              	bd.ClearCachedData();
				});
			
			props.visualization = new SettingsVisualizationMenuSelectEdit<BiomeDef>("Biomes","Select Biome",()=>DefDatabase<BiomeDef>.AllDefsListForReading,
               def => def.defName, DefLabelProducerGeneric<BiomeDef>.LabelProducerNotNull,key => DefDatabase<BiomeDef>.GetNamed(key),
               f=>"Edit \""+((ISettingsInstance)f.flowScope["edit"]).getLabel()+"\"");
		}
		
		private static IEnumerable<T> getEnumEntriesFromNames<T>(IEnumerable<string> names) {
        	List<T> entriesMatching = new List<T>();
        	if ( names == null || !names.Any() ) { entriesMatching.TrimExcess(); return entriesMatching; }
        	Array entries = Enum.GetValues(typeof(T));
        	foreach ( string name in names ) {
        		foreach ( T entry in entries ) {
        			if ( Enum.GetName(typeof(T),entry).Equals(name) ) {
        				entriesMatching.Add(entry);
        				break;
        			}
        		}
        	}
    		entriesMatching.TrimExcess();
        	return entriesMatching;
		}
		
		private static List<string> GetEnumFlagsNames<T>(T efs) where T : struct, IConvertible, IComparable, IFormattable {
			int iefs = efs.ToInt32(CultureInfo.CurrentCulture);
			List<string> names = new List<string>();
			Array arr = Enum.GetValues(typeof(T));
			foreach ( T o in arr ) {
				int io = o.ToInt32(CultureInfo.CurrentCulture);
				if ( io == 0 ) {
					if ( iefs == 0 ) {
						names.Add(Enum.GetName(typeof(T),o));
					}
				} else if ( (iefs & io) == io ) {
					names.Add(Enum.GetName(typeof(T),o));
				}
			}
			return names;
		}
	}
	
	static class DefLabelProducerGeneric<T> where T : Def {
		static public Func<T, String> LabelProducerNotNullUnique = o=>((Def)o).label.NullOrEmpty() ? ((Def)o).defName : ((Def)o).label+" ("+((Def)o).defName+")";
		static public Func<T, String> LabelProducerNotNull = o=>((Def)o).label.NullOrEmpty() ? ((Def)o).defName : ((Def)o).label;
		static public Func<T, String> LabelProducerUnique = o=>o != null ? ((Def)o).label.NullOrEmpty() ? ((Def)o).defName : ((Def)o).label+" ("+((Def)o).defName+")" : "NONE";
		static public Func<T, String> LabelProducer = o=>o != null ? ((Def)o).label.NullOrEmpty() ? ((Def)o).defName : ((Def)o).label : "NONE";
	}
	
	//transform configuration to translation entries
	//^[^<]+<[^>]+>\("([^"]*)".*$
	//<X.\1></X.\1>\n<X.\1.tooltip></X.\1.tooltip>
}
